!$Id: avhrr_pixel_aerosol.f90,v 1.6 2009/06/11 22:44:12 heidinger Exp $
module AEROSOL_PROPERTIES
!------------------------------------------------------------
!@***h* PATMOSX/avhrr_pixel_aerosol.f90
! 
! NAME
!   avhrr_pixel_aerosol.f90
! 
! LOCATION
!   /patmosx/src/avhrr_pixel_aerosol.f90
! 
! PURPOSE
!  a module for pixel level aerosol properties from AVHRR
!  this is the single channel algorithm used in PATMOS for
!  aerosol optical in ch1 over the ocean
! 
! DESCRIPTION
!  a module for pixel level aerosol properties from AVHRR
!  this is the single channel algorithm used in PATMOS for
!  aerosol optical in ch1 over the ocean
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   Note, aerosol product quality flags are computed elsewhere.
!
!  Lookup tables provided by A. Ignatov (02/2005)
!
!  Public routines used in this module:
!  READ_AER_CH123A_REF_LUTS - read the lookup tables (LUTS)
!  PIXEL_AER_RET_OCEAN - perform the estimation of optical depth
!
! INPUTS_OUTPUTS
!  Input and output to these routines is through public arrays
!  through the AVHRR_PIXEL_COMMON module. 
!
! File I/O 
!  Logical units are opened and closed during LUT read.
!
! public variables used
!  alb1 - channel 1 relflectance (%)
!  alb2 - channel 2 reflectance (%)
!  alb3a - channel 3a reflectance (%)
!  satzen - sensor zenith angle (deg)
!  solzen - solar zenith angle (deg)
!  relaz - relative azimuth angle (deg)
!
! output passed through AVHRR_PIXEL_COMMON
!  aot1 - channel 1 aerosol optical depth
!  aot2 - channel 2 aerosol optical depth
!  aot3a - channel 3a aerosol optical depth
!
! aerosol lookup table variables
!  tau_lut - optical depth used in tables - dimension (ntau)
!  solzen_lut - solar zenith angles used in tables - dimension(nsolzen)
!  zen_lut - sensor zenith angles used in tables -dimension (nzen)
!  az_lut - relative azimuth angles used in tables - dimension(naz)
!  ref_lut_ch1_aer - ch1 reflectances (0-1) of lookup table - 
!                    dimension(nsolzen,nzen,ntau,naz)
!  ref_lut_ch2_aer - ch2 reflectances (0-1) of lookup table
!  ref_lut_ch3a_aer - ch3a reflectances (0-1) of lookup table
!
!
!@*****
!------------------------------------------------------------------------
! a module for pixel level aerosol properties from AVHRR
! this is the single channel algorithm used in PATMOS for
! aerosol optical in ch1 over the ocean
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! Lookup tables provided by A. Ignatov (02/2005)
!
! Input and output to these routines is through public arrays
! through the AVHRR_PIXEL_COMMON module. 
!
! Public routines used in this module:
!  READ_AER_CH123A_REF_LUTS - read the lookup tables (LUTS)
!  PIXEL_AER_RET_OCEAN - perform the estimation of optical depth
!
! Private routines used in this module:
!  AER_RET - routine used in estimation aerosol optical depth.
!
! File I/O 
! Logical units are opened and closed during LUT read.
!
! public variables used
!  alb1 - channel 1 relflectance (%)
!  alb2 - channel 2 reflectance (%)
!  alb3a - channel 3a reflectance (%)
!  satzen - sensor zenith angle (deg)
!  solzen - solar zenith angle (deg)
!  relaz - relative azimuth angle (deg)
!
! output passed through AVHRR_PIXEL_COMMON
! aot1 - channel 1 aerosol optical depth
! aot2 - channel 2 aerosol optical depth
! aot3a - channel 3a aerosol optical depth
!
! aerosol lookup table variables
! tau_lut - optical depth used in tables - dimension (ntau)
! solzen_lut - solar zenith angles used in tables - dimension(nsolzen)
! zen_lut - sensor zenith angles used in tables -dimension (nzen)
! az_lut - relative azimuth angles used in tables - dimension(naz)
! ref_lut_ch1_aer - ch1 reflectances (0-1) of lookup table - 
!                    dimension(nsolzen,nzen,ntau,naz)
! ref_lut_ch2_aer - ch2 reflectances (0-1) of lookup table
! ref_lut_ch3a_aer - ch3a reflectances (0-1) of lookup table
!
! Note, aerosol product quality flags are computed elsewhere.
!
!------------------------------------------------------------------------
 use AVHRR_PIXEL_COMMON
 use CONSTANTS
 use FILE_UTILITY
 use NUMERICAL_ROUTINES
 implicit none
 private
 public:: READ_AER_CH123A_REF_LUTS,&
          PIXEL_AER_RET_OCEAN
 private:: AER_RET

!------------------------------------------------------------------------
 integer, parameter, private:: nzen = 15, naz = 19, nsolzen = 15, ntau = 7

!--- aerosol lut variables
 real, dimension(ntau), save, private::tau_lut
 real, dimension(nsolzen), save, private::solzen_lut
 real, dimension(nzen), save, private::zen_lut
 real, dimension(naz), save, private::az_lut
 real, save, public:: lambda_ref
 real, save, private:: delta_solzen,delta_zen,delta_az,delta_tau
 real, dimension(nsolzen,nzen,ntau,naz), save, private::  ref_lut_ch1_aer, &
                                                 ref_lut_ch2_aer,ref_lut_ch3a_aer

!--- other module-wide variables
 real, dimension(ntau), save, private::temp_ref_lut

 contains

!----------------------------------------------------------------------------
! grid_cell aerosol retrieval
!
! This uses linear in tau and angle interpolation - should improve this base
! on scheme used for operational aerosol retrieval.
!
! temp_ref_lut is a reflectance vector of dimension(ntau) that is already
!         interpolated in the angular dimensions.
!
!----------------------------------------------------------------------------
subroutine PIXEL_AER_RET_OCEAN(jmin,jmax)

   integer, intent(in):: jmin, jmax

   integer:: i,j,it,iu,iv,ifail
   real(kind=real4):: v,t,u,missing_value
 
   missing_value = -999.0
   ifail = 0

!--------------------------------------------------------------------------
! loop over pixels in scanlines
!--------------------------------------------------------------------------
scan_loop: do j = jmin, jmax + jmin - 1

pixel_loop:  do i = 1, num_pix

!--- check for a bad pixel
if (bad_pixel_mask(i,j) == sym%YES) then 
    cycle
endif

!--- check for valid illumination and surface

if ((solzen(i,j) > 87.0) .or. (sfc_type(i,j) /= sym%WATER_SFC) .or. (snow(i,j) /= sym%NO_SNOW)) then

   aot1(i,j) = missing_value_real4
   aot2(i,j) = missing_value_real4
   aot3a(i,j) = missing_value_real4


 else

  it = max(1,min(nsolzen-1,1+int( (solzen(i,j)-solzen_lut(1)) / (delta_solzen))))
  iu = max(1,min(nzen-1,1+int( (satzen(i,j)-zen_lut(1)) / (delta_zen))))
  iv = max(1,min(naz-1,1+int( min(relaz(i,j),360.0-relaz(i,j)) / (delta_az)) ))
  u = (satzen(i,j) - zen_lut(iu))/(zen_lut(iu+1)-zen_lut(iu))
  v = (min(relaz(i,j),360.0-relaz(i,j)) - az_lut(iv))/(az_lut(iv+1)-az_lut(iv))
  t =  (solzen(i,j) - solzen_lut(it))/(solzen_lut(it+1)-solzen_lut(it))

  ifail = 0
 
 
  !--  channel 1
  temp_ref_lut = (1.0-t)*(1.0-u)*(1.0-v)*ref_lut_ch1_aer(it,iu,:,iv) +  &
                 (1.0-t)*(1.0-u)*(v)*ref_lut_ch1_aer(it,iu,:,iv+1) +  &
                 (t)*(1.0-u)*(1.0-v)*ref_lut_ch1_aer(it+1,iu,:,iv) +  &
                 (t)*(1.0-u)*(v)*ref_lut_ch1_aer(it+1,iu,:,iv+1) +  &
                 (1.0-t)*(u)*(1.0-v)*ref_lut_ch1_aer(it,iu+1,:,iv) +  &
                 (1.0-t)*(u)*(v)*ref_lut_ch1_aer(it,iu+1,:,iv+1) +  &
                 (t)*(u)*(1.0-v)*ref_lut_ch1_aer(it+1,iu+1,:,iv) +  &
                 (t)*(u)*(v)*ref_lut_ch1_aer(it+1,iu+1,:,iv+1)
  call AER_RET(alb1(i,j)/100.0, aot1(i,j))

  !--  channel 2
  temp_ref_lut = (1.0-t)*(1.0-u)*(1.0-v)*ref_lut_ch2_aer(it,iu,:,iv) +  &
                 (1.0-t)*(1.0-u)*(v)*ref_lut_ch2_aer(it,iu,:,iv+1) +  &
                 (t)*(1.0-u)*(1.0-v)*ref_lut_ch2_aer(it+1,iu,:,iv) +  &
                 (t)*(1.0-u)*(v)*ref_lut_ch2_aer(it+1,iu,:,iv+1) +  &
                 (1.0-t)*(u)*(1.0-v)*ref_lut_ch2_aer(it,iu+1,:,iv) +  &
                 (1.0-t)*(u)*(v)*ref_lut_ch2_aer(it,iu+1,:,iv+1) +  &
                 (t)*(u)*(1.0-v)*ref_lut_ch2_aer(it+1,iu+1,:,iv) +  &
                 (t)*(u)*(v)*ref_lut_ch2_aer(it+1,iu+1,:,iv+1)
  call AER_RET(alb2(i,j)/100.0, aot2(i,j))

  !--  channel 3a
  temp_ref_lut = (1.0-t)*(1.0-u)*(1.0-v)*ref_lut_ch3a_aer(it,iu,:,iv) +  &
                 (1.0-t)*(1.0-u)*(v)*ref_lut_ch3a_aer(it,iu,:,iv+1) +  &
                 (t)*(1.0-u)*(1.0-v)*ref_lut_ch3a_aer(it+1,iu,:,iv) +  &
                 (t)*(1.0-u)*(v)*ref_lut_ch3a_aer(it+1,iu,:,iv+1) +  &
                 (1.0-t)*(u)*(1.0-v)*ref_lut_ch3a_aer(it,iu+1,:,iv) +  &
                 (1.0-t)*(u)*(v)*ref_lut_ch3a_aer(it,iu+1,:,iv+1) +  &
                 (t)*(u)*(1.0-v)*ref_lut_ch3a_aer(it+1,iu+1,:,iv) +  &
                 (t)*(u)*(v)*ref_lut_ch3a_aer(it+1,iu+1,:,iv+1)
  call AER_RET(alb3a(i,j)/100.0, aot3a(i,j))

 endif            !end of check for appropriate retrieval conditions

 end do pixel_loop
end do scan_loop
 
end subroutine PIXEL_AER_RET_OCEAN

!------------------------------------------------------------
! Actual routine to estimate aerosol optical depth from LUT
!------------------------------------------------------------
subroutine AER_RET(y,aot)

real, intent(in):: y
real, intent(out):: aot
integer:: is
real:: s

!--- linear interp
  call LOCATE(temp_ref_lut,ntau,y,is)
  is = max(1,min(ntau-1,is))
  s = (y - temp_ref_lut(is))/(temp_ref_lut(is+1) - temp_ref_lut(is))
  aot = (1.0-s)*tau_lut(is) + s*tau_lut(is+1) 
end subroutine AER_RET

!------------------------------------------------------------------------------
! READ IN THE NESDIS OPERATIONAL LOOKUP TABLES FOR CH1, CH2 and CH3A OCEANIC AEROSOL
!------------------------------------------------------------------------------
 subroutine READ_AER_CH123A_REF_LUTS(ancil_data_dir,satnum)
  character (len=*), intent(in):: ancil_data_dir
  integer, intent(in):: satnum
  character (len=128):: ch1_aer_lut_file,ch2_aer_lut_file,ch3a_aer_lut_file
  character (len=128):: header,basename
  integer:: isolzen,izen,itau
  integer:: nzen1,naz1,nsolzen1,ntau1
  integer:: nzen2,naz2,nsolzen2,ntau2
  integer:: nzen3,naz3,nsolzen3,ntau3
  real(kind=real4):: cos_solzen
  integer:: ch1_lun
  integer:: ch2_lun
  integer:: ch3a_lun


  if (satnum == 5) then
      basename = "Ch1F.lut.dat-NOAATN"
  elseif (satnum == 6) then
      basename = "Ch1F.lut.dat-NOAA06"
  elseif(satnum == 7) then
      basename = "Ch1F.lut.dat-NOAA07"
  elseif(satnum == 8) then
      basename = "Ch1F.lut.dat-NOAA08"
  elseif(satnum == 9) then
      basename = "Ch1F.lut.dat-NOAA09"
  elseif(satnum == 10) then
      basename = "Ch1F.lut.dat-NOAA10"
  elseif(satnum == 11) then
      basename = "Ch1F.lut.dat-NOAA11"
  elseif(satnum == 12) then
      basename = "Ch1F.lut.dat-NOAA12"
  elseif(satnum == 14) then
      basename = "Ch1F.lut.dat-NOAA14"
  elseif(satnum == 15) then
      basename = "Ch1F.lut.dat-NOAA15"
  elseif(satnum == 16) then
      basename = "Ch1F.lut.dat-NOAA16"
  elseif(satnum == 17) then
      basename = "Ch1F.lut.dat-NOAA17"
  elseif(satnum == 18) then
      basename = "Ch1F.lut.dat-NOAA18"
  elseif(satnum == 19) then
      basename = "Ch1F.lut.dat-NOAA19"
  elseif(satnum == 2) then
      basename = "Ch1F.lut.dat-MetOpA"
  else
      print *, "no aerosol lookup tables for satellite number = ", satnum
      print *, "stopping"
      stop
  endif

  ch1_aer_lut_file = trim(ancil_data_dir)//"luts/aerosol/"//trim(basename)//"_1"
  ch2_aer_lut_file = trim(ancil_data_dir)//"luts/aerosol/"//trim(basename)//"_2"
  ch3a_aer_lut_file = trim(ancil_data_dir)//"luts/aerosol/"//trim(basename)//"_3"

  if (satnum < 15) then
    ch3a_aer_lut_file = trim(ancil_data_dir)//"luts/aerosol/"//"Ch1F.lut.dat-NOAA15_3"
  endif

  ch1_lun = GET_LUN()
  open(unit=ch1_lun,file=ch1_aer_lut_file,status="old",position="rewind",action="read")
  ch2_lun = GET_LUN()
  open(unit=ch2_lun,file=ch2_aer_lut_file,status="old",position="rewind",action="read")
  ch3a_lun = GET_LUN()
  open(unit=ch3a_lun,file=ch3a_aer_lut_file,status="old",position="rewind",action="read")

   read(unit=ch1_lun,fmt="(4i5)") nzen1, naz1, nsolzen1,ntau1
   read(unit=ch2_lun,fmt="(4i5)") nzen2, naz2, nsolzen2,ntau2
   read(unit=ch3a_lun,fmt="(4i5)") nzen3, naz3, nsolzen3,ntau3

   ref_lut_ch1_aer = 0.0
   ref_lut_ch2_aer = 0.0
   ref_lut_ch3a_aer = 0.0
   tau_lut = 0.00
   solzen_lut = 0.00
   zen_lut = 0.00
   az_lut = 0.00
   lambda_ref = 0.63
   delta_tau = 0.15

   temp_ref_lut = 0.0

   delta_solzen = 90.0 / nsolzen
   do isolzen = 1,nsolzen
    solzen_lut(isolzen) = (isolzen-1)*delta_solzen
   enddo


   read(unit=ch1_lun, fmt=*) tau_lut
   read(unit=ch2_lun, fmt=*) tau_lut
   read(unit=ch3a_lun, fmt=*) tau_lut

   tau_lut = tau_lut * delta_tau

   do isolzen = 1,nsolzen
       cos_solzen = cos(solzen_lut(isolzen)*dtor)
     do itau = 1, ntau
       read(unit=ch1_lun,fmt="(a)") header
       read(unit=ch2_lun,fmt="(a)") header
       read(unit=ch3a_lun,fmt="(a)") header

       read(unit=ch1_lun,fmt="(10f6.0,/,9f6.0)") az_lut
       read(unit=ch2_lun,fmt="(10f6.0,/,9f6.0)") az_lut
       read(unit=ch3a_lun,fmt="(10f6.0,/,9f6.0)") az_lut
      
       do izen = 1, nzen
        read(unit=ch1_lun,fmt=*) zen_lut(izen), ref_lut_ch1_aer(isolzen,izen,itau,1:5)
        read(unit=ch1_lun,fmt=*) ref_lut_ch1_aer(isolzen,izen,itau,6:10)
        read(unit=ch1_lun,fmt=*) ref_lut_ch1_aer(isolzen,izen,itau,11:15)
        read(unit=ch1_lun,fmt=*) ref_lut_ch1_aer(isolzen,izen,itau,16:19)

        read(unit=ch2_lun,fmt=*) zen_lut(izen), ref_lut_ch2_aer(isolzen,izen,itau,1:5)
        read(unit=ch2_lun,fmt=*) ref_lut_ch2_aer(isolzen,izen,itau,6:10)
        read(unit=ch2_lun,fmt=*) ref_lut_ch2_aer(isolzen,izen,itau,11:15)
        read(unit=ch2_lun,fmt=*) ref_lut_ch2_aer(isolzen,izen,itau,16:19)

        read(unit=ch3a_lun,fmt=*) zen_lut(izen), ref_lut_ch3a_aer(isolzen,izen,itau,1:5)
        read(unit=ch3a_lun,fmt=*) ref_lut_ch3a_aer(isolzen,izen,itau,6:10)
        read(unit=ch3a_lun,fmt=*) ref_lut_ch3a_aer(isolzen,izen,itau,11:15)
        read(unit=ch3a_lun,fmt=*) ref_lut_ch3a_aer(isolzen,izen,itau,16:19)

       enddo 
     enddo
     ref_lut_ch1_aer(isolzen,:,:,:) = ref_lut_ch1_aer(isolzen,:,:,:)/cos_solzen
     ref_lut_ch2_aer(isolzen,:,:,:) = ref_lut_ch2_aer(isolzen,:,:,:)/cos_solzen
     ref_lut_ch3a_aer(isolzen,:,:,:) = ref_lut_ch3a_aer(isolzen,:,:,:)/cos_solzen
   enddo

  close(unit=ch1_lun)
  close(unit=ch2_lun)
  close(unit=ch3a_lun)

   delta_zen = zen_lut(2) - zen_lut(1)
   delta_az = az_lut(2) - az_lut(1)
 end subroutine READ_AER_CH123A_REF_LUTS

end module AEROSOL_PROPERTIES
