! $Id: cloud_mask.f90,v 1.38 2010/05/30 16:06:18 heidinger Exp $
module CLOUD_MASK
!---------------------------------------------------------------------
!@***h* PATMOSX/cloud_mask.f90
! 
! NAME
!   cloud_mask.f90
! 
! LOCATION
!   /patmosx/src/cloud_mask.f90
! 
! PURPOSE
!   This module performs a cloud mask 
! 
! DESCRIPTION
!  This module performs a cloud mask on 2 by 2 pixel arrays
!  resulting in cloud mask codes. In addition, this module 
!  it applies each test to each pixel and each test has its
!  own byte which is also an output of this routine.
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  none
!
! EXTERNALS
!  Subroutines included in module: 
!   CLOUD_MASK_3x3
!
!  DEPENDENCIES:
!	CONSTANTS
!	CLOUD_MASK_THRESHOLDS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!	CLOUD_MASK_ROUTINES
!
!@*****
!------------------------------------------------------------------------
! FUNCTION:
! This module performs a cloud mask on 2 by 2 pixel arrays
! resulting in cloud mask codes. In addition, this module 
! it applies each test to each pixel and each test has its
! own byte which is also an output of this routine. 
!
! Subroutines included in module: 
!   CLOUD_MASK_3x3
!
! DEPENDENCIES:
!	CONSTANTS
!	CLOUD_MASK_THRESHOLDS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!	CLOUD_MASK_ROUTINES
!
! SIDE EFFECTS: None
!
!----------------------------------------------------------------------
 use CONSTANTS
 use AVHRR_PIXEL_COMMON
 use NWP_COMMON
 use CLOUD_MASK_ROUTINES
 use CALIBRATION_CONSTANTS
 use NUMERICAL_ROUTINES
 use AWG_CLOUD_MASK_TESTS

 implicit none
 private
 public:: CLOUD_MASK_3x3

!--------------------------------------------------------------------
! define static thresholds here - should move to an include file
!--------------------------------------------------------------------
!
!--- include fixed thresholds via this include statement
!
!INCLUDE "awg_cloud_mask_tests_thresholds.inc"
include 'awg_cloud_mask_tests_thresholds.inc'

!---- uniformity thresholds - for a 3x3 box, this corresponds to CLAVR-1
 real(kind=real4), private, parameter:: &
                                        ref2_clr_uni_thresh_land = 0.20, &
                                        ref2_clr_uni_thresh_ocean = 0.05, &
                                        alb3b1_min_clr_ocean_lo = 0.5, &     !min expected alb3b/1 for clear glint
                                        alb3b1_min_clr_ocean_hi = 0.8, &     !min expected alb3b/1 for clear glint
                                        ch3a_polar_thresh   =   20.0          !ch3a threshold for polar areas
 real(kind=real4), private, parameter:: rfmft_thresh_ocean = 0.3
 real(kind=real4), private, parameter:: rfmft_thresh_land = 0.80
 real(kind=real4), private, parameter:: rtct_thresh_land = 5.0
 real(kind=real4), private, parameter:: rtct_thresh_ocean = 1.5
 real(kind=real4), private, parameter:: rtct_thresh_snow = 6.5
 real(kind=real4), private, parameter:: rtct_bt4_median_diff_thresh = 0.5

 contains
!------------------------------------------------------------------------
! SUBROUTINE NAME: CLOUD_MASK_3x3
!
!  Primary Cloud Mask Routine for OCEAN
!  Note this is longer assumes 2 scanlines
!
! INPUTS:
!  sc_id_new = spacecraft id
!  j1 - the first scan index to process
!  j2 - the last scan index to process
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!  cld_mask = 2 byte cloud mask for level 1b output
!		  cloud mask codes:
!			0 - clear
!			1 - probably clear
!			2 - probably cloudy
!			3 - cloud
!
!  mask - results of each cloud mask test. Byte description below:
!
! in addition to a 4-level cloud mask, CLAVR-x produces a series of 
! tests to allow for interrogation or modification of the cloud mask
!
! the test results are stored in the cld_test_vector array and the
! current ordering of tests on cld_test_vector is as follows
!
!--- ancillary data flags
! 1 - cloud mask attempted
! 2 - day
! 3 - terminator
! 4 - land
! 5 - coast
! 6 - glint
! 7 - desert
! 8 - snow
! 9 - cold surface
!--- clear-sky spatial uniformity tests
! 10 - RUT
! 11 - TUT
!--- cloud tests using thermal emission
! 12 - RTCT
! 13 - ETROP
! 14 - PFMFT
! 15 - NFMFT
! 16 - RFMFT
! 17 - CIRH2O (not on avhrr)
! 18 - TEMPIR (not on avhrr)
! 19 - Terminator Stability Test (not on avhrr)
!--- cloud tests using solar reflectance 
! 20 - RGCT
! 21 - RVCT
!--- cloud tests using swir solar reflectance 
! 22 - NIRREF
! 23 - CIRREF  (not on avhrr)
!--- cloud tests using swir thermal emission
! 24 - EMISS4
! 25 - ULST
!--- restoral tests
! 26 - probably clear restoral for cloud_mask
! 27 - cloud edge (prob cloudy)
! 28 - prob cloudy to prob clear restoral for cloud_mask
!--- non-cloud tests
! 29 - smoke
! 30 - dust 
! 31 - shadow
! 32 - fire
!
! CALLING SEQUENCE:  call CLOUD_MASK_3x3(sc_id_new,jmin,jmax)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES:
!
! DEPENDENCIES:
!	CONSTANTS
!	CLOUD_MASK_THRESHOLDS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!	CLOUD_MASK_ROUTINES
!
! SIDE EFFECTS: None
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
! cld_mask 		= cloud mask output
! mask		= results from each cloud mask test
! solzen		= solar zenith angle
! satzen		= satellite zenith angle
! coszen		= cosine of the solar zenith angle
! alb1_std_3x3		= Channel 1 spatial uniformity
! bt4_std_3x3		= Channel 4 Brightness Tempurature spatial uniformity
! bt4			= Channel 4 Brightness Tempertature
! q_flag		= pixel quality flag
! cld_test_vector		= this holds information pertaining to the type of pixel
!				this is and how the cloud mask treats it.
! zsfc		= altitude of pixel
! lat			= latitude of pixel
! lon			= longitude of pixel
! ch3a_on		= whether or not AVHRR channel 3a is used (0 = False, 1 = True)
! bt3b		= Channel 3b Brightness Temperature
! bt3b_uni		= Channel 3b Brightness Temperature uniformity
! zsfc_std_3x3		= surface altitude uniformity
! bt3b5 		= Channel 3b Brightness Tempruarture - Channel 5 Brightness Temperature 
! airmass 		= amount of air the satellite has to look at to get to the pixel
! alb1_sfc 		= channel 1 surface reflectance
! ndvi_sfc 		= calculated NDVI
! alb3b		= channel 3b reflectance
! alb3a 		= channel 3a reflectance
! alb31 		= ratio of channel 3ab albedo / channel 1 albedo
! alb21 		= ratio of channel 2 albedo / channel 1 albedo
! bt3b5		= Brightness Tempurature channel 3b - Brightness Tempurature channel 5
! bt5 		= Channel 5 Brightness Tempertature
! bt45 		= Brightness Tempurature channel 4 - Brightness Tempurature channel 5
! t4min_clim 	= minmum channel 4 climatology
! tmpair_nwp 	= surface air temperature
! sst_mask 		= an sst used for cloud masking only
! ems3b 		= channel 3b emissivity
! chan_counts	= raw channel 1 counts
! ch1_dark_count	= "clear" raw channel 1 count
! shadow		= is pixel shadow
! fire		= is pixel fire
! dust		= is pixel dust
! snow_         = snow mask
! num_pix	 	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! Is_Cold_Ch3b = a flag that determines if the ch3b quantities
!           are spatially averaged to reduce noise.
! Is_Cold_Surface = a flag that identifies cold surfaces
! j = which scanline is being tested
! i = which element is being tested
! k = do loop variable
! Is_Day = day/night flag (1 = day, 0 = night)
! Is_Coast = coast flag (1 = coast)
! Is_Glint = glint flag (1 = glint)
! Is_Snow = snow flag (1 = snow)
! Is_Land = land mask flag
! ems3b_temp 	= temporary ems3b array for pixels being analyized 
!
! Below are variables not used in this subroutine
! shadow, fire, dust
!
! -------------------------------------------------------------------------

 subroutine CLOUD_MASK_3x3(sc_id_new,jmin,jmax)

 integer, intent(in):: sc_id_new
 integer, intent(in):: jmin,jmax
 integer:: i,j,Is_Day,Is_Land, Is_Coast, Is_Glint, Is_Snow, Is_Desert, inwp, jnwp
 integer:: ilrc
 integer:: jlrc
 
 real(kind=real4):: ems3b_temp
 real(kind=real4):: bt45_temp
 real(kind=real4):: RGCT_Threshold

 integer:: Is_Cold_Ch3b
 integer:: Is_Cold_Surface
 integer:: itest
 integer(kind=int4):: Land_Type
 real(kind=real4):: Emiss_Tropo_Chn14

 real(kind=real4):: Emiss_Tropo_Chn14_LRC

 real(kind=real4):: bt4_median
 real(kind=real4):: bt4_mean

 integer:: i_min_bt4   !x-index relative to segment of local bt4 minimum 
 integer:: j_min_bt4   !y-index relative to segment of local bt4 minimum
 integer:: i_max_bt4   !x-index relative to segment of local bt4 maximum 
 integer:: j_max_bt4   !y-index relative to segment of local bt4 maximum

 !Subroutine to set up sensor specific thresholds
 ! This is because the old way did not set the values correctly inside the include file
 CALL SET_CMASK_THRESHOLDS('AVHRR')

!-------------------------------------------------------------------------
! 					EXECUTABLE CODE
!-------------------------------------------------------------------------

!--- Set ULST thresholds for this sensor

! loop over all scanlines
   j_loop: do j = jmin, jmax + jmin - 1

! loop over all pixels
   i_loop: do i = 1, num_pix

!   check for a bad pixel
    if (bad_pixel_mask(i,j) == sym%YES) then
         cld_test_vector(1,i,j) = sym%YES
         cycle
    endif

! This section checks to see if the pixel is valid

! check for a bad pixel
  cld_test_vector(1,i,j) = sym%NO

! thermal saturation
  if ((bt4_std_3x3(i,j) == 0.0) .and. (bt4(i,j) >= 325.0)) then
      cld_test_vector(1,i,j) = sym%YES
  endif

!--- solar impingement on sensor
  if ( (solzen(i,j) > 90.0) .and.  &
       (chan_counts(1,i,j)-ch1_dark_count >= 20)) then
      cld_test_vector(1,i,j) = sym%YES
  endif
  if ( (solzen(i,j) > 100.0) .and.  &
       (chan_counts(1,i,j)-ch1_dark_count >= 5)) then
      cld_test_vector(1,i,j) = sym%YES
  endif

!--- skip bad pixels
  if (cld_test_vector(1,i,j) == sym%YES) then
      cycle
  endif

!---- initialize outputs
!initialize cloud mask to prob clear (also our missing value)
 cld_mask(i,j) = 1

!---- set indices for thresholds
 inwp = i_nwp(i,j)
 jnwp = j_nwp(i,j)
 ilrc = i_lrc(i,j)
 jlrc = j_lrc(i,j)

 Is_Land = cld_test_vector(4,i,j)
 Is_Day = cld_test_vector(2,i,j)
 Is_Coast = cld_test_vector(5,i,j)
 Is_Glint = cld_test_vector(6,i,j)
 Is_Snow = sym%NO
 if (snow(i,j) /= sym%NO_SNOW) then
   Is_Snow = sym%YES
 endif
 Is_Desert = sym%NO
 if (desert_mask(i,j) == sym%YES) then
   Is_Desert = sym%YES
 endif

 !--- local alias
 Land_Type = land(i,j)
 Emiss_Tropo_Chn14 = emiss_11_tropo_rtm(i,j)
 if (ilrc > 0 .and. jlrc > 0) then 
   Emiss_Tropo_Chn14_LRC = emiss_11_tropo_nadir_rtm(ilrc,jlrc)
 else
   Emiss_Tropo_Chn14_LRC = missing_value_real4
 endif

 !---  flag cold surfaces
 Is_Cold_Surface = sym%NO 
 if (tsfc_rtm(i,j) < 265.0) then
    Is_Cold_Surface = sym%YES
 endif

 !  Handle noise potential in channel 3b
 Is_Cold_Ch3b = sym%NO

 ! Determine if pixel's ch3b is potentially noisy
 if (ch3a_on(j) == 0) then
     if (sc_id_new == 7)  then
        Is_Cold_Ch3b = sym%YES
     endif

     if (Is_Cold_Surface == sym%YES) then
       Is_Cold_Ch3b = sym%YES
     endif
 endif

  !--- choose size of box to search for maximum bt4
  if (gac == sym%YES) then
        i_min_bt4 = i_min_bt4_3x3(i,j)
        j_min_bt4 = j_min_bt4_3x3(i,j)
        i_max_bt4 = i_max_bt4_3x3(i,j)
        j_max_bt4 = j_max_bt4_3x3(i,j)
        bt4_median = bt4_median_3x3(i,j)
        bt4_mean = bt4_mean_3x3(i,j)
  else
        i_min_bt4 = i_min_bt4_3x3(i,j)
        j_min_bt4 = j_min_bt4_3x3(i,j)
        i_max_bt4 = i_max_bt4_3x3(i,j)
        j_max_bt4 = j_max_bt4_3x3(i,j)
        bt4_mean = bt4_mean_3x3(i,j)
        bt4_median = bt4_median_3x3(i,j)
  endif

!-------------------------------------------------------------------------
! Apply tests on spatial uniformity to filter clear pixels and 
! reclassify them as probably clear
!---------------------------------------------------------------------------

          !----------------------------------------------------------------------------
          !--- RUT: ch1 reflectance uniformity for clear pixels
          !----------------------------------------------------------------------------
          itest = 1 + Num_Ancil_Tests

          cld_test_vector(itest,i,j) = RUT_ROUTINE (&
                              Is_Land, &
                              Is_Snow, &
                              Is_Coast, &
                              alb1_sfc_white_sky(i,j), &
                              alb1_std_3x3(i,j), &
                              solzen(i,j))

          rut(i,j) = cld_test_vector(itest,i,j)

          !----------------------------------------------------------------------------
          !--- TUT: ch4 thermal uniformity for clear pixels
          !----------------------------------------------------------------------------
          itest = 2 + Num_Ancil_Tests

          cld_test_vector(itest,i,j) = TUT_ROUTINE (&
                              Is_Land, &
                              Is_Coast, &
                              bt4_std_3x3(i,j), &
                              zsfc_std_3x3(i,j))

          tut(i,j) = cld_test_vector(itest,i,j)

!----------------------------------------------------------------------------------
! If cold and ch3b is noisy, use local mean to reduce noise
!-----------------------------------------------------------------------------------
 ems3b_temp = ems3b(i,j)
 bt45_temp = bt45(i,j)
 if (Is_Cold_Ch3b == sym%YES) then
     ems3b_temp = ems3b_mean_3x3(i,j)
     bt45_temp = bt45_mean_3x3(i,j)
 endif

!======================================================================
! Thermal Emission Tests
!======================================================================

         !--------------------------------------------------------------
         !---  RTCT - look for cloudy pixels by local cold departures
         !---  of bt4 relative to its local maximum
         !--------------------------------------------------------------
         itest = first_ir_cld_mask_test + 0

         cld_test_vector(itest,i,j) = RTCT_ROUTINE(&
                              Is_Land,  &
                              Is_Coast,  &
                              Is_Snow, &
                              Is_Cold_Surface, &
                              bt4(i,j), &
                              bt4(i_min_bt4,j_min_bt4), &
                              bt4(i_max_bt4,j_max_bt4), &
                              zsfc_std_3x3(i,j))

         !--------------------------------------------------------------------
         !--- 11 micron Tropospheric Emissivity Test - ETROP
         !--------------------------------------------------------------------
         itest = First_IR_Cld_Mask_Test + 1

         cld_test_vector(itest,i,j) = ETROP_ROUTINE( &
                               Is_Snow, &
                               Is_Land, &
                               Is_Coast, &
                               Is_Desert, &
                               Is_Cold_Surface, &
                               Land_Type, &
                               bt4(i,j), &
                               bt_clear_ch4_rtm(i,j), &
                               Emiss_Tropo_Chn14, &
                               Emiss_Tropo_Chn14_LRC, &
                               bt4_std_3x3(i,j))

         !--------------------------------------------------------------------
         !--- PFMFT - Postive Four Minus Five Test
         !-------------------------------------------------------------------
         itest = First_IR_Cld_Mask_Test + 2

         !--- test for avhrr_1 series (no ch5)
         if (avhrr_1_flag == sym%NO) then

            cld_test_vector(itest,i,j) = PFMFT_ROUTINE( &
                      Is_Land, &
                      Is_Snow, &
                      Is_Cold_Surface, &
                      bt4(i,j), &
                      bt5(i,j), &
                      bt_clear_ch4_rtm(i,j), &
                      bt_clear_ch5_rtm(i,j))

         endif   !avhrr 1 check

         !------------------------------------------------------------
         !--- NFMFT - Negative Four Minus Five Test
         !------------------------------------------------------------
         itest = First_IR_Cld_Mask_Test + 3

         !--- test for avhrr_1 series (no ch5)
         if (avhrr_1_flag == sym%NO) then

           cld_test_vector(itest,i,j) = NFMFT_ROUTINE( &
                      Is_Land, &
                      Is_Snow, &
                      bt4(i,j), &
                      bt5(i,j), &
                      bt_clear_ch4_rtm(i,j), &
                      bt_clear_ch5_rtm(i,j))
         endif   !avhrr 1 check

         !--- turn off this test
         cld_test_vector(itest,i,j) = sym%NO


         !---------------------------------------------------------------
         !--- RFMFT - Relative Four Minus Five Test
         !--- 
         !--- thin cirrus should represent relatively 
         !--- decreasing t4 and increasing t45
         !---------------------------------------------------------------------
         itest = First_IR_Cld_Mask_Test + 4

         if (avhrr_1_flag == sym%NO) then

           cld_test_vector(itest,i,j) = RFMFT_ROUTINE( &
                      Is_Land, &
                      Is_Coast, &
                      bt4(i,j), &
                      bt5(i,j), &
                      bt45(i_max_bt4,j_max_bt4))
         endif

      !  pix_array_1(i,j) = (bt45(i_max_bt4,j_max_bt4) - bt45(i,j)) / &
      !                     (bt4(i_max_bt4,j_max_bt4) - bt4(i,j))
      !  pix_array_2(i,j) = bt45(i_max_bt4,j_max_bt4) - bt45(i,j)
!======================================================================
! Solar Reflectance Tests (aka VIS)
!======================================================================

         !------------------------------------------------------------------
         !--- Reflectance Gross Contrast Test  (RGCT)
         !------------------------------------------------------------------
         itest = First_Vis_Cld_Mask_Test + 0

         !--- determine threshold
         RGCT_Threshold = 44.0
         if (Is_Land == sym%YES .and. Is_Coast == sym%NO) then
           RGCT_Threshold =  20.0 + alb1_clear(i,j)
           !--- check in alb1 white sky is missing
           if (alb1_sfc_white_sky_min_3x3(i,j) <= 0.0) then
            RGCT_Threshold = 44.0
           endif
         endif

         if (Is_Land == sym%NO .and. Is_Coast == sym%NO) then
            RGCT_Threshold = 10.0 + alb1_clear(i,j)
         endif

         cld_test_vector(itest,i,j) = RGCT_ROUTINE(&
                                    Is_Snow, &
                                    Is_Glint, &
                                    solzen(i,j), &
                                    airmass(i,j), &
                                    alb1(i,j), &
                                    RGCT_Threshold)

         !--------------------------------------------------------------
         !---  RVCT - look for cloudy pixels by local bright departures
         !---  of alb1 relative to its local minimum
         !--------------------------------------------------------------
         itest = First_Vis_Cld_Mask_Test + 1

         cld_test_vector(itest,i,j) = RVCT_ROUTINE( &
                        Is_Coast, &
                        Is_Snow, &
                        Is_Land, &
                        alb1(i,j), &
                        scatangle(i,j), &
                        solzen(i,j), &
                        alb1_sfc_white_sky_std_3x3(i,j), &
                        alb1_min_3x3(i,j))

!======================================================================
! Short Wave Infrared that use SOLAR Tests 
!======================================================================

         !-------------------------------------------------------------
         !--- Channel 3a albedo test (C3AT)
         !-------------------------------------------------------------
         itest = First_SWIR_Solar_Cld_Mask_Test + 0

         !--- test to see if ch3a is on
         if (ch3a_on(j) == 1) then

            cld_test_vector(itest,i,j) = NIRREF_Chn5_ROUTINE( &
                         Is_Coast, &
                         Is_Snow, &
                         solzen(i,j), &
                         zsfc(i,j), &
                         ndsi(i,j), &
                         alb3a(i,j))

         endif

!======================================================================
! Short Wave Infrared that use Thermal Emission Tests 
!======================================================================

         !--------------------------------------------------------------------
         !--- EMISS4 Test (EMS3B)
         !--------------------------------------------------------------------
         itest = First_SWIR_Thermal_Cld_Mask_Test + 0

         if (ch3a_on(j) == 0 ) then     !ch3b must be on
             cld_test_vector(itest,i,j) = EMISS4_ROUTINE(&
                         Is_Glint,  &
                         Is_Coast,  &
                         Is_Land, &
                         Is_Snow, &
                         bt4(i,j), &
                         bt4_std_3x3(i,j), &
                         ems3b_temp, &
                         ems3b_clear_solar_rtm(i,j))
         endif

         !---------------------------------------------------------------------
         !--- ULST Test (Only at Night)
         !---------------------------------------------------------------------
         itest = First_SWIR_Thermal_Cld_Mask_Test + 1

         if (ch3a_on(j) == 0) then
            cld_test_vector(itest,i,j) = ULST_ROUTINE(&
                              Is_Land, &
                              Is_Day, &
                              Is_Snow, &
                              bt4(i,j), &
                              bt4_std_3x3(i,j), &
                              ems3b_temp,  &
                              ems3b_clear_solar_rtm(i,j),&
                              Emiss_Tropo_Chn14)
         endif

!close scanline and pixel loops
  end do i_loop
end do j_loop



!
! End subroutine CLOUD_MASK_3x3
!
 end subroutine CLOUD_MASK_3x3

!
! End module CLOUD_MASK
!
end module CLOUD_MASK
