! $Id: cloud_mask_routines.f90,v 1.13 2010/05/30 16:05:35 heidinger Exp $
module CLOUD_MASK_ROUTINES
!----------------------------------------------------------------------------
!@***h* PATMOSX/cloud_mask_routines.f90
! 
! NAME
!   cloud_mask_routines.f90
! 
! LOCATION
!   /patmosx/src/cloud_mask_routines.f90
! 
! PURPOSE
!   Calculation of the reflectance thresholds, dust detection,
!   fire detection, probably clear pixels, clear restorals, and putting bits into
!   cloud mask bits
! 
! DESCRIPTION
!  This module contains several subroutines needed for both cloud mask modules.
!  They include the calculation of the reflectance thresholds, dust detection,
!  fire detection, probably clear pixels, clear restorals, and putting bits into
!  cloud mask bits
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!  none
!
! EXTERNALS
!  Subroutines included in module: 
!       COMPUTE_2BIT_CLOUD_MASK
!       COMPUTE_2BIT_CLOUD_MASK_IR
!       COMPUTE_PROBABLY_CLOUDY
!       DUST_DETECTION_ALGORITHM
!       SMOKE_DETECTION_ALGORITHM
!       FIRE_DETECTION_ALGORITHM
!       COMPUTE_PROBABLY_CLEAR_RESTORAL
!       COMPUTE_PROBABLY_CLOUDY_RESTORAL
!
!  DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
!@*****
!------------------------------------------------------------------------
! FUNCTION:
! This module contains several subroutines needed for both cloud mask modules.
! They include the calculation of the reflectance thresholds, dust detection,
! fire detection, probably clear pixels, clear restorals, and putting bits into
! cloud mask bits
!
! Subroutines included in module: 
!       COMPUTE_2BIT_CLOUD_MASK
!       COMPUTE_2BIT_CLOUD_MASK_IR
!       COMPUTE_PROBABLY_CLOUDY
!       DUST_DETECTION_ALGORITHM
!       SMOKE_DETECTION_ALGORITHM
!       FIRE_DETECTION_ALGORITHM
!       COMPUTE_PROBABLY_CLEAR_RESTORAL
!       COMPUTE_PROBABLY_CLOUDY_RESTORAL
!
! DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!----------------------------------------------------------------------------
 use CONSTANTS
 use AVHRR_PIXEL_COMMON
 use NWP_COMMON
 use SURFACE_PROPERTIES
 use CALIBRATION_CONSTANTS
 use PLANCK
 implicit none
 private
 public:: &
          COMPUTE_2BIT_CLOUD_MASK, &
          COMPUTE_2BIT_CLOUD_MASK_IR, &
          COMPUTE_PROBABLY_CLOUDY,  &
          DUST_DETECTION_ALGORITHM, &
          SMOKE_DETECTION_ALGORITHM, &
          FIRE_DETECTION_ALGORITHM, &
          COMPUTE_PROBABLY_CLEAR_RESTORAL, &
          COMPUTE_PROBABLY_CLOUDY_RESTORAL, &
          PACK_CLOUD_MASK_BYTES

 INCLUDE 'awg_cloud_mask_tests_thresholds.inc'

 contains

!------------------------------------------------------------------------
! SUBROUTINE NAME: DUST_DETECTION_ALGORITHM
!
! Dust detection algorithm designed by Amato Evan - April 18, 2005
! Must be called after COMPUTE_DATA_MASK
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!
! CALLING SEQUENCE:  call DUST_DETECTION_ALGORITHM(j_min,num_scans_read)
!                       (called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES: Evan, A. T., A. K. Heidinger, and M. J. Pavolonis (2006)
!   Development of a new over-water advanced very high resolution radiometer
!   dust detection algorithm, Int. J. Remote Sens, 27(18), pp. 3903-3924.
!
! DEPENDENCIES:
!       CONSTANTS
!       CLOUD_MASK_THRESHOLDS
!       AVHRR_PIXEL_COMMON
!       NWP_COMMON
!       CLOUD_MASK
!
! SIDE EFFECTS: None
!
! MODIFICATIONS:
! 7/24/06 - modifed spatial filter to n = 2 from n = 1
! 2/7/07 - added aot2 threshold to initial criteria
! 2/7/07 - modifed spatial filter to n = 4 from n = 2
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
! cld_mask              = cloud mask output
! mask          = results from each cloud mask test
! solzen                = solar zenith angle
! bt4                   = Channel 4 Brightness Tempertature
! bad_pixel_mask        = pixel quality flag
! cld_test_vector             = this holds information pertaining to the type of pixel
!                               this is and how the cloud mask treats it.
! lat                   = latitude of pixel
! alb21                 = ratio of channel 2 albedo / channel 1 albedo
! bt45          = Brightness Tempurature channel 4 - Brightness Tempurature channel 5
! aot                   = channel 1 aerosol optical depth
! glintzen              = vector of glint zenith angle (degrees)
! num_pix               = number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j                     = which scanline is being tested
! i                     = which element is being tested
! ndusty                = number of dusty pixels
! ndusty_max    = total number of pixels done (scanlines * elements)
! i1                    = left element done
! i2                    = right element done
! j1                    = top scanline done
! j2                    = bottome pixel done
! n                     = size of box to check
! idust         = dust flag
!
!------------------------------------------------------------------------

 subroutine DUST_DETECTION_ALGORITHM(jmin,jmax)

  integer, intent(in):: jmin,jmax
  integer:: idust, i, j, ndusty, ndusty_max, n, j1,j2,i1,i2

! -------------------------------------------------------------------------
!                                       EXECUTABLE CODE
! -------------------------------------------------------------------------

!initalize dust bit in mask to "no dust"
 dust(:,jmin:jmax) = 0

!loop over all scanline
scan_loop: do j = jmin,jmax

!loop over all elements
pixel_loop:  do i = 1, num_pix

!--- check for a bad scan
  if (bad_pixel_mask(i,j) == sym%YES) then
    cycle
  endif

! set internal dust flag to "false"
   idust = 0

!---- Test for desert dust over water
 if (cld_test_vector(4,i,j) == sym%NO)  then

!--- Test works only during the day. This is to test whether or not pixel is "day"
   if (cld_test_vector(2,i,j) == sym%YES) then

!--- This finds dust and some clouds
     if( (aot1(i,j) < 4.5)    .and. (aot1(i,j) > 0.60)     .and. &
         (aot2(i,j) < 4.5)    .and. (aot2(i,j) > 0.60)     .and. &
         (bt4(i,j) > 270)     .and. (alb21(i,j) > 0.6)     .and. &
         (alb21(i,j) < 1.0)   .and. (abs(lat(i,j)) < 45.0) .and. &
         (solzen(i,j) < 70.0) .and. (glintzen(i,j) > 40.0) ) then


!--- Tests for a very low, dry dust layer, using BT4/BT5 ratio
       if( (bt45(i,j) <= -0.5) ) then
         idust = 1
       else

!--- Tests for "dark" dust using BT4/BT5 ratio and BT4
         if(  (cld_test_vector(9,i,j) == 0) .and. (cld_test_vector(10,i,j) == 0) ) then
           if( (aot1(i,j) <= 2.0) ) then
             if( (bt45(i,j) < 0.0) )then
               idust = 1
             else
               if( (bt45(i,j) < 3.5) .and. (bt4(i,j) > 287) ) then
                 idust = 1
               endif
             endif

!--- If it is not "dark" dust, we must test for "bright" dust
           else


!First test uses chan 1 albedo and BT4
             if( (bt4(i,j) > 285) .and. (alb1(i,j) < 55) ) then

!If the first criteria are satisfied,look at BT4/BT5 ratio
               if( (bt45(i,j) < 0.0) ) then
                 idust = 1
               else

!If BT4/BT5 ratio is greater than 0, use chan 1 aerosol opt. depth to check for dust
                 if( (bt45(i,j) < 0.7) .and. (aot1(i,j) < 3.25) ) then
                   idust = 1
                 endif
               endif

! Bright dust test end
             endif
           endif
         endif
       endif
     endif

! End of day/night test
   endif

! end of ocean/land dust test
 endif


!--- store idust into proper place in mask array
  dust(i,j) = idust

  end do pixel_loop !element loop end
end do scan_loop  !scanline loop end


!------------------------------------------------------
! spatial filter of dust
!------------------------------------------------------
!--- size of box to check (1 = 3x3 box, 2 = 5x5 box))
 n = 4

!--- store dust array into temporary array
temp_mask = dust

!loop over scan lines to perform checks
 scan_loop2: do j = jmin,jmax

!--- determine y-dimensions of array to check
       j1 = max(jmin,j-n)
       j2 = min(jmax,j+n)

!loop over elements to perform checks
 pixel_loop2:   do i = 1, num_pix

!--- determine x-dimensions of array to check
        i1 = max(1,i-n)
        i2 = min(num_pix,i+n)

!--- check to see if a dust pixel neighbors a non-dust pixel
       if (dust(i,j) == 1) then
         ndusty = sum(dust(i1:i2,j1:j2))
         ndusty_max = (i2-i1+1)*(j2-j1+1)

         if (ndusty < ndusty_max) then
            temp_mask(i,j) = 0
         endif
       endif
   end do pixel_loop2 !element loop end
 end do scan_loop2    !scanline loop end

 dust = temp_mask

!store data into cld_mask_vector
cld_test_vector(First_NonCld_Test+1,:,:) = dust


!--------------------------------------------------------------------------
! modify cloud mask
!--------------------------------------------------------------------------

!loop over elements to modify cloud mask
scan_loop3: do j = jmin,jmax

! loop over elements to modify cloud mask
 pixel_loop3: do i = 1, num_pix

!--- call all oceanic clear pixels classified as dust as partly clear
    if ((cld_mask(i,j) == sym%CLEAR) .and. (land_mask(i,j) == sym%NO) .and. (dust(i,j) == sym%YES) ) then
        cld_mask(i,j) = sym%PROB_CLEAR
        cld_mask_ir(i,j) = sym%PROB_CLEAR
    endif

! end loops
 end do pixel_loop3

end do scan_loop3

 end subroutine DUST_DETECTION_ALGORITHM
!------------------------------------------------------------------------
! SUBROUTINE NAME: SMOKE_DETECTION_ALGORITHM
!
!-------------------------------------------------------------------------
 subroutine SMOKE_DETECTION_ALGORITHM(jmin,jmax)

  integer, intent(in):: jmin,jmax
  integer:: i, j, n, i1, i2, j1, j2, nsmokey, nsmokey_max

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

!initalize dust bit in mask to "no dust"
 smoke(:,jmin:jmax) = sym%NO

!loop over all scanline
scan_loop: do j = jmin,jmax

  !no channel 3a option
  if (ch3a_on(j) == sym%YES) then
    cycle
  endif

!loop over all elements
pixel_loop:  do i = 1, num_pix

!--- check for a bad scan
  if (bad_pixel_mask(i,j) == sym%YES) then
    cycle
  endif

            if (sum(cld_test_vector(First_IR_Cld_Mask_Test:Last_IR_Cld_Mask_Test,i,j)) == 0) then   !clear in ir tests
              if ((solzen(i,j) < 70.0) .and. (sfc_type(i,j) == 0) .and. (snow(i,j) == 0)) then
                if (abs(tsfc(i,j)-tsfc_rtm(i,j)) < 5.0) then  !invisible in sst
                  if (abs( bt45(i,j) - (bt_clear_ch4_rtm(i,j) - bt_clear_ch5_rtm(i,j))) < 0.5) then   !no effet on t45

                 !--- thin smoke algorithm
                     if ((alb1_std_3x3(i,j) > 0.1) .and.  &
                         (bt45_std_3x3(i,j) < 0.2) .and. &
                         (alb3b_sfc(i,j) / alb1_sfc(i,j) < 0.5) .and. &
                         (ems3b_sfc(i,j) < 1.3) .and. &
                         (aot1(i,j) > 0.1) .and. &
                         (aot1(i,j) < 1.0)) then
                          smoke(i,j) = sym%YES
                     endif

               endif
              endif
             endif
           endif  !ir test check

           end do pixel_loop

end do scan_loop

!------------------------------------------------------
! spatial filter of smoke
!------------------------------------------------------
!--- size of box to check (1 = 3x3 box, 2 = 5x5 box))
 n = 2

!--- store dust array into temporary array
temp_mask = smoke

!loop over scan lines to perform checks
 scan_loop2: do j = jmin,jmax

!--- determine y-dimensions of array to check
       j1 = max(jmin,j-n)
       j2 = min(jmax,j+n)

!loop over elements to perform checks
 pixel_loop2:   do i = 1, num_pix
                                                                                                                                                                                                   
!--- determine x-dimensions of array to check
        i1 = max(1,i-n)
        i2 = min(num_pix,i+n)
                                                                                                                                                                                                   
!--- check to see if a dust pixel neighbors a non-dust pixel
       if (smoke(i,j) == 1) then
         nsmokey = sum(smoke(i1:i2,j1:j2))
         nsmokey_max = (i2-i1+1)*(j2-j1+1)
                                                                                                                                                                                                   
         if (nsmokey < nsmokey_max) then
            temp_mask(i,j) = 0
         endif
       endif
   end do pixel_loop2 !element loop end
 end do scan_loop2    !scanline loop end
                                                                                                                                                                                                   
  smoke = temp_mask

!--- save in mask array
cld_test_vector(First_NonCld_Test+0,:,:) = smoke

end subroutine SMOKE_DETECTION_ALGORITHM
!------------------------------------------------------------------------
! SUBROUTINE NAME: FIRE_DETECTION_ALGORITHM
!
! Fire detection algorithm - Based on Ivan Csiszars GAC work at ORA
! This algorithm as implemented is not tested.
! Algorithm must be called after COMPUTE_DATA_MASK
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!  
! CALLING SEQUENCE:  call FIRE_DETECTION_ALGORITHM(j_min,num_scans_read)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES: 
!
! DEPENDENCIES:
!	CONSTANTS
!	CLOUD_MASK_THRESHOLDS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!
! MODIFICATIONS: 
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
! data_bit		= results on how each pixel was processed
! alb1_std_3x3		= Channel 1 spatial uniformity
! bt4_std_3x3		= Channel 4 Brightness Tempurature spatial uniformity
! bt4			= Channel 4 Brightness Tempertature
! ch3a_on		= whether or not AVHRR channel 3a is used (0 = False, 1 = True)
! bt3b		= Channel 3b Brightness Temperature
! bt3b_std_3x3		= Channel 3b Brightness Temperature uniformity
! desert_mask 		= desert flag
! alb2		= channel 2 reflectance
! bt45 		= Brightness Tempurature channel 4 - Brightness Tempurature channel 5
! snow          = snow mask
! num_pix	 	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j 			= which scanline is being tested
! i			= which element is being tested
! iday = day/night flag (1 = day, 0 = night)
! icoast = coast flag (1 = coast)
! iglint = glint flag (1 = glint)
! isnow = snow flag (1 = snow)
! iland = land mask flag
! ifire = fire flag
!
!------------------------------------------------------------------------
 subroutine FIRE_DETECTION_ALGORITHM(jmin,jmax)

  integer, intent(in):: jmin,jmax
  integer:: ifire, i, j, iland, iday, icoast, iglint, isnow

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

!loop over scanlines
   scan_loop: do j = jmin, jmax

!loop over pixels
     pixel_loop: do i = 1, num_pix

!initalize land, day, coast, glint, snow flags
         iland = cld_test_vector(4,i,j)
         iday = cld_test_vector(2,i,j)
         icoast = cld_test_vector(5,i,j)
         iglint = glint_mask(i,j)
         isnow = sym%NO
         if (snow(i,j) /= sym%NO_SNOW) then
            isnow = sym%YES
         endif

!initialize fire flag to 0 (no fire)
         ifire = 0

!Fire detection algorithm only works channel 3b, during the day, on land, but not desert or coast
        if ((ch3a_on(j) == sym%NO).and. (iday == sym%YES) .and.  &
            (iland == sym%YES) .and.  &
            (desert_mask(i,j) == sym%NO) .and. (icoast == sym%NO))  then

!Fire detection thresholds                                                                                                                                                   
           if ( (bt3b(i,j) > 310.0) .and.  &
                (bt3b(i,j) - bt4(i,j) > 6.0) .and. &
                (bt4(i,j) > 260.0) .and. &
                (bt45(i,j) < 4.0) .and. &
                (alb2(i,j) < 25.0) .and. &
                (bt3b_std_3x3(i,j)> 8.0) .and. &
                (alb1_std_3x3(i,j)  < 10.0) .and. &
                (bt3b_std_3x3(i,j) / bt4_std_3x3(i,j) > 2.0) ) then

                 ifire = 1
            endif 
         endif

!--- update proper element of mask array
         fire(i,j) = ifire
         cld_test_vector(First_NonCld_Test+3,:,:) = fire(i,j)

!end loops
    end do pixel_loop
  end do scan_loop

!
! End subroutine FIRE_DETECTION_ALGORITHM
!
 end subroutine FIRE_DETECTION_ALGORITHM

!------------------------------------------------------------------------
! SUBROUTINE NAME: COMPUTE_PROBABLY_CLEAR_RESTORAL
!
! this routine interogates the cloud mask and determines if a probably clear
! pixel is actually clear.  This assumes that probably clear detection in 
! regions where there is no cloud detected are most likely clear
! the results are stored in mask(16)
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!  
! CALLING SEQUENCE:  call COMPUTE_PROBABLY_CLEAR_RESTORAL(j_min,num_scans_read)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES: 
!
! DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!
! MODIFICATIONS: 
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
! cld_mask 		= cloud mask output
! mask		= results from each cloud mask test
! data_bit		= results on how each pixel was processed
! num_pix	 	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j 			= which scanline is being tested
! i			= which element is being tested
! cld_mask_max	= maximum cloud mask value in box being tested
! i1			= left element done
! i2			= right element done
! j1			= top scanline done
! j2			= bottome pixel done
! n			= size of box to check
!
!------------------------------------------------------------------------
subroutine COMPUTE_PROBABLY_CLEAR_RESTORAL(jmin,jmax)

 integer, intent(in):: jmin, jmax
 integer:: i, j, i1,i2,j1,j2,n,cld_mask_max

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

n = 2   !2 = 5x5 window

!loop over scanlines
scan_loop: do j = jmin,jmax
                                                                                                                                                         
!--- determine y-dimensions of array to check
       j1 = max(jmin,j-n)
       j2 = min(jmax,j+n)

!loop over elements
pixel_loop:   do i = 1, num_pix

!--- skip bad pixels
        if (cld_test_vector(1,i,j) == 1) then
           cycle
        endif

!--- determine x-dimensions of array to check
        i1 = max(1,i-n)
        i2 = min(num_pix,i+n)

!--- check to see if a cloudy pixels is near a probably clear pixel
!--- if so, set a bit to 1 and restore to clear

       !--- do this for the full mask
       cld_test_vector(first_res_test+0,i,j) = 0
       if (cld_mask(i,j) == sym%PROB_CLEAR) then
         cld_mask_max = maxval(cld_mask(i1:i2,j1:j2))
         if (cld_mask_max == sym%CLEAR .or. cld_mask_max == sym%PROB_CLEAR) then   !if no cloudy or probably cloudy
            cld_test_vector(first_res_test + 0,i,j) = 1
            cld_mask(i,j) = sym%CLEAR
         endif
       endif

       !--- do this for the ir mask
       if (cld_mask_ir(i,j) == sym%PROB_CLEAR) then
         cld_mask_max = maxval(cld_mask_ir(i1:i2,j1:j2))
         if (cld_mask_max == sym%CLEAR .or. cld_mask_max == sym%PROB_CLEAR) then   !if no cloudy or probably cloudy
            cld_mask_ir(i,j) = sym%CLEAR
         endif
        endif
 
       !--- do this for the bayesian mask
       if (cld_mask_bayes(i,j) == sym%PROB_CLEAR) then
         cld_mask_max = maxval(cld_mask_bayes(i1:i2,j1:j2))
         if (cld_mask_max == sym%CLEAR .or. cld_mask_max == sym%PROB_CLEAR) then   !if no cloudy or probably cloudy
            cld_mask_bayes(i,j) = sym%CLEAR
         endif
        endif

!end loops                                                                                                                                                         
     end do pixel_loop
   end do scan_loop

!
! End subroutine COMPUTE_PROBABLY_CLEAR_RESTORAL
!
end subroutine COMPUTE_PROBABLY_CLEAR_RESTORAL

!------------------------------------------------------------------------
! SUBROUTINE NAME: COMPUTE_PROBABLY_CLOUDY_RESTORAL
!
! this routine interogates the cloud mask and determines if a probably cloudy
! pixel is actually clear.  This assumes that probably clear detection in 
! regions where there is no cloud detected are most likely clear
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!  
! CALLING SEQUENCE:  call COMPUTE_PROBABLY_CLOUDY_RESTORAL(j_min,num_scans_read)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES: 
!
! DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!
! MODIFICATIONS: 
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
! cld_mask 		= cloud mask output
! mask		= results from each cloud mask test
! data_bit		= results on how each pixel was processed
! num_pix	 	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j 			= which scanline is being tested
! i			= which element is being tested
! cld_mask_max	= maximum cloud mask value in box being tested
! i1			= left element done
! i2			= right element done
! j1			= top scanline done
! j2			= bottome pixel done
! n			= size of box to check
!
!------------------------------------------------------------------------
subroutine COMPUTE_PROBABLY_CLOUDY_RESTORAL(jmin,jmax)

 integer, intent(in):: jmin, jmax
 integer:: i, j, i1,i2,j1,j2,n,cld_mask_max

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

n = 2   !2 = 5x5 window

!loop over scanlines
scan_loop: do j = jmin,jmax
                                                                                                                                                         
!--- determine y-dimensions of array to check
       j1 = max(jmin,j-n)
       j2 = min(jmax,j+n)

!loop over elements
pixel_loop:   do i = 1, num_pix

!--- skip bad pixels
        if (cld_test_vector(1,i,j) == 1) then
           cycle
        endif

!--- determine x-dimensions of array to check
        i1 = max(1,i-n)
        i2 = min(num_pix,i+n)

!--- check to see if a cloudy pixels is near a probably clear pixel
!--- if so, set a bit to 1 and restore to clear

!--- do this for the full mask
       cld_test_vector(first_res_test+2,i,j) = 0
       if (cld_mask(i,j) == sym%PROB_CLOUDY) then
         cld_mask_max = maxval(cld_mask(i1:i2,j1:j2))
         if ((cld_mask_max /= sym%CLOUDY) .and. &  
            (cld_mask_max /= sym%PROB_CLOUDY)) then  
            cld_test_vector(first_res_test+2,i,j) = 1
            cld_mask(i,j) = sym%PROB_CLEAR
         endif
       endif

!--- do this for the ir mask
       if (cld_mask_ir(i,j) == sym%PROB_CLOUDY) then
         cld_mask_max = maxval(cld_mask_ir(i1:i2,j1:j2))
         if ((cld_mask_max /= sym%CLOUDY) .and. &  
            (cld_mask_max /= sym%PROB_CLOUDY)) then  
            cld_mask_ir(i,j) = sym%PROB_CLEAR
         endif
        endif

!end loops                                                                                                                                                         
     end do pixel_loop
   end do scan_loop

!
! End subroutine COMPUTE_PROBABLY_CLOUDY_RESTORAL
!
End Subroutine COMPUTE_PROBABLY_CLOUDY_RESTORAL

!------------------------------------------------------------------------
! SUBROUTINE NAME: COMPUTE_2BIT_CLOUD_MASK
!
! compute the 2bit cloud mask (0-3) based on the individual test results
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:

! CALLING SEQUENCE:  call COMPUTE_2BIT_CLOUD_MASK(j_min,num_scans_read)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES:
!
! DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!
! MODIFICATIONS:
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
!
! cld_mask 		= cloud mask output
! mask		= results from each cloud mask test
! n_cld_test 	= number of cloud tests on mask array
! num_pix	 	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j = which scanline is being tested
! i = which element is being tested
!
! -------------------------------------------------------------------------
subroutine COMPUTE_2BIT_CLOUD_MASK(jmin,jmax)
 integer, intent(in):: jmin,jmax
 integer:: i, j

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

!loop over elements
do i = 1, num_pix

!loop over scanlines
   do j = jmin,jmax


!--- set bad pixels to probably clear
     if (cld_test_vector(1,i,j) == sym%YES) then
       cld_mask(i,j) = sym%PROB_CLEAR
       cycle
     endif

!---- determine actual cloud mask value
     if (sum(cld_test_vector(First_Cld_Mask_Test:Last_Cld_Mask_Test,i,j)) == &
           Num_Cld_Mask_Tests*sym%NO) then  !check cld tests

       if (sum(cld_test_vector(First_Clr_Uni_Test:Last_Clr_Uni_Test,i,j)) == &
           Num_Clr_Uni_Tests*sym%NO) then   !check uni tests
         cld_mask(i,j) = sym%CLEAR
       else
         cld_mask(i,j) = sym%PROB_CLEAR
       endif

   else
         cld_mask(i,j) = sym%CLOUDY
   endif

!end loops
   end do
  end do

!
! End subroutine COMPUTE_2BIT_CLOUD_MASK
!
end subroutine COMPUTE_2BIT_CLOUD_MASK

!------------------------------------------------------------------------
! SUBROUTINE NAME: COMPUTE_PROBABLY_CLOUDY
!
! This routine uses a spatial filter to deterct eges of cloudes and 
! call them probably cloudy.  The current size is 3x3 box.  If a pixel is 
! cloudy (cld_mask = 3) but its neighbors are not, it is reclassified as
! probably cloudy (cld_mask = 2)
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!
! CALLING SEQUENCE:  call COMPUTE_PROBABLY_CLOUDY(j_min,num_scans_read)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES:
!
! DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!
! MODIFICATIONS:
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
! cld_mask 		= cloud mask output
! mask		= results from each cloud mask test
! temp 	= temporary mask array
! num_pix	 	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j 			= which scanline is being tested
! i			= which element is being tested
! cld_mask_max	= maximum cloud mask value in box being tested
! i1			= left element done
! i2			= right element done
! j1			= top scanline done
! j2			= bottome pixel done
! n			= size of box to check
! ncloudy		= sum of all of the mask values being tested
! ncloudy_max		= maximum value (if all values were cloudy pixels
!
! -------------------------------------------------------------------------
subroutine COMPUTE_PROBABLY_CLOUDY(jmin,jmax)
  integer, intent(in):: jmin, jmax
  integer:: i,j,i1,i2,j1,j2,ncloudy,ncloudy_max, n

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

!--- set size of box to check
   n = 1

!--- store cld mask in a temp array for modification
 temp_mask = cld_mask

!loop over scanlines
scan_loop: do j = jmin,jmax

!--- determine y-dimensions of array to check
       j1 = max(jmin,j-n)
       j2 = min(jmax,j+n)

!loop over elements of each scanline
pixel_loop:   do i = 1, num_pix

!--- determine x-dimensions of array to check
        i1 = max(1,i-n)
        i2 = min(num_pix,i+n)

!---------- perform restoral for vis+ir mask

!--- check to see if a cloudy pixel neighbors a non-cloudy pixel
       if (cld_mask(i,j) == sym%CLOUDY) then
         ncloudy = sum(cld_mask(i1:i2,j1:j2))
         ncloudy_max = sym%CLOUDY*(i2-i1+1)*(j2-j1+1) 
         if (ncloudy < ncloudy_max) then
            temp_mask(i,j) = sym%PROB_CLOUDY
            cld_test_vector(first_res_test + 1, i,j) = sym%YES
         endif
       endif

     end do pixel_loop
   end do scan_loop

!--- copy modifies mask into the cld_mask array
   cld_mask = temp_mask

!----------------------------------------------------------------------
! ir
!----------------------------------------------------------------------
!--- store cld mask in a temp array for modification
   temp_mask = cld_mask_ir

!loop over scanlines
scan_loop_ir: do j = jmin,jmax
       
!--- determine y-dimensions of array to check
       j1 = max(jmin,j-n)
       j2 = min(jmax,j+n)

!loop over elements of each scanline 
pixel_loop_ir:   do i = 1, num_pix
        
!--- determine x-dimensions of array to check
        i1 = max(1,i-n)
        i2 = min(num_pix,i+n)
         
       if (cld_mask_ir(i,j) == sym%CLOUDY) then
         ncloudy = sum(cld_mask_ir(i1:i2,j1:j2))
         ncloudy_max = sym%CLOUDY*(i2-i1+1)*(j2-j1+1)
         if (ncloudy < ncloudy_max) then
            temp_mask(i,j) = sym%PROB_CLOUDY
         endif

       endif
   
     end do pixel_loop_ir
   end do scan_loop_ir

!--- copy modifies mask into the cld_mask array
   cld_mask_ir = temp_mask

!----------------------------------------------------------------------
! bayes
!----------------------------------------------------------------------
!--- store cld mask in a temp array for modification
!  temp_mask = cld_mask_bayes

!loop over scanlines
!scan_loop_bayes: do j = jmin,jmax
       
!--- determine y-dimensions of array to check
!      j1 = max(jmin,j-n)
!      j2 = min(jmax,j+n)

!loop over elements of each scanline 
!pixel_loop_bayes:   do i = 1, num_pix
        
!--- determine x-dimensions of array to check
!       i1 = max(1,i-n)
!       i2 = min(num_pix,i+n)
         
!      if (cld_mask_bayes(i,j) == sym%CLOUDY) then
!        ncloudy = sum(cld_mask_bayes(i1:i2,j1:j2))
!        ncloudy_max = sym%CLOUDY*(i2-i1+1)*(j2-j1+1)
!        if (ncloudy < ncloudy_max) then
!           temp_mask(i,j) = sym%PROB_CLOUDY
!        endif

!      endif
   
!    end do pixel_loop_bayes
!  end do scan_loop_bayes

!--- copy modifies mask into the cld_mask array
!  cld_mask_bayes = temp_mask


!
! End subroutine COMPUTE_PROBABLY_CLOUDY
!
end subroutine COMPUTE_PROBABLY_CLOUDY

!------------------------------------------------------------------------
! SUBROUTINE NAME: COMPUTE_2BIT_CLOUD_MASK_IR
!
! compute the 2bit cloud mask (0-3) based on the individual test results
! using only the infrared tests
!
! INPUTS:
!  jmin - the first scan index to process
!  jmax - the last scan index to process
!
! OUTPUTS:
!
! CALLING SEQUENCE:  call COMPUTE_2BIT_CLOUD_MASK_IR(j_min,num_scans_read)
!			(called from PROCESS_AVHRR_CLAVR)
!
! REFERENCES:
!
! DEPENDENCIES:
!	CONSTANTS
!	AVHRR_PIXEL_COMMON
!	NWP_COMMON
!
! SIDE EFFECTS: None
!
! MODIFICATIONS:
!
!------------------------------------------------------------------------
!
! GLOBAL VARIABLES:
!
! cld_mask 		= cloud mask output
! mask		= results from each cloud mask test
! n_cld_test 	= number of cloud tests on mask array
! n_ref_cld_test =  number of reflectance cloud tests on mask array
! num_pix	= number of pixels in each scanline
!
! LOCAL VARIABLES:
!
! j = which scanline is being tested
! i = which element is being tested
!
! -------------------------------------------------------------------------

subroutine COMPUTE_2BIT_CLOUD_MASK_IR(jmin,jmax)
 integer, intent(in):: jmin,jmax
 integer:: i, j

! -------------------------------------------------------------------------
! 					EXECUTABLE CODE
! -------------------------------------------------------------------------

!Loop over elements of scanlines
do i = 1, num_pix

!Loop over scanlines
   do j = jmin,jmax

!--- set bad pixels to probably clear
     if (cld_test_vector(1,i,j) == sym%YES) then
       cld_mask_ir(i,j) = sym%PROB_CLEAR
       cycle
     endif

!---- determine actual cloud mask value
if ((sum(cld_test_vector(First_IR_Cld_Mask_Test:Last_IR_Cld_Mask_Test,i,j)) + &
     sum(cld_test_vector(First_SWIR_Thermal_Cld_Mask_Test:Last_SWIR_Thermal_Cld_Mask_Test,i,j)) ) == &
           (Num_IR_Cld_Mask_Tests + Num_SWIR_Thermal_Cld_Mask_Tests)*sym%NO) then  !check cld tests


!if uniform, call it clear
       if (sum(cld_test_vector(First_Clr_Uni_Test:Last_Clr_Uni_Test,i,j)) == &
           Num_Clr_Uni_Tests*sym%NO) then   !check uni tests

         cld_mask_ir(i,j) = sym%CLEAR

!otherwise, mark as probably clear
       else
         cld_mask_ir(i,j) = sym%PROB_CLEAR
       endif

!All other values mark as cloudy
    else
         cld_mask_ir(i,j) = sym%CLOUDY
    endif

!End loops
   end do
  end do

!
! End subroutine COMPUTE_2BIT_CLOUD_MASK_IR
!
  End subroutine COMPUTE_2BIT_CLOUD_MASK_IR

!----------------------------------------------------------------------
! Pack cld mask test results into a packed integer for output
!----------------------------------------------------------------------
  Subroutine PACK_CLOUD_MASK_BYTES(jmin,jmax)

  integer, intent(in):: jmin
  integer, intent(in):: jmax
  integer:: i   !local element index
  integer:: j   !local scanline index
!-- create vector of bit-depth for all values
  integer(kind=int4), dimension(Total_Num_Tests):: test_bit_depth = 1
 
   do i = 1, num_pix
     do j = jmin, jmax + jmin - 1

         call PACK_BYTES(cld_test_vector(1:Total_Num_Tests,i,j), &
                      test_bit_depth(1:Total_Num_Tests), &
                      cld_test_vector_packed(:,i,j))

     end do ! scanlines
   end do  ! pixels

  end subroutine PACK_CLOUD_MASK_BYTES

!
! End module CLOUD_MASK_ROUTINES
!                                                                                                                                                   
end module CLOUD_MASK_ROUTINES
