! $Id: equal_angle.f90,v 1.3 2010/09/23 20:45:59 heidinger Exp $
!------------------------------------------------------------------
!@***h* PATMOSX/equal_angle.f90
! 
! NAME
!   equal_angle.f90
! 
! LOCATION
!   /patmosx/src/equal_angle.f90
! 
! PURPOSE
!   This set of routines in this modules is used when the equal angle
!   options selected for the clavr-x gridcell
! 
! DESCRIPTION
!   This set of routines in this modules is used when the equal angle
!   options selected for the clavr-x gridcell
!   This cell-ordering is the same as the PATMOS ordering
!   first cell is at 0 longitude (the prime meridian) and at
!   the south pole. Cells go east then north.
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! EXTERNALS
!  Dependencies: (The following are names of modules used in CLAVR-x)
!  CONSTANTS
!
!  Public Routines within this module:
!  SETUP_EQUAL_ANGLE 
!  PLACE_EQUAL_ANGLE_CELL 
!  FIND_EQUAL_ANGLE_CELL
!  DESTROY_EQUAL_ANGLE
!
! NOTES
!  Calling Sequence:
!  use EQUAL_ANGLE_GRID
!
!@*****
!------------------------------------------------------------------------
!
! Source Code Name:
!  equal_angle.f90
!
! Module Name:
!  EQUAL_ANGLE_GRID
!
! Function:
!  a module within the CLAVR-x processing system
!
! Descripion:
!  This set of routines in this modules is used when the equal angle
!  options selected for the clavr-x gridcell.
!  The cell-ordering is the same as the PATMOS ordering
!  first cell is at 0 longitude (the prime meridian) and at
!  the south pole. Cells go east then north.
!
! Reference: None
!
! Dependencies: (The following are names of modules used in CLAVR-x)
!  CONSTANTS
!
! Calling Sequence:
!  use EQUAL_ANGLE_GRID
!
! Public Routines within this module:
!  SETUP_EQUAL_ANGLE 
!  PLACE_EQUAL_ANGLE_CELL 
!  FIND_EQUAL_ANGLE_CELL
!  DESTROY_EQUAL_ANGLE
!
! Author: Andrew K. Heidinger, National Oceanic and Atmospheric Administration
! (c) This code is copyrighted by the author and all NOAA restrictions apply
!-----------------------------------------------------------------------
 module EQUAL_ANGLE_GRID
  use CONSTANTS
  implicit none
  private
  public:: SETUP_EQUAL_ANGLE, &
           PLACE_EQUAL_ANGLE_CELL, &
           FIND_EQUAL_ANGLE_CELL, &
           DESTROY_EQUAL_ANGLE
          
  integer(kind=int4), dimension(:),allocatable,  &
                                 private, save:: num_zonal_cells
  integer(kind=int4), dimension(:),allocatable,  &
                                 private, save:: num_cells_spole
  integer(kind=int4), public, save:: ncells_angle
  integer, private, save:: num_lat
  integer, private, save:: num_lon
  real(kind=real8), private, save:: delta_lat_deg
  real(kind=real8), private, save:: delta_lon_deg

  contains
!----------------------------------------------------------------------
! setup the equal-area grid cells - this uses patmos convention
! which is cell numbering starts at the south pole 0 longitude
! and goes west and north
!
! this routine should be called before any other in this module
!
!-----------------------------------------------------------------------
  subroutine SETUP_EQUAL_ANGLE(dlat)
    real(kind=real4), intent(in):: dlat
    integer:: ilat

       delta_lat_deg = dlat
       delta_lon_deg = dlat
       num_lat = int(180.0/delta_lat_deg)    !number of latitude cells
       num_lon = int(360.0/delta_lat_deg)    !number of longitude cells

       allocate(num_zonal_cells(num_lat),num_cells_spole(num_lat))
       num_zonal_cells = 0
       num_cells_spole = 0


     do ilat = 1, num_lat
       num_zonal_cells(ilat) = num_lon
       num_cells_spole(ilat) = (ilat-1)*num_lon
     enddo

     ncells_angle = sum(num_zonal_cells)

  end subroutine SETUP_EQUAL_ANGLE
!------------------------------------------------------------------------
! deallocate the memory allocated in SETUP_EQUAL_ANGLE
!------------------------------------------------------------------------
  subroutine DESTROY_EQUAL_ANGLE()
       deallocate(num_zonal_cells, num_cells_spole)
  end subroutine DESTROY_EQUAL_ANGLE

!-------------------------------------------------------------------------
! Given a equal area cell index, find its central lat and lon
!-------------------------------------------------------------------------
  subroutine PLACE_EQUAL_ANGLE_CELL(num_box,lat_center,lon_center, &
                                   lat_width,lon_width)
      integer(kind=int4), intent(in):: num_box
      real(kind=real4), intent(out):: lat_center,lon_center,lat_width,lon_width
      integer:: latbin,lonbin

      do latbin = 1,num_lat-1
         if (num_cells_spole(latbin+1) >= num_box) then
             exit
         endif
      enddo
      lat_center = -90.0 + (latbin -1 + 0.5)*delta_lat_deg
      lonbin = num_box - num_cells_spole(latbin)
      lon_center = (lonbin - 1 + 0.5)*delta_lon_deg
      if (lon_center > 180.0) then
        lon_center = lon_center - 360.0
      endif
      lat_width = delta_lat_deg
      lon_width = delta_lon_deg
  end subroutine PLACE_EQUAL_ANGLE_CELL
!-------------------------------------------------------------------------
! given a latitude (-90 to 90) and a longitude (-180 to 180) determine
! which 1 deg equal area box this falls into.  These boxes are the
! 41258 boxes used in PATMOS mapping.
! This code is mainly taken from routines written by V. Rao Anne
!
!-------------------------------------------------------------------------
  subroutine FIND_EQUAL_ANGLE_CELL(latitude,longitude,num_box)
      real(kind=real4), intent(in):: latitude,longitude
      integer, intent(out):: num_box
      real(kind=real4):: rrlat,rrlon
      integer:: latbin,lonbin

!-------------------------------------------------------------------------
! determine equal area box
!-------------------------------------------------------------------------
   rrlat = latitude + 90.0
   rrlon = longitude
   if (rrlon < 0.00) then
      rrlon = 360.0 + rrlon
   endif
   latbin = min(int(rrlat/delta_lat_deg + 1.0),num_lat)
   lonbin = min(int(rrlon/delta_lon_deg+1.0) ,num_zonal_cells(latbin))
   num_box = num_cells_spole(latbin) + lonbin

  end subroutine FIND_EQUAL_ANGLE_CELL

end module EQUAL_ANGLE_GRID
