!------------------------------------------------------------------
!  $Id: gfs_hdf_module.f90,v 1.4 2010/09/23 21:35:33 heidinger Exp $
!------------------------------------------------------------------
!@***h* PATMOSX/gfs_hdf_module.f90
! 
! NAME
!   gfs_hdf_module.f90
! 
! LOCATION
!   /patmosx/src/gfs_hdf_module.f90
! 
! PURPOSE
!   This module houses all routines related to the GFS data
! 
! DESCRIPTION
!  This module houses all of the routines used to read and process the
!  GFS NWP data.  The data used here are already in hdf format from the
!  convert_grib_to_hdf utility.
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! EXTERNALS
!  Dependencies:
!  CONSTANTS
!  NUMERICAL_ROUTINES
!  Sort_Module
!  NWP_COMMON
!  HDF
!
!  Calling Sequence:
!  use GFS
!
!  Public Routines within this module:
!  READ_GFS_DATA
!
!@*****
!------------------------------------------------------------------------
! Source Code Name:
!  gfs_hdf_module.f90
!
! Module Name:
!  GFS
!
! Function:
!  a module in the CLAVR-x processing system
!
! Description:
!  This module houses all of the routines used to read and process the
!  GFS NWP data.  The data used here are already in hdf format from the
!  convert_grib_to_hdf utility.  
!
! Reference:
!
! Dependencies:
!  CONSTANTS
!  NUMERICAL_ROUTINES
!  Sort_Module
!  NWP_COMMON
!  HDF
!
! Calling Sequence:
!  use GFS
!
! Public Routines within this module:
!  READ_GFS_DATA
!
! Author: Andrew K. Heidinger, National Oceanic and Atmospheric Administration
! (c) This code is copyrighted by the author and all NOAA restrictions apply
!
!-------------------------------------------------------------------
module GFS
  use CONSTANTS
  use NUMERICAL_ROUTINES
  use Sort_Module
  use NWP_COMMON
  use HDF

  implicit none

  public:: READ_GFS_DATA
  private:: READ_DATA_2D
  private:: READ_DATA_3D

  real (kind=real4), parameter, private :: missing_gfs = 9.999E+20

contains

!-------------------------------------------------------------
! Subroutine to read in GFS data
!-------------------------------------------------------------
  subroutine READ_GFS_DATA(start_year, &
                           start_jday, &
                           start_itime,&
                           end_year,   &
                           end_jday,   &
                           end_itime,  &
                           nwp_path,   &
                           ierror_nwp)

!-------------------------------------------------------------
!   Local declarations
!-------------------------------------------------------------

!   Arguments
    character (len=*), intent(in) :: nwp_path
    integer(kind=int2), intent(in) :: start_year, start_jday, end_year, end_jday
    integer(kind=int4), intent(in) :: start_itime, end_itime
    integer, intent(out):: ierror_nwp
    integer :: year, jday

!   Parameters
    real, parameter :: missing_value = -9.999E+20

!   Local variables
    character (len=120) :: nwp_name_before
    character (len=120) :: nwp_name_after
    character (len=2)   :: year_string, hour_string, day_string, month_string
    character (len=3)   :: array_order_1, array_order_2 

    logical :: file_exists

    integer :: year2d, ileap_after, ileap_before, ileap_this_year, ileap_prev_year
    integer :: month_before, day_before, jday_before, year_before, hour_before
    integer :: month_after,  day_after,  jday_after,  year_after,  hour_after
    integer :: i, j, ii, jj, k

    real :: time,start_time, x

    integer:: nlevels, nlevels_o3mr, nlevels_rh, nlevels_clwmr, nlat_gfs, nlon_gfs
    real:: dlat_gfs, dlon_gfs, lat1_gfs, lon1_gfs

!-- values used for hdf reading
integer:: sd_id_1,sd_id_2,sds_id, istatus
integer, parameter:: sds_rank_1d = 1, sds_rank_2d = 2, sds_rank_3d = 3
integer, dimension(sds_rank_1d):: sds_start_1d, sds_stride_1d, sds_edges_1d
integer, dimension(sds_rank_2d):: sds_start_2d, sds_stride_2d, sds_edges_2d
integer, dimension(sds_rank_3d):: sds_start_3d, sds_stride_3d, sds_edges_3d

! HDF function declarations
 integer:: sfstart, sfendacc, sfend, &
           hishdff, sfselect, sfrdata, sfn2index, &
           sffattr, sfrnatt, sfrcatt


!--   Initializations
    ierror_nwp = 0
    nlat_gfs   = -1
    nlon_gfs   = -1

    missing_nwp = missing_gfs

!--- compute mean orbit time and account for orbiting end after midnight
    start_time = (start_itime)/60.0/60.0/1000.0
 
    time = 0.5*(start_itime + end_itime)/60.0/60.0/1000.0

    if (time < start_time) then
      time = time + 12.0
    endif

    if (time <= 24.00) then
      jday =  start_jday
      year =  start_year
    else
      jday =  end_jday
      year =  end_year
      time = time - 24.0
    endif

!------ determine if this year is a leap year
    ileap_this_year = 0
    if ((year == 1984) .or. (year == 1988) .or. (year == 1992) .or. (year == 1996) .or.  &
        (year == 2000) .or. (year == 2004) .or. (year == 2008)) then
       ileap_this_year = 1
    endif

    ileap_prev_year = 0
    if (year == 1985 .or. year == 1989 .or. year == 1993 .or. year == 1997 .or.  &
         year == 2001 .or. year == 2005 .or. year == 2009) then
       ileap_prev_year = 1
    endif

!---------------------------------------------------------------
!   Pick forecasts that surround this observation time
!---------------------------------------------------------------
    if ((time >= 0.00) .and. (time <= 6.00)) then
       jday_before = jday -1
       hour_before = 12
       jday_after = jday -1
       hour_after = 18
    elseif ((time > 6.00) .and. (time <= 12.00)) then
       jday_before = jday -1
       hour_before = 18
       jday_after = jday
       hour_after = 0
    elseif ((time > 12.00) .and. (time <= 18.00)) then
       jday_before = jday
       hour_before = 0
       jday_after = jday
       hour_after = 6
    elseif ((time > 18.00) .and. (time <= 24.00)) then
       jday_before = jday
       hour_before = 6
       jday_after = jday
       hour_after = 12
    else
       print *, EXE_PROMPT, "invalid time for forecast, stopping"
       ierror_nwp = 1
       stop 401
    endif

!   Adjust year if necessary
    year_before = year
    year_after = year
    ileap_after = ileap_this_year
    ileap_before = ileap_this_year
    if (jday_before < 1) then
       ileap_before = ileap_prev_year
       year_before = year -1
       jday_before = 365 + ileap_before
    endif

!   Compute weight of each forecast to this time
!   values near one mean you are close to time of the "after" file
!   values near zero mean you are close to time of the "before" file
    x = (time - 6.0*int(time/6.0))/6.0
    x = min(1.0,max(0.0,x))

!--------------------------------------------------------------------
!   Construct file name for forecast valid before obs time
!--------------------------------------------------------------------

!--- Compute month and day of month from year and Julian day
    month_before = COMPUTE_MONTH(jday_before,ileap_before)
    day_before = COMPUTE_DAY(jday_before,ileap_before)

!--- File name
    year2d = year_before - 100*int(year/100)      !two digit year
    write (year_string,  '(I2.2)') year2d
    write (hour_string,  '(I2.2)') hour_before
    write (month_string, '(I2.2)') month_before
    write (day_string,   '(I2.2)') day_before

    nwp_name_before = trim(nwp_path) // "gfs." // year_string // month_string // &
                       day_string // hour_string // "_F012.hdf"

!---------------------------------------------------------------------
! Construct file name for forecast valid after obs time
!---------------------------------------------------------------------

!--- Compute month and day of month from year and Julian day
    month_after= COMPUTE_MONTH(jday_after,ileap_after)
    day_after = COMPUTE_DAY(jday_after,ileap_after)

!--- File name
    year2d = year_after - 100*int(year/100)      !two digit year
    write (year_string,  '(I2.2)') year2d
    write (hour_string,  '(I2.2)') hour_after
    write (month_string, '(I2.2)') month_after
    write (day_string,   '(I2.2)') day_after

    nwp_name_after = trim(nwp_path)//"gfs."//year_string//month_string// &
                       day_string//hour_string//"_F012.hdf"


!--- Does before file exist?
    inquire(file = nwp_name_before, exist = file_exists)
    if (.not. file_exists) then
       write (*,*) "ERROR: Missing data file " // trim(nwp_name_before)
       write (*,*) "Setting GFS before to GFS after"
       nwp_name_before = nwp_name_after
       !stop 402
    endif

    print *, EXE_PROMPT, "NWP name before = ", trim(nwp_name_before)

!--- Does after file exist?
    inquire(file = nwp_name_after, exist = file_exists)
    if (.not. file_exists) then
       !--- check to see if before file was also missing
       if (nwp_name_after == nwp_name_before) then
           write (*,*) "ERROR: Neither GFS data files are present, processing stopped "
           stop 403
       endif  
       write (*,*) "ERROR: Missing data file " // trim(nwp_name_after)
       write (*,*) "Setting GFS after to GFS before"
       nwp_name_after = nwp_name_before
    endif


    print *, EXE_PROMPT, "NWP name after = ", trim(nwp_name_after)
!--------------------------------------------------------------
! open before file and read contents from file data before this time
!--------------------------------------------------------------
! Check if the file is of HDF type
     istatus = hishdff(trim(nwp_name_before))
     if (istatus == 0) then
        print *, EXE_PROMPT, "input cell file doesn't exist or is not recognized as HDF file, stopping"
        stop
     endif
   
!-- open files
     sd_id_1 = sfstart(trim(nwp_name_before), DFACC_READ)
     if (sd_id_1 == FAIL) then
      print *, EXE_PROMPT, "failed open for read on first gfs file, stopping"
      stop
     endif

     sd_id_2 = sfstart(trim(nwp_name_after), DFACC_READ)
     if (sd_id_2 == FAIL) then
      print *, EXE_PROMPT, "failed open for read on second gfs file, stopping"
      stop
     endif

!---- read attributes from file 1
     istatus = 0
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"NUMBER OF PRESSURE LEVELS"),nlevels) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"NUMBER OF O3MR LEVELS"),nlevels_o3mr) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"NUMBER OF RH LEVELS"),nlevels_rh) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"NUMBER OF CLWMR LEVELS"),nlevels_clwmr) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"NUMBER OF LATITUDES"),nlat_gfs) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"NUMBER OF LONGITUDES"),nlon_gfs) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"LATITUDE RESOLUTION"),dlat_gfs) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"LONGITUDE RESOLUTION"),dlon_gfs) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"FIRST LATITUDE"),lat1_gfs) + istatus
     istatus = sfrnatt(sd_id_1,sffattr(sd_id_1,"FIRST LONGITUDE"),lon1_gfs) + istatus
     istatus = sfrcatt(sd_id_1,sffattr(sd_id_1,"3D ARRAY ORDER"), array_order_1) + istatus

!--- don't read attributes from file 2 (assume they are the same)
     istatus = sfrcatt(sd_id_1,sffattr(sd_id_2,"3D ARRAY ORDER"), array_order_2) + istatus

!--- do some checking
     if (array_order_1 /= array_order_2) then 
       print *, EXE_PROMPT, "order of 3d arrays differ among gfs files, stopping = ",  &
              array_order_1, " ", array_order_2
       stop
     endif

!--- set reformat gfs flag
     REFORMAT_GFS_ZXY = 0
     if (array_order_1 == "XYZ" ) then
       REFORMAT_GFS_ZXY = 1
     endif

!--- set up need hdf dimensions

!---- define dimensions of 1d arrays
sds_start_1d = (/ 0 /)       !should this be 1
sds_stride_1d = (/ 1 /)
sds_edges_1d = (/ nlevels /)     !should this be nx-1,ny-1
                                                                                                                           
!---- define dimensions of 2d arrays
sds_start_2d = (/ 0, 0 /)       !should this be 1
sds_stride_2d = (/ 1, 1 /)
sds_edges_2d = (/ nlon_gfs, nlat_gfs /)     !should this be nx-1,ny-1
                                                                                                                           
!---- define dimensions of 3d arrays
sds_start_3d = (/ 0, 0, 0 /)       
sds_stride_3d = (/ 1, 1, 1 /)
sds_edges_3d = (/ nlevels, nlon_gfs, nlat_gfs /)     
if (REFORMAT_GFS_ZXY == 1) then
  sds_edges_3d = (/ nlon_gfs, nlat_gfs, nlevels /)    
endif

!---- store dimensions in public variables
    nlevels_nwp = nlevels
    nlat_nwp = nlat_gfs
    nlon_nwp = nlon_gfs

!---- specific to GFS grid
    lat1_nwp = lat1_gfs
    lon1_nwp = lon1_gfs
    dlon_nwp = dlon_gfs
    dlat_nwp = sign(dlat_gfs,-1.0*lat1_gfs)

!-----------------------------------------------------------------
! allocate NWP arrays
!-----------------------------------------------------------------
    call CREATE_NWP_ARRAYS()

!------------------------------------------------------------------
!   Read data from the first file
!------------------------------------------------------------------

!--- read in standard levels from first file
    istatus = 0
    sds_id = sfselect(sd_id_1, sfn2index(sd_id_1,"pressure levels"))
    istatus = sfrdata(sds_id, sds_start_1d, sds_stride_1d, sds_edges_1d,  &
                      P_std_nwp) + istatus
    istatus = sfendacc(sds_id) + istatus

!--- read in two dimensional arrays

!- land mask
    call READ_DATA_2D(sd_id_1,sd_id_2,"land mask",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,temp2d_nwp_1)
    land_nwp = int(temp2d_nwp_1,kind=int1)

!- ice mask
    call READ_DATA_2D(sd_id_1,sd_id_2,"ice fraction",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,ice_nwp )

!--- read in two dimensional arrays
    call READ_DATA_2D(sd_id_1,sd_id_2,"surface pressure",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,psfc_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"surface temperature",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,tmpsfc_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"surface height",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,zsfc_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"temperature at sigma=0.995",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,tmpair_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"rh at sigma=0.995",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,rhsfc_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"water equivalent snow depth",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,weasd_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"total precipitable water",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,tpw_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"tropopause temperature",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,t_trop_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"tropopause pressure",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,p_trop_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"u-wind at sigma=0.995",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,u_wnd_10m_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"v-wind at sigma=0.995",sds_start_2d, sds_stride_2d, &
                      sds_edges_2d,x,missing_nwp,v_wnd_10m_nwp)
    call READ_DATA_2D(sd_id_1,sd_id_2,"total ozone",sds_start_2d, sds_stride_2d,sds_edges_2d,x,missing_nwp,ozone_nwp)

!--- read in three dimensional arrays
    call READ_DATA_3D(sd_id_1,sd_id_2,"temperature",sds_start_3d, sds_stride_3d, &
                      sds_edges_3d,x,missing_nwp,t_prof_nwp)
    call READ_DATA_3D(sd_id_1,sd_id_2,"height",sds_start_3d, sds_stride_3d, &
                      sds_edges_3d,x,missing_nwp,z_prof_nwp)
    call READ_DATA_3D(sd_id_1,sd_id_2,"o3mr",sds_start_3d, sds_stride_3d, &
                      sds_edges_3d,x,missing_nwp,ozone_prof_nwp)
    call READ_DATA_3D(sd_id_1,sd_id_2,"rh",sds_start_3d, sds_stride_3d, &
                      sds_edges_3d,x,missing_nwp,rh_prof_nwp)
    call READ_DATA_3D(sd_id_1,sd_id_2,"clwmr",sds_start_3d, sds_stride_3d, &
                      sds_edges_3d,x,missing_nwp,clwmr_prof_nwp)

!--- close file
istatus = sfend(sd_id_1)
istatus = sfend(sd_id_2)

!--- fix GFS bug in RH
  call FIX_GFS_RH()

!--- compute UTH    - 300 to 400 mb   !--- NEED TO make this dynamic!!!!!!
do i = 1, nlon_nwp
 do j = 1, nlat_nwp
   uth_nwp(i,j) = sum(rh_prof_nwp(10:12,i,j))/3.0
 enddo
enddo
                                                                                
!--- store 500 mb heights - note convert back to meters
hght500_nwp = z_prof_nwp(14,:,:) * 1000.0

!---- compute wind speed
wnd_spd_10m_nwp = missing_value_real4
where((u_wnd_10m_nwp /= missing_gfs) .and. (v_wnd_10m_nwp /= missing_gfs))
 wnd_spd_10m_nwp = sqrt(u_wnd_10m_nwp**2 + v_wnd_10m_nwp**2)
endwhere
                                                                                
!---- compute wind direction
wnd_dir_10m_nwp = missing_value_real4
where((wnd_spd_10m_nwp >= 0.0) .and. (v_wnd_10m_nwp /= missing_gfs))
 wnd_dir_10m_nwp = acos(-1.0*v_wnd_10m_nwp/wnd_spd_10m_nwp) / dtor     !in degrees  (0 to 180)
endwhere
where((u_wnd_10m_nwp >= 0.0 ) .and. (u_wnd_10m_nwp /= missing_gfs))
 wnd_dir_10m_nwp = 360.0 - wnd_dir_10m_nwp
endwhere

!---- compute the 3x3 uniformity of tmpair
do i = 1, nlon_nwp
 ii = max(2,min(nlon_nwp-1,i))
 do j = 1, nlat_nwp
  jj = max(2,min(nlat_nwp-1,j))
  tmpair_uni_nwp(i,j) = (maxval(tmpair_nwp(ii-1:ii+1,jj-1:jj+1)) - &
                        minval(tmpair_nwp(ii-1:ii+1,jj-1:jj+1)) ) / 3.0
  tmpsfc_uni_nwp(i,j) = (maxval(tmpsfc_nwp(ii-1:ii+1,jj-1:jj+1)) - &
                        minval(tmpsfc_nwp(ii-1:ii+1,jj-1:jj+1)) ) / 3.0
 enddo
enddo


!----- check for missing t tropopause values and set to 200 K
do i = 1, nlon_nwp
 do j = 1, nlat_nwp
    if ((t_trop_nwp(i,j) < 180.0).or.(t_trop_nwp(i,j) > 240.0)) then
       t_trop_nwp(i,j) = 200.0
    endif
 enddo
enddo

 end subroutine READ_GFS_DATA

!--------------------------------------------------------------------------------------------
! generic HDF4 read routines
!--------------------------------------------------------------------------------------------

!------------------------------------------------------------------------------------------
! read in the 2d dimensional data from the GFS HDF files
!------------------------------------------------------------------------------------------
subroutine READ_DATA_2D(sd_id_1,sd_id_2,sds_name,sds_start,sds_stride,sds_edges,x,missing,temp)

  integer, intent(in):: sd_id_1, sd_id_2
  character(len=*), intent(in):: sds_name
  integer, dimension(:), intent(in):: sds_stride, sds_start, sds_edges
  real, intent(in):: x, missing
  real, dimension(:,:), intent(out):: temp

  integer:: sds_id_1,sds_id_2,istatus_1,istatus_2
  integer:: i1,i2,ndim1,ndim2

! HDF function declarations
 integer:: sfendacc, &
           sfselect, sfrdata, sfn2index


    istatus_1 = 0
    istatus_2 = 0

!--- open sds's
    sds_id_1 = sfselect(sd_id_1, sfn2index(sd_id_1,sds_name)) 
    sds_id_2 = sfselect(sd_id_2, sfn2index(sd_id_2,sds_name)) 

!--- read sds's
    istatus_1 = sfrdata(sds_id_1, sds_start, sds_stride, sds_edges, temp2d_nwp_1) + istatus_1
    istatus_2 = sfrdata(sds_id_2, sds_start, sds_stride, sds_edges, temp2d_nwp_2) + istatus_2

!--- interpolate
    ndim1 = size(temp,1)
    ndim2 = size(temp,2)
    do i1 = 1, ndim1
      do i2 = 1, ndim2
          if ((temp2d_nwp_1(i1,i2) /= missing) .and. (temp2d_nwp_2(i1,i2) /= missing)) then
           temp(i1,i2) = (1.0 - x) * temp2d_nwp_1(i1,i2)   + x * temp2d_nwp_2(i1,i2)
          else
           temp(i1,i2) = missing_nwp
          endif
      enddo
    enddo

!--- close sds's
    istatus_1 = sfendacc(sds_id_1) + istatus_1
    istatus_2 = sfendacc(sds_id_2) + istatus_2

!--- check for success
    if ((istatus_1 < 0) .or. (istatus_2 < 0)) then
        print *, EXE_PROMPT, "sds ",sds_name," not read in successfully from gfs files, stopping"
        stop
    endif


end subroutine READ_DATA_2D

!------------------------------------------------------------------------------------------
! read in the 3d dimensional data from the GFS HDF files
!------------------------------------------------------------------------------------------
subroutine READ_DATA_3D(sd_id_1,sd_id_2,sds_name,sds_start,sds_stride,sds_edges,x,missing,temp)

  integer, intent(in):: sd_id_1, sd_id_2
  character(len=*), intent(in):: sds_name
  integer, dimension(:), intent(in):: sds_stride, sds_start, sds_edges
  real, intent(in):: x, missing
  real, dimension(:,:,:), intent(out):: temp
  integer:: sds_id_1,sds_id_2,istatus_1,istatus_2
  integer:: i1,i2,i3,ndim1,ndim2,ndim3

! HDF function declarations
 integer:: sfendacc, &
           sfselect, sfrdata, sfn2index

    istatus_1 = 0
    istatus_2 = 0

!--- interpolate
    ndim1 = size(temp,1)
    ndim2 = size(temp,2)
    ndim3 = size(temp,3)
    
!--- open sds's
    sds_id_1 = sfselect(sd_id_1, sfn2index(sd_id_1,sds_name))
    sds_id_2 = sfselect(sd_id_2, sfn2index(sd_id_2,sds_name))

!--- read sds from first time
    istatus_1 = sfrdata(sds_id_1, sds_start, sds_stride, sds_edges, temp3d) + istatus_1

    if (REFORMAT_GFS_ZXY == 1) then
      do i1 = 1,ndim1 
         temp3d_nwp_1(i1,:,:) = temp3d(:,:,i1)  
      enddo
    else 
      temp3d_nwp_1 = temp3d
    endif

!--- read sds from second time
    istatus_2 = sfrdata(sds_id_2, sds_start, sds_stride, sds_edges, temp3d) + istatus_2

    if (REFORMAT_GFS_ZXY == 1) then
      do i1 = 1,ndim1 
         temp3d_nwp_2(i1,:,:) = temp3d(:,:,i1)  
      enddo
    else 
      temp3d_nwp_2 = temp3d
    endif

!-- note this where state is seg-faulting (replaced with equivalent do/if construct)
!   where ((temp3d_nwp_1 == missing) .or. (temp3d_nwp_2 == missing))
!      temp   = missing_nwp
!   elsewhere
!      temp   = (1.0 - x) * temp3d_nwp_1   + x * temp3d_nwp_2
!   endwhere
!----------------------------------------------------------------

    do i1 = 1, ndim1
      do i2 = 1, ndim2
        do i3 = 1, ndim3
          if ((temp3d_nwp_1(i1,i2,i3) /= missing) .and. (temp3d_nwp_2(i1,i2,i3) /= missing)) then
           temp(i1,i2,i3) = (1.0 - x) * temp3d_nwp_1(i1,i2,i3)   + x * temp3d_nwp_2(i1,i2,i3)
          else
           temp(i1,i2,i3) = missing_nwp
          endif
        enddo
      enddo
    enddo
            
!--- close sds's
    istatus_1 = sfendacc(sds_id_1) + istatus_1
    istatus_2 = sfendacc(sds_id_2) + istatus_2
                                                                                                                                                                                              
!--- check for success
    if ((istatus_1 /= 0).or.(istatus_2 /= 0)) then
        print *, EXE_PROMPT, "sds ",sds_name," not read in successfully from gfs files, stopping"
        stop
    endif

end subroutine READ_DATA_3D

!---------------------------------------------------------------------
! Fix GFS RH scaling
!
! In the current GFS output, the definition of RH varies between
! 253 and 273 K.  At 273 it is with respect to water. 
! At 253 it is defined with respect to ice.  
! At temperatures in between, it varies linearly.
! This routine attempts to define RH with respect to water for all temps
!---------------------------------------------------------------------
subroutine FIX_GFS_RH()

   integer:: i, j, k
   real:: es_water
   real:: es_ice
   real:: e
   real:: es
   real:: ice_weight

   do i = 1, nlon_nwp
      do j = 1, nlat_nwp

       do k = 1, nlevels_nwp
          if (rh_prof_nwp(k,i,j) > 0.0) then

           !--- compute saturation vapor pressures
           es_water = VAPOR(t_prof_nwp(k,i,j))
           es_ice = VAPOR_ICE(t_prof_nwp(k,i,j))

           !--- derive the ice/water weight used in gfs
           ice_weight = (273.16 - t_prof_nwp(k,i,j)) / &
                       (273.16-253.16)
           ice_weight = min(1.0,max(0.0,ice_weight))

           !--- derive es used in original rh definition
           es = ice_weight * es_ice + (1.0-ice_weight)*es_water

           !--- compute actual e 
           e = rh_prof_nwp(k,i,j) * es / 100.0

           !--- compute actual rh with respect to water
           rh_prof_nwp(k,i,j) = 100.0 * e / es_water

          endif
       enddo

      enddo
   enddo

end subroutine FIX_GFS_RH





end module GFS
