        module HDF_DCOMP
!--------------------------------------------------------------------------
!@***h* PATMOSX/hdf_dcomp.f90
! 
! NAME
!   hdf_dcomp.f90
! 
! LOCATION
!   /patmosx/src/hdf_dcomp.f90
! 
! PURPOSE
!   This module contains routines for hdf read functions for DCOMP algorithm
! 
! DESCRIPTION
!   This module contains routines for hdf read functions for DCOMP algorithm
!
! AUTHOR
!   A. Walther 
!   andi.walther@ssec.wisc.edu
! 
! CREATION DATE
!   28 Mar 2010
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
!@*****
!--------------------------------------------------------------------------
!  Module name: HDF_DCOMP
!
!  Description:
!  This module contains routines for hdf read functions for DCOMP algorithm
!
!    A. Walther 03/28/2010
!    andi.walther@ssec.wisc.edu
!
!--------------------------------------------------------------------------

 
IMPLICIT  NONE
 
! Some params 
INTEGER, PARAMETER, PRIVATE:: INT1 = selected_int_kind(1)
INTEGER, PARAMETER, PRIVATE:: REAL4 = selected_real_kind(6,37)
CHARACTER(*), PARAMETER, PRIVATE :: EXE_PROMPT = "hdf_dcomp>> "
INTEGER, PARAMETER, PRIVATE:: DFACC_READ = 1
 
INTEGER,PARAMETER,PRIVATE::FLAG_YES = 1
INTEGER,PARAMETER,PRIVATE::FLAG_NO = 0
 
INTEGER(KIND=INT1),PARAMETER , PRIVATE:: WATER_PHASE = 1
INTEGER(KIND=INT1),PARAMETER , PRIVATE:: ICE_PHASE = 2
 
INTEGER(KIND=INT1),PARAMETER , PRIVATE:: NUM_PHASE = 2
 
INTEGER(KIND=INT1), PARAMETER, PRIVATE :: VIS_CHN_NUMBER = 1
INTEGER(KIND=INT1), PARAMETER, PRIVATE :: IR_CHN_NUMBER_3A = 2      !--- AVHRR
INTEGER(KIND=INT1), PARAMETER, PRIVATE :: IR_CHN_NUMBER_3B = 3      !--- AVHRR

INTEGER (Kind=Int1), PARAMETER, PRIVATE :: NUM_SAT_ZEN = 45              !- size of zenith dimension
INTEGER (Kind=Int1), PARAMETER, PRIVATE :: NUM_SOL_ZEN = 45              !- size of solar zenith dimension
INTEGER (Kind=Int1), PARAMETER, PRIVATE :: NUM_REL_AZI = 45              !- size of relative az. dimension
INTEGER (Kind=Int1), PARAMETER, PRIVATE :: NUM_COD = 29
INTEGER (Kind=Int1), PARAMETER, PRIVATE :: NUM_CPS = 9
 
 ! -- look-up-tables structure for Cloud parameters 
TYPE :: Reflectance_Lookup_Table
  INTEGER (Kind=Int1) :: Flag                          !- flag whether populated or not
 
  REAL (Kind=Real4), DIMENSION(NUM_SAT_ZEN) :: Sat_Zen            !- satellite zenith vector
  REAL (Kind=Real4), DIMENSION(NUM_SOL_ZEN) :: Sol_Zen            !- solar zenith vector
  REAL (Kind=Real4), DIMENSION(NUM_REL_AZI) :: Rel_Azi            !- relative azimuth vector
  REAL (Kind=Real4), DIMENSION(NUM_COD) :: COD_Vec                !- log10 optical depth vector
  REAL (Kind=Real4), DIMENSION(NUM_CPS):: CPS_Vec                 !- log10 effective radius vector
  REAL (Kind=Real4), DIMENSION(NUM_CPS,NUM_COD) :: Sph_Alb        !- spherical albedo
  REAL (Kind=Real4), DIMENSION(NUM_CPS,NUM_COD,NUM_SOL_ZEN) :: Trans       !- flux transmission
  REAL (Kind=Real4), DIMENSION(NUM_CPS,NUM_COD,NUM_SOL_ZEN) :: Cld_Alb     !- cloud albedo
  REAL (Kind=Real4), DIMENSION(NUM_CPS,NUM_COD,NUM_SOL_ZEN,NUM_SAT_ZEN,NUM_REL_AZI) :: Refl  !- reflectance
END TYPE Reflectance_Lookup_Table

TYPE :: Phase_Reflectance_Table
  INTEGER (Kind=Int1) :: Flag                                 !- Phase Flag 
  TYPE(Reflectance_Lookup_Table),DIMENSION(Num_Phase) :: Phase  !- Cloud Phase
END TYPE Phase_Reflectance_Table

TYPE :: Channel_Reflectance_Tables1
  INTEGER (Kind=Int1) :: Flag                                  !-Channel Flag
  TYPE(Phase_Reflectance_Table), DIMENSION(3) :: Channel
END TYPE Channel_Reflectance_Tables1
  
TYPE (Channel_Reflectance_Tables1), SAVE, TARGET:: cld_refl_LUT

!-----------------------------------------------------------------------
! structures for emissivity tables
!-----------------------------------------------------------------------
TYPE, PRIVATE :: Emissivity_Lookup_Table
  INTEGER (kind=int1) :: flag
  REAL (kind=real4), DIMENSION(NUM_CPS,NUM_COD,NUM_SAT_ZEN) :: Trans 
  REAL (kind=real4), DIMENSION(NUM_CPS,NUM_COD,NUM_SAT_ZEN) :: Emiss 
END TYPE Emissivity_Lookup_Table

TYPE, private:: Phase_Emissivity_Tables
  INTEGER (kind=int1):: flag
  TYPE(Emissivity_Lookup_Table),DIMENSION(Num_Phase):: Phase
END TYPE Phase_Emissivity_Tables

Type (Phase_Emissivity_Tables), private, SAVE, TARGET:: cld_ems_LUT



!-- ancil data structure
TYPE :: ANCIL_DATA_TYPE
  INTEGER(KIND=int1)::Flag
  REAL(KIND=real4),DIMENSION(3,3) :: gas_trans  ! -- gas transmission coefficients
  REAL(KIND=real4),DIMENSION(3) :: ozone_trans        ! -- ozone transmission coefficients 
END TYPE ANCIL_DATA_TYPE
 
TYPE(Ancil_Data_Type),SAVE :: Ancil_Data
 
PUBLIC  :: POPULATE_ALL_LUT
PRIVATE :: POPULATE_CLOUD_LUT_SINGLE
PRIVATE :: POPULATE_ANCIL
PUBLIC  :: GET_LUT_DATA

CONTAINS
 
!====================================================================
! Function Name: POPULATE_ALL_LUT
!
! Function:     Populates all main LUTs and ancil LUT
!
! Inputs: lut_path - look-up table path
!         sensor_name - sensor name as appears in LUT file name
!         sensor_chn_number - channel number as appears in LUT file name
!         noaa_sat_string - noaa satellite string as appears in LUT file name
!
! Dependencies:
!         POPULATE_CLOUD_LUT_SINGLE(Lut_File,idx_chn,idx_phase)
!         CALL POPULATE_CLOUD_LUT_SINGLE_EMS(Lut_File,idx_phase)
!         POPULATE_ANCIL(ancil_file)
!
! History:
!
! Local Variables:
!         Phase_String - "wat" or "ice"
!         Chan_String - Channel string
!         Lut_File - LUT file string
!         Ancil_File - Ancillary file string
!         Idx_Chn, Idx_Phase, Identifier_old, Identifier_current
!
!====================================================================
SUBROUTINE POPULATE_ALL_LUT( &
      Lut_Path &          ! - input 
    , Sensor_Name &       ! - input
        , Sensor_Chn_Number & ! - input
        , NOAA_Sat_String)  ! - input

CHARACTER (LEN=*), INTENT(in) ::  lut_path  ! - path there LUT is stored
CHARACTER (LEN=*), INTENT(in):: sensor_name ! - sensor name as appeared in LUT file name
CHARACTER (LEN=*), INTENT(in), DIMENSION(:)  :: Sensor_Chn_Number            ! - channel number as appeared in LUT file name
CHARACTER (LEN=*), INTENT(in)  :: NOAA_sat_string            ! - channel number as appeared in LUT file name

!-locals
CHARACTER (LEN =3 ) :: Phase_String ! - "wat" or "ice"
CHARACTER (LEN =3 ) :: Chan_String  !
CHARACTER (LEN=1024):: Lut_File     !
CHARACTER (LEN=1024):: Ancil_File   !
INTEGER::Idx_Chn
INTEGER::Idx_Phase
CHARACTER(LEN=1024)::Identifier_old
CHARACTER(LEN=1024)::Identifier_current

!- executable

Identifier_current = TRIM(Lut_Path)//TRIM(NOAA_Sat_String)//TRIM(Sensor_Name)

IF (Identifier_current .EQ. Identifier_old) THEN
  ! PRINT*,TRIM(Identifier_current)//' already populated!'

ELSE

 PRINT "(a,'LUT  DCOMP starts')",EXE_PROMPT



  Loop_Channel: DO idx_chn = 1 , 3 
    !--- create a string to hold channel number
    Chan_String= trim(sensor_chn_number(Idx_Chn)) 
        IF (Chan_String .EQ. 'no') CYCLE 
    Loop_Phase: DO idx_phase = 1 , 2 
      IF (idx_phase .EQ. 1 ) THEN
            Phase_String = "wat"
            Lut_File = TRIM(Lut_Path)//TRIM(NOAA_Sat_String)//"_ch" &
            //TRIM(Chan_String)//"_ref_lut_"//Phase_String//"_cld.hdf"
          ENDIF  
          
      IF (idx_phase .eq. 2 ) THEN
            Phase_String = "ice"               
        Lut_File = TRIM(Lut_Path)//TRIM(Sensor_name)//"_ch" &
            //TRIM(Chan_String)//"_ref_lut_"//Phase_String//"_cld.hdf"
          ENDIF
        
          CALL POPULATE_CLOUD_LUT_SINGLE(Lut_File,idx_chn,idx_phase)            
    END DO Loop_Phase 
  END DO Loop_channel

  Emm_loop_phase: DO idx_phase = 1 ,2
    IF (idx_phase .EQ. 1 ) THEN
	  Phase_String = 'wat'
      Lut_File = TRIM(Lut_Path)//TRIM(NOAA_Sat_String)//"_ch" &
            //TRIM(Chan_String)//"_ems_lut_"//Phase_String//"_cld.hdf"
    ENDIF
	
	IF (idx_phase .EQ. 2 ) THEN
	  Phase_string='ice'
	  Lut_File = TRIM(Lut_Path)//TRIM(Sensor_name)//"_ch" &
            //TRIM(Chan_String)//"_ems_lut_"//Phase_String//"_cld.hdf"
	ENDIF
        CALL POPULATE_CLOUD_LUT_SINGLE_EMS(Lut_File,idx_phase)

  END DO Emm_loop_phase

  Ancil_file = TRIM(Lut_Path)//TRIM(sensor_name)//'_gas_coeff.hdf'  
  CALL POPULATE_ANCIL(ancil_file)
  Identifier_old = Identifier_current
ENDIF

END SUBROUTINE POPULATE_ALL_LUT

!====================================================================
! Subroutine Name: POPULATE_CLOUD_LUT_SINGLE(lut_file, Idx_Chn, Idx_Phase) 
!
! Function: populates cloud look-up table  
!
! Inputs:
!    Lut_file - file there LUT is stored
!    Idx_Chn - channel number  in LUT
!    Idx_Phase - phase number  in LUT
!
!====================================================================
 
SUBROUTINE POPULATE_CLOUD_LUT_SINGLE( &
      lut_file &    ! - input
    , Idx_Chn   &      ! - input
        , Idx_Phase )

IMPLICIT NONE

! ---  input
CHARACTER (LEN=*), INTENT(in) ::  Lut_file   ! - file there LUT is stored
INTEGER , INTENT(in)  ::Idx_Chn               ! - channel number  in LUT 
INTEGER , INTENT(in)  ::Idx_Phase             ! - phase number  in LUT 


   
! -  hdf stuff
INTEGER:: Astatus
INTEGER:: Tstatus
INTEGER:: Istatus
 
INTEGER:: Sfstart, Sfrcatt,Sfn2Index, Sffattr, Sfendacc,Sfend,Sfrdata,Sfselect
INTEGER:: Sd_Id
INTEGER:: Sds_Id
 

INTEGER, PARAMETER:: Sds_Rank_1D = 1
INTEGER, DIMENSION(Sds_Rank_1D):: Sds_Start_1D, Sds_Edge_1D, Sds_Stride_1D

INTEGER, PARAMETER:: Sds_Rank_2D = 2
INTEGER, DIMENSION(Sds_Rank_2D):: Sds_Start_2D, Sds_Edge_2D, Sds_Stride_2D, Sds_Dims_2D

INTEGER, PARAMETER:: Sds_Rank_3D = 3
INTEGER, DIMENSION(Sds_Rank_3D):: Sds_Start_3D, Sds_Edge_3D, Sds_Stride_3D, Sds_Dims_3D

INTEGER, PARAMETER:: Sds_Rank_5D = 5
INTEGER, DIMENSION(Sds_Rank_5D):: Sds_Start_5D, Sds_Edge_5D, Sds_Stride_5D, Sds_Dims_5D


!----------------------------------------------------------------------------------
! Executable Code
!----------------------------------------------------------------------------------
 
 
 

Sd_Id = Sfstart( &
                 TRIM(lut_file), &
                 DFACC_READ) 
  
  
Istatus = 0


!--- Read in values in the table
Istatus = 0

!-- read in vectors
 Sds_Start_1D = 0
 Sds_Stride_1D = 1

!-- sensor zenith angle   
Sds_Edge_1D = Num_Sat_zen
Sds_Id = Sfselect(Sd_Id,Sfn2Index(Sd_Id,"sensor_zenith_angle"))
Istatus = Sfrdata(Sds_Id,Sds_Start_1D, Sds_Stride_1D, Sds_Edge_1D, &
                  Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%Sat_Zen) +  &
                  Istatus
Istatus = Sfendacc(Sds_Id) + Istatus
 
!-- solar sensor zenith angle
Sds_edge_1d = Num_sol_zen
Sds_id = sfselect(sd_id,sfn2index(sd_id,"solar_zenith_angle"))
Istatus = sfrdata(sds_id,sds_start_1d, sds_stride_1d, sds_edge_1d, &
                  Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%sol_zen) +  &
                  istatus
Istatus = sfendacc(sds_id) + istatus
 
!-- relative azimith angle
 
sds_edge_1d = Num_Rel_Azi
sds_id = sfselect(sd_id,sfn2index(sd_id,"relative_azimuth_angle"))
istatus = sfrdata(sds_id,sds_start_1d, sds_stride_1d, sds_edge_1d, &
                  Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%Rel_Azi) +  &
                  istatus
istatus = sfENDacc(sds_id) + istatus


!-- log10 optical depth
 
sds_edge_1d = NUM_COD
sds_id = sfselect(sd_id,sfn2index(sd_id,"log10_optical_depth"))
istatus = sfrdata(sds_id,sds_start_1d, sds_stride_1d, sds_edge_1d, &
                  Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%COD_Vec) +  &
                  istatus
istatus = sfENDacc(sds_id) + istatus
 

!-- log10 effective radius
 
sds_edge_1d = NUM_CPS
sds_id = sfselect(sd_id,sfn2index(sd_id,"log10_eff_radius"))
istatus = sfrdata(sds_id,sds_start_1d, sds_stride_1d, sds_edge_1d, &
                  Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%CPS_Vec) +  &
                  istatus
istatus = sfENDacc(sds_id) + istatus
 

!--- 2d arrays
sds_start_2d = 0
sds_stride_2d = 1


!--- spherical albedo
sds_dims_2d(1) = NUM_CPS   ! NUM_CPS
sds_dims_2d(2) = NUM_COD  ! NUM_COD 
sds_edge_2d(1) = NUM_CPS
sds_edge_2d(2) = NUM_COD
sds_id = sfselect(sd_id,sfn2index(sd_id,"spherical_albedo"))
istatus = sfrdata(sds_id,sds_start_2d, sds_stride_2d, sds_edge_2d, &
                           Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%sph_alb) +  &
                           istatus
istatus = sfENDacc(sds_id) + istatus
 
 
!--- 3d arrays
sds_start_3d = 0
sds_stride_3d = 1
 
!--- albedo
sds_dims_3d(1) = NUM_CPS
sds_dims_3d(2) = NUM_COD
sds_dims_3d(3) = num_sol_zen
sds_edge_3d(1) = NUM_CPS
sds_edge_3d(2) = NUM_COD
sds_edge_3d(3) = num_sol_zen
sds_id = sfselect(sd_id,sfn2index(sd_id,"albedo"))
istatus = sfrdata(sds_id,sds_start_3d, sds_stride_3d, sds_edge_3d, &
                           Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%Cld_Alb) +  &
                           istatus
istatus = sfENDacc(sds_id) + istatus
 

 
!-- transmission
sds_dims_3d(1) = NUM_CPS
sds_dims_3d(2) = NUM_COD
sds_dims_3d(3) = num_sol_zen
sds_edge_3d(1) = NUM_CPS
sds_edge_3d(2) = NUM_COD
sds_edge_3d(3) = num_sol_zen
sds_id = sfselect(sd_id,sfn2index(sd_id,"transmission"))
istatus = sfrdata(sds_id,sds_start_3d, sds_stride_3d, sds_edge_3d, &
                           Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%trans) +  &
                           istatus
istatus = sfENDacc(sds_id) + istatus
 

!--- 5d arrays
sds_start_5d = 0
sds_stride_5d = 1


!--- reflectance
sds_dims_5d(1) = NUM_CPS
sds_dims_5d(2) = NUM_COD
sds_dims_5d(3) = num_sol_zen
sds_dims_5d(4) = num_sat_zen
sds_dims_5d(5) = Num_Rel_Azi

sds_edge_5d(1) = NUM_CPS
sds_edge_5d(2) = NUM_COD
sds_edge_5d(3) = num_sol_zen
sds_edge_5d(4) = num_sat_zen
sds_edge_5d(5) = Num_Rel_Azi

sds_id = sfselect(sd_id,sfn2index(sd_id,"reflectance"))
istatus = sfrdata(sds_id,sds_start_5d, sds_stride_5d, sds_edge_5d, &
                           Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%refl) +  &
                           istatus
istatus = sfENDacc(sds_id) + istatus
 
 
IF (istatus /= 0) THEN
  PRINT "(a,'Error reading sds data from cloud lut files, stopping')",EXE_PROMPT
   print*,trim(lut_file)
  STOP
END IF

!--- Close the lookup table file
istatus = sfEND(sd_id) 
IF (istatus /= 0) THEN
  PRINT "(a,'Error closing cloud lut files ')",EXE_PROMPT
  print*,trim(lut_file)
END IF
   
Cld_Refl_Lut%Channel(Idx_Chn)%Phase(Idx_Phase)%Flag = FLAG_YES

END SUBROUTINE POPULATE_CLOUD_LUT_SINGLE          





!====================================================================
! Subroutine Name: POPULATE_CLOUD_LUT_SINGLE_EMS(lut_file,Idx_Phase) 
!
! Function: populate cloud look-up table 
!
! Inputs: 
!     Lut_file - file there LUT is stored
!     Idx_Phase - phase number  in LUT
!
!====================================================================
 
SUBROUTINE POPULATE_CLOUD_LUT_SINGLE_EMS( &
      lut_file &    ! - input
        , Idx_Phase )

IMPLICIT NONE

! ---  input
CHARACTER (LEN=*), INTENT(in) ::  Lut_file   ! - file there LUT is stored
INTEGER , INTENT(in)  ::Idx_Phase             ! - phase number  in LUT 


   
! -  hdf stuff
INTEGER:: Astatus
INTEGER:: Tstatus
INTEGER:: Istatus
 
INTEGER:: Sfstart, Sfrcatt,Sfn2Index, Sffattr, Sfendacc,Sfend,Sfrdata,Sfselect
INTEGER:: Sd_Id
INTEGER:: Sds_Id
 
INTEGER, PARAMETER:: Sds_Rank_3D = 3
INTEGER, DIMENSION(Sds_Rank_3D):: Sds_Start_3D, Sds_Edge_3D, Sds_Stride_3D, Sds_Dims_3D




!----------------------------------------------------------------------------------
! Executable Code
!----------------------------------------------------------------------------------
 
 
 

Sd_Id = Sfstart( &
                 TRIM(lut_file), &
                 DFACC_READ) 
  
  
Istatus = 0


!--- 3d arrays
sds_start_3d = 0
sds_stride_3d = 1
 
!--- emissivity
sds_dims_3d(1) = NUM_CPS
sds_dims_3d(2) = NUM_COD
sds_dims_3d(3) = num_sat_zen
sds_edge_3d(1) = NUM_CPS
sds_edge_3d(2) = NUM_COD
sds_edge_3d(3) = num_sat_zen
sds_id = sfselect(sd_id,sfn2index(sd_id,"cloud_emissivity"))
istatus = sfrdata(sds_id,sds_start_3d, sds_stride_3d, sds_edge_3d, &
                           Cld_Ems_Lut%Phase(Idx_Phase)%Emiss) +  &
                           istatus
istatus = sfendacc(sds_id) + istatus
 

 
!-- transmission
sds_dims_3d(1) = NUM_CPS
sds_dims_3d(2) = NUM_COD
sds_dims_3d(3) = num_sol_zen
sds_edge_3d(1) = NUM_CPS
sds_edge_3d(2) = NUM_COD
sds_edge_3d(3) = num_sol_zen
sds_id = sfselect(sd_id,sfn2index(sd_id,"cloud_transmission"))
istatus = sfrdata(sds_id,sds_start_3d, sds_stride_3d, sds_edge_3d, &
                           Cld_Ems_Lut%Phase(Idx_Phase)%Trans) +  &
                           istatus
istatus = sfENDacc(sds_id) + istatus
  
 
IF (istatus /= 0) THEN
  PRINT "(a,'Error reading sds data from cloud EMD lut files, stopping')",EXE_PROMPT
PRINT*,TRIM(lut_file)
  STOP
END IF

!--- Close the lookup table file
istatus = sfEND(sd_id) 
IF (istatus /= 0) THEN
  PRINT "(a,'Error closing cloud EMSD lut files')",EXE_PROMPT
END IF
   
Cld_Ems_Lut%Phase(Idx_Phase)%Flag = FLAG_YES

END SUBROUTINE POPULATE_CLOUD_LUT_SINGLE_EMS      

 
!====================================================================
! SUBROUTINE Name: HDF_DCOMP_READ_ANCIL
!
! Function:     Reads ancillary data from HDF file
!
! Description: 
!
! Calling Sequence:
!
!      CALL HDF_DCOMP_READ_ANCIL(ancil_file,ozone,gas)
!
! Inputs: 
!     Ancil_file:  string which contains the ancillary HDF file
!
! Output:
!
!     Ozone : 3 ozone coefficients needed for transmission calculations in VIS channel
!     Ozone : 3 x 3 ozone coefficients needed for transmission calculations in VIS(0.6)  and 1.6 and 3.9 channels
!    
!
! History:    created April 2010
!
!  Global variables: 
!
! Local Variables:
!    
!     ozone_coeff
!     gas_coeff
!
!====================================================================
!

SUBROUTINE POPULATE_ANCIL(ancil_file)

!- input 
CHARACTER (LEN=*), INTENT(in):: Ancil_file

!- locals
REAL(KIND=real4), DIMENSION(3)::ozone_coeff
REAL(KIND=real4), DIMENSION(3,3)::gas_coeff

!-- hdf
INTEGER:: Sfstart, Sfrcatt,Sfn2Index, Sffattr, Sfendacc,Sfend,Sfrdata,Sfselect
INTEGER:: Sd_Id
INTEGER:: Sds_Id
INTEGER:: Istatus

INTEGER, PARAMETER:: Sds_Rank_1D = 1
INTEGER, DIMENSION(Sds_Rank_1D):: Sds_Start_1D, Sds_Edge_1D, Sds_Stride_1D

INTEGER, PARAMETER:: Sds_Rank_2D = 2
INTEGER, DIMENSION(Sds_Rank_2D):: Sds_Start_2D, Sds_Edge_2D, Sds_Stride_2D, Sds_Dims_2D

!----------------------------------------------------------------------------------
! Executable Code
!----------------------------------------------------------------------------------

Sd_Id = Sfstart( &
                                TRIM(ancil_file), &
                                DFACC_READ) 

Sds_Id = Sfselect(Sd_Id,Sfn2Index(Sd_Id,"Ozone_transmission"))
Istatus = Sfrdata(Sds_Id,0, 1, 3, ozone_coeff) 

sds_start_2d = 0
sds_stride_2d = 1

sds_edge_2d(1) = 3
sds_edge_2d(2) = 3

Sds_Id = Sfselect(Sd_Id,Sfn2Index(Sd_Id,"Gas_transmission"))
Istatus = Sfrdata(Sds_Id,sds_start_2d, sds_stride_2d, sds_edge_2d, gas_coeff)
istatus = sfENDacc(sds_id)

!--- initialize 
Ancil_Data%Gas_Trans = 0.0
Ancil_Data%Ozone_Trans = 0.0

!--- populate 
Ancil_Data%Gas_Trans  =  Gas_Coeff
Ancil_Data%Ozone_Trans  =  Ozone_Coeff
        
Ancil_Data%Flag = FLAG_YES

 END SUBROUTINE POPULATE_ANCIL
 
 !====================================================================
! SUBROUTINE Name: GET_LUT_DATA
!
! Function:      Interface to look-up-table data
!
! Description:    handles data access to LUT data. HDF read routines 
!
! Calling Sequence:
!
! Inputs: 
!    Idx_Chn     : Channel index 
!    Idx_Phase   : Phase index
!
! Output:
!    Sat_Zen_Vec_LUT - satellite zenith angle vector
!    Sol_Zen_Vec_LUT - solar zenith angle vector
!    Rel_Azi_Vec_LUT - relative azimuth vector
!    COD_Vec_LUT     - log10 optical depth vector
!    CPS_Vec_LUT     - log10 effective radius vector
!    Sph_Alb_2D_LUT  - spherical albedo
!    Trans_3D_LUT    - flux transmission
!    Cld_Alb_3D_LUT  - cloud albedo
!    Refl_5D_LUT - reflectance
!    Trans_Ems_3D_LUT - Transmission ems
!    Ems_3D_LUT - reflectance
!    Num_Sol_Zen_LUT - total # solar zeniths
!    Num_Sat_Zen_LUT - total # satellite zeniths
!    Num_Rel_Azi_LUT - total # relative azimuths
!    Ozone_Coeff
!    Gas_Coeff
!
! History:                   
!
!====================================================================
  
SUBROUTINE GET_LUT_DATA(Idx_Chn,Idx_Phase &   
    ,Num_Sol_Zen_LUT &
        ,Num_Sat_Zen_LUT &
        ,Num_Rel_Azi_LUT &
    ,Sat_Zen_Vec_LUT &
        ,Sol_Zen_Vec_LUT &
        ,Rel_Azi_Vec_LUT &
        ,COD_Vec_LUT &
        ,CPS_Vec_LUT &
        ,Sph_Alb_2D_LUT &
        ,Trans_3D_LUT &
        ,Cld_Alb_3d_LUT &
        ,Refl_5D_LUT &
        ,Trans_EMS_3D_LUT &
        ,Ems_3D_LUT &
        ,Gas_Coeff &
        ,Ozone_Coeff  )
        
! ---  input
INTEGER , INTENT(in) :: Idx_chn    ! channel number
INTEGER , INTENT(in) :: Idx_phase  ! phase number

! --- optional output
! - number of parameters 
INTEGER,INTENT(out), OPTIONAL:: Num_Sol_Zen_LUT
INTEGER,INTENT(out), OPTIONAL:: Num_Sat_Zen_LUT
INTEGER,INTENT(out), OPTIONAL:: Num_Rel_Azi_LUT           !- total number relative azimuths
 
! -- data 
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:) ::     Sat_Zen_Vec_LUT   ! - satellite zenith angle vector 
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:) ::     Sol_Zen_Vec_LUT   ! - solar zenith angle vector
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:) ::     Rel_Azi_Vec_LUT   ! - relative azimuth vector
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:) ::     COD_Vec_LUT       ! - log10 optical depth vector
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:) ::     CPS_Vec_LUT       ! - log10 effective radius vector
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:,:) ::   Sph_Alb_2D_LUT    ! - spherical albedo
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:,:,:) :: Trans_3D_LUT      ! - flux transmission
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:,:,:) :: Cld_Alb_3D_LUT    ! - cloud albedo
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:,:,:,:,:) :: Refl_5D_LUT   ! - reflectance
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:,:,:) :: Trans_Ems_3D_LUT   ! - Transmission ems
REAL (Kind=Real4),INTENT(out),OPTIONAL ,POINTER, DIMENSION(:,:,:) :: Ems_3D_LUT   ! - reflectance
  
! - data from ancil file 
REAL(KIND=real4),INTENT(out), OPTIONAL , DIMENSION(3)::Ozone_Coeff
REAL(KIND=real4),INTENT(out), OPTIONAL , DIMENSION(3)::Gas_Coeff
  

INTEGER::Idx_Chn_LUT  ! - substitutes sensor channel number with 1 for VIS or 2 for IR 
   
INTEGER::sensor_channel_idx
  
!-------------------------------------------------------------------------------
! Executable Code
!-------------------------------------------------------------------------------

! - set  table numbers  outdated stuff can be removed after testing
!IF (Idx_chn .EQ. VIS_CHN_NUMBER) Idx_Chn_LUT = 1
!IF (Idx_chn .EQ.  IR_CHN_NUMBER_3A) Idx_Chn_LUT = 2
!IF (Idx_chn .EQ.  IR_CHN_NUMBER_3B) Idx_Chn_LUT = 3

 Idx_chn_LUT = Idx_chn


 ! - populate optional output variables
IF (present(Num_Sol_Zen_LUT)) &
  Num_Sol_Zen_LUT = SIZE(Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Sol_Zen)
  
IF (present(Num_Sat_Zen_LUT)) &
  Num_Sat_Zen_LUT = SIZE(Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Sat_Zen)
  
IF (present(Num_Rel_Azi_LUT)) &
  Num_Rel_Azi_LUT = SIZE(Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Rel_Azi)  

IF (present(Sat_Zen_Vec_LUT)) &
  Sat_Zen_Vec_LUT => Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Sat_Zen  
  
IF (present(Sol_Zen_Vec_LUT)) &
  Sol_Zen_Vec_LUT => Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Sol_Zen 
  
IF (present(Rel_Azi_Vec_LUT)) &
  Rel_Azi_Vec_LUT => Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Rel_Azi 
  
IF (present(CPS_Vec_LUT)) &
  CPS_Vec_LUT =>         Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%CPS_Vec  
  
IF (present(COD_Vec_LUT)) &
  COD_Vec_LUT =>         Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%COD_Vec 
  
IF (present(Sph_Alb_2D_LUT)) &
  Sph_alb_2D_LUT =>  Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Sph_Alb  
  
IF (present(Trans_3D_LUT)) &
  Trans_3D_LUT =>        Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Trans 
  
IF (present(Cld_Alb_3D_LUT)) &
  Cld_Alb_3D_LUT =>  Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Cld_Alb 
  
IF (present(Refl_5D_LUT)) &
  Refl_5D_LUT =>         Cld_Refl_Lut%Channel(Idx_Chn_LUT)%Phase(Idx_Phase)%Refl 
   
IF (present(Trans_ems_3D_LUT)) &
  Trans_ems_3D_LUT =>   Cld_Ems_Lut%Phase(Idx_Phase)%Trans   
 
IF (present(Ems_3d_LUT)) &
   Ems_3D_LUT => Cld_Ems_Lut%Phase(Idx_Phase)%Emiss 

! - populate optional output ancil parameters
IF (present(Gas_coeff)) &
  Gas_coeff = Ancil_Data%Gas_Trans(Idx_Chn_LUT,:)
  
IF (present(Ozone_coeff)) THEN

  select case (Idx_Chn)
   case(1) 
      Ozone_coeff(:) = (/-0.000606266,9.77984e-05,-1.67962e-08/)
   case(2) 
      Ozone_coeff(:) = (/-0.000606266,9.77984e-05,-1.67962e-08/)
   case(5) 
      Ozone_coeff(:) = (/0.000178786,8.88510e-05,3.46661e-09/)
   case(6) 
      Ozone_coeff(:) = (/0.000986278,8.23759e-05,1.07635e-08/)
   case(7) 
      Ozone_coeff(:) = (/0.000638877,8.45589e-05,9.84943e-09/)
   case(8) 
      Ozone_coeff(:) = (/-2.74525e-05,9.02744e-05,2.31578e-09/)
   case(9) 
      Ozone_coeff(:) = (/-0.000856822,9.79630e-05,-1.48645e-08/)
   case(10) 
      Ozone_coeff(:) = (/0.000663486,8.40186e-05,1.07917e-08/)
   case(11) 
      Ozone_coeff(:) = (/0.000897866,8.26629e-05,1.45399e-08/)
   case(12) 
      Ozone_coeff(:) = (/0.000210246,8.78771e-05,4.71260e-09/)
   case(13) 
      Ozone_coeff(:) = (/0.00114584,8.06184e-05,1.62876e-08/)
   case(14) 
      Ozone_coeff(:) = (/-0.000665083,9.51923e-05,-8.21226e-09/)
   case(15)
      Ozone_coeff(:) = (/0.000898769,8.11015e-05,1.69703e-08/)
   case(16) 
      Ozone_coeff(:) = (/-0.000822729,9.65522e-05,-1.17455e-08/)
   case(17)
      Ozone_coeff(:) = (/0.000913279,8.22402e-05,1.27435e-08/)
   case(18)
      Ozone_coeff(:) = (/0.000258731,8.80321e-05,4.91110e-09/)
   case default
      Ozone_coeff(:) = (/0.000258731,8.80321e-05,4.91110e-09/)
 
 end select 

 
ENDIF
END SUBROUTINE GET_LUT_DATA


END MODULE HDF_DCOMP

