!$Id: level2b.f90,v 1.23 2010/09/23 15:38:12 wstraka Exp $
!----------------------------------------------------------------------------
!@***h* PATMOSX/level2b.f90
! 
! NAME
!   level2b.f90
! 
! LOCATION
!   /patmosx/src/level2b.f90
! 
! PURPOSE
!   Routines for creating, writing and closing  L2b output files
! 
! DESCRIPTION
!   Routines for creating, writing and closing  L2b output files
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   May 2009
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
! EXTERNALS
!  use CONSTANTS
!  use HDF
!  use LEVEL2_ROUTINES
!  use SCALING_PARAMETERS
!
!  public:: LAGRANGIAN_ANCHOR_INTERP, &
!           LINEAR_ANCHOR_INTERP, &
!           GNOMIC_ANCHOR_INTERP
!           DEFINE_SDS    !place in interface
!           READ_SDS     
!           UNSCALE_SDS   !place in interface
!           SCALE_SDS     !place in interface
!           WRITE_SDS     
!           COPY_GLOBAL_ATTRIBUTES    
!           REGRID
!           SUBSET_LEVEL2B
!
!@*****
!------------------------------------------------------------------------
!
! MODULE name: Level2b_routines
! 
! Routines for creating, writing and closing  L2b output files
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! History:
! Creation Date May 2009
!----------------------------------------------------------------------
module LEVEL2B_ROUTINES
 use CONSTANTS
 use HDF
 use LEVEL2_ROUTINES
 use SCALING_PARAMETERS

 implicit none

 public:: LAGRANGIAN_ANCHOR_INTERP, &
          LINEAR_ANCHOR_INTERP, &
          GNOMIC_ANCHOR_INTERP
 public:: DEFINE_SDS    !place in interface
 public:: READ_SDS     
 public:: UNSCALE_SDS   !place in interface
 public:: SCALE_SDS     !place in interface
 public:: WRITE_SDS     
 public:: COPY_GLOBAL_ATTRIBUTES    
 public:: REGRID
 public:: SUBSET_LEVEL2B

 interface READ_SDS
     module procedure  &
         READ_SDS_RANK1,  &
         READ_SDS_RANK2
 end interface

 interface WRITE_SDS
     module procedure  &
         WRITE_SDS_RANK1,  &
         WRITE_SDS_RANK2
 end interface

 
 type, public :: sds_struct
  character(len=100):: variable_name
  integer(kind=int4):: data_type
  integer(kind=int4):: rank
  integer(kind=int1):: scaling_type
  character(len=100):: units
  character(len=100):: long_name
  character(len=100):: standard_name
  real(kind=real4):: unscaled_min
  real(kind=real4):: unscaled_max
  real(kind=real4):: unscaled_missing
  real(kind=real4):: scale_factor
  real(kind=real4):: add_offset
  integer(kind=int4):: scaled_min
  integer(kind=int4):: scaled_max
  integer(kind=int4):: scaled_missing
  integer(kind=int4):: fill_value
  integer(kind=int4):: status_flag
  integer(kind=int4):: id_input
  integer(kind=int4):: id_output
  integer(kind=int4):: compression_type
  integer(kind=int4):: num_attrs
 end type sds_struct

 character(24), parameter, private :: MOD_PROMPT = " AVHRR_LEVEL2B_ROUTINES: "

 real(kind=real4), dimension(:,:), allocatable, public, save:: lat_input
 real(kind=real4), dimension(:,:), allocatable, public, save:: lon_input
 real(kind=real4), dimension(:,:), allocatable, public, save:: lat_output
 real(kind=real4), dimension(:,:), allocatable, public, save:: lon_output
 integer(kind=int4), dimension(:,:), allocatable, public, save:: ielem_output
 integer(kind=int4), dimension(:,:), allocatable, public, save:: iline_output

 integer, parameter, private:: subset_pixel_hdf_flag = 0
 integer, parameter, private:: comp_type = 0
 integer, parameter, private:: comp_prm = 0

 contains

!====================================================================
! SUBROUTINE Name: REGRID
!
! Function: 
!       routines to remap data to a regular lat,lon grid
!
! Description:
!   This subroutine is a routine to remap data from lat/lon to a 
!   static grid.
!
! Inputs:
!       lon_output = vector of the longitudes of the grid
!       lat_output = vector of the latitudes of the grid
!       ielem_output = array of element indices for each grid point
!       iline_output = array of line indices for each grid point
!====================================================================
subroutine REGRID( &
        lat_south, &
        dlat,      &
        nlat,      &
        lon_west,  &
        dlon,      &
        nlon)

real(kind=real4), intent(in):: lat_south
real(kind=real4), intent(in):: dlat
integer(kind=int4), intent(in):: nlat
real(kind=real4), intent(in):: lon_west
real(kind=real4), intent(in):: dlon
integer(kind=int4), intent(in):: nlon

real(kind=real4):: big_number
real(kind=real4):: max_allowable_distance
real(kind=real4):: max_allowable_scanline_latitude_spacing
real(kind=real4), dimension(:,:),allocatable:: min_dist_grid
integer:: ielem
integer:: iline
integer:: ilon
integer:: ilat
integer:: nx
integer:: ny
real:: lat_north
real:: lon_east
real:: xdist
real:: xdist_01
real:: xdist_02
real:: xdist_10
real:: xdist_20
real:: lat_width
real:: lon_width
integer:: ilon_1, ilon_2
integer:: ilon_min, ilon_max
real:: lon_1, lon_2
integer:: ilat_1, ilat_2
integer:: ilat_min, ilat_max
real:: lat_1, lat_2
integer:: i1,i2,j1,j2
integer:: dateline_flag
real:: lon_east_abs
real:: lon_input_abs

big_number = 999999.9

!--- construct boundaries
lon_east_abs = lon_west + dlon * (nlon-1)
lon_east = lon_east_abs
if (lon_east > 180.0) lon_east = lon_east - 360.0

dateline_flag = sym%NO
if (lon_west > 0 .and. lon_east < 0.0) dateline_flag = sym%YES

lat_north = lat_south + dlat * (nlat-1)

nx = size(lat_input,1)
ny = size(lat_input,2)

ielem_output = missing_value_int4
iline_output = missing_value_real4

max_allowable_distance = sqrt(dlat**2 + dlon**2)

ALLOCATE(min_dist_grid(nlon,nlat))
min_dist_grid = big_number

do ielem = 1, nx
  do iline = 1, ny
       
    if ((lat_input(ielem,iline) >= lat_south) .and. (lat_input(ielem,iline) <= lat_north)) then

      lon_input_abs = lon_input(ielem,iline)
      if (lon_input_abs < 0.0) lon_input_abs = lon_input_abs + 360.0

      if ( ( (dateline_flag == sym%NO) .and. &                !dateline check
             (lon_input(ielem,iline) >= lon_west) .and. &
             (lon_input(ielem,iline) <= lon_east)) .or. & 
           ( (dateline_flag == sym%YES) .and.  &
             (lon_input_abs >= lon_west) .and. &
             (lon_input_abs <= lon_east_abs))) then 

        i1 = max(1,ielem-1)
        i2 = min(nx,ielem+1)
        j1 = max(1,iline-1)
        j2 = min(ny,iline+1)

        lon_width = abs(lon_input(i1,iline) - lon_input(i2,iline))
        lon_1 = lon_input(ielem,iline) + lon_width/2.0
        lon_2 = lon_input(ielem,iline) - lon_width/2.0

        !-- lon width constraints prevents large values in presence of scanline jumps
        lat_width = min(lon_width,abs(lat_input(ielem,j1) - lat_input(ielem,j2)))
        lat_1 = lat_input(ielem,iline) + lat_width/2.0
        lat_2 = lat_input(ielem,iline) - lat_width/2.0

        call position_to_indices (ilat, ilon, lat_input(ielem,iline), lon_input(ielem,iline),  &
            lat_south, lat_north, dlat, nlat, lon_west, lon_east,dlon, nlon, xdist)

        call position_to_indices (ilat, ilon_1, lat_input(ielem,iline), lon_1,  &
            lat_south, lat_north, dlat, nlat, lon_west, lon_east,dlon, nlon, xdist_10)

        call position_to_indices (ilat, ilon_2, lat_input(ielem,iline), lon_2,  &
            lat_south, lat_north, dlat, nlat, lon_west, lon_east,dlon, nlon, xdist_20)

        call position_to_indices (ilat_1, ilon, lat_1, lon_input(ielem,iline),  &
            lat_south, lat_north, dlat, nlat, lon_west, lon_east,dlon, nlon, xdist_01)

        call position_to_indices (ilat_2, ilon, lat_2, lon_input(ielem,iline),  &
            lat_south, lat_north, dlat, nlat, lon_west, lon_east,dlon, nlon, xdist_02)

        ilon_min = min(ilon_1,ilon_2)
        ilon_max = max(ilon_1,ilon_2)
        ilat_min = min(ilat_1,ilat_2)
        ilat_max = max(ilat_1,ilat_2)

        !update closest grid point

        if ((ilon >= 1) .and. (ilat >= 1)) then    !check for valid regrid result

          if ((xdist < min_dist_grid(ilon,ilat) .and. &  !check on distance
             xdist < max_allowable_distance)) then

            ielem_output(ilon,ilat)  = ielem
            iline_output(ilon,ilat)  = iline
            min_dist_grid(ilon,ilat) = xdist

            !update surrounding grid points that fall within the pixel's footprint but
            !have not already been filled in by a closest pixel
            if ((ilon_min > 1).and.(ilat_min > 1)) then
              do ilon_1 = ilon_min, ilon_max
                do ilat_1 = ilat_min, ilat_max
                  !-- do not reset grid point closest to pixel
                  if ((ilon_1 == ilon) .and. (ilat_1 == ilat)) cycle
                  !-- fill in values for all grid points within pixel footprint
                  !-- ignore those already set previously
                  if (min_dist_grid(ilon_1,ilat_1) == big_number) then
                    ielem_output(ilon_1,ilat_1)  = ielem
                    iline_output(ilon_1,ilat_1)  = iline
                  endif
                end do
              end do
            endif

          end if   !end check on distance
        end if     ! valid regird

      end if      !dateline check

    end if      !latitude check longitude check

  end do
end do
   
DEALLOCATE(min_dist_grid)
!--- make grid lat and lons
do ilat = 1, nlat
  do ilon = 1, nlon
       lat_output(ilon,ilat) = lat_south + (ilat-1)*dlat
       lon_output(ilon,ilat) = lon_west + (ilon-1)*dlon
    end do
  end do

  where(lon_output > 180.0)
     lon_output = lon_output - 360.0
  end where

  end subroutine REGRID

!====================================================================
! SUBROUTINE Name: indices_to_position
!
! Function: 
!       Converts grid indicies to lat/lon position
!
! Description:
!   This subroutine is a routine to convert the grid index to a lat/lon
!
!====================================================================
subroutine indices_to_position (ilat,ilon, lat, lon,  &
                                lat_south, lat_north,  &
                                dlat, nlat, lon_west, lon_east,dlon, nlon)
      integer(kind=int4), intent(in):: ilat
      integer(kind=int4), intent(in):: ilon
      integer(kind=int4), intent(in):: lat_south
      integer(kind=int4), intent(in):: lat_north
      integer(kind=int4), intent(in):: dlat
      integer(kind=int4), intent(in):: nlat
      integer(kind=int4), intent(in):: lon_west
      integer(kind=int4), intent(in):: lon_east
      integer(kind=int4), intent(in):: dlon
      integer(kind=int4), intent(in):: nlon
      real(kind=real4), intent(out):: lat
      real(kind=real4), intent(out):: lon
   
      !--- initialize
      lat = -999.0
      lon = -999.0

      if ((ilat >= 1) .and. (ilat <= nlat) .and. (ilon >= 1) .and. (ilon <= nlon)) then
       lat = lat_south + (ilat-1)*dlat
       lon = lon_west + (ilon-1)*dlon
      endif

      if (lon > 180.0) then 
           lon = lon - 360.0
      endif
end subroutine indices_to_position

!====================================================================
! SUBROUTINE Name: position_to_indices
!
! Function: 
!       Converts lat/lon position to grid indicies 
!
! Description:
!   This subroutine is a routine to convert lat/lon position to grid index
!
!====================================================================

subroutine position_to_indices(ilat,ilon, lat, lon,  &
                               lat_south, lat_north, &
                               dlat, nlat, lon_west, lon_east, &
                               dlon, nlon, distance)

       real(kind=real4), intent(in):: lat
       real(kind=real4), intent(in):: lon
       real(kind=real4), intent(in):: lat_south
       real(kind=real4), intent(in):: lat_north
       real(kind=real4), intent(in):: dlat
       integer(kind=int4), intent(in):: nlat
       real(kind=real4), intent(in):: lon_west
       real(kind=real4), intent(in):: lon_east
       real(kind=real4), intent(in):: dlon
       integer(kind=int4), intent(in):: nlon
       integer(kind=int4), intent(out):: ilat
       integer(kind=int4), intent(out):: ilon
       real(kind=real4), intent(out):: distance
       real(kind=real4):: lon_abs
       real(kind=real4):: lon_east_abs
       real(kind=real4):: lat_abs

       ilat = -999
       ilon = -999

      lon_east_abs = lon_east
      if (lon_east_abs < 0.0) lon_east_abs = lon_east_abs + 360.0
      lon_abs = lon
      if (lon_abs < 0.0) lon_abs = lon_abs + 360.0

      if ((lat >= lat_south) .and. (lat <= lat_north)) then
         ilat = nint((lat - lat_south) / dlat) + 1
         ilat = min(nlat,max(1,ilat))
      endif

     !-- -no date line
     if (lon_west  < lon_east) then
      if ((lon >= lon_west) .and. (lon <= lon_east)) then
        ilon = nint((lon - lon_west) / dlon) + 1
        ilon = min(nlon,max(1,ilon))
      end if
     else 
      if (lon < 0.0) lon_abs = lon + 360.0
      if ((lon_abs <= lon_east_abs) .and. (lon_abs >= lon_west)) then
        ilon = nint((lon_abs - lon_west) / dlon) + 1
        ilon = min(nlon,max(1,ilon))
      endif
     end if

     distance = 999.9
     if ((ilon >= 1) .and. (ilat >= 1)) then
             lat_abs = lat_south + (ilat-1)*dlat
             lon_east_abs = lon_west + (ilon-1)*dlon
             lon_abs = lon
             if (lon_west > 0.0 .and. lon < 0.0) lon_abs = lon + 360.0
             distance = (lat-lat_abs)**2 + (lon_abs-lon_east_abs)**2
     end if

end subroutine position_to_indices


!---------------------------------------------------------------------
! SUBROUTINE Name: LAGRANGIAN_ANCHOR_INTERP

! EXPLANATION OF LAC TO GAC GEOLOCATION CONVERSION:
!
! A LAC scan line is converted to GAC line in the following way:
!
!  1. GAC processing starts from the 3rd LAC pixel; 1st and 2nd LAC pixels are
!     not used.
!
!  2. The 1st GAC pixel is formed from the 3rd, 4th, 5th and 6th LAC pixel.
!
!  3. The 7th LAC pixel is skipped and the processing for the 2nd GAC pixel uses
!     the 8th, 9th, 10th and 11th LAC pixel.
!
!  4. LAC pixels contributing to a GAC pixel can be described with a formula:
!
!        l, l+1, l+2, l+3 for l = 5*g-2 where g = 1,..,409
!
!     `l' and `g' being LAC and GAC pixel index, respectively, in a scan line.
!
!  5. Geolocation of the third LAC pixel of the four used for a GAC pixel is
!     assigned to that GAC pixel. For the 1st and 2nd GAC pixels these are the
!     5th and 10th LAX pixels. Or in short:
!
!        l = 5*g (g=1,..,409)
!
!  6. The 2048th LAC pixel is not used for GAC pixel processing. The processing
!     ends with the 2047th LAX pixels, for a total of 2045 LAX pixels used
!     (2045/5=409).
!
! This subroutine performs a five-point Lagrangian interpolation for 2048 LAC
! pixels regardless of the actual length of the scan line (GAC or LAC). If it's
! a GAC scan line, the geolocations of the 2nd and 3rd LAC pixels for each GAC
! pixel are averaged to yield the final GAC pixel's geolocation.
!---------------------------------------------------------------------


 subroutine LAGRANGIAN_ANCHOR_INTERP(num_pix,anchor,y)
   integer, intent(in):: num_pix
   real, dimension(:), intent(in):: anchor
   real, dimension(:), intent(out):: y
   real, dimension(2048):: temp_y
   real, dimension(size(anchor)):: temp_anchor
   real:: x,L,xa_i,xa_j
   integer:: m,i,j,ia,ia1,ia2,ipix,num_anchor,num_skip,num_start
   integer:: discon

   m = 2    !2 means 5 point interp
   num_anchor = size(anchor,1)  !should be 51

! I'm leaving `num_skip' and `num_start' here as useful constants although they
! are not used in this subroutine.
   if (num_pix == 409) then
      ! It's a GAC scan line.
      num_start = 5 !number of pixels between anchors
      num_skip = 8  !pixel number for first anchor point
   else
      ! Must be a LAC scan line.
      num_start = 25
      num_skip = 40
   endif

!--- initialize
   y = 0.0
   temp_y = 0.0
   temp_anchor = anchor

!-------- check for a discontinuity (ie. crossing the 180W meridian)
    discon = 0
    if ((minval(temp_anchor) < -140.0).and.(maxval(temp_anchor) > 140.0)) then
     discon = 1
     where (temp_anchor < 0.0)
        temp_anchor =  temp_anchor + 360.0
     endwhere
    endif

!-- perform 2m+1 point Lagrangian interpolation
      do ipix = 1, 2048
         ia = (ipix-25)/40+1
         ia1 =  min(num_anchor,max(1,ia-m))
         ia2 =  max(1,min(num_anchor,ia+m))
         x = ipix
         do i = ia1,ia2
           L = 1.0
           xa_i = (i-1)*(40)+25
           do j = ia1,ia2
            xa_j = (j-1)*(40)+25
            if (i /= j) L = L*(x-xa_j)/(xa_i-xa_j)
          enddo
          temp_y(ipix) = temp_y(ipix)+L*temp_anchor(i)
         enddo
      enddo

! If it's a GAC scan line perform averaging of `temp_y' for the 2nd and 3rd LAC
! pixel going into every GAC pixel for the final value of `y'.
if (num_pix == 409) then
   do ipix = 1, 409
      y(ipix) = 0.5*(temp_y(5*ipix)+temp_y(5*ipix-1))
   enddo
else
   y = temp_y
endif

!----- account for any discontinuities
   where(y > 180.0)
    y = y - 360.0
   endwhere
   where(y < -180.0)
    y = 360.0 + y
   endwhere

 end subroutine LAGRANGIAN_ANCHOR_INTERP

!---------------------------------------------------------------------
! SUBROUTINE Name: GNOMIC_ANCHOR_INTERP
! 
! Function: 
!       This is the gnomic navigation anchor interpolation routine
!       note, this is only valid for lat and lon, do not apply to angles
!---------------------------------------------------------------------
 subroutine GNOMIC_ANCHOR_INTERP(num_pix,lon_anchor_deg,lat_anchor_deg,longitude,latitude)

   integer, intent(in):: num_pix
   real, dimension(:), intent(in):: lon_anchor_deg,lat_anchor_deg
   real, dimension(:), intent(out):: longitude,latitude
   real, dimension(size(lon_anchor_deg)):: k,x_anchor,y_anchor, &
                                           lon_anchor,lat_anchor
   real, dimension(num_pix):: x,y
   real:: yy,xx,xa1,xa2,rho,v,v_prev,c,lat_center,lon_center,offset
   integer:: ia1,ia2,ipix,num_anchor,num_skip,num_start


   num_anchor = size(lat_anchor)  !should be 51
   num_skip = num_pix / num_anchor
   num_start = (num_pix - num_skip*(num_anchor-1))/2 + 1

!--- initialize
     latitude = 0.0
     longitude = 0.0

!--- convert anchors to radians
     lat_anchor = lat_anchor_deg * dtor
     lon_anchor = lon_anchor_deg * dtor

!--- pick center for projection (in radians)
     lat_center = lat_anchor(num_anchor/2)
     lon_center = lon_anchor(num_anchor/2)

!--- convert to gnomic space
     k = sin(lat_center) * sin(lat_anchor) +  &
         cos(lat_center) * cos(lat_anchor) * cos(lon_anchor - lon_center)
     k = 1.0 / k

     x_anchor = k * cos(lat_anchor) * sin(lon_anchor - lon_center)
     y_anchor = k * (cos(lat_center) * sin(lat_anchor) -  &
                     sin(lat_center) * cos(lat_anchor) * cos(lon_anchor - lon_center))

!--- linear interp in gnomic space
      do ipix = 1, num_pix

!--- determine two closest anchor points
         ia1 = max(1,min(num_anchor-1,(ipix - num_start)/num_skip + 1))
         ia2 = ia1 + 1

!--- determine weights of two closest anchor points
         xa1 = (ia1-1)*(num_skip) + num_start
         xa2 = (ia2-1)*(num_skip) + num_start

!--- determine point to interpolate to
         xx = ipix

!--- interp x at xx
         yy = (xx-xa1)*(x_anchor(ia2)-x_anchor(ia1))/(xa2-xa1) +  &
                   x_anchor(ia1)
         x(ipix) = yy
         
!--- interp y at xx
         yy = (xx-xa1)*(y_anchor(ia2)-y_anchor(ia1))/(xa2-xa1) +  &
                   y_anchor(ia1)
         y(ipix) = yy

      enddo

!--- convert other quantities
      v_prev = 0.0
      offset = 0
      do ipix = 1,num_pix

!--- some needed constants
        rho = sqrt(x(ipix)**2 + y(ipix)**2)

!--- handle center point where rho = 0.0
        if (rho == 0.0) then 
           latitude(ipix) = lat_center
           longitude(ipix) = lon_center
           cycle
        endif
        c = atan(rho)

!--- v is the longitude in radians relative to the center longitude
        v = atan( x(ipix) * sin(c) /  &
          (rho * cos(lat_center) * cos(c) - y(ipix) * sin(lat_center) * sin(c)) )

!--- look for a jump and determine offset
        if (ipix == 1) then
          v_prev = v
        endif

        latitude(ipix) = asin(cos(c) * sin(lat_center) + (y(ipix)*sin(c)*cos(lat_center) / rho))
        longitude(ipix) = lon_center + v

        v_prev = v

      enddo

!--- convert to degrees
      latitude = latitude / dtor
      longitude = longitude / dtor
     

!----- account for discontinuity if there was one
  where (longitude > 180.0)
    longitude = longitude - 360.0
  endwhere
  where (longitude < -180.0)
    longitude = longitude + 360.0
  endwhere

  end subroutine GNOMIC_ANCHOR_INTERP

!---------------------------------------------------------------------
! SUBROUTINE Name: LINEAR_ANCHOR_INTERP
!
! Function:
!       Perfoms linear interpolation of the various anchor points.
!---------------------------------------------------------------------
 subroutine LINEAR_ANCHOR_INTERP(num_pix,anchor,y)
   integer, intent(in):: num_pix
   real, dimension(:), intent(in):: anchor
   real, dimension(:), intent(out):: y
   real, dimension(size(anchor)):: temp_anchor
   real:: x,xa1,xa2
   integer:: ia1,ia2,ipix,num_anchor,num_skip,num_start
   integer:: discon

   num_anchor = size(anchor)  !should be 51
   num_skip = num_pix / num_anchor
   num_start = (num_pix - num_skip*(num_anchor-1))/2 + 1

!--- initialize
     y = 0.0
     temp_anchor = anchor

!-------- check for a discontinuity (ie. crossing dateline)
    discon = 0
    if ((minval(temp_anchor) < -175.0).and.(maxval(temp_anchor) > 175.0)) then
     discon = 1
     where( temp_anchor < 0.0)
        temp_anchor =  temp_anchor + 360.0
     endwhere
    endif

!-----------------------------------------------------------
      do ipix = 1, num_pix
         ia1 = max(1,min(num_anchor-1,(ipix - num_start)/num_skip + 1))
         ia2 = ia1 + 1

         xa1 = (ia1-1)*(num_skip) + num_start
         xa2 = (ia2-1)*(num_skip) + num_start

         x = ipix

         y(ipix) = (x-xa1)*(temp_anchor(ia2)-temp_anchor(ia1))/(xa2-xa1) +  &
                   temp_anchor(ia1)

      enddo

!----- account for discontinuity if there was one
  if (discon == 1) then
   where (y > 180.0)
    y = y - 360.0
   endwhere
  endif

  end subroutine LINEAR_ANCHOR_INTERP
!---------------------------------------------------------------
! SUBROUTINE Name: DEFINE_SDS
!
! Function:
!       Define the SDS's of the Level 2b file
!---------------------------------------------------------------
  subroutine DEFINE_SDS(sd_id,     &
                        sds_dims,   &
                        sds)

    integer, intent(in):: sd_id
    type(sds_struct), intent(inout):: sds
    integer, dimension(2), intent(in):: sds_dims
    character(len=100):: sds_name_temp

    integer:: dim_id
    integer:: len

    integer:: sfcreate
    integer:: sfsnatt
    integer:: sfscatt
    integer:: sfdimid
    integer:: sfsdmname
    integer:: sfschnk
    integer:: istatus_sum

    istatus_sum = 0

     sds_name_temp = trim(sds%variable_name)

     !--- strip of anchor prefix
     !if (sds_name_temp(1:6) == "anchor") then
     !    len = len_trim(sds_name_temp)
     !    sds_name_temp = sds_name_temp(8:len)
     ! endif

     sds%id_output = sfcreate(sd_id,trim(sds_name_temp),sds%data_type,sds%rank,sds_dims)

     istatus_sum = sfsnatt(sds%id_output, "SCALED", DFNT_INT8, 1, sds%scaling_type) + istatus_sum
     istatus_sum = sfscatt(sds%id_output, "UNITS", DFNT_CHAR8, len_trim(sds%units), trim(sds%units)) + istatus_sum
     istatus_sum = sfscatt(sds%id_output, "STANDARD_NAME", DFNT_CHAR8, len_trim(sds%standard_name),  &
                           trim(sds%standard_name)) + istatus_sum
     istatus_sum = sfscatt(sds%id_output, "LONG_NAME", DFNT_CHAR8, len_trim(sds%long_name),  &
                           trim(sds%long_name)) + istatus_sum

     dim_id = sfdimid(sds%id_output, 0)
     istatus_sum = sfsdmname(dim_id,"longitude index") + istatus_sum

     dim_id = sfdimid(sds%id_output, 1)
     istatus_sum = sfsdmname(dim_id,"latitude index") + istatus_sum

     istatus_sum = sfsnatt(sds%id_output, "RANGE_MISSING", DFNT_FLOAT32, 1, sds%unscaled_missing) + istatus_sum

     !--- determined if a scaled sds, if so write needed attributes for scaling
     if (sds%scaling_type > 0) then

      !--- write remaining attributes
      istatus_sum = sfsnatt(sds%id_output, "SCALE_FACTOR", DFNT_FLOAT32, 1, sds%scale_factor) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "ADD_OFFSET", DFNT_FLOAT32, 1, sds%add_offset) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "RANGE_MIN", DFNT_FLOAT32, 1, sds%unscaled_min) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "RANGE_MAX", DFNT_FLOAT32, 1, sds%unscaled_max) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "SCALED_MIN", DFNT_INT32, 1, sds%scaled_min) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "SCALED_MAX", DFNT_INT32, 1, sds%scaled_max) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "SCALED_MISSING", DFNT_INT32, 1, sds%scaled_missing) + istatus_sum
      istatus_sum = sfsnatt(sds%id_output, "_FILLVALUE", DFNT_INT32, 1, sds%fill_value) + istatus_sum

   endif

   sds%status_flag = istatus_sum

  end subroutine DEFINE_SDS
!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: READ_SDS_RANK1
!
! Function:
!     Reads 1dimensional SDSs from Level 2b files. 
!-----------------------------------------------------------------------------------------------
  subroutine READ_SDS_RANK1(sd_id,     &
                            scaled_sds_data,   &
                            sds)

    integer, intent(in):: sd_id
    type(sds_struct), intent(inout):: sds
    real(kind=real4), dimension(:), intent(out):: scaled_sds_data
    integer, dimension(1):: sds_dims
    integer, dimension(1):: sds_start
    integer, dimension(1):: sds_stride
    integer, dimension(1):: sds_edges
    integer(kind=int1), dimension(:), allocatable:: temp_i1
    integer(kind=int2), dimension(:), allocatable:: temp_i2
    integer(kind=int4), dimension(:), allocatable:: temp_i4
    real(kind=real4), dimension(:), allocatable:: temp_r4
    integer:: num_attrs
    integer:: sfn2index
    integer:: sfselect
    integer:: sffattr
    integer:: sfrnatt
    integer:: sfrcatt
    integer:: sfginfo
    integer:: sfrdata
    integer:: sfendacc
    integer:: istatus
    character(72):: sds_name_temp

    istatus = 0

    !--- open sds for reading
    sds%id_input = sfselect(sd_id, sfn2index(sd_id,trim(sds%variable_name)))

    istatus = sfginfo(sds%id_input, sds_name_temp, sds%rank, sds_dims, sds%data_type, num_attrs) + istatus
    sds_start = (/ 0 /)
    sds_stride = (/ 1 /)
    sds_edges = sds_dims

    !--- read sds attributes
    istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED"), sds%scaling_type)
    istatus = sfrcatt(sds%id_input, sffattr(sds%id_input,"UNITS"), sds%units)
    istatus = sfrcatt(sds%id_input, sffattr(sds%id_input,"STANDARD_NAME"), sds%standard_name)
    istatus = sfrcatt(sds%id_input, sffattr(sds%id_input,"LONG_NAME"), sds%long_name)
    istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"RANGE_MISSING"), sds%unscaled_missing)

    !-- if scaled, read attributes that allow unscaling
    if (sds%scaling_type > 0) then
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED_MISSING"), sds%scaled_missing)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED_MIN"), sds%scaled_min)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED_MAX"), sds%scaled_max)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"RANGE_MIN"), sds%unscaled_min)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"RANGE_MAX"), sds%unscaled_max)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALE_FACTOR"), sds%scale_factor)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"ADD_OFFSET"), sds%add_offset)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"_FILLVALUE"), sds%fill_value)
    endif

    !--- allocate arrays for holding data, read data and store in output array
    if (sds%data_type == DFNT_INT8) then
       allocate(temp_i1(sds_dims(1)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_i1) + istatus
       scaled_sds_data = real(temp_i1)
    elseif (sds%data_type == DFNT_INT16) then
       allocate(temp_i2(sds_dims(1)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_i2) + istatus
       scaled_sds_data = real(temp_i2)
    elseif (sds%data_type == DFNT_INT32) then
       allocate(temp_i4(sds_dims(1)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_i4) + istatus
       scaled_sds_data = real(temp_i4)
    elseif (sds%data_type == DFNT_FLOAT32) then
       allocate(temp_r4(sds_dims(1)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_r4) + istatus
       scaled_sds_data = temp_r4
    else
       print *, "attempt to read unsupported data type, stopping"
       stop
    endif

    !---deallocate temp arrays
    if (allocated(temp_i1)) deallocate(temp_i1)
    if (allocated(temp_i2)) deallocate(temp_i2)
    if (allocated(temp_i4)) deallocate(temp_i4)
    if (allocated(temp_r4)) deallocate(temp_r4)

    !--- close sds
    istatus = sfendacc(sds%id_input) + istatus

   end subroutine READ_SDS_RANK1


!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: READ_SDS_RANK2
!
! Function:
!     Reads 2-dimensional SDSs from Level 2b files. 
!-----------------------------------------------------------------------------------------------
  subroutine READ_SDS_RANK2(sd_id,     &
                            scaled_sds_data,   &
                            sds)

    integer, intent(in):: sd_id
    type(sds_struct), intent(inout):: sds
    real(kind=real4), dimension(:,:), intent(out):: scaled_sds_data
    integer, dimension(2):: sds_dims
    integer, dimension(2):: sds_start
    integer, dimension(2):: sds_stride
    integer, dimension(2):: sds_edges
    integer(kind=int1), dimension(:,:), allocatable:: temp_i1
    integer(kind=int2), dimension(:,:), allocatable:: temp_i2
    integer(kind=int4), dimension(:,:), allocatable:: temp_i4
    real(kind=real4), dimension(:,:), allocatable:: temp_r4
    integer:: num_attrs
    integer:: sfn2index
    integer:: sfselect
    integer:: sffattr
    integer:: sfrnatt
    integer:: sfrcatt
    integer:: sfginfo
    integer:: sfrdata
    integer:: sfendacc
    integer:: istatus
    character(72):: sds_name_temp

    istatus = 0

    !--- open sds for reading
    sds%id_input = sfselect(sd_id, sfn2index(sd_id,trim(sds%variable_name)))
    istatus = sfginfo(sds%id_input, sds_name_temp, sds%rank, sds_dims, sds%data_type, num_attrs) + istatus
    sds_start = (/ 0, 0 /)
    sds_stride = (/ 1, 1 /)
    sds_edges = sds_dims

    !--- read sds attributes
    istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED"), sds%scaling_type)
    istatus = sfrcatt(sds%id_input, sffattr(sds%id_input,"UNITS"), sds%units)
    istatus = sfrcatt(sds%id_input, sffattr(sds%id_input,"STANDARD_NAME"), sds%standard_name)
    istatus = sfrcatt(sds%id_input, sffattr(sds%id_input,"LONG_NAME"), sds%long_name)
    istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"RANGE_MISSING"), sds%unscaled_missing)

    !-- if scaled, read attributes that allow unscaling
    if (sds%scaling_type > 0) then
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED_MISSING"), sds%scaled_missing)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED_MIN"), sds%scaled_min)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALED_MAX"), sds%scaled_max)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"RANGE_MIN"), sds%unscaled_min)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"RANGE_MAX"), sds%unscaled_max)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"SCALE_FACTOR"), sds%scale_factor)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"ADD_OFFSET"), sds%add_offset)
      istatus = sfrnatt(sds%id_input, sffattr(sds%id_input,"_FILLVALUE"), sds%fill_value)
    endif

    !--- allocate arrays for holding data, read data and store in output array
    if (sds%data_type == DFNT_INT8) then
       allocate(temp_i1(sds_dims(1),sds_dims(2)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_i1) + istatus
       scaled_sds_data = real(temp_i1)
    elseif (sds%data_type == DFNT_INT16) then
       allocate(temp_i2(sds_dims(1), sds_dims(2)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_i2) + istatus
       scaled_sds_data = real(temp_i2)
    elseif (sds%data_type == DFNT_INT32) then
       allocate(temp_i4(sds_dims(1),sds_dims(2)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_i4) + istatus
       scaled_sds_data = real(temp_i4)
    elseif (sds%data_type == DFNT_FLOAT32) then
       allocate(temp_r4(sds_dims(1),sds_dims(2)))
       istatus = sfrdata(sds%id_input, sds_start, sds_stride, sds_edges, temp_r4) + istatus
       scaled_sds_data = temp_r4
    else
       print *, "attempt to read unsupported data type, stopping"
       stop
    endif

    !---deallocate temp arrays
    if (allocated(temp_i1)) deallocate(temp_i1)
    if (allocated(temp_i2)) deallocate(temp_i2)
    if (allocated(temp_i4)) deallocate(temp_i4)
    if (allocated(temp_r4)) deallocate(temp_r4)

    !--- close sds
    istatus = sfendacc(sds%id_input) + istatus


   end subroutine READ_SDS_RANK2

!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: UNSCALE_SDS
!
! Function:
!     Unscales scaled SDS data from level2b files
!-----------------------------------------------------------------------------------------------
   subroutine UNSCALE_SDS(sds, scaled_data, unscaled_data)
      type(sds_struct), intent(in):: sds
      real(kind=real4), dimension(:,:), intent(in):: scaled_data
      real(kind=real4), dimension(:,:), intent(out):: unscaled_data

      !--- unscale sds

      if (sds%scaling_type /= sym%NO_SCALING) then

        !---- linear
        if (sds%scaling_type == sym%LINEAR_SCALING) then
           unscaled_data = min(1.0,max(0.0,real(scaled_data - sds%scaled_min)/real(sds%scaled_max - sds%scaled_min)))
           unscaled_data = sds%unscaled_min + unscaled_data * (sds%unscaled_max - sds%unscaled_min)
        endif

        !---- log10
        if (sds%scaling_type == sym%LOG10_SCALING) then
           unscaled_data = min(1.0,max(0.0,real(scaled_data - sds%scaled_min)/real(sds%scaled_max - sds%scaled_min)))
           unscaled_data = sds%unscaled_min + unscaled_data * (sds%unscaled_max - sds%unscaled_min)
           unscaled_data = 10**(unscaled_data)
        endif

        !---- square root
        if (sds%scaling_type == sym%SQUARE_ROOT_SCALING) then
           unscaled_data = min(1.0,max(0.0,real(scaled_data - sds%scaled_min)/real(sds%scaled_max - sds%scaled_min)))
           unscaled_data = sds%unscaled_min + (unscaled_data**2) * (sds%unscaled_max - sds%unscaled_min)
        endif

        !--- set scaled missing values
        where (scaled_data == sds%scaled_missing)
         unscaled_data = sds%unscaled_missing
        endwhere

      else

        unscaled_data = scaled_data

      endif

   end subroutine UNSCALE_SDS

!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: SCALE_SDS
!
! Function:
!     Scales unscaled SDS data from level2b files
!-----------------------------------------------------------------------------------------------
   subroutine SCALE_SDS(sds, unscaled_data, scaled_data)
      type(sds_struct), intent(in):: sds
      real(kind=real4), dimension(:,:), intent(in):: unscaled_data
      real(kind=real4), dimension(:,:), intent(out):: scaled_data

      !--- scale sds

      if (sds%scaling_type /= sym%NO_SCALING) then

        !---- linear
        if (sds%scaling_type == sym%LINEAR_SCALING) then
           scaled_data = min(1.0,max(0.0,(unscaled_data - sds%unscaled_min)/(sds%unscaled_max - sds%unscaled_min)))
           scaled_data = sds%scaled_min + scaled_data * (sds%scaled_max - sds%scaled_min)
        endif

        !---- log10
        if (sds%scaling_type == sym%LOG10_SCALING) then
          scaled_data = sds%unscaled_missing
          where(unscaled_data > 0.0)
            scaled_data = log10(unscaled_data)
          end where
          scaled_data = sds%scaled_min + ((max(sds%unscaled_min,min(sds%unscaled_max,scaled_data)) - sds%unscaled_min)/ &
                    (sds%unscaled_max - sds%unscaled_min)) * (sds%scaled_max - sds%scaled_min)
        endif

        !---- square root
        if (sds%scaling_type == sym%SQUARE_ROOT_SCALING) then
         scaled_data = min(1.0,max(0.0,(unscaled_data - sds%unscaled_min)/(sds%unscaled_max - sds%unscaled_min)))
         scaled_data = sds%scaled_min + sqrt(scaled_data) * (sds%scaled_max - sds%scaled_min)
       endif

       !--- set scaled missing values
       where (unscaled_data == sds%unscaled_missing)
         scaled_data = sds%scaled_missing 
       endwhere

     else
   
       scaled_data = unscaled_data
 
     endif

   end subroutine SCALE_SDS

!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: WRITE_SDS_RANK1
!
! Function:
!     Write 1 dimensional data to a HDF file
!-----------------------------------------------------------------------------------------------

  subroutine WRITE_SDS_RANK1(sd_id,     &
                             scaled_sds_data,   &
                             sds)

    integer, intent(in):: sd_id
    type(sds_struct), intent(in):: sds
    real(kind=real4), dimension(:), intent(in):: scaled_sds_data
    integer, dimension(1):: sds_dims
    integer, dimension(1):: sds_start
    integer, dimension(1):: sds_stride
    integer, dimension(1):: sds_edges
    integer(kind=int1), dimension(:), allocatable:: temp_i1
    integer(kind=int2), dimension(:), allocatable:: temp_i2
    integer(kind=int4), dimension(:), allocatable:: temp_i4
    real(kind=real4), dimension(:), allocatable:: temp_r4
    integer(kind=int4):: istatus
    integer:: sfwdata
    integer:: sfendacc

    sds_dims(1) = size(scaled_sds_data,1)
    sds_start = (/ 0 /)
    sds_stride = (/ 1 /)
    sds_edges = sds_dims


    istatus = 0

    sds_dims(1) = size(scaled_sds_data,1)

    if (sds%data_type == DFNT_INT8) then
       allocate(temp_i1(sds_dims(1)))
       temp_i1 = int(scaled_sds_data,kind=int1)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_i1) + istatus
    elseif (sds%data_type == DFNT_INT16) then
       allocate(temp_i2(sds_dims(1)))
       temp_i2 = int(scaled_sds_data,kind=int2)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_i2) + istatus
    elseif (sds%data_type == DFNT_INT32) then
       allocate(temp_i4(sds_dims(1)))
       temp_i4 = int(scaled_sds_data,kind=int4)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_i4) + istatus
    elseif (sds%data_type == DFNT_FLOAT32) then
       allocate(temp_r4(sds_dims(1)))
       temp_r4 = real(scaled_sds_data,kind=real4)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_r4) + istatus
    endif


   !---deallocate temp arrays
   if (allocated(temp_i1)) deallocate(temp_i1)
   if (allocated(temp_i2)) deallocate(temp_i2)
   if (allocated(temp_i4)) deallocate(temp_i4)
   if (allocated(temp_r4)) deallocate(temp_r4)

   end subroutine WRITE_SDS_RANK1
!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: WRITE_SDS_RANK2
!
! Function:
!     Write 2 dimensional data to a HDF file
!-----------------------------------------------------------------------------------------------

   subroutine WRITE_SDS_RANK2(sd_id,     &
                             scaled_sds_data,   &
                             sds)

    integer, intent(in):: sd_id
    type(sds_struct), intent(in):: sds
    real(kind=real4), dimension(:,:), intent(in):: scaled_sds_data
    integer, dimension(2):: sds_dims
    integer, dimension(2):: sds_start
    integer, dimension(2):: sds_stride
    integer, dimension(2):: sds_edges
    integer(kind=int1), dimension(:,:), allocatable:: temp_i1
    integer(kind=int2), dimension(:,:), allocatable:: temp_i2
    integer(kind=int4), dimension(:,:), allocatable:: temp_i4
    real(kind=real4), dimension(:,:), allocatable:: temp_r4
    integer(kind=int4):: istatus
    integer:: sfwdata
    integer:: sfendacc

    sds_dims(1) = size(scaled_sds_data,1)
    sds_dims(2) = size(scaled_sds_data,2)
    sds_start = (/ 0, 0 /)
    sds_stride = (/ 1, 1 /)
    sds_edges = sds_dims

    istatus = 0

    if (sds%data_type == DFNT_INT8) then
       allocate(temp_i1(sds_dims(1),sds_dims(2)))
       temp_i1 = int(scaled_sds_data,kind=int1)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_i1) + istatus
    elseif (sds%data_type == DFNT_INT16) then
       allocate(temp_i2(sds_dims(1),sds_dims(2)))
       temp_i2 = int(scaled_sds_data,kind=int2)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_i2) + istatus
    elseif (sds%data_type == DFNT_INT32) then
       allocate(temp_i4(sds_dims(1),sds_dims(2)))
       temp_i4 = int(scaled_sds_data,kind=int4)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_i4) + istatus
    elseif (sds%data_type == DFNT_FLOAT32) then
       allocate(temp_r4(sds_dims(1),sds_dims(2)))
       temp_r4 = real(scaled_sds_data,kind=real4)
       istatus = sfwdata(sds%id_output, sds_start, sds_stride, sds_edges, temp_r4) + istatus
    endif


   !---deallocate temp arrays
   if (allocated(temp_i1)) deallocate(temp_i1)
   if (allocated(temp_i2)) deallocate(temp_i2)
   if (allocated(temp_i4)) deallocate(temp_i4)
   if (allocated(temp_r4)) deallocate(temp_r4)

   end subroutine WRITE_SDS_RANK2

!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: COPY_GLOBAL_ATTRIBUTES
!
! Function:
!    Copies the global attributes from one file and puts them in a different file
!-----------------------------------------------------------------------------------------------

    subroutine COPY_GLOBAL_ATTRIBUTES(sd_id_input,sd_id_output)
   
   integer(kind=int4):: sd_id_input
   integer(kind=int4):: sd_id_output
   integer:: istatus
   integer(kind=int4):: num_global_attrs
   integer(kind=int4):: num_sds_input
   integer(kind=int4):: iattr
   character(len=100):: attr_name
   integer(kind=int4):: data_type
   integer(kind=int4):: count
   integer(kind=int4):: attr_buffer_i4
   integer(kind=int2):: attr_buffer_i2
   integer(kind=int1):: attr_buffer_i1
   real(kind=real4):: attr_buffer_r4
   character(len=100):: attr_buffer_char

   integer:: sffinfo
   integer:: sfgainfo
   integer:: sfrnatt
   integer:: sfsnatt
   
   
   istatus = sffinfo(sd_id_input,num_sds_input,num_global_attrs)

   do iattr = 0, num_global_attrs-1

       istatus = sfgainfo(sd_id_input,iattr,attr_name,data_type,count)

       if (data_type == DFNT_INT8) then
           istatus = sfrnatt(sd_id_input,iattr,attr_buffer_i1)
           istatus = sfsnatt(sd_id_output,trim(attr_name),data_type,count,attr_buffer_i1)
       endif

       if (data_type == DFNT_INT16) then
           istatus = sfrnatt(sd_id_input,iattr,attr_buffer_i2)
           istatus = sfsnatt(sd_id_output,trim(attr_name),data_type,count,attr_buffer_i2)
       endif

       if (data_type == DFNT_INT32) then
           istatus = sfrnatt(sd_id_input,iattr,attr_buffer_i4)
           istatus = sfsnatt(sd_id_output,trim(attr_name),data_type,count,attr_buffer_i4)
       endif

       if (data_type == DFNT_FLOAT32) then
           istatus = sfrnatt(sd_id_input,iattr,attr_buffer_r4)
           istatus = sfsnatt(sd_id_output,trim(attr_name),data_type,count,attr_buffer_r4)
       endif

       if (data_type == DFNT_CHAR) then
           istatus = sfrnatt(sd_id_input,iattr,attr_buffer_char)
           istatus = sfsnatt(sd_id_output,trim(attr_name),data_type,count,trim(attr_buffer_char))
       endif
   enddo

   end subroutine 

!------------------------------------------------------------------------------------------------
! SUBROUTINE Name: SUBSET_LEVEL2b
!
! Function:
!    Reads in a level-2b file and then write out a subset of it
!-----------------------------------------------------------------------------------------------

  subroutine SUBSET_LEVEL2b(file_input, &
                            path_input, & 
                            file_output, &
                            path_output, & 
                            lon_west_out, &
                            lon_east_out, &
                            dlon_out, &
                            nlon_out, &
                            lat_south_out, &
                            lat_north_out, &
                            dlat_out, &
                            nlat_out)

  character(len=*), intent(in):: file_input
  character(len=*), intent(in):: path_input
  character(len=*), intent(in):: file_output
  character(len=*), intent(in):: path_output
  real(kind=real4), intent(in):: lon_west_out
  real(kind=real4), intent(in):: lon_east_out
  real(kind=real4), intent(in):: dlon_out
  real(kind=real4), intent(in):: lat_south_out
  real(kind=real4), intent(in):: lat_north_out
  real(kind=real4), intent(in):: dlat_out

  integer(kind=int4):: sd_id_input
  integer(kind=int4):: sd_id_output

  type(sds_struct) :: sds_input
  type(sds_struct) :: sds_output

  integer(kind=int4) :: nlon_out
  integer(kind=int4) :: nlat_out
  real(kind=real4):: lon_west_in
  real(kind=real4):: lon_east_in
  real(kind=real4):: dlon_in
  integer(kind=int4):: nlon_in
  real(kind=real4):: lat_south_in
  real(kind=real4):: lat_north_in
  real(kind=real4):: dlat_in
  integer(kind=int4):: nlat_in

  integer(kind=int4):: lon_stride
  integer(kind=int4):: lat_stride


  real(kind=real4), dimension(:,:), allocatable:: scaled_sds_data_input

  integer:: sfstart, sfend, sfsnatt, sfrnatt, sfginfo, sfselect, sffattr
  integer:: isds, num_sds_input
  integer:: istatus
  integer(kind=int4), dimension(2):: sds_dims

  !------------------------------------------------------------------
  ! open input file for reading
  !------------------------------------------------------------------
  sd_id_input = sfstart(trim(path_input)//trim(file_input), DFACC_READ)

  !------------------------------------------------------------------
  ! open output file for writing 
  !------------------------------------------------------------------
  sd_id_output = sfstart(trim(path_output)//trim(file_output), DFACC_WRITE)

  !------------------------------------------------------------------
  ! copy attributes from input to output file 
  !------------------------------------------------------------------
  call COPY_GLOBAL_ATTRIBUTES(sd_id_input,sd_id_output)

  !------------------------------------------------------------------
  !  determine size of input fields
  !------------------------------------------------------------------
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"NUMBER_OF_LONGITUDES"),nlon_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"NUMBER_OF_LATITUDES"),nlat_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"LONGITUDE_SPACING"),dlat_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"LATITUDE_SPACING"),dlat_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"WESTERN_MOST_LONGITUDE"),lon_west_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"EASTERN_MOST_LONGITUDE"),lon_east_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"NORTHERN_MOST_LATITUDE"),lat_north_in)
  istatus = sfrnatt(sd_id_input,sffattr(sd_id_input,"SOUTHERN_MOST_LATITUDE"),lat_south_in)
  allocate(scaled_sds_data_input(nlon_in,nlat_in))


  !------------------------------------------------------------------
  ! determine stride and length of output data
  !------------------------------------------------------------------
  nlon_out = int((lon_east_out - lon_west_out) / dlon_out) + 1
  nlat_out = int((lat_north_out - lat_south_out) / dlat_out) + 1
  lon_stride = (nlon_in-1) / (nlon_out-1)   !????
  lat_stride = (nlat_in-1) / (nlat_out-1)

  !------------------------------------------------------------------
  ! replace attributes that differ from the input and output files 
  !------------------------------------------------------------------
  istatus = sfsnatt(sd_id_output,"NUMBER_OF_LONGITUDES",DFNT_INT32,1,nlon_out)
  istatus = sfsnatt(sd_id_output,"NUMBER_OF_LATITUDES",DFNT_INT32,1,nlat_out)
  istatus = sfsnatt(sd_id_output,"LONGITUDE_SPACING",DFNT_FLOAT32,1,dlon_out)
  istatus = sfsnatt(sd_id_output,"LATITUDE_SPACING",DFNT_FLOAT32,1,dlat_out)
  istatus = sfsnatt(sd_id_output,"WESTERN_MOST_LONGITUDE",DFNT_FLOAT32,1,lon_west_out)
  istatus = sfsnatt(sd_id_output,"EASTERN_MOST_LONGITUDE",DFNT_FLOAT32,1,lon_east_out)
  istatus = sfsnatt(sd_id_output,"NORTHERN_MOST_LATITUDE",DFNT_FLOAT32,1,lat_north_out)
  istatus = sfsnatt(sd_id_output,"SOUTHERN_MOST_LATITUDE",DFNT_FLOAT32,1,lat_south_out)


  sds_input%id_input = sfselect(sd_id_input,isds) !note, sds index starts at zero

  sds_loop: do isds = 0, num_sds_input-1

  istatus = sfginfo(sds_input%id_input,  &
                    sds_input%variable_name, &
                    sds_input%rank, &
                    sds_dims, &
                    sds_input%data_type, &
                    sds_input%num_attrs) + istatus

  call READ_SDS(sd_id_input,scaled_sds_data_input,sds_input)

  sds_output = sds_input

  call DEFINE_SDS(sd_id_output, (/nlon_out,nlat_out/), sds_output)

! call WRITE_SDS(sd_id_output,scaled_sds_data_input,sds_output)


  end do sds_loop


  deallocate(scaled_sds_data_input)

  !----- close input hdf file
  istatus = sfend(sd_id_input) + istatus

  !----- close output hdf file
  istatus = sfend(sd_id_output) + istatus

  end subroutine SUBSET_LEVEL2b

end module LEVEL2B_ROUTINES
