! $Id: level3.f90,v 1.13 2010/05/30 16:04:21 heidinger Exp $
module LEVEL3_ROUTINES
!-------------------------------------------------------------------------
!@***h* PATMOSX/level3.f90
! 
! NAME
!   level3.f90
! 
! LOCATION
!   /patmosx/src/level3.f90
! 
! PURPOSE
!   Module that houses routines to generate gridded parameters from the
!   pixel level parameters
! 
! DESCRIPTION
!   Module that houses routines to generate gridded parameters from the
!   pixel level parameters
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
! EXTERNALS
!  Public routines used is this module:
!   COMPILE_GRIDCELL_ARRAYS - compile gridcells stats from pixel parameters
!   COMPUTE_GRIDCELL_ARRAYS - once all pixels are processed, compute the final
!                             gridcell parameters
!   CREATE_GRIDCELL_ARRAYS - allocate memory for grid-cell arrays
!   DESTROY_GRIDCELL_ARRAYS - deallocate memory for grid-cell arrays
!   INITIALIZE_GRIDCELL_ARRAYS - initialize the values of the grid-cell 
!                                parameters
!   READ_GRIDCELL_SFCTYPE - read the gridcell surface type which is
!                           precomputed for each grid format and resolution
!
! INPUTS_OUTPUTS
!   Unit 7 is opened and closed in READ_GRIDCELL_SFCTYPE
!
!@*****
!------------------------------------------------------------------------
! Module that houses routines to generate gridded parameters from the
! pixel level parameters
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! File I/O - Unit 7 is opened and closed in READ_GRIDCELL_SFCTYPE
!
! Public routines used is this module:
! COMPILE_GRIDCELL_ARRAYS - compile gridcells stats from pixel parameters
! COMPUTE_GRIDCELL_ARRAYS - once all pixels are processed, compute the final
!                           gridcell parameters
! CREATE_GRIDCELL_ARRAYS - allocate memory for grid-cell arrays
! DESTROY_GRIDCELL_ARRAYS - deallocate memory for grid-cell arrays
! INITIALIZE_GRIDCELL_ARRAYS - initialize the values of the grid-cell 
!                              parameters
! READ_GRIDCELL_SFCTYPE - read the gridcell surface type which is
!                         precomputed for each grid format and resolution
!
! Private routines used is this module: None
!
!-------------------------------------------------------------------------
 use CONSTANTS
 use AVHRR_PIXEL_COMMON
 use EQUAL_AREA_GRID
 use EQUAL_ANGLE_GRID
 use FILE_UTILITY
 use NWP_COMMON
 implicit none
 private
 public:: COMPILE_GRIDCELL_ARRAYS,  &
          COMPUTE_GRIDCELL_ARRAYS, &
          CREATE_GRIDCELL_ARRAYS, &
          DESTROY_GRIDCELL_ARRAYS, &
          INITIALIZE_GRIDCELL_ARRAYS,&
          READ_GRIDCELL_SFCTYPE

!------- grid-cell array declarations
integer (kind=int1), dimension(:), allocatable, public:: cell_sfc_type
integer (kind=int1), dimension(:), allocatable, public:: cell_asc_des
integer (kind=int1), dimension(:), allocatable, public:: cell_ch3a_on
integer (kind=int1), dimension(:), allocatable, public:: cell_sc_id
real(kind=real4), dimension(:,:,:), allocatable, public:: cell_mean
real(kind=real4), dimension(:,:,:), allocatable, public:: cell_std
real(kind=real4), dimension(:,:), allocatable, public:: cell_count_sum
real(kind=real4), dimension(:,:), allocatable, public:: cell_count_mean
integer (kind=int4), dimension(:), allocatable, public, save:: cell_orbit

real(kind=real4), dimension(:), allocatable, public:: satzen_mean
real(kind=real4), dimension(:), allocatable, public:: solzen_mean
real(kind=real4), dimension(:), allocatable, public:: relaz_mean
real(kind=real4), dimension(:), allocatable, public:: glint_mean
real(kind=real4), dimension(:), allocatable, public:: zsfc_mean
integer(kind=int2), dimension(:,:,:), allocatable, public, save:: cell_mask_count
integer(kind=int2), dimension(:,:,:), allocatable, public, save:: cell_mask_count_ir
integer(kind=int2), dimension(:), allocatable, public, save:: cell_multilayer_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_pixel_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_snow_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_day
integer(kind=int2), dimension(:), allocatable, public, save:: cell_year
integer(kind=int2), dimension(:), allocatable, public, save:: cell_ocean_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_land_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_clear_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_night_count
integer(kind=int2), dimension(:), allocatable, public, save:: cell_sw_cld_count
real(kind=real4), dimension(:), allocatable, public, save:: term_frac
real(kind=real4), dimension(:), allocatable, public, save:: snow_frac
real(kind=real4), dimension(:), allocatable, public, save:: dust_frac
real(kind=real4), dimension(:), allocatable, public, save:: fire_frac
real(kind=real4), dimension(:), allocatable, public, save:: smoke_frac
integer(kind=int2), dimension(:), allocatable, public, save:: cell_water_count_cm2
integer(kind=int2), dimension(:), allocatable, public, save:: cell_water_count_cm3
integer(kind=int2), dimension(:,:), allocatable, public, save:: cell_wat_cld_count
integer(kind=int2), dimension(:,:), allocatable, public, save:: cell_ice_cld_count
real(kind=real4), dimension(:,:), allocatable, public, save:: cell_space_counts

real(kind=real4), dimension(:,:), allocatable, public, save:: cell_dark_obs(:,:)
real(kind=real4), dimension(:,:), allocatable, public, save:: cell_warm_obs(:,:)
real(kind=real4), dimension(:,:), allocatable, public, save:: cell_green_obs(:,:)

!--- clear-sky property arrays
real(kind=real4), dimension(:),allocatable, public, save:: cell_utc_time
real(kind=real4), dimension(:),allocatable, public, save:: cell_gpi_235
real(kind=real4), dimension(:),allocatable, public, save:: cell_gpi_255
real(kind=real4), dimension(:),allocatable, public, save:: cell_gpi_275
real(kind=real4), dimension(:),allocatable, public, save:: cell_dcc_frac
real(kind=real4), dimension(:),allocatable, public, save:: cell_mean_dcc_ch1
real(kind=real4), dimension(:),allocatable, public, save:: cell_std_dcc_ch1

!--- sst arrays
 real(kind=real4), dimension(:), allocatable, public, save:: cell_sst_back

!--- ndvi arrays
 real(kind=real4), dimension(:), allocatable, public, save:: cell_mean_ndvi_sfc
 integer(kind=int2), dimension(:), allocatable, private, save:: ndvi_count

!--- rsr arrays
 real(kind=real4), dimension(:), allocatable, public, save:: cell_mean_rsr
 real(kind=real4), dimension(:), allocatable, public, save:: cell_std_rsr
 integer(kind=int2), dimension(:), allocatable, private, save:: rsr_count

!--- tsfc arrays
 real(kind=real4), dimension(:), allocatable, public, save:: cell_mean_tsfc
 real(kind=real4), dimension(:), allocatable, public, save:: cell_std_tsfc
 integer(kind=int2), dimension(:), allocatable, private, save:: tsfc_count

!--- cloud property arrays
real(kind=real4), dimension(:), allocatable, public, save:: frac_all
real(kind=real4), dimension(:), allocatable, public, save:: frac_all_ir
real(kind=real4), dimension(:), allocatable, public, save:: frac_water
real(kind=real4), dimension(:), allocatable, public, save:: frac_ice
real(kind=real4), dimension(:), allocatable, public, save:: frac_multilayer

real(kind=real4), dimension(:), allocatable, public, save:: cld_layer_sum_sw
real(kind=real4), dimension(:), allocatable, public, save:: frac_high_sw
real(kind=real4), dimension(:), allocatable, public, save:: frac_mid_sw
real(kind=real4), dimension(:), allocatable, public, save:: frac_low_sw

real(kind=real4), dimension(:), allocatable, public, save:: Tc_sw_mean
real(kind=real4), dimension(:), allocatable, public, save:: Tc_sw_std
integer(kind=int2), dimension(:), allocatable, public, save:: Tc_sw_count
real(kind=real4), dimension(:), allocatable, public, save:: ec_sw_mean
real(kind=real4), dimension(:), allocatable, public, save:: ec_sw_std
integer(kind=int2), dimension(:), allocatable, public, save:: ec_sw_count
real(kind=real4), dimension(:), allocatable, public, save:: Pc_sw_mean
real(kind=real4), dimension(:), allocatable, public, save:: Zc_sw_mean
real(kind=real4), dimension(:), allocatable, public, save:: tau_sw_mean
real(kind=real4), dimension(:), allocatable, public, save:: tau_sw_std
integer(kind=int2), dimension(:), allocatable, public, save:: tau_sw_count
real(kind=real4), dimension(:), allocatable, public, save:: beta_sw_mean
real(kind=real4), dimension(:), allocatable, public, save:: beta_sw_std
integer(kind=int2), dimension(:), allocatable, public, save:: beta_sw_count
real(kind=real4), dimension(:), allocatable, public, save:: tau_vis_mean
real(kind=real4), dimension(:), allocatable, public, save:: tau_vis_std
integer(kind=int2), dimension(:), allocatable, public, save:: tau_vis_count
real(kind=real4), dimension(:), allocatable, public, save:: re_vis_mean
real(kind=real4), dimension(:), allocatable, public, save:: re_vis_std
integer(kind=int2), dimension(:), allocatable, public, save:: re_vis_count
real(kind=real4), dimension(:), allocatable, public, save:: lwp_vis_mean
real(kind=real4), dimension(:), allocatable, public, save:: lwp_vis_std
integer(kind=int2), dimension(:), allocatable, public, save:: lwp_vis_count
real(kind=real4), dimension(:), allocatable, public, save:: iwp_vis_mean
real(kind=real4), dimension(:), allocatable, public, save:: iwp_vis_std
integer(kind=int2), dimension(:), allocatable, public, save:: iwp_vis_count


!--- water cloud props
real(kind=real4), dimension(:), allocatable, public, save:: Tc_sw_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: Tc_sw_std_wat
integer(kind=int2), dimension(:), allocatable, public, save:: Tc_sw_count_wat
real(kind=real4), dimension(:), allocatable, public, save:: ec_sw_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: ec_sw_std_wat
integer(kind=int2), dimension(:), allocatable, public, save:: ec_sw_count_wat
real(kind=real4), dimension(:), allocatable, public, save:: Pc_sw_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: Zc_sw_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: tau_sw_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: tau_sw_std_wat
integer(kind=int2), dimension(:), allocatable, public, save:: tau_sw_count_wat
real(kind=real4), dimension(:), allocatable, public, save:: beta_sw_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: beta_sw_std_wat
integer(kind=int2), dimension(:), allocatable, public, save:: beta_sw_count_wat
real(kind=real4), dimension(:), allocatable, public, save:: tau_vis_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: tau_vis_std_wat
integer(kind=int2), dimension(:), allocatable, public, save:: tau_vis_count_wat
real(kind=real4), dimension(:), allocatable, public, save:: re_vis_mean_wat
real(kind=real4), dimension(:), allocatable, public, save:: re_vis_std_wat
integer(kind=int2), dimension(:), allocatable, public, save:: re_vis_count_wat

!--- ice cloud props
real(kind=real4), dimension(:), allocatable, public, save:: Tc_sw_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: Tc_sw_std_ice
integer(kind=int2), dimension(:), allocatable, public, save:: Tc_sw_count_ice
real(kind=real4), dimension(:), allocatable, public, save:: ec_sw_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: ec_sw_std_ice
integer(kind=int2), dimension(:), allocatable, public, save:: ec_sw_count_ice
real(kind=real4), dimension(:), allocatable, public, save:: Pc_sw_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: Zc_sw_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: tau_sw_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: tau_sw_std_ice
integer(kind=int2), dimension(:), allocatable, public, save:: tau_sw_count_ice
real(kind=real4), dimension(:), allocatable, public, save:: beta_sw_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: beta_sw_std_ice
integer(kind=int2), dimension(:), allocatable, public, save:: beta_sw_count_ice
real(kind=real4), dimension(:), allocatable, public, save:: tau_vis_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: tau_vis_std_ice
integer(kind=int2), dimension(:), allocatable, public, save:: tau_vis_count_ice
real(kind=real4), dimension(:), allocatable, public, save:: re_vis_mean_ice
real(kind=real4), dimension(:), allocatable, public, save:: re_vis_std_ice
integer(kind=int2), dimension(:), allocatable, public, save:: re_vis_count_ice


!--- aerosol property arrays
real(kind=real4), dimension(:), allocatable, public, save:: aot1_mean
real(kind=real4), dimension(:), allocatable, public, save:: aot1_std
real(kind=real4), dimension(:), allocatable, public, save:: aot2_mean
real(kind=real4), dimension(:), allocatable, public, save:: aot2_std
integer(kind=int2), dimension(:), allocatable, private, save:: aot1_count
integer(kind=int2), dimension(:), allocatable, private, save:: aot2_count

!--- outgoing longwave radiation (OLR)
real(kind=real4), dimension(:), allocatable, public, save:: &
                                        olr_mean,olr_std, &
                                        olr_clr_mean,olr_clr_std, &
                                        olr_cldy_mean,olr_cldy_std
integer(kind=int2), dimension(:), allocatable, private, save:: &
                                        olr_count,olr_clr_count,olr_cldy_count

!--- atmospherically corrected reflectances
real(kind=real4), dimension(:), allocatable, public, save:: alb1_clr_sfc_mean, alb2_clr_sfc_mean, &
                                                            alb1_sfc_mean,alb2_sfc_mean
real(kind=real4), dimension(:), allocatable, public, save:: alb3a_clr_sfc_mean, alb3a_sfc_mean
real(kind=real4), dimension(:), allocatable, public, save:: alb3b_clr_sfc_mean, alb3b_sfc_mean
real(kind=real4), dimension(:), allocatable, public, save:: ems3b_clr_mean, ems3b_mean
real(kind=real4), dimension(:), allocatable, public, save:: ems3b_clr_sfc_mean, ems3b_sfc_mean
real(kind=real4), dimension(:), allocatable, public, save:: rad3b_clr_sfc_mean, rad3b_sfc_mean, &
                                                            rad4_clr_sfc_mean, rad4_sfc_mean, &
                                                            rad5_clr_sfc_mean, rad5_sfc_mean
!--- nwp  fields in gridded output                                                          
real(kind=real4), dimension(:), allocatable, public, save:: &
                  cell_tsfc_nwp, cell_tair_nwp,cell_tpw_nwp,cell_weasd_nwp, &
                  cell_wnd_spd_10m_nwp, cell_wnd_dir_10m_nwp,cell_ttrop_nwp, &
                  cell_uth_nwp,cell_hght500_nwp

!--- nwp cloudy radiances and cloud products                    
real(kind=real4), dimension(:), allocatable, public, save:: ch4_rad_clr_nwp_mean
real(kind=real4), dimension(:), allocatable, public, save:: ch5_rad_clr_nwp_mean

!--- nwp cloud props
real(kind=real4), dimension(:), allocatable, public, save:: cell_Pc_nwp
real(kind=real4), dimension(:), allocatable, public, save:: cell_iwp_nwp
real(kind=real4), dimension(:), allocatable, public, save:: cell_lwp_nwp
real(kind=real4), dimension(:), allocatable, public, save:: cell_ncld_layers_nwp

!--- cell lon and lat
real, dimension(:), allocatable, public:: cell_lon
real, dimension(:), allocatable, public:: cell_lat


real(kind=real4), dimension(0:3), parameter, private::  &
                    frac_weight = reshape((/0.0,0.35,1.00,1.0/),(/4/))

!--------------------------------------------------------------------------------
! these parameters control the coverage and quality of the products
!--------------------------------------------------------------------------------
real(kind=real4), parameter, public:: day_solzen_thresh_ref = 89.5   !solzen at which reflectances are missing

integer, parameter, public::  count_thresh = 1, &      !minimum number of pixels need to report a value
                              cld_qf_min = 1, &        !minimum quality flag for use in CLD compositing
                              aot_qf_min = 3, &        !minimum quality flag for use in AOT compositing
                              olr_qf_min = 2, &        !minimum quality flag for use in OLR compositing
                              sst_qf_min = 3, &        !minimum quality flag for use in SST compositing
                              tsfc_qf_min = 3, &       !minimum quality flag for use in IST compositing
                              ndvi_qf_min = 3, &       !minimum quality flag for use in NDVI compositing
                              rsr_qf_min = 3           !minimum quality flag for use in NDVI compositing

!---- 
 character(*), parameter, private :: MOD_PROMPT = " CLAVR_ROUTINES: "

 contains
!-----------------------------------------------------------------------
! GRIDCELL ROUTINES
!----------------------------------------------------------------------- 
subroutine CREATE_GRIDCELL_ARRAYS(nchan_clavr,nmask_clavr,ntype,ncell)

  integer(kind=int4),intent(in):: nchan_clavr,nmask_clavr,ntype,ncell

!-- grid-cell arrays
  allocate(cell_mean(nchan_clavr,0:nmask_clavr-1,ncell), &
   cell_std(nchan_clavr,0:nmask_clavr-1,ncell), &
   cell_count_sum(5,ncell), &
   cell_count_mean(5,ncell), &
   cell_space_counts(5,ncell), &
   cell_mask_count(nchan_clavr,0:nmask_clavr-1,ncell), &
   cell_mask_count_ir(nchan_clavr,0:nmask_clavr-1,ncell), &
   cell_asc_des(ncell), cell_ch3a_on(ncell),cell_sc_id(ncell),&
   cell_multilayer_count(ncell), &
   cell_pixel_count(ncell),&
   cell_snow_count(ncell), &
   cell_ocean_count(ncell),cell_land_count(ncell), &
   cell_clear_count(ncell), cell_night_count(ncell),&
   fire_frac(ncell),dust_frac(ncell), smoke_frac(ncell),&
   cell_water_count_cm2(ncell),cell_water_count_cm3(ncell),&
   cell_wat_cld_count(nchan_clavr,ncell),cell_ice_cld_count(nchan_clavr,ncell),&
   cell_sw_cld_count(ncell), &
   satzen_mean(ncell),solzen_mean(ncell),&
   relaz_mean(ncell), glint_mean(ncell), &
   zsfc_mean(ncell), &
   cell_orbit(ncell),cell_utc_time(ncell), &
   snow_frac(ncell),term_frac(ncell), &
   cell_day(ncell),cell_year(ncell))
   
   allocate(cell_gpi_235(ncell))
   allocate(cell_gpi_255(ncell))
   allocate(cell_gpi_275(ncell))
   allocate(cell_dcc_frac(ncell))
   allocate(cell_mean_dcc_ch1(ncell))
   allocate(cell_std_dcc_ch1(ncell))

cell_multilayer_count = 0

   allocate(cell_sst_back(ncell))

   allocate(cell_mean_ndvi_sfc(ncell),ndvi_count(ncell))
 
   allocate(cell_mean_rsr(ncell),cell_std_rsr(ncell),rsr_count(ncell))

   allocate(cell_mean_tsfc(ncell),cell_std_tsfc(ncell),tsfc_count(ncell))


allocate(cell_tsfc_nwp(ncell),cell_tair_nwp(ncell),cell_tpw_nwp(ncell), cell_weasd_nwp(ncell), &
         cell_wnd_spd_10m_nwp(ncell),cell_wnd_dir_10m_nwp(ncell),cell_ttrop_nwp(ncell),  &
         cell_uth_nwp(ncell),cell_hght500_nwp(ncell))
cell_tsfc_nwp = 0.0
cell_tair_nwp = 0.0
cell_tpw_nwp = 0.0
cell_weasd_nwp = 0.0
cell_wnd_spd_10m_nwp = 0.0
cell_wnd_dir_10m_nwp = 0.0
cell_ttrop_nwp = 0.0
cell_uth_nwp = 0.0
cell_hght500_nwp = 0.0

allocate(cell_dark_obs(nchan_clavr,ncell), cell_warm_obs(nchan_clavr,ncell),cell_green_obs(nchan_clavr+1,ncell))
cell_dark_obs = 0.0
cell_warm_obs = 0.0
cell_green_obs = 0.0

allocate(cell_Pc_nwp(ncell))
allocate(cell_lwp_nwp(ncell))
allocate(cell_iwp_nwp(ncell))
allocate(cell_ncld_layers_nwp(ncell))
cell_Pc_nwp = 0.0
cell_lwp_nwp = 0.0
cell_iwp_nwp = 0.0
cell_ncld_layers_nwp = 0.0

!--- these must be set to missing
allocate(cell_lon(ncell), cell_lat(ncell))
cell_lon = missing_value_real4
cell_lat = missing_value_real4

allocate(cell_sfc_type(ncell))
cell_sfc_type = 0

allocate(frac_all(ncell))
allocate(frac_all_ir(ncell))
allocate(frac_ice(ncell))
allocate(frac_water(ncell))
allocate(frac_multilayer(ncell))

!--- layered cloud fraction
allocate(frac_high_sw(ncell),frac_mid_sw(ncell),frac_low_sw(ncell),cld_layer_sum_sw(ncell))

frac_all = 0.0
frac_all_ir = 0.0
frac_ice = 0.0
frac_water = 0.0
frac_high_sw = 0.0
frac_mid_sw = 0.0
frac_low_sw = 0.0
cld_layer_sum_sw = 0
frac_multilayer = 0.0

allocate(Tc_sw_mean(ncell),ec_sw_mean(ncell),tau_vis_mean(ncell),re_vis_mean(ncell),  &
         Tc_sw_std(ncell),ec_sw_std(ncell),tau_vis_std(ncell),re_vis_std(ncell), &
         Tc_sw_count(ncell),ec_sw_count(ncell), &
         Pc_sw_mean(ncell),Zc_sw_mean(ncell),tau_sw_mean(ncell),tau_sw_std(ncell), &
         beta_sw_mean(ncell), beta_sw_std(ncell), beta_sw_count(ncell), &
         tau_vis_count(ncell),re_vis_count(ncell)) 

allocate(Tc_sw_mean_wat(ncell),ec_sw_mean_wat(ncell),tau_vis_mean_wat(ncell),re_vis_mean_wat(ncell),  &
         Tc_sw_std_wat(ncell),ec_sw_std_wat(ncell),tau_vis_std_wat(ncell),re_vis_std_wat(ncell), &
         Tc_sw_count_wat(ncell),ec_sw_count_wat(ncell),&
         Pc_sw_mean_wat(ncell),Zc_sw_mean_wat(ncell),tau_sw_mean_wat(ncell),tau_sw_std_wat(ncell),&
         beta_sw_mean_wat(ncell), beta_sw_std_wat(ncell), beta_sw_count_wat(ncell),&
         tau_vis_count_wat(ncell),re_vis_count_wat(ncell))

allocate(Tc_sw_mean_ice(ncell),ec_sw_mean_ice(ncell),tau_vis_mean_ice(ncell),re_vis_mean_ice(ncell),  &
         Tc_sw_std_ice(ncell),ec_sw_std_ice(ncell),tau_vis_std_ice(ncell),re_vis_std_ice(ncell), &
         Tc_sw_count_ice(ncell),ec_sw_count_ice(ncell), &
         Pc_sw_mean_ice(ncell),Zc_sw_mean_ice(ncell),tau_sw_mean_ice(ncell),tau_sw_std_ice(ncell),&
         beta_sw_mean_ice(ncell), beta_sw_std_ice(ncell), beta_sw_count_ice(ncell), &
         tau_vis_count_ice(ncell),re_vis_count_ice(ncell))

allocate(iwp_vis_mean(ncell),iwp_vis_std(ncell),lwp_vis_mean(ncell),lwp_vis_std(ncell), &
         lwp_vis_count(ncell),iwp_vis_count(ncell))

Tc_sw_mean = 0.0
Tc_sw_std = 0.0
ec_sw_mean = 0.0
ec_sw_std = 0.0
tau_vis_mean = 0.0
tau_vis_std = 0.0
tau_sw_mean = 0.0
tau_sw_std = 0.0
re_vis_mean = 0.0
re_vis_std = 0.0
Tc_sw_count = 0
ec_sw_count = 0
tau_vis_count = 0
re_vis_count = 0
Zc_sw_mean = 0.0
Pc_sw_mean = 0.0
beta_sw_mean = 0.0
beta_sw_std = 0.0

Tc_sw_mean_wat = 0.0
Tc_sw_std_wat = 0.0
ec_sw_mean_wat = 0.0
ec_sw_std_wat = 0.0
tau_vis_mean_wat = 0.0
tau_vis_std_wat = 0.0
tau_sw_mean_wat = 0.0
tau_sw_std_wat = 0.0
re_vis_mean_wat = 0.0
re_vis_std_wat = 0.0
Tc_sw_count_wat = 0
ec_sw_count_wat = 0
beta_sw_count_wat = 0
tau_vis_count_wat = 0
re_vis_count_wat = 0
Zc_sw_mean_wat = 0.0
Pc_sw_mean_wat = 0.0
beta_sw_mean_wat = 0.0
beta_sw_std_wat = 0.0

Tc_sw_mean_ice = 0.0
Tc_sw_std_ice = 0.0
ec_sw_mean_ice = 0.0
ec_sw_std_ice = 0.0
tau_vis_mean_ice = 0.0
tau_vis_std_ice = 0.0
tau_sw_mean_ice = 0.0
tau_sw_std_ice = 0.0
re_vis_mean_ice = 0.0
re_vis_std_ice = 0.0
Tc_sw_count_ice = 0
ec_sw_count_ice = 0
beta_sw_count_ice = 0
tau_vis_count_ice = 0
re_vis_count_ice = 0
Zc_sw_mean_ice = 0.0
Pc_sw_mean_ice = 0.0
beta_sw_mean_ice = 0.0
beta_sw_std_ice = 0.0

lwp_vis_mean = 0.0
lwp_vis_std = 0.0
iwp_vis_mean = 0.0
iwp_vis_std = 0.0
lwp_vis_count = 0
iwp_vis_count = 0

allocate(aot1_mean(ncell),aot2_mean(ncell),aot1_std(ncell),aot2_std(ncell),  &
         aot1_count(ncell),aot2_count(ncell))
aot1_mean = 0.0
aot1_std = 0.0
aot1_count = 0
aot2_mean = 0.0
aot2_std = 0.0
aot2_count = 0

!----- radiative flux parameters
allocate(olr_mean(ncell),olr_std(ncell),olr_count(ncell),&
         olr_cldy_mean(ncell),olr_cldy_std(ncell),olr_cldy_count(ncell),&
         olr_clr_mean(ncell),olr_clr_std(ncell),olr_clr_count(ncell))

olr_mean = 0.0
olr_std = 0.0
olr_count = 0
olr_cldy_mean = 0.0
olr_cldy_std = 0.0
olr_cldy_count = 0
olr_clr_mean = 0.0
olr_clr_std = 0.0
olr_clr_count = 0

!------ atmospherically corrected radiances
allocate(alb1_clr_sfc_mean(ncell), &
         alb2_clr_sfc_mean(ncell), &
         alb1_sfc_mean(ncell), &
         alb2_sfc_mean(ncell),&
         alb3a_clr_sfc_mean(ncell), &
         alb3a_sfc_mean(ncell), &
         alb3b_clr_sfc_mean(ncell), &
         alb3b_sfc_mean(ncell), &
         rad3b_sfc_mean(ncell), &
         rad3b_clr_sfc_mean(ncell),&
         rad4_sfc_mean(ncell), &
         rad4_clr_sfc_mean(ncell), &
         rad5_sfc_mean(ncell), &
         rad5_clr_sfc_mean(ncell), &
         ems3b_clr_mean(ncell), &
         ems3b_mean(ncell), &
         ems3b_clr_sfc_mean(ncell), &
         ems3b_sfc_mean(ncell))

alb1_sfc_mean = 0.0
alb1_clr_sfc_mean = 0.0
alb2_sfc_mean = 0.0
alb2_clr_sfc_mean = 0.0
alb3a_sfc_mean = 0.0
alb3a_clr_sfc_mean = 0.0
alb3b_sfc_mean = 0.0
alb3b_clr_sfc_mean = 0.0
rad3b_sfc_mean = 0.0
rad3b_clr_sfc_mean = 0.0
rad4_sfc_mean = 0.0
rad4_clr_sfc_mean = 0.0
rad5_sfc_mean = 0.0
rad5_clr_sfc_mean = 0.0
ems3b_clr_sfc_mean = 0.0
ems3b_sfc_mean = 0.0
ems3b_clr_mean = 0.0
ems3b_mean = 0.0

!------------------------------------------------------------------------------------------
! NWP Cloudy radiances and cloud products
!------------------------------------------------------------------------------------------
allocate(ch4_rad_clr_nwp_mean(ncell),  &
         ch5_rad_clr_nwp_mean(ncell))
ch4_rad_clr_nwp_mean = 0.0
ch5_rad_clr_nwp_mean = 0.0


end subroutine CREATE_GRIDCELL_ARRAYS

!-------------------------------------------------
subroutine INITIALIZE_GRIDCELL_ARRAYS()

  cell_mean = 0.0
  cell_std = 0.0
  cell_count_sum = 0.0
  cell_count_mean = 0
  satzen_mean =  0.0
  solzen_mean = 0.0
  relaz_mean =  0.0
  glint_mean = 0.0
  zsfc_mean = 0.0
  cell_orbit = missing_value_int4
  cell_utc_time =  0.0
  cell_pixel_count = 0
  cell_snow_count = 0
  cell_sw_cld_count = 0
  snow_frac = 0.0
  term_frac = 0.0
  fire_frac = 0.0
  dust_frac = 0.0
  smoke_frac = 0.0
  cell_mask_count = 0
  cell_mask_count_ir = 0
  cell_asc_des = 0
  cell_ch3a_on =  missing_value_int1
  cell_sc_id =  missing_value_int1
  cell_multilayer_count = 0
  cell_gpi_235 = 0.0
  cell_gpi_255 = 0.0
  cell_gpi_275 = 0.0
  cell_dcc_frac = 0.0
  cell_mean_dcc_ch1 = 0.0
  cell_std_dcc_ch1 = 0.0
  cell_mean_ndvi_sfc = 0.0
  cell_mean_tsfc = 0.0
  cell_std_tsfc = 0.0
  cell_night_count = 0
  cell_clear_count = 0
  cell_ocean_count = 0
  cell_land_count = 0
  cell_water_count_cm2 = 0
  cell_water_count_cm3 = 0
  cell_wat_cld_count = 0
  cell_ice_cld_count = 0
  cell_day = missing_value_int2
  cell_year = missing_value_int2
  frac_all = 0.0
  frac_all_ir = 0.0
  frac_ice = 0.0
  frac_water = 0.0
  frac_multilayer = 0.0
  frac_high_sw = 0.0
  frac_mid_sw = 0.0
  frac_low_sw = 0.0
  cld_layer_sum_sw = 0

  tau_vis_mean = 0.0
  tau_sw_mean = 0.0
  re_vis_mean = 0.0
  Tc_sw_mean = 0.0
  ec_sw_mean = 0.0
  beta_sw_mean = 0.0
  tau_vis_std = 0.0
  tau_sw_std = 0.0
  re_vis_std = 0.0
  Tc_sw_std = 0.0
  ec_sw_std = 0.0
  beta_sw_std = 0.0
  tau_vis_count = 0
  re_vis_count = 0
  Tc_sw_count = 0
  ec_sw_count = 0
  beta_sw_count = 0
  Zc_sw_mean = 0.0
  Pc_sw_mean = 0.0

  tau_vis_mean_wat = 0.0
  tau_sw_mean_wat = 0.0
  re_vis_mean_wat = 0.0
  Tc_sw_mean_wat = 0.0
  ec_sw_mean_wat = 0.0
  beta_sw_mean_wat = 0.0
  tau_vis_std_wat = 0.0
  tau_sw_std_wat = 0.0
  re_vis_std_wat = 0.0
  Tc_sw_std_wat = 0.0
  ec_sw_std_wat = 0.0
  beta_sw_std_wat = 0.0
  tau_vis_count_wat = 0
  re_vis_count_wat = 0
  Tc_sw_count_wat = 0
  ec_sw_count_wat = 0
  beta_sw_count_wat = 0
  Zc_sw_mean_wat = 0.0
  Pc_sw_mean_wat = 0.0

  tau_vis_mean_ice = 0.0
  tau_sw_mean_ice = 0.0
  re_vis_mean_ice = 0.0
  Tc_sw_mean_ice = 0.0
  ec_sw_mean_ice = 0.0
  beta_sw_mean_ice = 0.0
  tau_vis_std_ice = 0.0
  tau_sw_std_ice = 0.0
  re_vis_std_ice = 0.0
  Tc_sw_std_ice = 0.0
  ec_sw_std_ice = 0.0
  beta_sw_std_ice = 0.0
  tau_vis_count_ice = 0
  re_vis_count_ice = 0
  Tc_sw_count_ice = 0
  ec_sw_count_ice = 0
  beta_sw_count_ice = 0
  Zc_sw_mean_ice = 0.0
  Pc_sw_mean_ice = 0.0

  lwp_vis_count = 0
  iwp_vis_count = 0

  cell_space_counts = 0.0
  cell_dark_obs = 0.0
  cell_warm_obs = 0.0
  cell_green_obs = 0.0
  cell_sst_back = 0.0
  cell_mean_ndvi_sfc = 0.0
  aot1_mean = 0.0
  aot1_std = 0.0
  aot1_count = 0
  aot2_mean = 0.0
  aot2_std = 0.0
  aot2_count = 0
  olr_mean = 0.0 
  olr_std = 0.0
  olr_count = 0
  olr_cldy_mean = 0.0 
  olr_cldy_std = 0.0
  olr_cldy_count = 0
  olr_clr_mean = 0.0 
  olr_clr_std = 0.0
  olr_clr_count = 0
  alb1_clr_sfc_mean = 0.0
  alb1_sfc_mean = 0.0
  alb2_clr_sfc_mean = 0.0
  alb2_sfc_mean = 0.0
  alb3a_clr_sfc_mean = 0.0
  alb3a_sfc_mean = 0.0
  alb3b_clr_sfc_mean = 0.0
  alb3b_sfc_mean = 0.0
  rad3b_clr_sfc_mean = 0.0
  rad3b_sfc_mean = 0.0
  rad4_clr_sfc_mean = 0.0
  rad4_sfc_mean = 0.0
  rad5_clr_sfc_mean = 0.0
  rad5_sfc_mean = 0.0
  ems3b_clr_sfc_mean = 0.0
  ems3b_sfc_mean = 0.0
  ems3b_clr_mean = 0.0
  ems3b_mean = 0.0

  tsfc_count = 0
  ndvi_count = 0

  cell_mean_rsr = 0.0
  cell_std_rsr = 0.0
  rsr_count = 0

!----------------------------------------
! NWP Cloudy radiances and layers
!----------------------------------------
ch4_rad_clr_nwp_mean = 0.0
ch5_rad_clr_nwp_mean = 0.0

cell_tsfc_nwp = 0.0
cell_tair_nwp = 0.0
cell_ttrop_nwp = 0.0
cell_tpw_nwp = 0.0
cell_uth_nwp = 0.0
cell_hght500_nwp = 0.0
cell_weasd_nwp = 0.0
cell_wnd_spd_10m_nwp = 0.0
cell_wnd_dir_10m_nwp = 0.0

cell_Pc_nwp = 0.0
cell_lwp_nwp = 0.0
cell_iwp_nwp = 0.0
cell_ncld_layers_nwp = 0.0

end subroutine INITIALIZE_GRIDCELL_ARRAYS


!-------------------------------------------------------------------------------
! this routine removes the grid-cell arrays out of memory
!-------------------------------------------------------------------------------
 subroutine DESTROY_GRIDCELL_ARRAYS()
     deallocate(cell_asc_des,cell_ch3a_on,cell_sc_id,cell_space_counts,&
      cell_mean,cell_std,cell_count_sum,cell_count_mean,cell_sfc_type, &
      cell_multilayer_count,cell_mask_count,cell_mask_count_ir,cell_pixel_count, &
      cell_snow_count,fire_frac,dust_frac,term_frac,smoke_frac,snow_frac, &
      satzen_mean,solzen_mean,relaz_mean,glint_mean,zsfc_mean, &
      cell_orbit,cell_utc_time, &
      cell_day,cell_year, &
      cell_mean_tsfc,cell_std_tsfc, &
      cell_water_count_cm2,cell_water_count_cm3,cell_wat_cld_count,cell_ice_cld_count, &
      cell_sst_back,cell_mean_ndvi_sfc, cell_ocean_count, cell_land_count)

    deallocate(cell_gpi_235)
    deallocate(cell_gpi_255)
    deallocate(cell_gpi_275)
    deallocate(cell_dcc_frac)
    deallocate(cell_mean_dcc_ch1)
    deallocate(cell_std_dcc_ch1)

    deallocate(ndvi_count, tsfc_count)

    deallocate(cell_dark_obs,cell_warm_obs,cell_green_obs)

    deallocate(cell_lat,cell_lon)

    deallocate(frac_all)
    deallocate(frac_all_ir)
    deallocate(frac_ice)
    deallocate(frac_water)
    deallocate(frac_multilayer)
    
    deallocate(cld_layer_sum_sw,frac_high_sw,frac_mid_sw,frac_low_sw)

    deallocate(iwp_vis_mean,iwp_vis_std,lwp_vis_mean,lwp_vis_std,iwp_vis_count,lwp_vis_count)

    deallocate(Tc_sw_mean,ec_sw_mean, Tc_sw_std, ec_sw_std, &
               tau_vis_mean,re_vis_mean,tau_vis_std,re_vis_std,&
               Pc_sw_mean,Zc_sw_mean,tau_sw_mean,tau_sw_std, &
               beta_sw_mean, beta_sw_std,  &
               tau_vis_count, re_vis_count, &
               Tc_sw_count,ec_sw_count, beta_sw_count)

    deallocate(Tc_sw_mean_wat,ec_sw_mean_wat, Tc_sw_std_wat, ec_sw_std_wat, &
               tau_vis_mean_wat,re_vis_mean_wat,&
               tau_vis_std_wat,re_vis_std_wat, &
               Pc_sw_mean_wat,Zc_sw_mean_wat,tau_sw_mean_wat,tau_sw_std_wat, &
               beta_sw_mean_wat, beta_sw_std_wat,  &
               tau_vis_count_wat, re_vis_count_wat, &
               Tc_sw_count_wat,ec_sw_count_wat, beta_sw_count_wat)

    deallocate(Tc_sw_mean_ice,ec_sw_mean_ice, Tc_sw_std_ice, ec_sw_std_ice, &
               tau_vis_mean_ice,re_vis_mean_ice, &
               tau_vis_std_ice,re_vis_std_ice,  &
               Pc_sw_mean_ice,Zc_sw_mean_ice,tau_sw_mean_ice,tau_sw_std_ice, &
               beta_sw_mean_ice, beta_sw_std_ice,  &
               tau_vis_count_ice, re_vis_count_ice, &
               Tc_sw_count_ice,ec_sw_count_ice, beta_sw_count_ice)

    deallocate(aot1_mean,aot1_std,aot1_count,aot2_mean,aot2_std,aot2_count)

    deallocate(olr_mean,olr_std,olr_count, &
               olr_cldy_mean,olr_cldy_std,olr_cldy_count,&
               olr_clr_mean,olr_clr_std,olr_clr_count)
 
    deallocate(cell_night_count,cell_clear_count,alb1_clr_sfc_mean,alb2_clr_sfc_mean,alb1_sfc_mean,alb2_sfc_mean,&
               alb3b_sfc_mean,alb3b_clr_sfc_mean,rad3b_sfc_mean,rad3b_clr_sfc_mean, &
               rad4_sfc_mean,rad4_clr_sfc_mean,rad5_sfc_mean,rad5_clr_sfc_mean, &
               ems3b_mean,ems3b_clr_mean,ems3b_sfc_mean,ems3b_clr_sfc_mean)

    if (allocated(alb3a_sfc_mean))         deallocate(alb3a_sfc_mean)
    if (allocated(alb3a_clr_sfc_mean))     deallocate(alb3a_clr_sfc_mean)

    deallocate(cell_tsfc_nwp,cell_tair_nwp,cell_tpw_nwp,cell_weasd_nwp,cell_wnd_spd_10m_nwp, &
               cell_wnd_dir_10m_nwp,cell_ttrop_nwp,cell_uth_nwp,cell_hght500_nwp)

    if (allocated(ch4_rad_clr_nwp_mean))  deallocate(ch4_rad_clr_nwp_mean)
    if (allocated(ch5_rad_clr_nwp_mean))  deallocate(ch5_rad_clr_nwp_mean)

    if (allocated(cell_Pc_nwp))           deallocate(cell_Pc_nwp)
    if (allocated(cell_lwp_nwp))          deallocate(cell_lwp_nwp)
    if (allocated(cell_iwp_nwp))          deallocate(cell_iwp_nwp)
    if (allocated(cell_ncld_layers_nwp))  deallocate(cell_ncld_layers_nwp)

 end subroutine DESTROY_GRIDCELL_ARRAYS

!---------------------------------------------------------------------------------------------
! take the scanline pixel data and map into the grid-cell data
!---------------------------------------------------------------------------------------------
 subroutine COMPILE_GRIDCELL_ARRAYS(level3_format,nmask_clavr,nwp_flag,j1,j2)
   integer, intent(in):: j1,j2
   integer::erstat
   integer, intent(in):: level3_format
   integer(kind=int4), intent(in):: nmask_clavr
   integer, intent(in):: nwp_flag
   integer(kind=int4):: j,ipix,imask,iterm,icoast,ifire,isnow,idust, &
                        itype, iphase, icell,ivalid
   real:: dummy_r4, cell_time_temp
   integer:: icompile


 scan_loop_map:  do j = j1,j2


!-- check for bad scans
     if (bad_scan_flag(j) == sym%YES) then
        cycle
     endif

!----- populate the data array
       pix_data(1,:) = alb1(:,j)
       pix_data(2,:) = alb2(:,j)
       if (ch3a_on(j) == 1) then
        pix_data(3,:) = alb3a(:,j)
       else
        pix_data(3,:) = alb3b(:,j) 
       endif
       if (ch3a_on(j) == 0) then
         pix_data(4,:) = rad3b(:,j)
       else
         pix_data(4,:) = missing_value_real4
       endif
       pix_data(5,:) = rad4(:,j)
       pix_data(6,:) = rad5(:,j)

 
!-------------------------------------------------
! loop through each pixel on scanline and map it
!-------------------------------------------------
 pix_loop_map:    do ipix = 1,num_pix

!-- check for bad scans
     if (bad_pixel_mask(ipix,j) == sym%YES) then
        cycle
     endif

!--- make local aliases
           imask = cld_mask(ipix,j)

           if (cloud_type_ir_flag == sym%YES) then
               itype = cld_type_ir(ipix,j)
           else
               itype = cld_type(ipix,j)
           endif
           iterm = solar_contamination_mask(ipix,j)
           icoast = coast_mask(ipix,j)
           isnow = snow(ipix,j)
           ifire = fire(ipix,j)
           idust = dust(ipix,j)
           ivalid = bad_pixel_mask(ipix,j)

!---- make a phase from type (0=water, 1=ice)
           iphase = sym%WATER_PHASE
           if ((itype == sym%OPAQUE_ICE_TYPE) .or. &
               (itype == sym%CIRRUS_TYPE) .or. &
               (itype == sym%OVERLAP_TYPE) .or. &
               (itype == sym%OVERSHOOTING_TYPE) ) then
               iphase = sym%ICE_PHASE
            endif



!----- determine if a valid cloud mask was made
           if (ivalid == sym%YES) then
             cycle
           endif

!----- cloud type each pixel

         if (level3_format == 0) then
          call FIND_EQUAL_AREA_CELL(lat(ipix,j),lon(ipix,j),icell)
         elseif (level3_format == 1) then
          call FIND_EQUAL_ANGLE_CELL(lat(ipix,j),lon(ipix,j),icell)
         else
          print *, "gridcell format is not valid in clavr_mod, stopping"
          erstat = 41
          stop 41
         endif

!--------------------------------------------------------------------------
! map data for each pixel only if the land mask of that pixel is consistent
! with the predominant sfc type and excluding coast - this is the PATMOS logic
!---------------------------------------------------------------------------
 icompile = sym%YES
 if ( land_mask(ipix,j) /= min(1_int1,cell_sfc_type(icell)) ) then
   icompile = sym%NO
 endif

!---- this causes some cells to be missing in some parts of the world
!if (icoast == sym%YES) then
!  icompile = sym%NO
!  cycle
!endif
!--------------------------------------------------

!-- check to see if there is a discontinuity in the times going each cell - ie a day change
!-- if so, only keep one set of times - assume time should not vary by 15 minutes in a cell.
 if ((icompile == sym%YES) .and. (cell_pixel_count(icell) > 0)) then
   cell_time_temp = cell_utc_time(icell) / cell_pixel_count(icell)
   if (abs(utc_scan_time_hours(j) - cell_time_temp) > 0.25) then
     icompile = sym%NO
   endif
 endif


 if (icompile == sym%YES) then           !compile_check

!---- find lat and lon for this cell if it is missing
      if (cell_lat(icell) == missing_value_real4) then 
       if (level3_format == 0) then
          call PLACE_EQUAL_AREA_CELL(icell, cell_lat(icell), cell_lon(icell), dummy_r4,dummy_r4)
       elseif (level3_format == 1) then
         call PLACE_EQUAL_ANGLE_CELL(icell, cell_lat(icell), cell_lon(icell), dummy_r4,dummy_r4)
       else
         print *, EXE_PROMPT, MOD_PROMPT, "gridcell format not valid in COMPUTE_GRIDCELL_ARRAYS, stopping "
         erstat = 42
         stop 42
         stop
       endif
      endif

           cell_asc_des(icell) = ascend(j)        !the last scan line determines asc/des
           if (cell_ch3a_on(icell) < 0) then      !initially cell_ch3a_on is set to a negative number
             cell_ch3a_on(icell) = ch3a_on(j)
           endif
           if (cell_ch3a_on(icell) /= ch3a_on(j)) then         !if any nonuniformity of this value, set to 2
             cell_ch3a_on(icell) = 2
           endif

           cell_pixel_count(icell) = cell_pixel_count(icell) + 1

           if (solzen(ipix,j) > day_solzen_thresh_ref) then
              cell_night_count(icell) = cell_night_count(icell) + 1
           endif

           if (iterm == 1) then
             term_frac(icell) = term_frac(icell) + 1.0
           endif
           if (ifire == 1) then
             fire_frac(icell) = fire_frac(icell) + 1.0
           endif
           if (idust == 1) then
             dust_frac(icell) = dust_frac(icell) + 1.0
           endif
           if (isnow == 1) then
             snow_frac(icell) = snow_frac(icell) + 1.0
           endif
           if (smoke(ipix,j) == 1) then
             smoke_frac(icell) = smoke_frac(icell) + 1.0
           endif

!----- calibration parameters
           cell_space_counts(:,icell) = cell_space_counts(:,icell) + scan_space_counts(:,j)

!----- darkest pixel (channel 1)
           if ((pix_data(1,ipix) <= cell_dark_obs(1,icell)).or.(cell_dark_obs(1,icell) <= 0)) then
                cell_dark_obs(:,icell) = pix_data(:,ipix)
                cell_dark_obs(1,icell) = alb1_sfc(ipix,j)
                cell_dark_obs(2,icell) = alb2_sfc(ipix,j)
           endif

!----- warmest pixel  (channel 4)
           if ((pix_data(5,ipix) >= cell_dark_obs(5,icell)).or.(cell_dark_obs(5,icell) <= 0)) then
                cell_warm_obs(:,icell) = pix_data(:,ipix)
                cell_warm_obs(1,icell) = alb1_sfc(ipix,j)
                cell_warm_obs(2,icell) = alb2_sfc(ipix,j)
           endif

!----- warmest pixel  (channel 4)
           if ((ndvi(ipix,j) >= cell_green_obs(7,icell)).or.(cell_green_obs(7,icell) <= 0)) then
                cell_green_obs(1:6,icell) = pix_data(:,ipix)
                cell_green_obs(1,icell) = alb1_sfc(ipix,j)
                cell_green_obs(2,icell) = alb2_sfc(ipix,j)
                cell_green_obs(7,icell) = ndvi(ipix,j)
           endif


!------- global precipitation indices
           if (bt4(ipix,j) < 235.0) then
            cell_gpi_235(icell) = cell_gpi_235(icell) + 1.0
           endif
           if (bt4(ipix,j) < 255.0) then
            cell_gpi_255(icell) = cell_gpi_255(icell) + 1.0
           endif
           if (bt4(ipix,j) < 275.0) then
            cell_gpi_275(icell) = cell_gpi_275(icell) + 1.0
           endif

!------ fraction covered by DCC's
           if (dcc_mask(ipix,j) == sym%YES) then
            cell_dcc_frac(icell) = cell_dcc_frac(icell) + 1.0
            cell_mean_dcc_ch1(icell) = cell_mean_dcc_ch1(icell) + alb1(ipix,j)
            cell_std_dcc_ch1(icell) = cell_std_dcc_ch1(icell) + alb1(ipix,j)**2
           endif

!------ compile derived clear variables that don't have separate counts reserved for them
           if (imask == 0) then
             cell_clear_count(icell) = cell_clear_count(icell) + 1
             alb1_clr_sfc_mean(icell) = alb1_clr_sfc_mean(icell) + alb1_sfc(ipix,j)
             alb2_clr_sfc_mean(icell) = alb2_clr_sfc_mean(icell) + alb2_sfc(ipix,j)
             alb3b_clr_sfc_mean(icell) = alb3b_clr_sfc_mean(icell) + alb3b_sfc(ipix,j)
             rad3b_clr_sfc_mean(icell) = rad3b_clr_sfc_mean(icell) + rad3b_sfc(ipix,j)
             rad4_clr_sfc_mean(icell) = rad4_clr_sfc_mean(icell) + rad4_sfc(ipix,j)
             rad5_clr_sfc_mean(icell) = rad5_clr_sfc_mean(icell) + rad5_sfc(ipix,j)
             ems3b_clr_sfc_mean(icell) = ems3b_clr_sfc_mean(icell) + ems3b_sfc(ipix,j)
             ems3b_clr_mean(icell) = ems3b_clr_mean(icell) + ems3b(ipix,j)
           endif

!--- count snow pixels
            if (isnow == 1) then
                 cell_snow_count(icell) = cell_snow_count(icell) + 1
            endif

!--- compile background sst and ndvi for this cell
               if (land_mask(ipix,j) == sym%NO) then
                 cell_ocean_count(icell) = cell_ocean_count(icell) + 1
                 if (sst_anal(ipix,j) /= missing_value_real4) then
                  cell_sst_back(icell) = cell_sst_back(icell) + sst_anal(ipix,j)
                 else
                  cell_sst_back(icell) = cell_sst_back(icell) + tmpsfc_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
                 endif
               elseif (land_mask(ipix,j)==1) then
                 cell_land_count(icell) = cell_land_count(icell) + 1
               endif

!- compile mean and standard deviation of observations for each cloud mask value
            cell_mask_count(:,imask,icell) = cell_mask_count(:,imask,icell) + 1
            cell_mean(:,imask,icell) = cell_mean(:,imask,icell) + pix_data(:,ipix)
            cell_std(:,imask,icell) = cell_std(:,imask,icell) + pix_data(:,ipix)**2

!- multilayer cloud count
          if (itype == sym%OVERLAP_TYPE) then
            cell_multilayer_count(icell) = cell_multilayer_count(icell) + 1
          endif

! do for imask - nmask-1 which is reserved for all pixels
          cell_mask_count(:,nmask_clavr-1,icell) = cell_pixel_count(icell)
          cell_mean(:,nmask_clavr-1,icell) = cell_mean(:,nmask_clavr-1,icell)+ pix_data(:,ipix)
          cell_std(:,nmask_clavr-1,icell) =  cell_std(:,nmask_clavr-1,icell) + pix_data(:,ipix)**2
          alb1_sfc_mean(icell) = alb1_sfc_mean(icell) + alb1_sfc(ipix,j)
          alb2_sfc_mean(icell) = alb2_sfc_mean(icell) + alb2_sfc(ipix,j)
          alb3b_sfc_mean(icell) = alb3b_sfc_mean(icell) + alb3b_sfc(ipix,j)
          rad3b_sfc_mean(icell) = rad3b_sfc_mean(icell) + rad3b_sfc(ipix,j)
          rad4_sfc_mean(icell) = rad4_sfc_mean(icell) + rad4_sfc(ipix,j)
          rad5_sfc_mean(icell) = rad5_sfc_mean(icell) + rad5_sfc(ipix,j)
          ems3b_sfc_mean(icell) = ems3b_sfc_mean(icell) + ems3b_sfc(ipix,j)
          ems3b_mean(icell) = ems3b_mean(icell) + ems3b(ipix,j)

!-- all sky counts --------NOTE USING SINGLE GAIN COUNTS
          cell_count_sum(:,icell) = cell_count_sum(:,icell) + chan_counts_sg(:,ipix,j)

!--------- compute fraction of cloudy pixels that are water
           if (imask == 2 .and. iphase == sym%WATER_PHASE) then
               cell_water_count_cm2(icell) = cell_water_count_cm2(icell) + 1
           endif
           if (imask == 3 .and. iphase == sym%WATER_PHASE) then
               cell_water_count_cm3(icell) = cell_water_count_cm3(icell) + 1
           endif

!----------- map  angles for each pixel
           satzen_mean(icell) = satzen_mean(icell) + satzen(ipix,j)
           solzen_mean(icell) = solzen_mean(icell) + solzen(ipix,j)
           relaz_mean(icell) = relaz_mean(icell) + relaz(ipix,j)
           glint_mean(icell) = glint_mean(icell) + glintzen(ipix,j)
           zsfc_mean(icell) = zsfc_mean(icell) + zsfc(ipix,j)

!----------- map time for each pixel
             cell_utc_time(icell) = cell_utc_time(icell) + utc_scan_time_hours(j)

!---------- nwp stuff
           if (nwp_flag > 0) then

            cell_tpw_nwp(icell) = cell_tpw_nwp(icell) + tpw_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_uth_nwp(icell) = cell_uth_nwp(icell) + uth_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_hght500_nwp(icell) = cell_hght500_nwp(icell) + hght500_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_tair_nwp(icell) = cell_tair_nwp(icell) + tmpair_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_ttrop_nwp(icell) = cell_ttrop_nwp(icell) + ttropo_rtm(ipix,j)
            cell_tsfc_nwp(icell) = cell_tsfc_nwp(icell) + tmpsfc_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_weasd_nwp(icell) = cell_weasd_nwp(icell) + weasd_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_wnd_spd_10m_nwp(icell) = cell_wnd_spd_10m_nwp(icell) + wnd_spd_10m_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_wnd_dir_10m_nwp(icell) = cell_wnd_dir_10m_nwp(icell) + wnd_dir_10m_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            ch4_rad_clr_nwp_mean(icell) = ch4_rad_clr_nwp_mean(icell) + rad_clear_ch4_rtm(ipix,j)
            ch5_rad_clr_nwp_mean(icell) = ch5_rad_clr_nwp_mean(icell) + rad_clear_ch5_rtm(ipix,j)
            cell_Pc_nwp(icell) = cell_Pc_nwp(icell) + Pc_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_lwp_nwp(icell) = cell_lwp_nwp(icell) + lwp_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_iwp_nwp(icell) = cell_iwp_nwp(icell) + iwp_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
            cell_ncld_layers_nwp(icell) = cell_ncld_layers_nwp(icell) + ncld_layers_nwp(i_nwp(ipix,j),j_nwp(ipix,j))
           endif


!------- compile cloud layered cloud fraction from split window
          if (cld_layer_sw(ipix,j) > 0) then
            cld_layer_sum_sw(icell) = cld_layer_sum_sw(icell) +  1.0
            if (cld_layer_sw(ipix,j) == 3) then
               frac_high_sw(icell)  =  frac_high_sw(icell) + 1.0
            elseif (cld_layer_sw(ipix,j) == 2) then
               frac_mid_sw(icell)  =  frac_mid_sw(icell) + 1.0
            elseif (cld_layer_sw(ipix,j) == 1) then
               frac_low_sw(icell)  =  frac_low_sw(icell) + 1.0
            endif
          endif

       !--------- compile all cloud property statistics
       if (cld_flag == sym%YES) then

          if (Tc_sw_qf(ipix,j) >= cld_qf_min) then
              Tc_sw_mean(icell) = Tc_sw_mean(icell) + Tc_sw(ipix,j)
              Pc_sw_mean(icell) = Pc_sw_mean(icell) + Pc_sw(ipix,j)
              Zc_sw_mean(icell) = Zc_sw_mean(icell) + Zc_sw(ipix,j)
              Tc_sw_std(icell) = Tc_sw_std(icell) + Tc_sw(ipix,j)**2
              Tc_sw_count(icell) = Tc_sw_count(icell) + 1
          endif

          if (ec_sw_qf(ipix,j) >= cld_qf_min) then
              ec_sw_mean(icell) = ec_sw_mean(icell) + ec_sw(ipix,j)
              ec_sw_std(icell) = ec_sw_std(icell) + ec_sw(ipix,j)**2
              ec_sw_count(icell) = ec_sw_count(icell) + 1
              tau_sw_mean(icell) = tau_sw_mean(icell) + tau_sw(ipix,j)
              tau_sw_std(icell) = tau_sw_std(icell) + tau_sw(ipix,j)**2
          endif

          if (beta_sw_qf(ipix,j) >= cld_qf_min) then
              beta_sw_mean(icell) = beta_sw_mean(icell) + beta_sw(ipix,j)
              beta_sw_std(icell) = beta_sw_std(icell) + beta_sw(ipix,j)**2
              beta_sw_count(icell) = beta_sw_count(icell) + 1
          endif

          if (tau_vis_qf(ipix,j) >= cld_qf_min) then
              tau_vis_mean(icell) = tau_vis_mean(icell) + tau_vis(ipix,j)
              tau_vis_std(icell) = tau_vis_std(icell) + tau_vis(ipix,j)**2
              tau_vis_count(icell) = tau_vis_count(icell) + 1
          endif
          if (re_vis_qf(ipix,j) >= cld_qf_min) then
              re_vis_mean(icell) = re_vis_mean(icell) + re_vis(ipix,j)
              re_vis_std(icell) = re_vis_std(icell) + re_vis(ipix,j)**2
              re_vis_count(icell) = re_vis_count(icell) + 1
          endif

!--------- compile water cloud property statistics
         if (iphase == sym%WATER_PHASE) then

          if (Tc_sw_qf(ipix,j) >= cld_qf_min) then
              Tc_sw_mean_wat(icell) = Tc_sw_mean_wat(icell) + Tc_sw(ipix,j)
              Pc_sw_mean_wat(icell) = Pc_sw_mean_wat(icell) + Pc_sw(ipix,j)
              Zc_sw_mean_wat(icell) = Zc_sw_mean_wat(icell) + Zc_sw(ipix,j)
              Tc_sw_std_wat(icell) = Tc_sw_std_wat(icell) + Tc_sw(ipix,j)**2
              Tc_sw_count_wat(icell) = Tc_sw_count_wat(icell) + 1
          endif
          if (ec_sw_qf(ipix,j) >= cld_qf_min) then
              ec_sw_mean_wat(icell) = ec_sw_mean_wat(icell) + ec_sw(ipix,j)
              ec_sw_std_wat(icell) = ec_sw_std_wat(icell) + ec_sw(ipix,j)**2
              ec_sw_count_wat(icell) = ec_sw_count_wat(icell) + 1
              tau_sw_mean_wat(icell) = tau_sw_mean_wat(icell) + tau_sw(ipix,j)
              tau_sw_std_wat(icell) = tau_sw_std_wat(icell) + tau_sw(ipix,j)**2
          endif
          if (beta_sw_qf(ipix,j) >= cld_qf_min) then
              beta_sw_mean_wat(icell) = beta_sw_mean_wat(icell) + beta_sw(ipix,j)
              beta_sw_std_wat(icell) = beta_sw_std_wat(icell) + beta_sw(ipix,j)**2
              beta_sw_count_wat(icell) = beta_sw_count_wat(icell) + 1
          endif
          if (tau_vis_qf(ipix,j) >= cld_qf_min) then
              tau_vis_mean_wat(icell) = tau_vis_mean_wat(icell) + tau_vis(ipix,j)
              tau_vis_std_wat(icell) = tau_vis_std_wat(icell) + tau_vis(ipix,j)**2
              tau_vis_count_wat(icell) = tau_vis_count_wat(icell) + 1
          endif
          if (re_vis_qf(ipix,j) >= cld_qf_min) then
              re_vis_mean_wat(icell) = re_vis_mean_wat(icell) + re_vis(ipix,j)
              re_vis_std_wat(icell) = re_vis_std_wat(icell) + re_vis(ipix,j)**2
              re_vis_count_wat(icell) = re_vis_count_wat(icell) + 1
          endif
        endif

!--------- compile ice cloud property statistics
         if (iphase == sym%ICE_PHASE) then

          if (Tc_sw_qf(ipix,j) >= cld_qf_min) then
              Tc_sw_mean_ice(icell) = Tc_sw_mean_ice(icell) + Tc_sw(ipix,j)
              Pc_sw_mean_ice(icell) = Pc_sw_mean_ice(icell) + Pc_sw(ipix,j)
              Zc_sw_mean_ice(icell) = Zc_sw_mean_ice(icell) + Zc_sw(ipix,j)
              Tc_sw_std_ice(icell) = Tc_sw_std_ice(icell) + Tc_sw(ipix,j)**2
              Tc_sw_count_ice(icell) = Tc_sw_count_ice(icell) + 1
          endif
          if (ec_sw_qf(ipix,j) >= cld_qf_min) then
              ec_sw_mean_ice(icell) = ec_sw_mean_ice(icell) + ec_sw(ipix,j)
              ec_sw_std_ice(icell) = ec_sw_std_ice(icell) + ec_sw(ipix,j)**2
              ec_sw_count_ice(icell) = ec_sw_count_ice(icell) + 1
              tau_sw_mean_ice(icell) = tau_sw_mean_ice(icell) + tau_sw(ipix,j)
              tau_sw_std_ice(icell) = tau_sw_std_ice(icell) + tau_sw(ipix,j)**2
          endif
          if (beta_sw_qf(ipix,j) >= cld_qf_min) then
              beta_sw_mean_ice(icell) = beta_sw_mean_ice(icell) + beta_sw(ipix,j)
              beta_sw_std_ice(icell) = beta_sw_std_ice(icell) + beta_sw(ipix,j)**2
              beta_sw_count_ice(icell) = beta_sw_count_ice(icell) + 1
          endif

          if (tau_vis_qf(ipix,j) >= cld_qf_min) then
              tau_vis_mean_ice(icell) = tau_vis_mean_ice(icell) + tau_vis(ipix,j)
              tau_vis_std_ice(icell) = tau_vis_std_ice(icell) + tau_vis(ipix,j)**2
              tau_vis_count_ice(icell) = tau_vis_count_ice(icell) + 1
          endif
          if (re_vis_qf(ipix,j) >= cld_qf_min) then
              re_vis_mean_ice(icell) = re_vis_mean_ice(icell) + re_vis(ipix,j)
              re_vis_std_ice(icell) = re_vis_std_ice(icell) + re_vis(ipix,j)**2
              re_vis_count_ice(icell) = re_vis_count_ice(icell) + 1
          endif
        endif

          !-------- ice water path
          if ((tau_vis_qf(ipix,j) >= cld_qf_min).and.(iphase == sym%ICE_PHASE)) then          
              iwp_vis_mean(icell) = iwp_vis_mean(icell) + iwp_vis(ipix,j)
              iwp_vis_std(icell) = iwp_vis_std(icell) + iwp_vis(ipix,j)**2
              iwp_vis_count(icell) = iwp_vis_count(icell) + 1
          endif

          !-------- liquid water path
          if ((tau_vis_qf(ipix,j) >= cld_qf_min)  .and. &
              (re_vis_qf(ipix,j) >= cld_qf_min)   .and. &
              (iphase == sym%WATER_PHASE)) then
              lwp_vis_mean(icell) = lwp_vis_mean(icell) + lwp_vis(ipix,j) 
              lwp_vis_std(icell) = lwp_vis_std(icell) + lwp_vis(ipix,j)**2
              lwp_vis_count(icell) = lwp_vis_count(icell) + 1
          endif
       endif



          !--------- compile aerosol property statistics
          if (aer_flag == sym%YES) then
            if (aot_qf(ipix,j) >= aot_qf_min) then
              aot1_mean(icell) = aot1_mean(icell) + aot1(ipix,j)
              aot1_std(icell) = aot1_std(icell) + aot1(ipix,j)**2
              aot1_count(icell) = aot1_count(icell) + 1
            endif

            if (aot_qf(ipix,j) >= aot_qf_min) then
              aot2_mean(icell) = aot2_mean(icell) + aot2(ipix,j)
              aot2_std(icell) = aot2_std(icell) + aot2(ipix,j)**2
              aot2_count(icell) = aot2_count(icell) + 1
            endif
          endif

!--------- compile ndvi statistics
          if (ndvi_qf(ipix,j) >= ndvi_qf_min) then
              cell_mean_ndvi_sfc(icell) = cell_mean_ndvi_sfc(icell) + ndvi_sfc(ipix,j)
              ndvi_count(icell) = ndvi_count(icell) + 1
          endif

!--------- compile rsr statistics
          if (rsr_qf(ipix,j) >= rsr_qf_min) then
              cell_mean_rsr(icell) = cell_mean_rsr(icell) + rsr(ipix,j)
              cell_std_rsr(icell) = cell_std_rsr(icell) + rsr(ipix,j)**2
              rsr_count(icell) = rsr_count(icell) + 1
          endif

!--------- compile tsfc statistics
          if ((tsfc_qf(ipix,j) >= tsfc_qf_min)) then
              cell_mean_tsfc(icell) = cell_mean_tsfc(icell) + tsfc(ipix,j)
              cell_std_tsfc(icell) = cell_std_tsfc(icell) + tsfc(ipix,j)**2
              tsfc_count(icell) = tsfc_count(icell) + 1
          endif

!--------- compile olr statistics
          if (erb_flag == sym%YES) then
            if (olr_qf(ipix,j) >= olr_qf_min) then
              !--- all-sky
               olr_mean(icell) = olr_mean(icell) + olr(ipix,j)
               olr_std(icell) = olr_std(icell) + olr(ipix,j)**2
               olr_count(icell) = olr_count(icell) + 1
              !--- cloudy 
              if ((imask == sym%PROB_CLOUDY).or.(imask == sym%CLOUDY)) then
                olr_cldy_mean(icell) = olr_cldy_mean(icell) + olr(ipix,j)
                olr_cldy_std(icell) = olr_cldy_std(icell) + olr(ipix,j)**2
                olr_cldy_count(icell) = olr_cldy_count(icell) + 1
              endif
              !--- clear 
              if ((imask == sym%PROB_CLEAR).or.(imask == sym%CLEAR)) then
               olr_clr_mean(icell) = olr_clr_mean(icell) + olr(ipix,j)
               olr_clr_std(icell) = olr_clr_std(icell) + olr(ipix,j)**2
               olr_clr_count(icell) = olr_clr_count(icell) + 1
              endif
            endif
         endif


   end if  !compile_check

!--------------------------------------------------------------------------
!  compute cell_mask_count_ir using similar strategy
!---------------------------------------------------------------------------
      if (icompile == sym%YES) then
           imask = cld_mask_ir(ipix,j)
           cell_mask_count_ir(:,imask,icell) = cell_mask_count_ir(:,imask,icell) + 1
      end if 


       end do  pix_loop_map
     end do scan_loop_map

 end subroutine COMPILE_GRIDCELL_ARRAYS 

!-----------------------------------------------------------------------------
! compute records the clavr file
!-----------------------------------------------------------------------------
 subroutine COMPUTE_GRIDCELL_ARRAYS(ncells,orbit_number,start_day,end_day, &
                                    start_year,end_year, &
                                    nchan_clavr,nmask_clavr,sc_id_new,ncells_with_data)
                                    
  integer(kind=int4),intent(in):: ncells,orbit_number,nchan_clavr,nmask_clavr
  integer(kind=int2),intent(in):: start_day,start_year,end_day,end_year
  integer,intent(in):: sc_id_new
  integer(kind=int4), intent(out):: ncells_with_data
  real(kind=real4):: solzen_temp


  integer(kind=int4):: icell,i,j
  

!-------------------------------------------------------------------------------------
! determine cell ancillary and geolocation properties
!--------------------------------------------------------------------------------------
 ncells_with_data = 0
 do icell = 1, ncells

   if (cell_pixel_count(icell) >= count_thresh) then     !check for minimum number of pixels

        cell_sc_id(icell) = sc_id_new
        cell_utc_time(icell) = cell_utc_time(icell) / cell_pixel_count(icell)
        cell_orbit(icell) = orbit_number
        cell_day(icell) = start_day
   
!--- check for data across UTC midnight
        if (start_day /= end_day) then
          if (cell_utc_time(icell) > 12.00) then
            cell_day(icell) = start_day
          else
            cell_day(icell) = end_day
          endif
        endif

        if (cell_day(icell) < 366) then
            cell_year(icell) = start_year
       else
            cell_year(icell) = end_year
       endif

       ncells_with_data = ncells_with_data + 1

      else
       cell_sc_id(icell) = missing_value_int1
       cell_utc_time(icell) = missing_value_real4
       cell_day(icell) = missing_value_int2
       cell_year(icell) = missing_value_int2
       cell_orbit(icell) = missing_value_int4
       cell_lon(icell) = missing_value_real4
       cell_lat(icell) = missing_value_real4
      endif

    enddo

    print *, EXE_PROMPT, MOD_PROMPT, "Number of cells with data = ", ncells_with_data


!---------------------------------------------------------------
! determine cloud properties
!---------------------------------------------------------------
 do icell = 1, ncells

   if (cell_pixel_count(icell) >= count_thresh) then     !check for minimum number of pixels

!---- total cloud amount
      frac_all(icell) =  sum(frac_weight(0:3)*cell_mask_count(1,0:3,icell))/cell_pixel_count(icell)
      frac_all_ir(icell) =  sum(frac_weight(0:3)*cell_mask_count_ir(1,0:3,icell))/cell_pixel_count(icell)

!---- water and ice cloud
      frac_ice(icell) = 0.0
      frac_water(icell) = 0.0
      if (sum(cell_mask_count(1,2:3,icell)) > 0) then
       frac_water(icell) = (cell_water_count_cm2(icell)*frac_weight(2) + cell_water_count_cm3(icell)*frac_weight(3))/ &
             (frac_weight(2)*cell_mask_count(1,2,icell) + frac_weight(3)*cell_mask_count(1,3,icell))
       frac_ice(icell) =  1.0 - frac_water(icell)
       frac_water(icell) = frac_water(icell)*frac_all(icell)
       frac_ice(icell) = frac_ice(icell)*frac_all(icell)
      endif

!--- High, Mid and Low Cloud Amounts from Split Window
      if (cld_layer_sum_sw(icell) > 0.0) then
        frac_high_sw(icell) = frac_all(icell) * frac_high_sw(icell)/cld_layer_sum_sw(icell)
        frac_mid_sw(icell) = frac_all(icell) * frac_mid_sw(icell)/cld_layer_sum_sw(icell)
        frac_low_sw(icell) = frac_all(icell) * frac_low_sw(icell)/cld_layer_sum_sw(icell)
      else
        frac_high_sw(icell) = 0.0
        frac_mid_sw(icell) = 0.0
        frac_low_sw(icell) = frac_all(icell)
      endif

!---- cloud type fractions
      if (cell_multilayer_count(icell) > 0) then
        frac_multilayer(icell) = real(cell_multilayer_count(icell)) / cell_pixel_count(icell)
      else
        frac_multilayer(icell) = 0.0
      endif

!---------------------------------------------------------------
!---- cloud properties 
!---------------------------------------------------------------

!--- all cloud
     if (tau_vis_count(icell) >= count_thresh) then
        tau_vis_mean(icell) = tau_vis_mean(icell)/tau_vis_count(icell) 
        tau_vis_std(icell) = sqrt( max(0.0,(tau_vis_std(icell)/tau_vis_count(icell) - &
             tau_vis_mean(icell)**2)))
     else
        tau_vis_mean(icell) = missing_value_real4
        tau_vis_std(icell) = missing_value_real4
     endif 

     if (re_vis_count(icell) >= count_thresh) then
        re_vis_mean(icell) = re_vis_mean(icell)/re_vis_count(icell) 
        re_vis_std(icell) = sqrt( max(0.0,(re_vis_std(icell)/re_vis_count(icell) - &
                              re_vis_mean(icell)**2)))
     else
        re_vis_mean(icell) = missing_value_real4
        re_vis_std(icell) = missing_value_real4
     endif 

     if (Tc_sw_count(icell) >= count_thresh) then
        Tc_sw_mean(icell) = Tc_sw_mean(icell)/Tc_sw_count(icell) 
        Pc_sw_mean(icell) = Pc_sw_mean(icell)/Tc_sw_count(icell) 
        Zc_sw_mean(icell) = Zc_sw_mean(icell)/Tc_sw_count(icell) 
        Tc_sw_std(icell) = sqrt( max(0.0,(Tc_sw_std(icell)/Tc_sw_count(icell) - &
                              Tc_sw_mean(icell)**2)))
     else
        Tc_sw_mean(icell) = missing_value_real4
        Pc_sw_mean(icell) = missing_value_real4
        Zc_sw_mean(icell) = missing_value_real4
        Tc_sw_std(icell) = missing_value_real4
     endif

     if (ec_sw_count(icell) >= count_thresh) then
        ec_sw_mean(icell) = ec_sw_mean(icell)/ec_sw_count(icell) 
        ec_sw_std(icell) = sqrt( max(0.0,(ec_sw_std(icell)/ec_sw_count(icell) - &
                              ec_sw_mean(icell)**2)))
        tau_sw_mean(icell) = tau_sw_mean(icell)/ec_sw_count(icell) 
        tau_sw_std(icell) = sqrt( max(0.0,(tau_sw_std(icell)/ec_sw_count(icell) - &
                              tau_sw_mean(icell)**2)))
     else
        ec_sw_mean(icell) = missing_value_real4
        ec_sw_std(icell) = missing_value_real4
        tau_sw_mean(icell) = missing_value_real4
        tau_sw_std(icell) = missing_value_real4
     endif 

     if (beta_sw_count(icell) >= count_thresh) then
        beta_sw_mean(icell) = beta_sw_mean(icell)/beta_sw_count(icell) 
        beta_sw_std(icell) = sqrt( max(0.0,(beta_sw_std(icell)/beta_sw_count(icell) - &
                              beta_sw_mean(icell)**2)))
     else
        beta_sw_mean(icell) = missing_value_real4
        beta_sw_std(icell) = missing_value_real4
     endif 

!--- water cloud
     if (tau_vis_count_wat(icell) >= count_thresh) then
        tau_vis_mean_wat(icell) = tau_vis_mean_wat(icell)/tau_vis_count_wat(icell) 
        tau_vis_std_wat(icell) = sqrt( max(0.0,(tau_vis_std_wat(icell)/tau_vis_count_wat(icell) - &
             tau_vis_mean_wat(icell)**2)))
     else
        tau_vis_mean_wat(icell) = missing_value_real4
        tau_vis_std_wat(icell) = missing_value_real4
     endif 

     if (re_vis_count_wat(icell) >= count_thresh) then
        re_vis_mean_wat(icell) = re_vis_mean_wat(icell)/re_vis_count_wat(icell) 
        re_vis_std_wat(icell) = sqrt( max(0.0,(re_vis_std_wat(icell)/re_vis_count_wat(icell) - &
                              re_vis_mean_wat(icell)**2)))
     else
        re_vis_mean_wat(icell) = missing_value_real4
        re_vis_std_wat(icell) = missing_value_real4
     endif 

     if (Tc_sw_count_wat(icell) >= count_thresh) then
        Tc_sw_mean_wat(icell) = Tc_sw_mean_wat(icell)/Tc_sw_count_wat(icell) 
        Pc_sw_mean_wat(icell) = Pc_sw_mean_wat(icell)/Tc_sw_count_wat(icell) 
        Zc_sw_mean_wat(icell) = Zc_sw_mean_wat(icell)/Tc_sw_count_wat(icell) 
        Tc_sw_std_wat(icell) = sqrt( max(0.0,(Tc_sw_std_wat(icell)/Tc_sw_count_wat(icell) - &
                              Tc_sw_mean_wat(icell)**2)))
     else
        Tc_sw_mean_wat(icell) = missing_value_real4
        Pc_sw_mean_wat(icell) = missing_value_real4
        Zc_sw_mean_wat(icell) = missing_value_real4
        Tc_sw_std_wat(icell) = missing_value_real4
     endif 

     if (ec_sw_count_wat(icell) >= count_thresh) then
        ec_sw_mean_wat(icell) = ec_sw_mean_wat(icell)/ec_sw_count_wat(icell) 
        ec_sw_std_wat(icell) = sqrt( max(0.0,(ec_sw_std_wat(icell)/ec_sw_count_wat(icell) - &
                              ec_sw_mean_wat(icell)**2)))
        tau_sw_mean_wat(icell) = tau_sw_mean_wat(icell)/ec_sw_count_wat(icell) 
        tau_sw_std_wat(icell) = sqrt( max(0.0,(tau_sw_std_wat(icell)/ec_sw_count_wat(icell) - &
                              tau_sw_mean_wat(icell)**2)))
     else
        ec_sw_mean_wat(icell) = missing_value_real4
        ec_sw_std_wat(icell) = missing_value_real4
        tau_sw_mean_wat(icell) = missing_value_real4
        tau_sw_std_wat(icell) = missing_value_real4
     endif 

     if (beta_sw_count_wat(icell) >= count_thresh) then
        beta_sw_mean_wat(icell) = beta_sw_mean_wat(icell)/beta_sw_count_wat(icell) 
        beta_sw_std_wat(icell) = sqrt( max(0.0,(beta_sw_std_wat(icell)/beta_sw_count_wat(icell) - &
                              beta_sw_mean_wat(icell)**2)))
     else
        beta_sw_mean_wat(icell) = missing_value_real4
        beta_sw_std_wat(icell) = missing_value_real4
     endif 

!--- ice cloud
     if (tau_vis_count_ice(icell) >= count_thresh) then
        tau_vis_mean_ice(icell) = tau_vis_mean_ice(icell)/tau_vis_count_ice(icell) 
        tau_vis_std_ice(icell) = sqrt( max(0.0,(tau_vis_std_ice(icell)/tau_vis_count_ice(icell) - &
             tau_vis_mean_ice(icell)**2)))
     else
        tau_vis_mean_ice(icell) = missing_value_real4
        tau_vis_std_ice(icell) = missing_value_real4
     endif 

     if (re_vis_count_ice(icell) >= count_thresh) then
        re_vis_mean_ice(icell) = re_vis_mean_ice(icell)/re_vis_count_ice(icell) 
        re_vis_std_ice(icell) = sqrt( max(0.0,(re_vis_std_ice(icell)/re_vis_count_ice(icell) - &
                              re_vis_mean_ice(icell)**2)))
     else
        re_vis_mean_ice(icell) = missing_value_real4
        re_vis_std_ice(icell) = missing_value_real4
     endif 

     if (Tc_sw_count_ice(icell) >= count_thresh) then
        Tc_sw_mean_ice(icell) = Tc_sw_mean_ice(icell)/Tc_sw_count_ice(icell) 
        Pc_sw_mean_ice(icell) = Pc_sw_mean_ice(icell)/Tc_sw_count_ice(icell) 
        Zc_sw_mean_ice(icell) = Zc_sw_mean_ice(icell)/Tc_sw_count_ice(icell) 
        Tc_sw_std_ice(icell) = sqrt( max(0.0,(Tc_sw_std_ice(icell)/Tc_sw_count_ice(icell) - &
                              Tc_sw_mean_ice(icell)**2)))
     else
        Tc_sw_mean_ice(icell) = missing_value_real4
        Pc_sw_mean_ice(icell) = missing_value_real4
        Zc_sw_mean_ice(icell) = missing_value_real4
        Tc_sw_std_ice(icell) = missing_value_real4
     endif 

     if (ec_sw_count_ice(icell) >= count_thresh) then
        ec_sw_mean_ice(icell) = ec_sw_mean_ice(icell)/ec_sw_count_ice(icell) 
        ec_sw_std_ice(icell) = sqrt( max(0.0,(ec_sw_std_ice(icell)/ec_sw_count_ice(icell) - &
                              ec_sw_mean_ice(icell)**2)))
        tau_sw_mean_ice(icell) = tau_sw_mean_ice(icell)/ec_sw_count_ice(icell) 
        tau_sw_std_ice(icell) = sqrt( max(0.0,(tau_sw_std_ice(icell)/ec_sw_count_ice(icell) - &
                              tau_sw_mean_ice(icell)**2)))
     else
        ec_sw_mean_ice(icell) = missing_value_real4
        ec_sw_std_ice(icell) = missing_value_real4
        tau_sw_mean_ice(icell) = missing_value_real4
        tau_sw_std_ice(icell) = missing_value_real4
     endif 

     if (beta_sw_count_ice(icell) >= count_thresh) then
        beta_sw_mean_ice(icell) = beta_sw_mean_ice(icell)/beta_sw_count_ice(icell) 
        beta_sw_std_ice(icell) = sqrt( max(0.0,(beta_sw_std_ice(icell)/beta_sw_count_ice(icell) - &
                              beta_sw_mean_ice(icell)**2)))
     else
        beta_sw_mean_ice(icell) = missing_value_real4
        beta_sw_std_ice(icell) = missing_value_real4
     endif 

!--- liquid water path
     if (lwp_vis_count(icell) >= count_thresh) then
        lwp_vis_mean(icell) = lwp_vis_mean(icell)/lwp_vis_count(icell) 
        lwp_vis_std(icell) = sqrt( max(0.0,(lwp_vis_std(icell)/lwp_vis_count(icell) - &
                              lwp_vis_mean(icell)**2)))
     else
        lwp_vis_mean(icell) = missing_value_real4
        lwp_vis_std(icell) = missing_value_real4
     endif 

!---- ice water path
     if (iwp_vis_count(icell) >= count_thresh) then
        iwp_vis_mean(icell) = iwp_vis_mean(icell)/iwp_vis_count(icell) 
        iwp_vis_std(icell) = sqrt( max(0.0,(iwp_vis_std(icell)/iwp_vis_count(icell) - &
                              iwp_vis_mean(icell)**2)))
     else
        iwp_vis_mean(icell) = missing_value_real4
        iwp_vis_std(icell) = missing_value_real4
     endif 

!---------------------------------------------------------------
!---- aerosol properties 
!---------------------------------------------------------------
     if (aot1_count(icell) >= count_thresh) then
        aot1_mean(icell) = aot1_mean(icell)/aot1_count(icell) 
        aot1_std(icell) = sqrt( max(0.0,(aot1_std(icell)/aot1_count(icell) - &
             aot1_mean(icell)**2)))
     else
        aot1_mean(icell) = missing_value_real4
        aot1_std(icell) = missing_value_real4
     endif 

     if (aot2_count(icell) >= count_thresh) then
        aot2_mean(icell) = aot2_mean(icell)/aot2_count(icell) 
        aot2_std(icell) = sqrt( max(0.0,(aot2_std(icell)/aot2_count(icell) - &
             aot2_mean(icell)**2)))
     else
        aot2_mean(icell) = missing_value_real4
        aot2_std(icell) = missing_value_real4
     endif 


!------------------------------------------------------------
! clear surface reflectances
!------------------------------------------------------------
     if (cell_pixel_count(icell) >= count_thresh) then
         alb1_sfc_mean(icell) = alb1_sfc_mean(icell) / cell_pixel_count(icell)
         alb2_sfc_mean(icell) = alb2_sfc_mean(icell) / cell_pixel_count(icell)
         rad3b_sfc_mean(icell) = rad3b_sfc_mean(icell) / cell_pixel_count(icell)
         rad4_sfc_mean(icell) = rad4_sfc_mean(icell) / cell_pixel_count(icell)
         rad5_sfc_mean(icell) = rad5_sfc_mean(icell) / cell_pixel_count(icell)
         alb3b_sfc_mean(icell) = alb3b_sfc_mean(icell) / cell_pixel_count(icell)
         ems3b_sfc_mean(icell) = ems3b_sfc_mean(icell) / cell_pixel_count(icell)
         ems3b_mean(icell) = ems3b_mean(icell) / cell_pixel_count(icell)
     else
         alb1_sfc_mean(icell) = missing_value_real4
         alb2_sfc_mean(icell) = missing_value_real4
         rad3b_sfc_mean(icell) = missing_value_real4
         rad4_sfc_mean(icell) = missing_value_real4
         rad5_sfc_mean(icell) = missing_value_real4
         alb3b_sfc_mean(icell) = missing_value_real4
         ems3b_sfc_mean(icell) = missing_value_real4
         ems3b_mean(icell) = missing_value_real4
     endif

     if (cell_clear_count(icell) >= count_thresh) then
         alb1_clr_sfc_mean(icell) = alb1_clr_sfc_mean(icell) / cell_clear_count(icell)
         alb2_clr_sfc_mean(icell) = alb2_clr_sfc_mean(icell) / cell_clear_count(icell)
         alb3b_clr_sfc_mean(icell) = alb3b_clr_sfc_mean(icell) / cell_clear_count(icell)
         rad3b_clr_sfc_mean(icell) = rad3b_clr_sfc_mean(icell) / cell_clear_count(icell)
         rad4_clr_sfc_mean(icell) = rad4_clr_sfc_mean(icell) / cell_clear_count(icell)
         rad5_clr_sfc_mean(icell) = rad5_clr_sfc_mean(icell) / cell_clear_count(icell)
         ems3b_clr_sfc_mean(icell) = ems3b_clr_sfc_mean(icell) / cell_clear_count(icell)
         ems3b_clr_mean(icell) = ems3b_clr_mean(icell) / cell_clear_count(icell)
     else
         alb1_clr_sfc_mean(icell) = missing_value_real4
         alb2_clr_sfc_mean(icell) = missing_value_real4
         alb3b_clr_sfc_mean(icell) = missing_value_real4
         rad3b_clr_sfc_mean(icell) = missing_value_real4
         rad4_clr_sfc_mean(icell) = missing_value_real4
         rad5_clr_sfc_mean(icell) = missing_value_real4
         ems3b_clr_sfc_mean(icell) = missing_value_real4
         ems3b_clr_mean(icell) = missing_value_real4
     endif

!----- RSR
     if (rsr_count(icell) >= count_thresh) then
        cell_mean_rsr(icell) = cell_mean_rsr(icell)/rsr_count(icell) 
        cell_std_rsr(icell) = sqrt( max(0.0,(cell_std_rsr(icell)/rsr_count(icell) - &
             cell_mean_rsr(icell)**2)))
     else
        cell_mean_rsr(icell) = missing_value_real4
        cell_std_rsr(icell) = missing_value_real4
     endif

!----- Tsfc
     if (tsfc_count(icell) >= count_thresh) then
        cell_mean_tsfc(icell) = cell_mean_tsfc(icell)/tsfc_count(icell) 
        cell_std_tsfc(icell) = sqrt( max(0.0,(cell_std_tsfc(icell)/tsfc_count(icell) - &
             cell_mean_tsfc(icell)**2)))
     else
        cell_mean_tsfc(icell) = missing_value_real4
        cell_std_tsfc(icell) = missing_value_real4
     endif

!----- NDVI
     if (ndvi_count(icell) >= count_thresh) then
        cell_mean_ndvi_sfc(icell) = cell_mean_ndvi_sfc(icell)/ndvi_count(icell) 
     else
        cell_mean_ndvi_sfc(icell) = missing_value_real4
     endif

!---------------------------------------------------------------
!----  olr
!---------------------------------------------------------------

!--- all sky
     if (olr_count(icell) >= count_thresh) then
        olr_mean(icell) = olr_mean(icell)/olr_count(icell) 
        olr_std(icell) = sqrt( max(0.0,(olr_std(icell)/olr_count(icell) - &
             olr_mean(icell)**2)))
     else
        olr_mean(icell) = missing_value_real4
        olr_std(icell) = missing_value_real4
     endif

!--- cloudy
     if (olr_cldy_count(icell) >= count_thresh) then
        olr_cldy_mean(icell) = olr_cldy_mean(icell)/olr_cldy_count(icell) 
        olr_cldy_std(icell) = sqrt( max(0.0,(olr_cldy_std(icell)/olr_cldy_count(icell) - &
             olr_cldy_mean(icell)**2)))
     else
        olr_cldy_mean(icell) = missing_value_real4
        olr_cldy_std(icell) = missing_value_real4
     endif

!--- clear
     if (olr_clr_count(icell) >= count_thresh) then
        olr_clr_mean(icell) = olr_clr_mean(icell)/olr_clr_count(icell) 
        olr_clr_std(icell) = sqrt( max(0.0,(olr_clr_std(icell)/olr_clr_count(icell) - &
             olr_clr_mean(icell)**2)))
     else
        olr_clr_mean(icell) = missing_value_real4
        olr_clr_std(icell) = missing_value_real4
     endif

   endif   !end number of pixel check

 enddo


!------------------------------------------------------------------------------
!--- compute means and standard deviations of observations in each grid-cell
!------------------------------------------------------------------------------
     do i = 1, nchan_clavr
      do j = 0,nmask_clavr-1
        do icell = 1,ncells

!---- compute stats if sufficient pixels
          if (cell_mask_count(i,j,icell) >= count_thresh) then
             cell_mean(i,j,icell) = cell_mean(i,j,icell) / cell_mask_count(i,j,icell)
             cell_std(i,j,icell) =  sqrt(max(0.0, &
                                    cell_std(i,j,icell)/cell_mask_count(i,j,icell)-cell_mean(i,j,icell)**2))
          else
             cell_mean(i,j,icell) = missing_value_real4
             cell_std(i,j,icell) =  missing_value_real4
          endif

!--- set all reflectances to missing if any night pixels in this cell (not for alb3b)
          if (cell_night_count(icell) > 0) then
           cell_mean(1:2,:,icell) = missing_value_real4
           cell_std(1:2,:,icell) = missing_value_real4
           alb1_clr_sfc_mean(icell) = missing_value_real4
           alb2_clr_sfc_mean(icell) = missing_value_real4
           alb1_sfc_mean(icell) = missing_value_real4
           alb2_sfc_mean(icell) = missing_value_real4
          endif
          if ((cell_ch3a_on(icell) == 1).and.(cell_night_count(icell) > 0)) then
           cell_mean(3,:,icell) = missing_value_real4
           cell_std(3,:,icell) = missing_value_real4
          endif 
        end do
       end do
     end do


!-----------------------------------------------------------------
! compute mean counts for each channel in each cell
!-----------------------------------------------------------------
     do i = 1, 5
      do icell = 1,ncells
       if ((cell_pixel_count(icell) >= count_thresh) .and. (cell_count_sum(i,icell) > 0.0)) then
         cell_count_mean(i,icell) = nint(cell_count_sum(i,icell)/cell_pixel_count(icell))
       else
         cell_count_mean(i,icell) = missing_value_int2
       endif
      enddo
    enddo

!-----------------------------------------------------------------
! compute stats for dcc
!-----------------------------------------------------------------
     do icell = 1,ncells
       if (cell_pixel_count(icell) >= count_thresh)  then
         cell_dcc_frac(icell) = cell_dcc_frac(icell) / cell_pixel_count(icell)
         cell_mean_dcc_ch1(icell) = cell_mean_dcc_ch1(icell) / cell_pixel_count(icell)
         cell_std_dcc_ch1(icell) = sqrt(max(0.0, &
               cell_std_dcc_ch1(icell)/cell_pixel_count(icell) - cell_mean_dcc_ch1(icell)**2))
       else
         cell_dcc_frac(icell) = missing_value_real4
         cell_mean_dcc_ch1(icell) = missing_value_real4
         cell_std_dcc_ch1(icell) = missing_value_real4
       endif
     enddo

!------- compute means of ever present values
  where (cell_pixel_count >= count_thresh)
   snow_frac = real(cell_snow_count) / cell_pixel_count
   term_frac = term_frac / cell_pixel_count
   fire_frac = fire_frac / cell_pixel_count
   dust_frac = dust_frac / cell_pixel_count
   smoke_frac = smoke_frac / cell_pixel_count
   satzen_mean = satzen_mean / cell_pixel_count
   solzen_mean = solzen_mean / cell_pixel_count
   relaz_mean = relaz_mean / cell_pixel_count
   glint_mean = glint_mean / cell_pixel_count
   zsfc_mean = zsfc_mean / cell_pixel_count
   cell_gpi_235 = cell_gpi_235 / cell_pixel_count
   cell_gpi_255 = cell_gpi_255 / cell_pixel_count
   cell_gpi_275 = cell_gpi_275 / cell_pixel_count
   cell_space_counts(1,:) = cell_space_counts(1,:) / cell_pixel_count
   cell_space_counts(2,:) = cell_space_counts(2,:) / cell_pixel_count
   cell_space_counts(3,:) = cell_space_counts(3,:) / cell_pixel_count
   cell_space_counts(4,:) = cell_space_counts(4,:) / cell_pixel_count
   cell_space_counts(5,:) = cell_space_counts(5,:) / cell_pixel_count
   cell_tsfc_nwp = cell_tsfc_nwp / cell_pixel_count
   cell_tair_nwp = cell_tair_nwp / cell_pixel_count
   cell_ttrop_nwp = cell_ttrop_nwp / cell_pixel_count
   cell_tpw_nwp = cell_tpw_nwp / cell_pixel_count
   cell_uth_nwp = cell_uth_nwp / cell_pixel_count
   cell_hght500_nwp = cell_hght500_nwp / cell_pixel_count
   cell_weasd_nwp = cell_weasd_nwp / cell_pixel_count
   cell_wnd_spd_10m_nwp = cell_wnd_spd_10m_nwp / cell_pixel_count
   cell_wnd_dir_10m_nwp = cell_wnd_dir_10m_nwp / cell_pixel_count

   ch4_rad_clr_nwp_mean = ch4_rad_clr_nwp_mean / cell_pixel_count
   ch5_rad_clr_nwp_mean = ch5_rad_clr_nwp_mean / cell_pixel_count

   cell_Pc_nwp = cell_Pc_nwp / cell_pixel_count
   cell_lwp_nwp = cell_lwp_nwp / cell_pixel_count
   cell_iwp_nwp = cell_iwp_nwp / cell_pixel_count
   cell_ncld_layers_nwp = cell_ncld_layers_nwp / cell_pixel_count

  elsewhere
   satzen_mean = missing_value_real4
   solzen_mean = missing_value_real4
   relaz_mean = missing_value_real4
   glint_mean = missing_value_real4
   zsfc_mean = missing_value_real4
   cell_gpi_235 = missing_value_real4
   cell_gpi_255 = missing_value_real4
   cell_gpi_275 = missing_value_real4
   cell_space_counts(1,:) = missing_value_real4
   cell_space_counts(2,:) = missing_value_real4
   cell_space_counts(3,:) = missing_value_real4
   cell_space_counts(4,:) = missing_value_real4
   cell_space_counts(5,:) = missing_value_real4
   cell_tsfc_nwp = missing_value_real4
   cell_tair_nwp = missing_value_real4
   cell_ttrop_nwp = missing_value_real4
   cell_tpw_nwp = missing_value_real4
   cell_uth_nwp = missing_value_real4
   cell_hght500_nwp = missing_value_real4
   cell_weasd_nwp = missing_value_real4
   cell_wnd_spd_10m_nwp = missing_value_real4
   cell_wnd_dir_10m_nwp = missing_value_real4

   !--- ncep cloudy radiances and products
   ch4_rad_clr_nwp_mean = missing_value_real4
   ch5_rad_clr_nwp_mean = missing_value_real4

   !--- nwp cloud props
   cell_Pc_nwp = missing_value_real4
   cell_iwp_nwp = missing_value_real4
   cell_lwp_nwp = missing_value_real4
   cell_ncld_layers_nwp = missing_value_real4

  end where

  !---- set dust frac to missing where aot1 is missing and over water
  where (solzen_mean > 70.0 .or. glint_mean < 40.0)
    dust_frac = missing_value_real4
  endwhere
  where(cell_sfc_type > 0)
     dust_frac = missing_value_real4
  endwhere


   !--- background sst
   where((cell_ocean_count >= count_thresh).and.(cell_sfc_type == 0))
     cell_sst_back = cell_sst_back / cell_ocean_count
   elsewhere
     cell_sst_back = missing_value_real4
   endwhere

  !---------------------------------------------------------------------------
  ! setting ch3a/b radiances to missing if cell has both
  ! --------------------------------------------------------------------------
  solzen_temp = acos(cossolzen_min_solar)/dtor   !limit expressed in degrees
  do icell = 1, ncells
     !--- test for ch3a transition and reset values
     if (cell_ch3a_on(icell) == 2) then
         cell_mean(3,:,icell) = missing_value_real4
         cell_std(3,:,icell) = missing_value_real4
         cell_mean(4,:,icell) = missing_value_real4
         cell_std(4,:,icell) = missing_value_real4
     endif

 enddo

end subroutine COMPUTE_GRIDCELL_ARRAYS


!------------------------------------------------------------------
! READ_GRIDCELL_SFCTYPE - reads in a precomputed file containing 
!    the dominant surface type for each grid-cell
!
! input
!   dlat - the resolution of the grid-cells
!   data_dir - the directory containing the sfc_data subdirectory 
!
! output - cell_sfc_type, passed only through shared memory
!
!-------------------------------------------------------------------
  subroutine READ_GRIDCELL_SFCTYPE(level3_format,ncells,dlat,data_dir)
   integer:: erstat,ios0
   integer, intent(in):: level3_format, ncells
   real(kind=real4), intent(in):: dlat
   character(len=*), intent(in):: data_dir
   character(len=128):: cell_sfc_file
   integer:: cell_sfc_file_lun

!--- determine file to use

  if (level3_format == 0) then
    if (dlat == 0.25) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_area_025.dat"
    elseif (dlat == 0.5) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_area_05.dat"
    elseif (dlat == 1.0) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_area_1.dat"
    elseif (dlat == 2.5) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_area_25.dat"
    else
         print *,EXE_PROMPT, MOD_PROMPT, "no sfc file available for this resolution, stopping"
         erstat = 43
         stop 43
         stop
    endif

  elseif (level3_format == 1) then
    if (dlat == 0.25) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_angle_025.dat"
    elseif (dlat == 0.5) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_angle_05.dat"
    elseif (dlat == 1.0) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_angle_1.dat"
    elseif (dlat == 2.5) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_angle_25.dat"
    else
         print *,EXE_PROMPT, MOD_PROMPT, "no sfc file available for this resolution, stopping"
         erstat = 43
         stop 43
         stop
    endif

  else
    print *, EXE_PROMPT, MOD_PROMPT, "gridcell format not valid in reading sfc type, stopping "
    erstat = 44
    stop 44
    stop

  endif


!------- read in grid-cell surface
    print *,EXE_PROMPT, MOD_PROMPT, "grid-cell surface file read ==>",trim(data_dir)//trim(cell_sfc_file)
    erstat = 0
    cell_sfc_file_lun = get_lun()
    open(unit=cell_sfc_file_lun,file=trim(cell_sfc_file),status="old",action="read", &
         access="direct",recl=ncells,iostat=ios0)
    if (ios0 /= 0) then
      erstat = 45
      print *, EXE_PROMPT, MOD_PROMPT, "error opening grid cell surface file, ios0 = ", ios0
      stop 45
    endif
    read(unit=cell_sfc_file_lun,rec=1,iostat=ios0) cell_sfc_type
    if (ios0 /= 0) then
      erstat = 46
      print *, EXE_PROMPT, MOD_PROMPT, "error reading grid cell surface file, ios0 = ", ios0
      stop 46
    endif
    close(unit=cell_sfc_file_lun)

 end subroutine READ_GRIDCELL_SFCTYPE


end module LEVEL3_ROUTINES
