!---------------------------------------------------------------
!@***h* PATMOSX/planck.f90
! 
! NAME
!   planck.f90
! 
! LOCATION
!   /patmosx/src/planck.f90
! 
! PURPOSE
!   this module holds the routine to do rapid Planck
!   computations using a table lookup approach.  
! 
! DESCRIPTION
!   this module holds the routine to do rapid Planck
!   computations using a table lookup approach.  This
!   has been shown to speed up CLAVR-x over using the
!   explicit planck function with expontentials.
!   This routine is a modication of that in GEOCAT
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   Aug 2006
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! EXTERNALS
!   public::
!   POPULATE_PLANCK_TABLES
!   PLANCK_RAD_FAST
!   PLANCK_TEMP_FAST
!   PLANCK_RAD
!   PLANCK_TEMP
!
!@*****
!------------------------------------------------------------------------
! PLANCK.F90 - this module holds the routine to do rapid Planck
!              computations using a table lookup approach.  This
!              has been shown to speed up CLAVR-x over using the
!              explicit planck function with expontentials
!              This routine is a modication of that in GEOCAT
!
! author: Andrew Heidinger
! Date: August 2006
!
! Modification History
!   August 31, 2006 - Created
!
! ROUTINES:
! public::
!   POPULATE_PLANCK_TABLES
!   PLANCK_RAD_FAST
!   PLANCK_TEMP_FAST
!   PLANCK_RAD
!   PLANCK_TEMP
!
!---------------------------------------------------------------
 module PLANCK
  use CONSTANTS
  use CALIBRATION_CONSTANTS
  use NUMERICAL_ROUTINES

  implicit none
   
  public:: POPULATE_PLANCK_TABLES,PLANCK_RAD_FAST,PLANCK_TEMP_FAST,&
           PLANCK_RAD,PLANCK_TEMP

!-- planck tables arrays
  integer, parameter, private:: nplanck = 161
  real (kind=int4), parameter, private:: T_planck_min = 180.0,  &
                                         delta_T_planck = 1.0
  real(kind=int4), dimension(nplanck), save, private:: B3b,B4,B5,T_planck

  contains

!-------------------------------------------------------------------------
! subroutine POPULATE_PLANCK_TABLES(a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5)
!
! compute planck function tables
!-------------------------------------------------------------------------
 subroutine POPULATE_PLANCK_TABLES(a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5)
  real, intent(in):: a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5
  integer:: i

  do i = 1, nplanck
    T_planck(i) = T_planck_min + (i-1)*delta_T_planck
    B3b(i) = c1*(nu_3b**3)/(exp((c2*nu_3b)/ &
              ((T_planck(i)-a1_3b)/a2_3b))-1.0)
    B4(i) = c1*(nu_4**3)/(exp((c2*nu_4)/ &
              ((T_planck(i)-a1_4)/a2_4))-1.0)
    B5(i) = c1*(nu_5**3)/(exp((c2*nu_5)/ &
              ((T_planck(i)-a1_5)/a2_5))-1.0)
  enddo

  end subroutine POPULATE_PLANCK_TABLES

!------------------------------------------------------------------
! function PLANCK_RAD_FAST(ichan, T, dB_dT) result(B)
!
! Subroutine to convert brightness temperature to radiance using a
! look-up table *Function that returns a scalar*.
!------------------------------------------------------------------
  function PLANCK_RAD_FAST(ichan, T, dB_dT) result(B)
    integer (kind=int4), intent(in) :: ichan
    real (kind=real4), intent(in) :: T 
    real (kind=real4), optional, intent(out) :: dB_dT
    real (kind=real4) :: B, dB_dT_tmp
    integer:: l

!--- compute planck emission for cloud temperature
     l = (T - T_planck_min)/delta_T_planck
     l = max(1,min(nplanck-1,l))

    if (ichan == 3) then 
      dB_dT_tmp = (B3b(l+1)-B3b(l))/(T_planck(l+1)-T_planck(l))
      B = B3b(l) + (T - T_planck(l)) * (dB_dT_tmp)
    elseif (ichan == 4) then 
      dB_dT_tmp = (B4(l+1)-B4(l))/(T_planck(l+1)-T_planck(l))
      B = B4(l) + (T - T_planck(l)) * (dB_dT_tmp)
    elseif (ichan == 5) then 
      dB_dT_tmp = (B5(l+1)-B5(l))/(T_planck(l+1)-T_planck(l))
      B = B5(l) + (T - T_planck(l)) * (dB_dT_tmp)
    else
      print *, "unsupported channel number in Planck Computation, stopping"
      stop
    endif

    if (present(dB_dT)) dB_dT = dB_dT_tmp

    return

  end function PLANCK_RAD_FAST

!------------------------------------------------------------------------
! function PLANCK_TEMP_FAST(ichan, B, dB_dT) result(T)
!
! Subroutine to convert radiance (B) to brightness temperature(T) using a
! look-up table *Function that returns a scalar*.
!------------------------------------------------------------------
  function PLANCK_TEMP_FAST(ichan, B, dB_dT) result(T)
    integer (kind=int4), intent(in) :: ichan
    real (kind=real4), intent(in) :: B 
    real (kind=real4), optional, intent(out) :: dB_dT
    real (kind=real4) :: T, dB_dT_tmp
    integer:: l

    if (ichan == 3) then 
      call locate(B3b,nplanck,B,l)
      l = max(1,min(nplanck-1,l))
      dB_dT_tmp = (B3b(l+1)-B3b(l))/(T_planck(l+1)-T_planck(l))
      T = T_planck(l) + (B - B3b(l)) / (dB_dT_tmp)
    elseif (ichan == 4) then 
      call locate(B4,nplanck,B,l)
      l = max(1,min(nplanck-1,l))
      dB_dT_tmp = (B4(l+1)-B4(l))/(T_planck(l+1)-T_planck(l))
      T = T_planck(l) + (B - B4(l)) / (dB_dT_tmp)
    elseif (ichan == 5) then 
      call locate(B5,nplanck,B,l)
      l = max(1,min(nplanck-1,l))
      dB_dT_tmp = (B5(l+1)-B5(l))/(T_planck(l+1)-T_planck(l))
      T = T_planck(l) + (B - B5(l)) / (dB_dT_tmp)
    else
      print *, "unsupported channel number in Planck Computation, stopping"
      stop
    endif

    if (present(dB_dT)) dB_dT = dB_dT_tmp

    return

  end function PLANCK_TEMP_FAST

!----------------------------------------------------------------------
! function PLANCK_RAD(ichan, T) result(B)
!----------------------------------------------------------------------

  function PLANCK_RAD(ichan, T) result(B)
    integer (kind=int4), intent(in) :: ichan
    real (kind=real4), intent(in) :: T 
    real (kind=real4) :: B

    if (ichan == 3) then 
       B = c1*(nu_3b**3)/(exp((c2*nu_3b)/((T-a1_3b)/a2_3b))-1.0)
    elseif (ichan == 4) then 
       B = c1*(nu_4**3)/(exp((c2*nu_4)/((T-a1_4)/a2_4))-1.0)
    elseif (ichan == 5) then 
       B = c1*(nu_5**3)/(exp((c2*nu_5)/((T-a1_5)/a2_5))-1.0)
    else
      print *, "unsupported channel number in PLANCK_RAD, stopping"
      stop
    endif

    return

  end function PLANCK_RAD
  
!----------------------------------------------------------------------
! function PLANCK_TEMP(ichan, B) result(T) 
!----------------------------------------------------------------------
  function PLANCK_TEMP(ichan, B) result(T)
    integer (kind=int4), intent(in) :: ichan
    real (kind=real4), intent(in) :: B 
    real (kind=real4) :: T

    if (ichan == 3) then 
       T = a1_3b + a2_3b * ((c2*nu_3b) / log( 1.0 + (c1*(nu_3b**3))/B))
    elseif (ichan == 4) then 
       T = a1_4 + a2_4 * ((c2*nu_4) / log( 1.0 + (c1*(nu_4**3))/B))
    elseif (ichan == 5) then 
       T = a1_5 + a2_5 * ((c2*nu_5) / log( 1.0 + (c1*(nu_5**3))/B))
    else
      print *, "unsupported channel number in PLANCK_TEMP, stopping"
      stop
    endif

    return

  end function PLANCK_TEMP

 end module PLANCK
