!$Id: process_avhrr_clavr.f90,v 1.76.2.1 2010/09/23 23:01:42 heidinger Exp $
  program PROCESS_AVHRR_CLAVR
!------------------------------------------------------------------
!@***h* PATMOSX/process_avhrr_clavr.f90
! 
! NAME
!   process_avhrr_clavr.f90
! 
! LOCATION
!   /patmosx/src/process_avhrr_clavr.f90
! 
! PURPOSE
!  This code generates the executable, clavrxorb, in the CLAVR-x processing
!  system
! 
! DESCRIPTION
!  This code serves as the NESDIS operational AVHRR cloud processing system
!  (CLAVR-x) and the AVHRR climate data generation system (PATMOS-x)
!
!  The input to this code is controlled through three mechanisms
!    1. command-line options  (type clavrxorb --help to see documentation)
!    2. a FILELIST - a list of level-1b files and directories (default name is
!      clavrxorb_file_list)
!    3. a OPTIONSLIST - a list of processing options (default is
!                     clavrxorb_default_options)
!
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   2004
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   Clouds from AVHRR Extended (CLAVR-x) 1b PROCESSING SOFTWARE Version 5.2
!
!   Overview of capabilities.
!    CLAVRXORB can
!       - use level-1b calibration or apply new calibration routines
!       - use level-1b geolocation or apply new geolocation routines
!       - generate pixel level cloud, aerosol and surface products
!       - write to a series of pixel-level hdf files
!       - write a level-3 file (gridded data for each orbit)
!       - write to optional binary diagnostic files
!
!  In general, CLAVRXORB uses global data arrays and structures to pass data
!
!  CLAVRXORB operates on all NESDIS Level-1b formats and on AAPP data
!
!  Note, comments the begin with "Marker" refer to flowchart
!
!  Web-page:  http://cimss.ssec.wisc.edu/clavr or http://cimss.ssec.wisc.edu/patmosx
!
!  Other points of contact:  William Straka, CIMSS, wstraka@ssec.wisc.edu
!
!@*****
!------------------------------------------------------------------------
! Clouds from AVHRR Extended (CLAVR-x) 1b PROCESSING SOFTWARE Version 5.2
!
! Copyright (C) 2004  Andrew K Heidinger
! National Oceanic and Atmospheric Administration
!
! This copyright pertains to all routines in the CLAVR-x system unless stated
!
! Source Code Name:
!  process_avhrr_clavr.f90
!
! Program Name:
!  PROCESS_AVHRR_CLAVR
!
! Executable Name:
!  CLAVRXORB
!
! Function:
!  This code generates the executable, clavrxorb, in the CLAVR-x processing
!  system
!
! Description:
!  This code serves as the NESDIS operational AVHRR cloud processing system
!  (CLAVR-x) and the AVHRR climate data generation system (PATMOS-x)
!
!  The input to this code is controlled through three mechanisms
!    1. command-line options  (type clavrxorb --help to see documentation)
!    2. a FILELIST - a list of level-1b files and directories (default name is
!      clavrxorb_file_list)
!    3. a OPTIONSLIST - a list of processing options (default is
!                     clavrxorb_default_options)
!
!  Overview of capabilities.
!    CLAVRXORB can
!       - use level-1b calibration or apply new calibration routines
!       - use level-1b geolocation or apply new geolocation routines
!       - generate pixel level cloud, aerosol and surface products
!       - write to a series of pixel-level hdf files
!       - write a level-3 file (gridded data for each orbit)
!       - write to optional binary diagnostic files
!
!  In general, CLAVRXORB uses global data arrays and structures to pass data
!
!  CLAVRXORB operates on all NESDIS Level-1b formats and on AAPP data
!
!  Note, comments the begin with "Marker" refer to flowchart
!
! Author: Andrew Heidinger, NOAA/NESDIS, Andrew.Heidinger@noaa.gov
!
! Web-page:  http://cimss.ssec.wisc.edu/clavr or http://cimss.ssec.wisc.edu/patmosx
!
! Other points of contact:  William Straka, CIMSS, wstraka@ssec.wisc.edu
!-------------------------------------------------------------------------
 
!*****************************************************************************
! Marker: ACCESS MODULES 
!******************************************************************************
   use CONSTANTS
   use HDF
   use AVHRR_PIXEL_COMMON
   use AVHRR_PIXEL_ROUTINES
   use AVHRR_DIAGNOSTIC_ROUTINES
   use AVHRR_CALNAV_ROUTINES
   use LEVEL2_ROUTINES
   use LEVEL3_ROUTINES
   use OISST_ANALYSIS
   use SURFACE_PROPERTIES
   use CLOUD_MASK_ROUTINES
   use CLOUD_MASK
   use CLOUD_TYPING
   use EQUAL_AREA_GRID  
   use EQUAL_ANGLE_GRID
   use CLOUD_HEIGHT_11_12_1DVAR
   use CLOUD_TAU_RE_SOLAR_MODULE
   use AEROSOL_PROPERTIES
   use HDF_PARAMS
   use LAND_SFC_PROPERTIES
   use CELL_HDF_ROUTINES_1
   use GFS
   use NCEP_REANALYSIS
   use RT_UTILITIES
   use NWP_COMMON
   use SCALING_PARAMETERS
   use SFC_EMISS
   use PLANCK
   use AVHRR_REPOSITION_ROUTINES
   use NAIVE_BAYESIAN_CLOUD_MASK
   use NAIVE_BAYESIAN_CLOUD_PHASE
 
   implicit none
 
!***********************************************************************
! Marker: DECLARE VARIABLES
!***********************************************************************
   integer(kind=int4), parameter:: nchan_clavr = 6
   integer(kind=int4), parameter:: nmask_clavr = 5
   integer(kind=int4), parameter:: ntype = 7
   integer(kind=int4)::  nword_clavr
   integer(kind=int4)::  nword_clavr_start
   integer:: file_list_lun                 !logical unit number for file_list
   integer:: iseg
   integer:: ilat
   integer:: ilon
   integer:: ivza
   integer(kind=int4):: nrec_header
   integer(kind=int4):: ncells_with_data
   integer(kind=int4):: i      !generic local index
   integer(kind=int4):: j      !generic local index
   integer(kind=int4):: ielem  !generic pixel (along scan) index
   integer(kind=int4):: iline  !generic line (across scan) index
   integer(kind=int2):: dummy_i2
   logical:: level1b_exists
   integer, parameter:: num_segment_time_points=13
   real(kind=real4), dimension(num_segment_time_points):: segment_time_point_seconds
   real(kind=real4) :: start_time_point_hours
   real(kind=real4) :: end_time_point_hours
   real(kind=real4) :: total_processing_start_time_hours
   real(kind=real4) :: total_processing_end_time_hours
   real(kind=real4) :: total_processing_time_seconds
   real(kind=real4) :: orbital_processing_start_time_hours
   real(kind=real4) :: orbital_processing_end_time_hours
   real(kind=real4) :: orbital_processing_time_seconds
   character(len=7):: l1b_ext
   character(len=120):: file_1b_temp
   character(len=120):: file_1b_full
   character(len=180):: system_string
   integer(kind=int4):: ncells, erstat
   real(kind=real4):: time_since_launch
   integer(kind=int4):: err_reposnx_flag
 
   integer(kind=real4):: ios
   integer(kind=real4):: file_number
   integer(kind=real4):: clevernav_read_status
   integer(kind=real4):: ierror_nwp
   integer(kind=real4):: iperiod16   
   character(len=3):: day_string
   character(len=100):: modis_white_sky_0_66_name
   character(len=100):: modis_white_sky_1_64_name
   character(len=100):: snow_mask_file_name
   character(len=100):: oisst_file_name
   character(*), parameter :: PROGRAM_NAME = 'CLAVRXORB'

   integer(kind=int4):: emiss_file_id = missing_value_int4
   integer(kind=int4):: coast_mask_id = missing_value_int4
   integer(kind=int4):: land_mask_id = missing_value_int4
   integer(kind=int4):: sfc_type_id = missing_value_int4
   integer(kind=int4):: volcano_mask_id = missing_value_int4
   integer(kind=int4):: surface_elev_id = missing_value_int4
   integer(kind=int4):: modis_alb_0_66_id = missing_value_int4
   integer(kind=int4):: modis_alb_1_64_id = missing_value_int4
   integer(kind=int4):: snow_mask_id = missing_value_int4
   TYPE(land_grid_description) :: coast_mask_str
   TYPE(land_grid_description) :: sfc_type_str
   TYPE(land_grid_description) :: land_mask_str
   TYPE(land_grid_description) :: volcano_mask_str
   TYPE(land_grid_description) :: surface_elev_str
   TYPE(land_grid_description) :: modis_alb_0_66_str
   TYPE(land_grid_description) :: modis_alb_1_64_str
   TYPE(land_grid_description) :: snow_mask_str


  !***********************************************************************
  ! Begin Executable Code
  !***********************************************************************
  print "(a,'<----------  Start of CLAVRXORB ---------->')",EXE_PROMPT

  !----------------------------------------------------------------------------
  ! Determine time of the start of all processing
  !----------------------------------------------------------------------------
  total_processing_start_time_hours = COMPUTE_TIME_HOURS()

  !------------------------------------------------------------------------------
  ! initialize previous date variables
  !-------------------------------------------------------------------------------
  month_prev = 0
  sc_id_new_prev = 0
  start_year_prev = 0
  start_day_prev = 0

  !---- Find the machine name to use in HDF global attribute
  call GET_MACHINE_NAME()

  !--- Determine machine byte ordering
  machine_byte_ordering = sym%LITTLE_ENDIAN
  dummy_i2 = 1
  if (ishft(ishft(dummy_i2,15),-15) == 1) then
    machine_byte_ordering = sym%LITTLE_ENDIAN
  else
    ! XLF doesn't like sym%BIG_ENDIAN.
    ! A conversion from type DERIVED TYPE is not permitted
    !Thus, change to 1, which is what sym%BIG_ENDIAN is
    machine_byte_ordering = 1 
  endif

!*************************************************************************
! Marker: Read and Quality Check User Defined Options
!*************************************************************************
  
    !-----------------------------------------------------------------
    !--- open and read default options file, if -no_default not on command line
    !--- then read in any comand line options, overriding clavrx_default_options.
    !--- If -no_default option used, then it will use default options unless
    !--- specified via command line options.
    !--- Added by WCS3
    !-----------------------------------------------------------------
 
    call READ_CLAVRXORB_COMMANDLINE_OPTIONS()

    call QC_CLAVRXORB_OPTIONS()
    
   !------------------------------------------------------------------------------
   ! determine read size here - should move to avhrr_input
   !-------------------------------------------------------------------------------
   j_min = 1
   j_max = num_scans_per_segment

!*************************************************************************
! Marker: Open hgh spatial resolution ancillary data files
!*************************************************************************
 
   !------------------------------------------------------------------------------
   !--- Read elevation data 
   !------------------------------------------------------------------------------
   if (read_surface_elevation /= 0) then
     print *, EXE_PROMPT,"Opening surface elevation file"
     surface_elev_str%sds_name = SURFACE_ELEV_SDS_NAME
     
     !--- read in which elevation type that is specified.
     if (read_surface_elevation == 1) then
        surface_elev_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"sfc_data/", &
                                      "GLOBE_1km_digelev.hdf", &
                                      grid_str=surface_elev_str)
     else ! low resolution, read_surface_elevation = 2
        surface_elev_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"sfc_data/", &
                                     "GLOBE_8km_digelev.hdf", &
                                      grid_str=surface_elev_str)
     endif


   endif

   !------------------------------------------------------------------
   ! Open coast mask file (Used GEOCAT 1km Coast mask)
   !------------------------------------------------------------------
   if (read_coast_mask == sym%YES) then
     print *, EXE_PROMPT,"Opening coast file"
     coast_mask_str%sds_name = COAST_MASK_SDS_NAME
     coast_mask_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"sfc_data/", &
                                      "coast_mask_1km.hdf", &
                                      grid_str=coast_mask_str)
   endif

   !------------------------------------------------------------------
   ! Open land surface type file
   !------------------------------------------------------------------
   print *, EXE_PROMPT, "Opening land surface type file"
   sfc_type_str%sds_name = SFC_TYPE_SDS_NAME

   if (read_hires_sfc_type == sym%YES) then
      sfc_type_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"/sfc_data/", &
                                     "gl-latlong-1km-landcover.hdf", &
                                      grid_str=sfc_type_str)
   else
      sfc_type_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"/sfc_data/", &
                                     "gl-latlong-8km-landcover.hdf", &
                                      grid_str=sfc_type_str)
   endif

   !------------------------------------------------------------------
   ! Open land mask file
   !------------------------------------------------------------------
    if (read_land_mask == sym%YES) then
      print *, EXE_PROMPT, "Opening land mask file"
      land_mask_str%sds_name = LAND_MASK_SDS_NAME
      land_mask_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"/sfc_data/", &
                                    "lw_geo_2001001_v03m.hdf", &
                                     grid_str=land_mask_str)
    endif
 
    !------------------------------------------------------------------
    ! Open volcano mask file
    !------------------------------------------------------------------
    if (read_volcano_mask == sym%YES) then

     print *, EXE_PROMPT, "Opening volcano mask file"

     volcano_mask_str%sds_name = VOLCANO_MASK_SDS_NAME
     volcano_mask_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"/sfc_data/", &
                                        "volcano_mask_1km.hdf", &
                                        grid_str=volcano_mask_str)
    endif

    !-----------------------------------------------------------------------
    !--- set up surface radiative properties 
    !-----------------------------------------------------------------------
    call SETUP_UMD_PROPS()
 
    !--------------------------------------------------------------------
    !--- setup clock corrections in memory
    !--------------------------------------------------------------------
    if (nav_flag == 2) then
      call SETUP_CLOCK_CORRECTIONS()
    endif

!**********************************************************************
!Marker:  Initialize Level-3 Arrays and Grid
!**********************************************************************
    !-----------------------------------------------------------------------
    !--- set up level3 and associated arrays
    !-----------------------------------------------------------------------
    if (level3_flag == 1) then  
        if (level3_format == 0) then
         call SETUP_EQUAL_AREA(dlat)   !maybe just pass ncells
         ncells = ncells_area
        elseif (level3_format == 1) then
         call SETUP_EQUAL_ANGLE(dlat)
         ncells = ncells_angle
        else
         erstat = 7
         print *, EXE_PROMPT, "level3 format choice ", level3_format,  &
                  " not valid, stopping"
         stop 7
        endif
     endif

     !-----------------------------------------------------------------------
     !--- allocate memory for level3 arrays
     !-----------------------------------------------------------------------
     if (level3_flag == 1) then
        call CREATE_GRIDCELL_ARRAYS(nchan_clavr,nmask_clavr,ntype,ncells)
     endif

    !-----------------------------------------------------------------------
    !--- read in grid-cell surface
    !-----------------------------------------------------------------------
    if (level3_flag == 1 ) then  
     call READ_GRIDCELL_SFCTYPE(level3_format,ncells,dlat,ancil_data_dir)
    endif
 
!----------------------------------------------------------------------
! Marker: Read file directories in FILELIST
!----------------------------------------------------------------------

  !--- print to screen which file list is used
  print *, EXE_PROMPT, "CLAVR-x FILE LIST FILE USED: ", trim(file_list)

  !--- open file containing list of level1b data to process
  file_list_lun = GET_LUN()
  open(unit=file_list_lun,file = trim(file_list),status="old",action="read",iostat=ios)
  if (ios /= 0) then
    print *, EXE_PROMPT, "Error opening clavrxorb_file_list, iostat = ", ios
    stop
  endif

  !--- read directories from clavrxorb_input_files
  read(unit=file_list_lun,fmt="(a)") dir_1b
  read(unit=file_list_lun,fmt="(a)") dir_1bx
  read(unit=file_list_lun,fmt="(a)") dir_nav_in
  read(unit=file_list_lun,fmt="(a)") dir_nav_out
  read(unit=file_list_lun,fmt="(a)") dir_cmr
  read(unit=file_list_lun,fmt="(a)") dir_sst
  read(unit=file_list_lun,fmt="(a)") dir_cld
  read(unit=file_list_lun,fmt="(a)") dir_obs
  read(unit=file_list_lun,fmt="(a)") dir_geo
  read(unit=file_list_lun,fmt="(a)") dir_rtm
  read(unit=file_list_lun,fmt="(a)") dir_ash
  read(unit=file_list_lun,fmt="(a)") dir_level2
  read(unit=file_list_lun,fmt="(a)") dir_level3

  !--- reset file counter
  file_number = 1
  cloud_phase_bayesian_flag = sym%NO

 !----------------------------------------------------------------------
 ! Marker: BEGIN LOOP OVER FILES
 !----------------------------------------------------------------------
 file_loop: do
 
    !----------------------------------------------------------------------
    ! Marker: READ IN CLAVRXORB_FILE_LIST AND SET FLAGS 
    !----------------------------------------------------------------------
    read(unit=file_list_lun,fmt="(a)",iostat=ios) file_1b_temp
    if (ios /= 0) then
      if (ios /= -1) then
        !-- non eof error
        erstat = 8
        print *, EXE_PROMPT, "Problem reading orbit names from control file, ios = ",ios
        stop 8
      else
        !-- end of orbits
        if (file_number == 1) then
          print *, EXE_PROMPT, "no orbits to process, stopping"
          stop
        endif
        exit
      endif
    endif

   !----------------------------------------------------------------------------
   ! Determine time of the start of the processing of this orbit
   !----------------------------------------------------------------------------
   orbital_processing_start_time_hours = COMPUTE_TIME_HOURS()

  !--------------------------------------------------------------
  ! Determine if this level-1b file can be opended, if not skip
  !--------------------------------------------------------------

!**********************************************************************
! Marker: Prepare to read Level-1b file
!**********************************************************************

  !-- see if level-1b file exists
  level1b_exists = file_exists(trim(dir_1b)//trim(file_1b_temp))
  if (level1b_exists .eqv. .FALSE.) then
            print *, EXE_PROMPT, "Level-1b file not found, skipping"
            cycle
  endif

  !--------------------------------------------------------------
  !  Determine based on the L1b file's extension if the input file
  !  is gzipped. Announce this fact thru a boolean var.
  !--------------------------------------------------------------
  l1b_ext = file_1b_temp(len_trim(file_1b_temp)-2:len_trim(file_1b_temp))

  !-- determine if gzipped
  if (trim(l1b_ext) == '.gz') then
     l1b_gzip = sym%YES
  else
     l1b_gzip = sym%NO
  endif

 !--- check if bzipped
  if (trim(l1b_ext) == 'bz2') then
     l1b_bzip2 = sym%YES
  else
     l1b_bzip2 = sym%NO
  endif
 
  if (l1b_gzip == sym%YES) then
     file_1b = file_1b_temp(1:len(trim(file_1b_temp))-3)
     system_string = "gunzip -c "//trim(dir_1b)//trim(file_1b_temp)// &
        " > temp_1b_file"
     call system(system_string)
     file_1b_temp = "temp_1b_file"
  elseif (l1b_bzip2 == sym%YES) then
     file_1b = file_1b_temp(1:len(trim(file_1b_temp))-4)
     system_string = "bunzip2 -c "//trim(dir_1b)//trim(file_1b_temp)// &
        " > temp_1b_file"
     call system(system_string)
     file_1b_temp = "temp_1b_file"
  else
     file_1b = trim(file_1b_temp)
     file_1b_temp = trim(dir_1b)//trim(file_1b_temp)
  endif

   !--- make a full file name
   if (l1b_gzip == sym%YES .or. l1b_bzip2 == sym%YES) then
     file_1b_full = trim(file_1b_temp)
   else
     file_1b_full = trim(dir_1b)//trim(file_1b)
   endif

   !--- print to screen the file name
   print *,EXE_PROMPT
   print *,EXE_PROMPT, "<------------- Next Orbit ---------------> "
   print *,EXE_PROMPT, "file name = ", trim(file_1b)


   !-------------------------------------------------------
   ! reset record counters
   !-------------------------------------------------------
   file_number = file_number + 1
  
   !-------------------------------------------------------
   ! Determine the type of level 1b file
   !-------------------------------------------------------
   call DETERMINE_FILE_TYPE(trim(file_1b_full),gac,klm,aapp,ver_1b,data_type,byte_swap_1b)

   !--- based on klm and gac flags, check for option consistency
      if ((klm == sym%NO) .and. (cloud_mask_1b == sym%YES)) then
        erstat = 2
        print *, EXE_PROMPT,  &
           "Error: using cloud mask from 1bx option not available for pre-klm data, stopping"
        stop 2
      endif
      if ((klm == sym%NO) .and. (bx_file_flag == sym%YES)) then
        erstat = 3
        print *,EXE_PROMPT, "Error: 1bx option not available for pre-klm data, stopping"
        stop 3
     endif

   !-------------------------------------------------------------------
   !-- based on file type (klm and gac), determine parameters needed
   !-- to read in header and data records for this orbit
   !------------------------------------------------------------------- 
    call DEFINE_1B_DATA(gac,klm,aapp,nrec_header,nword_clavr_start,nword_clavr)

   !-----------------------------------------------------------------------
   !--- set up pixel level arrays (size depends on gac flag)
   !-----------------------------------------------------------------------
   call CREATE_PIXEL_ARRAYS(nchan_clavr)

   !----------------------------------------------------------------------
   !-- reset thermal calibration stats for this orbit
   !----------------------------------------------------------------------
   valid_ref_cal = .false.
   valid_therm_cal = .false.
   spinup_new_therm_cal = .true.
   iprt = 0
   jcal = 0
   mean_T_prt = 0.0
   mean_space_count_1 = 0.0
   mean_space_count_2 = 0.0
   mean_space_count_3 = 0.0
   mean_space_count_4 = 0.0
   mean_space_count_5 = 0.0
   mean_bb_count_3 = 0.0
   mean_bb_count_4 = 0.0
   mean_bb_count_5 = 0.0
   ch1_gain_low = 0.0
   ch1_gain_high = 0.0
   ch2_gain_low = 0.0
   ch2_gain_high = 0.0
   ch3a_gain_low = 0.0
   ch3a_gain_high = 0.0
 
!-----------------------------------------------------------------------
!--- Compute the time stamp for use in all generated HDF output files
!-----------------------------------------------------------------------
   call HDF_TSTAMP()
   
!-----------------------------------------------------------------------
!--- reset level3 arrays for this orbit
!-----------------------------------------------------------------------
   if (level3_flag == 1) then
      call INITIALIZE_GRIDCELL_ARRAYS()
   endif
 
!*************************************************************************
! Marker:  READ IN HEADER AND DETERMINE SOME CONSTANTS
!*************************************************************************

   !-------------------------------------------------------------------
   !-- read in header
   !-------------------------------------------------------------------
   call READ_AVHRR_LEVEL1B_HEADER(trim(file_1b_full))

   !-----------------------------------------------------------------
   !-- determine if this data is from the AVHRR/1 series
   !-----------------------------------------------------------------
   call DETERMINE_AVHRR_1(start_year,klm,avhrr_1_flag)

   !------------------------------------------------------------------
   ! compute the month and day of month
   !------------------------------------------------------------------
   ileap = 0
   if (start_year == 1984 .or. start_year == 1988 .or. start_year == 1992 .or. start_year == 1996 .or.  &
      start_year == 2000 .or. start_year == 2004 .or. start_year == 2008 .or. start_year == 2012) then
     ileap = 1
   endif
 
   month = COMPUTE_MONTH(int(start_day, kind=int4), int(ileap, kind=int4))
   day_of_month = COMPUTE_DAY(int(start_day, kind=int4), int(ileap, kind=int4))

   !-------------------------------------------------------------------
   !-- define a sc_id that is unique value for each satellite
   !-------------------------------------------------------------------
   call COMPUTE_NEW_SPACECRAFT_ID(sc_id,sc_id_new,sc_id_char)
 
   !--------------------------------------------------------------
   !--- make an integer orbit_number from proc_block_id
   !--------------------------------------------------------------
    orbit_number = 0
    do i = 1,len(proc_block_id)
      orbit_number = orbit_number + (ichar(proc_block_id(i:i))-48)* &
       (10**(len(proc_block_id) - i))
    enddo
 
    !--------------------------------------------------------------
    !--- output orbit parameters to screen
    !--------------------------------------------------------------
    print *,EXE_PROMPT, "Orbit Number = ", orbit_number
    print *,EXE_PROMPT, "spacecraft number = ", sc_id_new
    print *,EXE_PROMPT, "data type = ", data_type
    print *,EXE_PROMPT, "level 1b version = ", ver_1b
    print *,EXE_PROMPT, "gac flag = ", gac
    print *,EXE_PROMPT, "klm flag = ", klm
    print *,EXE_PROMPT, "aapp flag = ", aapp
    print *,EXE_PROMPT, "avhrr/1 flag = ", avhrr_1_flag
    print *,EXE_PROMPT, "start year, day, time = ",start_year,start_day, &
             start_time/60.0/60.0/1000.0
    print *,EXE_PROMPT, "end year, day, time = ", end_year,end_day, &
             end_time/60.0/60.0/1000.0
    print *,EXE_PROMPT, "number of scans = ", num_scans

   !--------------------------------------------------------------
   !--- based on spacecraft id, set up constants
   !--------------------------------------------------------------
   if (sc_id_new < 10) then
    instr_const_file = trim(ancil_data_dir)//"avhrr_data/"// &
              "avhrr_"//char(sc_id_new+48)//"_instr.dat"
    algo_const_file = trim(ancil_data_dir)//"avhrr_data/"// &
              "avhrr_"//char(sc_id_new+48)//"_algo.dat"
   else
    instr_const_file = trim(ancil_data_dir)//"avhrr_data/"// &
             "avhrr_1"//char(sc_id_new - 10 + 48)//"_instr.dat" 
    algo_const_file = trim(ancil_data_dir)//"avhrr_data/"// &
             "avhrr_1"//char(sc_id_new - 10 + 48)//"_algo.dat" 
   endif
 
!*************************************************************************
! Marker:  READ IN AVHRR-SPECIFIC CONSTANTS
!*************************************************************************
   !--- read in AVHRR Instrument Constants from appropriate file
   call READ_AVHRR_INSTR_CONSTANTS(trim(instr_const_file))

   !--- read in AVHRR Algorithm Constants from appropriate file
   call READ_AVHRR_ALGO_CONSTANTS(trim(algo_const_file))

!*************************************************************************
! Marker:  Open non-static high spatial resolution ancillary data
!*************************************************************************

   !--------------------------------------------------------------------
   ! read in sst analysis file
   !--------------------------------------------------------------------

   !Set to use default options. This will be turned off if there is NO daily OISST data
   use_sst_anal = use_sst_anal_default

   if (use_sst_anal == sym%YES ) then  
          if (sst_anal_opt < 2) then
           if(start_year /= start_year_prev .or. start_day /= start_day_prev) then
                 oisst_file_name= get_oisst_map_filename(start_year,start_day, &
                                                 trim(oisst_data_dir), &
                                                 sst_anal_opt)
                if (oisst_file_name == "no_file") then
                   ! insert temp sst flag off here
                     use_sst_anal = sym%NO
                     print *, EXE_PROMPT, "Could not find daily OISST file"
                else
                     call READ_OISST_ANALYSIS_MAP(oisst_file_name)
                endif
           endif
          else
            print *, EXE_PROMPT, "NESDIS 100 km SST Option not yet available"
          endif
     endif

    !--------------------------------------------------------------------
    !--- 5 km surface emiss
    !--------------------------------------------------------------------
    if (use_seebor == sym%YES) then
      print *, EXE_PROMPT, "Reading in SeeBor Data for month = ", month
      call open_seebor_emiss(trim(ancil_data_dir)//"sfc_data", month, emiss_file_id)
    endif

    !----------------------------------------------------------------------
    ! Read in Modis White Sky Albedo Map appropriate for this day
    !----------------------------------------------------------------------
    if (modis_clr_alb_flag == sym%YES) then

     print *,EXE_PROMPT, "Opening modis clear albedo map"

    !--- determine 16 day period
     iperiod16 = 16 * ((start_day-1) / 16) + 1 

     if (iperiod16 < 10) then
         write(day_string,fmt="(i1)") iperiod16
         day_string = "00"//day_string
       elseif (iperiod16 < 100) then
         write(day_string,fmt="(i2)") iperiod16
         day_string = "0"//day_string
       else
         write(day_string,fmt="(i3)") iperiod16
     endif

     modis_white_sky_0_66_name = "AlbMap.WS.c004.v2.0.2004."// &
                                 day_string//".0.659_x4.hdf"
     modis_white_sky_1_64_name = "AlbMap.WS.c004.v2.0.2004."// &
                                 day_string//".1.64_x4.hdf"

     !--- Open modis white sky 0.66 um albedo
     modis_alb_0_66_str%sds_name = MODIS_ALB_0_66_SDS_NAME
     modis_alb_0_66_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"/sfc_data/", &
                                        trim(modis_white_sky_0_66_name), &
                                        grid_str=modis_alb_0_66_str)

     !--- Open modis white sky 1.66 um albedo
     modis_alb_1_64_str%sds_name = MODIS_ALB_1_64_SDS_NAME
     modis_alb_1_64_id = OPEN_LAND_SFC_HDF(trim(ancil_data_dir)//"/sfc_data/", &
                                        trim(modis_white_sky_1_64_name), &
                                        grid_str=modis_alb_1_64_str)
    endif

    !------------------------------------------------------------------
    ! Open snow mask file
    !------------------------------------------------------------------
    if (read_snow_mask == sym%YES) then

     failed_hires_snow_mask_flag = sym%NO

     snow_mask_str%sds_name = SNOW_MASK_SDS_NAME
     snow_mask_file_name = GET_SNOW_MAP_FILENAME(start_year,start_day, &
                                                 trim(snow_data_dir))
     if (snow_mask_file_name == "no_file") then
         failed_hires_snow_mask_flag = sym%YES
         print *, EXE_PROMPT, "Could not find snow mask file"
     else
         snow_mask_id = OPEN_LAND_SFC_HDF(trim(snow_data_dir), &
                                      snow_mask_file_name, &
                                      grid_str=snow_mask_str)
         if (snow_mask_id > 0) then
             failed_hires_snow_mask_flag = sym%NO
             print *, EXE_PROMPT, "Snow mask file opened successfully "
         else
             failed_hires_snow_mask_flag = sym%YES
             print *, EXE_PROMPT, "Snow mask file open failed "
         endif
      endif
    endif
    

!*************************************************************************
! Marker:  READ IN NWP DATA
!*************************************************************************

   !--- GFS
   if (nwp_flag == 1) then
     call READ_GFS_DATA(start_year, start_day, start_time, end_year, end_day, end_time, gfs_data_dir, ierror_nwp) 
   endif

   !--- NCEP Reanalysis
   if (nwp_flag == 2) then
    call READ_NCEP_REANALYSIS_DATA(start_year, start_day, start_time, end_year, end_day, end_time, ncep_data_dir)
   endif

   !---- if NWP is being read in, then proceeed in allocating RTM, NWP arrays
   if (nwp_flag /= 0) then

    !--- Quality control NWP fields
    call QC_NWP()

    !--- create temporary NWP vectors needed for RTM
    call CREATE_TEMP_NWP_VECTORS()

    !--- Compute mappings for NWP and RTM vertical coordinates
    call MAP_NWP_RTM()

    !--- allocate RTM structures
    call ALLOCATE_RTM(nlon_nwp,nlat_nwp)

    !--- read in RTM coefficients
    if (rtm_flag == 1) then
     call pfcomavhrr(ancil_data_dir,sc_id_char,*200)
    endif

  endif

  !------- store previous day and year to prevent reading of same data for next orbit
  start_year_prev = start_year
  start_day_prev = start_day

!*************************************************************************
! Marker: Populate or read in other lookup tables
!*************************************************************************

    !--- planck and aerosol tables
    if (sc_id_new /= sc_id_new_prev) then

      call  POPULATE_PLANCK_TABLES(a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5)
 
      if (aer_flag == sym%YES) then
        call READ_AER_CH123A_REF_LUTS(ancil_data_dir,sc_id_new)
      endif

      sc_id_new_prev = sc_id_new
 
    endif
 
    !--------------------------------------------------------------------
    ! read in Bayesian Cloud Mask
    !----------------------------------------------------------------------
    if (cloud_mask_bayesian_flag == sym%YES) then
       call READ_NAIVE_BAYES()
    endif
    if (cloud_phase_bayesian_flag == sym%YES) then
       call READ_NAIVE_BAYES_PHASE()
    endif

    !--------------------------------------------------------------
    !--- name and open and write header to 1bx file
    !--------------------------------------------------------------
    if (bx_file_flag == sym%YES) then
      call WRITE_HEADER_1BX(file_1b)
    endif

    !--- compute sun-earth distance
    sun_earth_distance = 1.0 - 0.016729*cos(0.9856*(start_day-4.0)*dtor)
 
    !--- compute time since launch needed for reflectance calibration
    time_since_launch = start_year + start_day / 365.25  - launch_date
 
    !--------------------------------------------------------------
    !-- Marker: Interpolate a clock correction for this orbit
    !--------------------------------------------------------------
    if (nav_flag == 2) then
      call INTERPOLATE_CLOCK_ERROR(start_year, start_day, start_time,  &
                                   end_year, end_day, end_time, &
                                   sc_id_new,timerr_seconds)
      print *, EXE_PROMPT, "Clock Error = ", timerr_seconds
    endif

   !------------------------------------------------------------------
   !--- Marker: Open diagnostic files - not must follow pfaast read
   !------------------------------------------------------------------
   if (diag_flag == 1) then
     call OPEN_DIAG_FILES()
   endif

!*************************************************************************
! READ IN DATA RECORDS AND PERFORM QUALITY CHECKING
!*************************************************************************

!-------------------------------------------------------------------------------------------
! Marker: Begin loop over orbit segments
!-------------------------------------------------------------------------------------------
print *,EXE_PROMPT
print *,EXE_PROMPT, "Started Processing All Orbital Segments"


!--- compute number of segments in this orbit 
num_segments = num_scans / num_scans_per_segment + 1

!--- initialize time counters
segment_time_point_seconds = 0.0

segment_loop: do iseg = 1,num_segments

    !--- reset pixel arrays to missing for this segment
    call RESET_PIXEL_ARRAYS_TO_MISSING()

    !-----------------------------------------------------------------
    !---- Marker: Read AVHRR level-1b data
    !-----------------------------------------------------------------
    start_time_point_hours = COMPUTE_TIME_HOURS()

    call READ_AVHRR_LEVEL1B_DATA(trim(file_1b_full), &
             klm,aapp,therm_cal_1b,&
             time_since_launch,nrec_header,iseg)

    end_time_point_hours = COMPUTE_TIME_HOURS()

    !--- update time summation for level-1b processing
    segment_time_point_seconds(1) =  segment_time_point_seconds(1) + &
              60.0*60.0*(end_time_point_hours - start_time_point_hours)



   !--- check to see that some data was read in
   if (num_scans_read == 0) then
      print *, EXE_PROMPT, "Error: no scans read in, exiting segment processing loop"
      exit
   endif

   !*******************************************************************
   ! Marker: Definition of pixel hdf files 
   !  if first segment, define pixel hdf files
   ! this has to be here because cal coefficients are only known
   ! after data has been read in and calibrated
   !*******************************************************************
   if (iseg == 1) then
     num_scans_hdf = 0

     call DEFINE_HDF_FILE_STRUCTURES(num_scans, &
                              dir_cmr, &
                              dir_nav_out, &
                              dir_sst, &
                              dir_cld, &
                              dir_obs, &
                              dir_geo, &
                              dir_rtm, &
                              dir_ash, &
                              dir_level2, &
                              file_1b, &
                              cmr_file_flag, &
                              nav_file_flag,&
                              sst_file_flag, &
                              cld_file_flag, &
                              obs_file_flag, &
                              geo_file_flag, &
                              rtm_file_flag, &
                              ash_file_flag, &
                              level2_file_flag, &
                              c1,c2,a1_3b,a2_3b,nu_3b, &
                              a1_4,a2_4,nu_4,a1_5,a2_5,nu_5,solar_3b_nu,&
                              sun_earth_distance,therm_cal_1b, &
                              ref_cal_1b,nav_flag,use_sst_anal, &
                              sst_anal_opt,cloud_type_ir_flag,modis_clr_alb_flag,nwp_flag, &
                              ch1_gain_low,ch1_gain_high, &
                              ch1_switch_count_cal,ch1_dark_count_cal, &
                              ch2_gain_low,ch2_gain_high, &
                              ch2_switch_count_cal,ch2_dark_count_cal, &
                              ch3a_gain_low,ch3a_gain_high, &
                              ch3a_switch_count_cal,ch3a_dark_count_cal, &
                              start_year,end_year,start_day,end_day,&
                              start_time,end_time)
   endif

   !----------------------------------------------------------------------
   ! check to see if this segment fits desired subset if subsetting
   !----------------------------------------------------------------------
   if (subset_pixel_hdf_flag == sym%YES) then

     !--- check to see if data fell within solar zenith angle bounds
     if ((minval(solzen(num_pix/2,:)) > solzen_max_limit) .or.   &
         (maxval(solzen(num_pix/2,:)) < solzen_min_limit)) then
        cycle   !go to next segment in the segment_loop
      endif

     !--- check to see if in latitude range (using diag values)
     if ((minval(lat_1b(num_pix/2,:)) > lat_max_diag) .or.   &
         (maxval(lat_1b(num_pix/2,:)) < lat_min_diag)) then
       cycle   !go to next segment in the segment_loop
     endif

   endif

   !*******************************************************************
   ! Marker: Recompute geolocation
   !*******************************************************************

   !--- use level 1b navigation
   if (nav_flag == 0) then
    lat = lat_1b
    lon = lon_1b
   endif

   !---  Repositioning
   if (nav_flag == 2) then
       if ((timerr_seconds /= missing_value_real4) .and. &
           (timerr_seconds /= 0.0)) then 
             call REPOSITION_FOR_CLOCK_ERROR(j_min,num_scans_read, &
                                  timerr_seconds,err_reposnx_flag)
       else
            lat = lat_1b
            lon = lon_1b
       endif
   endif

   !-----------------------------------------------------------------------
   !--- Marker: Compute some fundamental pixel-level arrays
   !-----------------------------------------------------------------------

   !--- compute time (local and utc) variables for this segment
   call CONVERT_TIME(j_min,num_scans_read)

   !--- set values in low quality scans to missing
   call SET_LOW_QUALITY_SCANS_TO_MISSING(j_min,num_scans_read)

   !--- compute some common used pixel arrays (ie bt45 = bt4 - b5)
   call COMPUTE_PIXEL_ARRAYS(j_min,num_scans_read)

   !*******************************************************************
   ! Marker: Interpolate ancillary data to each pixel in this segment
   !*******************************************************************
   start_time_point_hours = COMPUTE_TIME_HOURS()

   !--- surface emissivity
   if (use_seebor == sym%YES) then

     !--- surface emissivity in channel 3b
     call READ_SEEBOR_EMISS(emiss_file_id, 3, lat, lon, space_mask, sfc_emiss_3b)

     !--- surface emissivity in channel 4
     call READ_SEEBOR_EMISS(emiss_file_id, 4, lat, lon, space_mask, sfc_emiss_4)

     !--- surface emissivity in channel 5
     call READ_SEEBOR_EMISS(emiss_file_id, 5, lat, lon, space_mask, sfc_emiss_5)

   endif

   !--- map each each into correct NWP cell
   if (nwp_flag /= 0) then
    call MAP_PIXEL_NWP(j_min,num_scans_read)
   endif

   !--- mandatory fields - check for substitution of bad_pixel for space 
    call READ_LAND_SFC_HDF(sfc_type_id, sfc_type_str, lat, lon, space_mask, sfc_type)

   if (read_surface_elevation /= 0) then

    !--- read the high res data
    call READ_LAND_SFC_HDF(surface_elev_id, surface_elev_str, lat, lon, space_mask, two_byte_temp)

    !---  convert to a real number
    zsfc = real(two_byte_temp,kind=real4)

    !--- values over water are missing, set to zero
    where(zsfc == missing_value_real4)
            zsfc = 0.0
    endwhere

   else

     zsfc = 0.0

   endif


   !--- read coast mask
   if (read_coast_mask == sym%YES) then
     call READ_LAND_SFC_HDF(coast_mask_id, coast_mask_str, lat, lon, space_mask, coast)
   endif

   !--- read land mask
   if (read_land_mask == sym%YES) then
    call READ_LAND_SFC_HDF(land_mask_id, land_mask_str, lat, lon, space_mask, land)
   endif

   !--- read volcano mask
   if (read_volcano_mask == sym%YES) then
    call READ_LAND_SFC_HDF(volcano_mask_id, volcano_mask_str, lat, lon, space_mask, volcano_mask)
   endif

   !--- read snow mask
   if (read_snow_mask == sym%YES .and. failed_hires_snow_mask_flag == sym%NO) then
    call READ_LAND_SFC_HDF(snow_mask_id, snow_mask_str, lat, lon, space_mask, snow_hires)
   endif

   !--- define binary land and coast masks (yes/no) from land and coast flags
   call COMPUTE_BINARY_LAND_COAST_MASKS(j_min,num_scans_read)   

   !--- interpolate sst analyses to each pixel
   if (use_sst_anal == 1) then
     if (sst_anal_opt < 2) then
      call GET_PIXEL_SST_ANALYSIS(j_min,num_scans_read)
     endif
   endif

   !--- compute a coast mask relative to nwp data
   if (nwp_flag /= 0) then
    call COMPUTE_COAST_MASK_NWP(j_min,num_scans_read)
   endif

   !--- compute the channel 3b albedo arrays
   call NORMALIZE_REFLECTANCES(sun_earth_distance,j_min,num_scans_read)

   !--- compute the channel 3b albedo arrays
   call CH3B_ALB(sun_earth_distance,j_min,num_scans_read)

   !--- compute pixel level snow map based on all ancillary data
   if (nwp_flag /= 0) then
     call COMPUTE_SNOW_FIELD(j_min,num_scans_read)
   endif

   !--- interpolate white sky albedoes to each pixel in segment
   if (modis_clr_alb_flag == sym%YES) then
      if (minval(solzen) < 90.0) then
         call READ_MODIS_WHITE_SKY_ALBEDO(modis_alb_0_66_id, modis_alb_1_64_id, &
                                          modis_alb_0_66_str, modis_alb_1_64_str)
      endif
   endif

   !--- interpolate surface type field to each pixel in segment
   call GET_PIXEL_SFC_EMISS_FROM_SFC_TYPE(j_min,num_scans_read)   

   !--- Produce pixel level surface temperature arrays
   if (sst_flag > 0) then
        call OPERATIONAL_SST(j_min,num_scans_read)
   endif

   end_time_point_hours = COMPUTE_TIME_HOURS()
                        
   !--- update time summation for level-1b processing
   segment_time_point_seconds(2) =  segment_time_point_seconds(2) + &
                60.0*60.0*(end_time_point_hours - start_time_point_hours)

   !*******************************************************************
   ! Marker: Compute nwp mapping and rtm values for each pixel in segment
   !*******************************************************************

  !--- needs an NWP to run.
  if (nwp_flag /= 0) then

      start_time_point_hours = COMPUTE_TIME_HOURS()
      !-- temporally interp skin temp for each segment (only ncep reanalysis)
      if (nwp_flag == 2) then
        call TEMPORAL_INTERP_TMPSFC_NWP(scan_time(j_min), scan_time(j_min+num_scans_read-1))
      endif
      !--- compute a surface temperature from the NWP
      call COMPUTE_TSFC_NWP(1,num_pix,j_min,num_scans_read,smooth_nwp)

      !--- compute pixel-level rtm parameters
      call GET_PIXEL_NWP_RTM(j_min,num_scans_read)

      !--- compute a surface temperature - needs rtm results
      call COMPUTE_TSFC(j_min,num_scans_read)

      !--- apply atmospheric correction - needs rtm results
      call ATMOS_CORR(sc_id_new,nwp_flag,j_min,num_scans_read)

      end_time_point_hours = COMPUTE_TIME_HOURS()
      segment_time_point_seconds(3) =  segment_time_point_seconds(3) + &
           60.0*60.0*(end_time_point_hours - start_time_point_hours)
  endif

  !*******************************************************************
  ! Marker: Spatial metrics processing
  !*******************************************************************
  start_time_point_hours = COMPUTE_TIME_HOURS()

  !--- populate needed spatial uniformity arrays
  call COMPUTE_SPATIAL_UNIFORMITY(j_min,num_scans_read)

  !--- apply median filter to bt4
  call COMPUTE_MEDIAN_SEGMENT(bt4,bad_pixel_mask, 1, &
                              1, num_pix,j_min,j_min+num_scans_read-1, &
                              bt4_median_3x3,  &
                              bt4_std_median_3x3)

  !--- apply median filter to bt3b
  call COMPUTE_MEDIAN_SEGMENT(bt3b,bad_pixel_mask, 1, &
                              1, num_pix,j_min,j_min+num_scans_read-1, &
                              bt3b_median_3x3,  &
                              bt3b_std_median_3x3)

  !--- apply median filter to ems3b
  call COMPUTE_MEDIAN_SEGMENT(ems3b,bad_pixel_mask, 1, &
                              1, num_pix,j_min,j_min+num_scans_read-1, &
                              ems3b_median_3x3,  &
                              ems3b_std_median_3x3)

   !--- populate local radiative center arrays
   if (lrc_flag == sym%YES) then

     temp_mask = sym%YES

     call GRADIENT_MEANDER(emiss_11_tropo_rtm(:,j_min:num_scans_read-j_min+1),  &
                     num_pix, j_min, num_scans_read,  &
                     0,     &        !0 = uphill, 1 = downhill
                     temp_mask(:,j_min:num_scans_read-j_min+1), &
                     0.05,  &        !min valid
                     0.70,  &        !max valid
                     0, &            !number of gradient direction changes
                     i_lrc(:,j_min:num_scans_read-j_min+1), &
                     j_lrc(:,j_min:num_scans_read-j_min+1))

   end if

   end_time_point_hours = COMPUTE_TIME_HOURS()
   segment_time_point_seconds(4) =  segment_time_point_seconds(4) + &
            60.0*60.0*(end_time_point_hours - start_time_point_hours)

  !--- compute the glint mask
  call COMPUTE_GLINT(j_min,num_scans_read)

  !--- populate the ancillary data flags in cloud mask
  !--- only applied if nwp/rtm information available
   if (nwp_flag > 0) then
     call COMPUTE_DATA_MASK(j_min,num_scans_read)
   endif

   !*******************************************************************
   ! Marker: Generate pixel-level products
   !*******************************************************************

   !--- compute single gain counts for calibration studies
   call CONVERT_COUNTS_SINGLE_GAIN(klm,j_min,num_scans_read)

   !---- pixel level aerosol
    if (aer_flag == sym%YES) then

         start_time_point_hours = COMPUTE_TIME_HOURS()

         call PIXEL_AER_RET_OCEAN(j_min,num_scans_read)

         end_time_point_hours = COMPUTE_TIME_HOURS()
         segment_time_point_seconds(5) =  segment_time_point_seconds(5) + &
                 60.0*60.0*(end_time_point_hours - start_time_point_hours)
    endif

   !--- only apply cloud mask and type routines if nwp/rtm information available
   if (nwp_flag > 0) then

    start_time_point_hours = COMPUTE_TIME_HOURS()

    call FIND_OPAQUE_LEVEL(j_min,num_scans_read)

    !--- cloud mask
    if (cloud_mask_1b == sym%NO) then
     if (cloud_mask_bayesian_flag == sym%YES) then
       call CLOUD_MASK_NAIVE_BAYES(j_min,num_scans_read,iseg)
       call COMPUTE_CLOUD_FRACTION_3x3(j_min,num_scans_read)
     else
       call CLOUD_MASK_3x3(sc_id_new,j_min,num_scans_read)
       call COMPUTE_2BIT_CLOUD_MASK(j_min,num_scans_read)
       call COMPUTE_2BIT_CLOUD_MASK_IR(j_min,num_scans_read)
       call COMPUTE_PROBABLY_CLOUDY(j_min,num_scans_read)
       if (prob_clear_res_flag == sym%YES) then
        call COMPUTE_PROBABLY_CLEAR_RESTORAL(j_min,num_scans_read)
        call COMPUTE_PROBABLY_CLOUDY_RESTORAL(j_min,num_scans_read)
       endif
     endif
     clavr_status = 1

     if (cloud_mask_bayesian_flag == sym%YES) then
       cld_mask = cld_mask_bayes
     endif

    else
     cld_mask = cld_mask_1b
    endif

    end_time_point_hours = COMPUTE_TIME_HOURS()
    segment_time_point_seconds(6) =  segment_time_point_seconds(6) + &
            60.0*60.0*(end_time_point_hours - start_time_point_hours)

    !--- cloud type
    start_time_point_hours = COMPUTE_TIME_HOURS()
    if (avhrr_1_flag == sym%NO) then  
      call CLOUD_TYPE(j_min,num_scans_read)
      call COMPUTE_INOUE_CLOUD_TYPE(j_min,num_scans_read)
    else
      call CLOUD_TYPE_AVHRR1(j_min,num_scans_read)
    endif

    if (lrc_flag == sym%YES) then
      call CLOUD_RETYPE(j_min,num_scans_read,cld_type)
    endif

    !--- bayesian cloud phase
    if (cloud_phase_bayesian_flag == sym%YES) then
       call CLOUD_PHASE_NAIVE_BAYES(j_min,num_scans_read,iseg)
       if (lrc_flag == sym%YES) then
          call CLOUD_RETYPE(j_min,num_scans_read,cld_type_bayes)
       endif
    endif

    !--- compute deep convective cloud mask
    call COMPUTE_DCC_MASK(j_min,num_scans_read)

    end_time_point_hours = COMPUTE_TIME_HOURS()
    segment_time_point_seconds(7) =  segment_time_point_seconds(7) + &
            60.0*60.0*(end_time_point_hours - start_time_point_hours)

  endif

   if (cld_flag == sym%YES) then

    start_time_point_hours = COMPUTE_TIME_HOURS()

    start_time_point_hours = COMPUTE_TIME_HOURS()
    if (avhrr_1_flag == sym%NO) then    !currently, no avhrr_1_flag algorithm

      !--- cloud temperature and emissivity from split-window approach
      call SPLIT_WIN_CLOUD_BETA(j_min,num_scans_read)

    else

      !--- cloud temperature and emissivity from opaque approach
      call OPAQUE_CLOUD_HEIGHT(j_min,num_scans_read)

    endif

    end_time_point_hours = COMPUTE_TIME_HOURS()
    segment_time_point_seconds(8) =  segment_time_point_seconds(8) + &
            60.0*60.0*(end_time_point_hours - start_time_point_hours)

    !--- cloud optical depth and effective radius from vis/nir approach
    start_time_point_hours = COMPUTE_TIME_HOURS()
    call CLOUD_TAU_RE_SOLAR(j_min,num_scans_read,iseg,num_segments)

    end_time_point_hours = COMPUTE_TIME_HOURS()
    segment_time_point_seconds(9) =  segment_time_point_seconds(9) + &
            60.0*60.0*(end_time_point_hours - start_time_point_hours)

   endif

    !--- Non-cloud Detection
    if (aer_flag == sym%YES) then

         start_time_point_hours = COMPUTE_TIME_HOURS()

         call DUST_DETECTION_ALGORITHM(j_min,num_scans_read)
         call SMOKE_DETECTION_ALGORITHM(j_min,num_scans_read)

         end_time_point_hours = COMPUTE_TIME_HOURS()
         segment_time_point_seconds(10) =  segment_time_point_seconds(10) + &
                   60.0*60.0*(end_time_point_hours - start_time_point_hours)
    endif
    
    !--- radiative flux parameters
    if (erb_flag == sym%YES) then
      if (avhrr_1_flag == sym%NO) then    !currently, no avhrr_1_flag algorithm
         call COMPUTE_ERB(j_min,num_scans_read)
      endif
    endif

    if (nwp_flag > 0) then

      !--- assign clear sky quality flags
      call ASSIGN_CLEAR_SKY_QUALITY_FLAGS(j_min,num_scans_read)

      !--- pack cloud test bits into bytes for output
      call PACK_CLOUD_MASK_BYTES(j_min,num_scans_read)

    endif

   !--- generated cloud masked sst field
   call COMPUTE_MASKED_SST(j_min,num_scans_read)

   !*******************************************************************
   ! Marker: Write to output files (pixel-level)
   !*******************************************************************
    start_time_point_hours = COMPUTE_TIME_HOURS()

    call WRITE_PIXEL_HDF_RECORDS(cmr_file_flag, &
                                 nav_file_flag, &
                                 sst_file_flag, &
                                 cld_file_flag, &
                                 obs_file_flag, &
                                 geo_file_flag, &
                                 rtm_file_flag, &
                                 ash_file_flag, &
                                 level2_file_flag, &
                                 iseg)

    end_time_point_hours = COMPUTE_TIME_HOURS()
    segment_time_point_seconds(12) =  segment_time_point_seconds(12) + &
                   60.0*60.0*(end_time_point_hours - start_time_point_hours)

   !*******************************************************************
   ! Marker: Write to diagnostic output files
   !*******************************************************************
    if (diag_flag == sym%YES) then
      do iline = j_min, j_min + num_scans_read - 1

         if ((lat(num_pix/2,iline) >= lat_min_diag) .and.  &
             (lat(num_pix/2,iline) <= lat_max_diag) .and. &
             ((ascend(iline) == asc_flag_diag) .or. (asc_flag_diag == -1))) then 
             call WRITE_TO_DIAG_FILES(iline) 

         endif 

      end do

    endif

   !*************************************************************************
   ! Marker: Update Level-3 arrays based on pixel-level results
   !*************************************************************************
    if (level3_flag == sym%YES) then

       start_time_point_hours = COMPUTE_TIME_HOURS()

       call COMPILE_GRIDCELL_ARRAYS(level3_format,nmask_clavr,nwp_flag,j_min,num_scans_read)

       end_time_point_hours = COMPUTE_TIME_HOURS()
       segment_time_point_seconds(13) =  segment_time_point_seconds(13) + &
                   60.0*60.0*(end_time_point_hours - start_time_point_hours)
    endif

   !*************************************************************************
   ! Marker: RTM Structure Memory Deallocation
   !*************************************************************************

   !--- deallocate rtm profile arrays (only do if NWP is used)
   if (nwp_flag /= 0) then

    do iline = j_min, j_min + num_scans_read - 1
     do ielem = 1, num_pix
          ilon = i_nwp(ielem,iline)
          ilat = j_nwp(ielem,iline)
          ivza = ivza_rtm(ielem,iline)
          !--- check to see if ilon and ilat are valid, if deallocate
          if ((ilon >= 0) .and. (ilon <= nlon_nwp) .and. &
              (ilat >= 0) .and. (ilat <= nlat_nwp) .and. &
              (ivza >= 0) .and. (ivza <= rtm_nvzen)) then

           if (rtm(ilon,ilat)%d(ivza_rtm(ielem,iline))%flag == sym%YES) then
             call DEALLOCATE_RTM_VARS(ilon,ilat,ivza_rtm(ielem,iline))
           endif

          endif
       enddo
    enddo

   !--- deallocate rtm cells
    do iline = j_min, j_min + num_scans_read - 1
     do ielem = 1, num_pix

          ilon = i_nwp(ielem,iline)
          ilat = j_nwp(ielem,iline)

          !--- check to see if ilon and ilat are valid, if deallocate
          if ((ilon >= 0) .and. (ilon <= nlon_nwp) .and. &
              (ilat >= 0) .and. (ilat <= nlat_nwp)) then

            if (rtm(ilon,ilat)%flag == sym%YES) then
             call DEALLOCATE_RTM_CELL(ilon,ilat)
            endif

          endif
       enddo
    enddo
    
   endif

   !--- screen output to mark progress through orbit
   print *, EXE_PROMPT, "processed segment #",iseg," scanlines = ", scan_number(j_min),  &
                        scan_number(num_scans_read)

!*************************************************************************
! Marker: End of loop over orbital segments
!*************************************************************************
end do segment_loop

print *,EXE_PROMPT, "Finished Processing All Orbital Segments"
print *,EXE_PROMPT


!*************************************************************************
! Marker: Compute final level-3 arrays and write to Level-3 files
!*************************************************************************
 if (level3_flag == sym%YES) then
!---- compute final grid-cell stats
    call COMPUTE_GRIDCELL_ARRAYS(ncells, &
                             orbit_number,start_day,end_day, &
                             start_year,end_year, &
                             nchan_clavr,nmask_clavr,sc_id_new,ncells_with_data)
                                                                                                                 
  if (ncells_with_data > 0) then
    call WRITE_GRIDCELL_DATA(ncells,ncells_with_data, &
        dir_level3,file_1b,level3_format,dlat, &
        c1,c2,a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5,a2_5,nu_5, &
        solar_3b_nu,sun_earth_distance,therm_cal_1b, ref_cal_1b, &
        nav_flag,use_sst_anal,sst_anal_opt,cloud_type_ir_flag,modis_clr_alb_flag,nwp_flag, &
        ch1_gain_low,ch1_gain_high,ch1_switch_count_cal,ch1_dark_count_cal, &
        ch2_gain_low,ch2_gain_high,ch2_switch_count_cal,ch2_dark_count_cal, &
        ch3a_gain_low,ch3a_gain_high,ch3a_switch_count_cal,ch3a_dark_count_cal,&
        start_year,end_year,start_day,end_day,&
        start_time,end_time,data_comp_flag,timerr_seconds,sc_id_new)
  else
    print *, EXE_PROMPT, "Error, no gridcells with data, not creating grid-cell output ",  &
              ncells_with_data
  endif
                                                                                                                 
 endif
                                                                                                                 
!*************************************************************************
! Marker: Close output pixel-level files
!*************************************************************************

    !--- close pixel level hdf files
    call CLOSE_PIXEL_HDF_FILES(cmr_file_flag, &
                               nav_file_flag, &
                               sst_file_flag, &
                               cld_file_flag, &
                               obs_file_flag, &
                               geo_file_flag, &
                               rtm_file_flag, &
                               ash_file_flag, &
                               level2_file_flag)

   !--- close diagnostic files
   if (diag_flag == sym%YES) then
     call CLOSE_DIAG_FILES()
     print *, EXE_PROMPT, "Number of scans written to diagnostic files = ", j_diag_output
   endif

  !*************************************************************************
  ! Marker: Close non-static ancillary data files
  !*************************************************************************

  !--- Close high resolution surface emissivity maps, if needed
  if (use_seebor == sym%YES) then
    call close_seebor_emiss(emiss_file_id)
  endif

  !--- Close modis white sky albedo maps
  if (modis_clr_alb_flag == sym%YES) then
    call CLOSE_LAND_SFC_HDF(modis_alb_0_66_id)
    call CLOSE_LAND_SFC_HDF(modis_alb_1_64_id)
  endif

  !*************************************************************************
  !Marker: Deallocate remaining arrays
  !*************************************************************************

   !--- main RTM structures and NWP arrays
   if (nwp_flag > 0) then
    call DESTROY_NWP_ARRAYS()
    call DESTROY_TEMP_NWP_VECTORS()
    call DEALLOCATE_RTM()
   endif

   !--- Deallocate memory from pixels arrays
   call DESTROY_PIXEL_ARRAYS()

   !--- remove uncompressed file
       if (l1b_gzip == sym%YES .or. l1b_bzip2 == sym%YES) then 
          call system("rm "//trim(file_1b_temp))
       endif

   !--- Determine time of the end of the processing of this orbit
   orbital_processing_end_time_hours = COMPUTE_TIME_HOURS()
   orbital_processing_time_seconds = 60.0*60.0*(orbital_processing_end_time_hours -  &
                                                orbital_processing_start_time_hours)

  !--- diagnostic screen output
   print *, EXE_PROMPT, "<----- Timing Results ----->"
   print *, EXE_PROMPT, "Time for Level-1b Processing (sec) = ", segment_time_point_seconds(1)
   print *, EXE_PROMPT, "Time for Ancil. Data Processing (sec) = ", segment_time_point_seconds(2)
   print *, EXE_PROMPT, "Time for RTM Processing (sec) = ", segment_time_point_seconds(3)
   print *, EXE_PROMPT, "Time for Spatial Processing (sec) = ", segment_time_point_seconds(4)
   print *, EXE_PROMPT, "Time for Aerosol Retrieval (sec) = ", segment_time_point_seconds(5)
   print *, EXE_PROMPT, "Time for Cloud Mask (sec) = ", segment_time_point_seconds(6)
   print *, EXE_PROMPT, "Time for Cloud Type (sec) = ", segment_time_point_seconds(7)
   print *, EXE_PROMPT, "Time for Cloud Height (sec) = ", segment_time_point_seconds(8)
   print *, EXE_PROMPT, "Time for Cloud Opt/Micro (sec) = ", segment_time_point_seconds(9)
   print *, EXE_PROMPT, "Time for Dust/Smoke Detection (sec) = ", segment_time_point_seconds(10)
   print *, EXE_PROMPT, "Time for Volcanic Ash (sec) = ", segment_time_point_seconds(11)
   print *, EXE_PROMPT, "Time for Pixel-HDF Write (sec) = ", segment_time_point_seconds(12)
   print *, EXE_PROMPT, "Time for Grid-cell Compilation (sec) = ", segment_time_point_seconds(13)
   print *, EXE_PROMPT, "Total Time for Processing This Orbit (sec) = ", orbital_processing_time_seconds

  !*************************************************************************
  ! Marker: End loop over files
  !*************************************************************************
  end do file_loop

  !*************************************************************************
  ! Marker: Close FILELIST
  !*************************************************************************
  close(unit=file_list_lun)

!----------------------------------------------------------------------
! DEALLOCATE MEMORY AND END PROGRAM
!----------------------------------------------------------------------

   !*************************************************************************
   !-- Marker: Close static ancillary data files
   !*************************************************************************

   !--- Close surface type
   call CLOSE_LAND_SFC_HDF(sfc_type_id)

   !--- Close coast mask
   if (read_coast_mask == sym%YES) then
    call CLOSE_LAND_SFC_HDF(coast_mask_id)
   endif

   !--- Close land mask
   if (read_land_mask == sym%YES) then
    call CLOSE_LAND_SFC_HDF(land_mask_id)
   endif

   !--- Close volcano mask
   if (read_volcano_mask == sym%YES) then
    call CLOSE_LAND_SFC_HDF(volcano_mask_id)
   endif

   !--- Close surface elevation
   if (read_surface_elevation /= 0) then
    call CLOSE_LAND_SFC_HDF(surface_elev_id)
   endif

   !--- Close snow mask
   if (read_snow_mask == sym%YES .and. failed_hires_snow_mask_flag == sym%NO) then
    call CLOSE_LAND_SFC_HDF(snow_mask_id)
   endif

   !*************************************************************************
   ! Marker: Final remaining memory deallocation
   !*************************************************************************

   !--- Deallocate memory from level3 arrays
   if (level3_flag == 1) then
     call DESTROY_GRIDCELL_ARRAYS()
   endif

   !-- deallocate level3 mapping arrays
   if (level3_flag == 1 ) then
     if (level3_format == 0) then
         call DESTROY_EQUAL_AREA()
     else
         call DESTROY_EQUAL_ANGLE()
     endif
   endif

   !--- Determine time of the start of all processing
   total_processing_end_time_hours = COMPUTE_TIME_HOURS()
   total_processing_time_seconds = 60.0*60.0*(total_processing_end_time_hours -  &
                                              total_processing_start_time_hours)
   print *, EXE_PROMPT, "Total Time for All Processing (sec) = ", total_processing_time_seconds

   !---- print to screen that processing is done
   print *, EXE_PROMPT, "<--------- End of CLAVRXORB ---------->"

stop
!--------- PFAST error statements
 200   print *, EXE_PROMPT, "error reading PFAST RTM coefficients, stopping"

!*************************************************************************
! Marker: End of code
!*************************************************************************
 end program PROCESS_AVHRR_CLAVR
