!$Id: rt_utils.f90,v 1.33 2010/09/23 22:11:02 wstraka Exp $
!--------------------------------------------------------------------------------------
!@***h* PATMOSX/rt_utils.f90
! 
! NAME
!   rt_utils.f90
! 
! LOCATION
!   /patmosx/src/rt_utils.f90
! 
! PURPOSE
!   This module holds the routines needed to compute the clear-sky RT parameters
! 
! DESCRIPTION
!   This module holds the routines needed to compute the clear-sky RT parameters
!   RT_UTILITIES houses most routines used for processing the RTM data structure
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! EXTERNALS
!  Dependencies:  (Following are names of other CLAVR-x modules)
!  CONSTANTS
!  NWP_COMMON
!  AVHRR_PIXEL_COMMON
!  NUMERICAL_ROUTINES
!  PLANCK
!  SURFACE_PROPERTIES
!
!  Calling Sequence:
!  use RT_UTILITIES
!
!  Public Routines within this Module:
!  MAP_NWP_RTM
!  CREATE_TEMP_NWP_VECTORS
!  DESTROY_TEMP_NWP_VECTORS
!  CONVERT_ATMOS_PROF_NWP_RTM
!  COMPUTE_CLEAR_RAD_PROFILES_RTM
!  GET_PIXEL_NWP_RTM
!  COMPUTE_TPW_PROFILE_NWP
!  CONVERT_NWP_H2O
!  ALLOCATE_RTM
!  DEALLOCATE_RTM
!  DEALLOCATE_RTM_VARS
!  ALLOCATE_RTM_CELL
!  DEALLOCATE_RTM_CELL
!  INTERP_RTM_KNOWING_Z
!  FIND_RTM_LEVELS
!  KNOWING_T_COMPUTE_P_Z_RTM
!  KNOWING_P_COMPUTE_T_Z_RTM
!  KNOWING_Z_COMPUTE_T_P_RTM
!  calculate_cloud_emissivity
!  nadir_cloud_emissivity
!  calculate_cloud_beta
!  calculate_demiss_dBsfc
!  calculate_demiss_dTsfc
!  calculate_dbeta_demiss
!  calculate_dbeta_dTsfc
!  calculate_demiss_dBcld
!  calculate_demiss_dTcld
!  calculate_demiss_dRcld
!  apply_rtm_sensitivity
!  bias_correct_radiance_using_bt
!  get_cloud_emissivity_level
!  Scaled_Reflectance_Ratio
!
!@*****
!------------------------------------------------------------------------
! Source Code Name:
!  rt_utils.f90
!
! Module Name:
!  RT_UTILITIES   (aka RADIATIVE TRANSFER UTILITIES)
!
! Function:
!  a module within the CLAVR-x processing system
!
! Description:
!  RT_UTILITIES houses most routines used for processing the RTM data structure
!
! Reference:
!
! Dependencies:  (Following are names of other CLAVR-x modules)
!  CONSTANTS
!  NWP_COMMON
!  AVHRR_PIXEL_COMMON
!  NUMERICAL_ROUTINES
!  PLANCK
!  SURFACE_PROPERTIES
!
! Calling Sequence:
!  use RT_UTILITIES
!
! Public Routines within this Module:
!  MAP_NWP_RTM
!  CREATE_TEMP_NWP_VECTORS
!  DESTROY_TEMP_NWP_VECTORS
!  CONVERT_ATMOS_PROF_NWP_RTM
!  COMPUTE_CLEAR_RAD_PROFILES_RTM
!  GET_PIXEL_NWP_RTM
!  COMPUTE_TPW_PROFILE_NWP
!  CONVERT_NWP_H2O
!  ALLOCATE_RTM
!  DEALLOCATE_RTM
!  DEALLOCATE_RTM_VARS
!  ALLOCATE_RTM_CELL
!  DEALLOCATE_RTM_CELL
!  INTERP_RTM_KNOWING_Z
!  FIND_RTM_LEVELS
!  KNOWING_T_COMPUTE_P_Z_RTM
!  KNOWING_P_COMPUTE_T_Z_RTM
!  KNOWING_Z_COMPUTE_T_P_RTM
!  calculate_cloud_emissivity
!  nadir_cloud_emissivity
!  calculate_cloud_beta
!  calculate_demiss_dBsfc
!  calculate_demiss_dTsfc
!  calculate_dbeta_demiss
!  calculate_dbeta_dTsfc
!  calculate_demiss_dBcld
!  calculate_demiss_dTcld
!  calculate_demiss_dRcld
!  apply_rtm_sensitivity
!  bias_correct_radiance_using_bt
!  get_cloud_emissivity_level
!  Scaled_Reflectance_Ratio
!
! Author: Andrew K. Heidinger, National Oceanic and Atmospheric Administration
! (c) This code is copyrighted by the author and all NOAA restrictions apply
! 
!--------------------------------------------------------------------------------------
module RT_UTILITIES
 
   use CONSTANTS
   use NWP_COMMON
   use AVHRR_PIXEL_COMMON
   use NUMERICAL_ROUTINES
   use PLANCK
   use SURFACE_PROPERTIES

   implicit none
   private
   public:: MAP_NWP_RTM,  &
            CREATE_TEMP_NWP_VECTORS,  &
            DESTROY_TEMP_NWP_VECTORS, &
            CONVERT_ATMOS_PROF_NWP_RTM,  &
            COMPUTE_CLEAR_RAD_PROFILES_RTM, &
            GET_PIXEL_NWP_RTM, &
            COMPUTE_TPW_PROFILE_NWP, &
            CONVERT_NWP_H2O, &
            ALLOCATE_RTM, &
            DEALLOCATE_RTM, &
            DEALLOCATE_RTM_VARS, &
            ALLOCATE_RTM_CELL, &
            DEALLOCATE_RTM_CELL, &
            INTERP_RTM_KNOWING_Z, &
            FIND_RTM_LEVELS, &
            KNOWING_T_COMPUTE_P_Z_RTM, &
            KNOWING_P_COMPUTE_T_Z_RTM, &
            KNOWING_Z_COMPUTE_T_P_RTM, &
            calculate_cloud_emissivity, &
            nadir_cloud_emissivity, &
            calculate_cloud_beta, &
            calculate_demiss_dBsfc, &
            calculate_demiss_dTsfc, &
            calculate_dbeta_demiss, &
            calculate_dbeta_dTsfc, &
            calculate_demiss_dBcld, &
            calculate_demiss_dTcld, &
            calculate_demiss_dRcld, &
            apply_rtm_sensitivity, &
            bias_correct_radiance_using_bt, &
            get_cloud_emissivity_level, &
            Scaled_Reflectance_Ratio

   integer, parameter, public:: nlevels_rtm = 101

   real, dimension(nlevels_rtm), parameter, public:: P_std_rtm = (/ &
          0.0050,    0.0161,    0.0384,    0.0769,    0.1370,  &  
          0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972,  &
          1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204,  &
          5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,  &
         14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,  &
         29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882,  &
         51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396,  &
         83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,  &
        125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,  &
        180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338,  &
        247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369,  &
        328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,  &
        424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,  &
        535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398,  &
        661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897,  &
        802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,  &
        958.5911,  986.0666, 1013.9476, 1042.2319, 1070.9170, 1100.0000/)
                                                                                                                                                    
    real, dimension(nlevels_rtm), parameter, public:: T_std_rtm = (/    &
                                190.19, 203.65, 215.30, 226.87, 237.83, &
        247.50, 256.03, 263.48, 267.09, 270.37, 266.42, 261.56, 256.40, & 
        251.69, 247.32, 243.27, 239.56, 236.07, 232.76, 230.67, 228.71, &
        227.35, 226.29, 225.28, 224.41, 223.61, 222.85, 222.12, 221.42, &
        220.73, 220.07, 219.44, 218.82, 218.23, 217.65, 217.18, 216.91, &
        216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.70, &
        216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.70, 216.71, &
        216.71, 216.72, 216.81, 217.80, 218.77, 219.72, 220.66, 222.51, &
        224.57, 226.59, 228.58, 230.61, 232.61, 234.57, 236.53, 238.48, &
        240.40, 242.31, 244.21, 246.09, 247.94, 249.78, 251.62, 253.45, &
        255.26, 257.04, 258.80, 260.55, 262.28, 264.02, 265.73, 267.42, &
        269.09, 270.77, 272.43, 274.06, 275.70, 277.32, 278.92, 280.51, &
        282.08, 283.64, 285.20, 286.74, 288.25, 289.75, 291.22, 292.68/) 

    real, dimension(nlevels_rtm), parameter, public:: wvmr_std_rtm = (/ &
                                 0.001,  0.001,  0.002,  0.003,  0.003, &
         0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, &
         0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, &
         0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, &
         0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, &
         0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003,  0.003, &
         0.003,  0.003,  0.004,  0.004,  0.005,  0.005,  0.007,  0.009, &
         0.011,  0.012,  0.014,  0.020,  0.025,  0.030,  0.035,  0.047, &
         0.061,  0.075,  0.089,  0.126,  0.162,  0.197,  0.235,  0.273, &
         0.310,  0.356,  0.410,  0.471,  0.535,  0.601,  0.684,  0.784, &
         0.886,  0.987,  1.094,  1.225,  1.353,  1.519,  1.686,  1.852, &
         2.036,  2.267,  2.496,  2.721,  2.947,  3.170,  3.391,  3.621, &
         3.848,  4.084,  4.333,  4.579,  4.822,  5.061,  5.296,  5.528/)
                                                                                                                                                    
    real, dimension(nlevels_rtm), parameter, public:: ozmr_std_rtm = (/ &
                               0.47330,0.27695,0.28678,0.51816,0.83229, &
       1.18466,1.69647,2.16633,3.00338,3.76287,4.75054,5.61330,6.33914, &
       7.03675,7.50525,7.75612,7.81607,7.69626,7.56605,7.28440,7.01002, &
       6.72722,6.44629,6.17714,5.92914,5.69481,5.47387,5.26813,5.01252, &
       4.68941,4.35141,4.01425,3.68771,3.37116,3.06407,2.77294,2.50321, &
       2.24098,1.98592,1.74840,1.54451,1.34582,1.17824,1.02513,0.89358, &
       0.78844,0.69683,0.62654,0.55781,0.50380,0.45515,0.42037,0.38632, &
       0.35297,0.32029,0.28832,0.25756,0.22739,0.19780,0.16877,0.14901, &
       0.13190,0.11511,0.09861,0.08818,0.07793,0.06786,0.06146,0.05768, &
       0.05396,0.05071,0.04803,0.04548,0.04301,0.04081,0.03983,0.03883, &
       0.03783,0.03685,0.03588,0.03491,0.03395,0.03368,0.03349,0.03331, &
       0.03313,0.03292,0.03271,0.03251,0.03190,0.03126,0.03062,0.02990, &
       0.02918,0.02850,0.02785,0.02721,0.02658,0.02596,0.02579,0.02579/)

    integer, dimension(nlevels_rtm), public:: k_rtm_nwp
    real, dimension(nlevels_rtm), public:: x_rtm_nwp
    real, dimension(nlevels_rtm), public, save:: T_rtm,  &
                     Z_rtm,  &
                     wvmr_rtm,  &
                     ozmr_rtm, &
                     ch3b_trans_prof_rtm, &
                     ch4_trans_prof_rtm,  &
                     ch5_trans_prof_rtm, &
                     ch3b_rad_prof_rtm, &
                     ch4_rad_prof_rtm, &
                     ch5_rad_prof_rtm, &
                     ch3b_trans_prof_solar_rtm, &
                     ch3b_trans_prof_solar_total_rtm, &
                     ch3b_cloud_prof_rtm, &
                     ch4_cloud_prof_rtm, &
                     ch5_cloud_prof_rtm

!---
    integer, dimension(:), allocatable, save, private:: k_nwp_rtm
    real, dimension(:), allocatable, save, private:: x_nwp_rtm
    real, dimension(:), allocatable, private, save:: T_nwp
    real, dimension(:), allocatable, private, save:: Z_nwp
    real, dimension(:), allocatable, private, save:: wvmr_nwp
    real, dimension(:), allocatable, private, save:: ozmr_nwp

!---------------------------------------------------------------------
! RTM structure definition
!---------------------------------------------------------------------
 TYPE, public :: rtm_prof
    integer (kind=int1) :: flag = 0
    REAL (kind=real4), DIMENSION(:), pointer :: trans_atm_clr3b
    REAL (kind=real4), DIMENSION(:), pointer :: trans_atm_clr4
    REAL (kind=real4), DIMENSION(:), pointer :: trans_atm_clr5
    REAL (kind=real4), DIMENSION(:), pointer :: trans_atm_clr3b_solar_total
    REAL (kind=real4), DIMENSION(:), pointer :: trans_atm_clr3b_solar
    REAL (kind=real4), DIMENSION(:), pointer :: rad_atm_clr3b
    REAL (kind=real4), DIMENSION(:), pointer :: rad_atm_clr4
    REAL (kind=real4), DIMENSION(:), pointer :: rad_atm_clr5
    REAL (kind=real4), dimension(:), pointer :: cloud_prof3b
    REAL (kind=real4), dimension(:), pointer :: cloud_prof4
    REAL (kind=real4), dimension(:), pointer :: cloud_prof5
  end TYPE rtm_prof

  TYPE, public :: rtm_params
    integer (kind=int1) :: flag = 0
    integer (kind=int1) :: sfc_level = 0
    integer (kind=int1) :: tropo_level = 0
    integer (kind=int1) :: inversion_level = 0
    integer (kind=int1) :: level850 = 0
    TYPE (rtm_prof), DIMENSION(:), ALLOCATABLE :: d
    REAL (kind=real4), dimension(:), allocatable :: T_prof
    REAL (kind=real4), dimension(:), allocatable :: Z_prof
    REAL (kind=real4), dimension(:), allocatable :: wvmr_prof
    REAL (kind=real4), dimension(:), allocatable :: ozmr_prof
  end TYPE rtm_params

!---------------------------------------------------------------------
! Declare a 2D pointer of "rtm_params" structures
!---------------------------------------------------------------------

  TYPE (rtm_params), PUBLIC, DIMENSION(:, :), ALLOCATABLE :: rtm

! integer, parameter, public:: rtm_nvzen = 100
! real, parameter, public::  rtm_vza_binsize = 0.01
  integer, parameter, public:: rtm_nvzen = 20
  real, parameter, public::  rtm_vza_binsize = 0.05

contains

!====================================================================
! SUBROUTINE Name: CREATE_TEMP_NWP_VECTORS
!
! Function:
!   create and initialize NWP vectors used for RTM calcs
!
!====================================================================
 subroutine CREATE_TEMP_NWP_VECTORS()

  allocate(T_nwp(nlevels_nwp), &
           Z_nwp(nlevels_nwp), &
           wvmr_nwp(nlevels_nwp), &
           ozmr_nwp(nlevels_nwp), &
           k_nwp_rtm(nlevels_nwp), &
           x_nwp_rtm(nlevels_nwp))

  T_nwp = 0.0
  Z_nwp = 0.0
  wvmr_nwp = 0.0
  ozmr_nwp = 0.0
  k_nwp_rtm = 0.0
  x_nwp_rtm = 0.0

 end subroutine CREATE_TEMP_NWP_VECTORS
!====================================================================
! SUBROUTINE Name: DESTROY_TEMP_NWP_VECTORS
!
! Function:
!   destroy and initialize NWP vectors used for RTM calcs
!
!====================================================================
 subroutine DESTROY_TEMP_NWP_VECTORS()

  deallocate(T_nwp,Z_nwp,wvmr_nwp,ozmr_nwp,k_nwp_rtm, x_nwp_rtm)

 end subroutine DESTROY_TEMP_NWP_VECTORS

!====================================================================
! SUBROUTINE Name: MAP_NWP_RTM
!
! Function:
!   develops the mapping of the NWP and RTM profile levels
!
!====================================================================
 subroutine MAP_NWP_RTM()
    integer:: k,k_temp

!--- map NWP levels to RTM levels
     do k = 1, nlevels_nwp
      
!--- locate the nwp level within the rtm levels 
!--- P_std_nwp(k) should fall between rtm levels k_temp and k_temp +1

       call locate(P_std_rtm, nlevels_rtm, P_std_nwp(k), k_temp)
       k_nwp_rtm(k) = min(nlevels_rtm-1,max(1,k_temp))

!-- compute linear weighting factor
       x_nwp_rtm(k) = (P_std_nwp(k) - P_std_rtm(k_nwp_rtm(k))) / &
                      (P_std_rtm(k_nwp_rtm(k)+1) - P_std_rtm(k_nwp_rtm(k))) 

     enddo

     !--- map RTM levels to NWP levels
     do k = 1, nlevels_rtm
      
       !--- locate the rtm level within the nwp levels 
       !--- P_std_rtm(k) should fall between nwp levels k_temp and k_temp +1
       call locate(P_std_nwp, nlevels_nwp, P_std_rtm(k), k_temp)
       k_rtm_nwp(k) = min(nlevels_nwp-1,max(1,k_temp))

       !-- compute linear weighting factor
       x_rtm_nwp(k) = (P_std_rtm(k) - P_std_nwp(k_rtm_nwp(k))) / &
                      (P_std_nwp(k_rtm_nwp(k)+1) - P_std_nwp(k_rtm_nwp(k)))
     
     enddo 
      
 end subroutine MAP_NWP_RTM

!====================================================================
! SUBROUTINE Name: CONVERT_ATMOS_PROF_NWP_RTM
!
! Description:
! This routine interpolate the NWP profiles to profiles with the
! vertical spacing defined by the RTM model.  It operates on profiles
! stored in this module
!
! INPUTS:
!
! highest_level_rtm_nwp - highest rtm level that is below the highest nwp level
! lowest_level_rtm_nwp - lowest rtm level that is above the lowest nwp level
! sfc_level_rtm - lowest rtm level above the surface
! P_near_sfc_nwp - lowest standard nwp level above surface pressure
!
!====================================================================
 subroutine CONVERT_ATMOS_PROF_NWP_RTM(ilon,ilat)

    integer, intent(in):: ilon, ilat
    integer:: k, lowest_level_rtm_nwp, highest_level_rtm_nwp,  &
              sfc_level_rtm
    real:: dT_dP_near_sfc
    real:: dwvmr_dP_near_sfc
    real:: dZ_dP_near_sfc
    real:: P_near_sfc_nwp
    real:: wvmr_sfc
    real:: es
    real:: e


    lowest_level_rtm_nwp = nlevels_rtm
    highest_level_rtm_nwp = 1
    sfc_level_rtm = nlevels_rtm
    P_near_sfc_nwp = P_std_nwp(sfc_level_nwp(ilon,ilat))
    
!--- make wvmr at sfc
    es = vapor(tmpsfc_nwp(ilon,ilat))
    e = rhsfc_nwp(ilon,ilat) * es / 100.0
    wvmr_sfc = 1000.0*0.622 * (e / (Psfc_nwp(ilon,ilat) - e))  !(g/kg)

!--- determine some critical levels in the rtm profile
    do k = 1, nlevels_rtm
       if (P_std_rtm(k) > P_std_nwp(1)) then
            highest_level_rtm_nwp = k
            exit
       endif
    enddo

    do k = nlevels_rtm,1,-1
       if (P_std_rtm(k) < psfc_nwp(ilon,ilat)) then
            sfc_level_rtm = k
            exit
       endif
    enddo

    do k = nlevels_rtm,1,-1
       if (P_std_rtm(k) < P_near_sfc_nwp) then
            lowest_level_rtm_nwp = k
            exit
       endif
    enddo

!--- compute T and wvmr lapse rate near the surface
    dT_dP_near_sfc = 0.0
    dZ_dP_near_sfc = 0.0
    dwvmr_dP_near_sfc = 0.0
    if (psfc_nwp(ilon,ilat) /= P_std_nwp(nlevels_nwp)) then
     dT_dP_near_sfc =  &
             (T_nwp(sfc_level_nwp(ilon,ilat)) - tmpair_nwp(ilon,ilat))/ &
             (P_std_nwp(sfc_level_nwp(ilon,ilat)) - psfc_nwp(ilon,ilat))
     dwvmr_dP_near_sfc =  &
             (wvmr_nwp(sfc_level_nwp(ilon,ilat)) - wvmr_sfc)/ &
             (P_std_nwp(sfc_level_nwp(ilon,ilat)) - psfc_nwp(ilon,ilat))
     dZ_dP_near_sfc =  &
             (Z_nwp(sfc_level_nwp(ilon,ilat)) - 0.0)/ &
             (P_std_nwp(sfc_level_nwp(ilon,ilat)) - psfc_nwp(ilon,ilat))
    endif

!--- for rtm levels above the highest nwp level, use rtm standard values
    do k = 1,highest_level_rtm_nwp-1
          Z_rtm(k) = Z_nwp(1)
          T_rtm(k) = T_std_rtm(k)
          wvmr_rtm(k) = wvmr_std_rtm(k)
          ozmr_rtm(k) = ozmr_std_rtm(k)
    enddo

!--- rtm levels within standard nwp levels above the surface
    do k = highest_level_rtm_nwp, lowest_level_rtm_nwp
          T_rtm(k) = T_nwp(k_rtm_nwp(k)) + x_rtm_nwp(k) *  &
                     (T_nwp(k_rtm_nwp(k)+1) - T_nwp(k_rtm_nwp(k)))
          Z_rtm(k) = Z_nwp(k_rtm_nwp(k)) + x_rtm_nwp(k) *  &
                     (Z_nwp(k_rtm_nwp(k)+1) - Z_nwp(k_rtm_nwp(k)))
          wvmr_rtm(k) = wvmr_nwp(k_rtm_nwp(k)) + x_rtm_nwp(k) *  &
                     (wvmr_nwp(k_rtm_nwp(k)+1) - wvmr_nwp(k_rtm_nwp(k)))
          ozmr_rtm(k) = ozmr_nwp(k_rtm_nwp(k)) + x_rtm_nwp(k) *  &
                     (ozmr_nwp(k_rtm_nwp(k)+1) - ozmr_nwp(k_rtm_nwp(k)))
    enddo

!--- rtm levels that are below the lowest nwp level but above the surface
   do k = lowest_level_rtm_nwp+1, sfc_level_rtm
           T_rtm(k) = tmpair_nwp(ilon,ilat) + dT_dP_near_sfc * &
                      (P_std_rtm(k) - psfc_nwp(ilon,ilat))
           wvmr_rtm(k) = wvmr_sfc + dwvmr_dP_near_sfc * &
                      (P_std_rtm(k) - psfc_nwp(ilon,ilat))
           Z_rtm(k) = dZ_dP_near_sfc * &
                      (P_std_rtm(k) - Psfc_nwp(ilon,ilat))
           ozmr_rtm(k) = ozmr_nwp(nlevels_nwp)
   enddo

!--- rtm levels below the surface
    do k = sfc_level_rtm +1, nlevels_rtm
          T_rtm(k) = tmpair_nwp(ilon,ilat)
          Z_rtm(k) = 0.0
          wvmr_rtm(k) = wvmr_sfc
          ozmr_rtm(k) = ozmr_std_rtm(k)
    enddo

!--- if using NCEP reanalysis which has no ozone profile, use default
       if (nwp_flag == 2) then
          ozmr_rtm = ozmr_std_rtm
       endif


 end subroutine CONVERT_ATMOS_PROF_NWP_RTM


!====================================================================
! SUBROUTINE Name: COMPUTE_CLEAR_RAD_PROFILES_RTM
!
! Function:
! Computes clear sky radiance profiles
!
!====================================================================
subroutine COMPUTE_CLEAR_RAD_PROFILES_RTM()

  integer:: k
  real:: T_mean, B_mean, B_level


 ch3b_rad_prof_rtm = 0.0
 ch4_rad_prof_rtm = 0.0
 ch5_rad_prof_rtm = 0.0
 ch3b_cloud_prof_rtm = 0.0
 ch4_cloud_prof_rtm = 0.0
 ch5_cloud_prof_rtm = 0.0

 do k = 2, nlevels_rtm
    
   T_mean = 0.5*(T_rtm(k-1) + T_rtm(k))

!-- ch3b
   B_mean = PLANCK_RAD_FAST(3,T_mean)

   ch3b_rad_prof_rtm(k) = ch3b_rad_prof_rtm(k-1) +  &
           (ch3b_trans_prof_rtm(k-1) - ch3b_trans_prof_rtm(k)) * B_mean

   B_level = PLANCK_RAD_FAST(3,T_rtm(k))
   ch3b_cloud_prof_rtm(k) = ch3b_rad_prof_rtm(k) +  &
                            (ch3b_trans_prof_rtm(k) * B_level)

!-- ch4
   B_mean = PLANCK_RAD_FAST(4,T_mean)
   ch4_rad_prof_rtm(k) = ch4_rad_prof_rtm(k-1) +  &
           (ch4_trans_prof_rtm(k-1) - ch4_trans_prof_rtm(k)) * B_mean

   B_level = PLANCK_RAD_FAST(4,T_rtm(k))
   ch4_cloud_prof_rtm(k) = ch4_rad_prof_rtm(k) +  &
                            (ch4_trans_prof_rtm(k) * B_level)


!-- ch5
   B_mean = PLANCK_RAD_FAST(5,T_mean)
   ch5_rad_prof_rtm(k) = ch5_rad_prof_rtm(k-1) +  &
           (ch5_trans_prof_rtm(k-1) - ch5_trans_prof_rtm(k)) * B_mean

   B_level = PLANCK_RAD_FAST(5,T_rtm(k))
   ch5_cloud_prof_rtm(k) = ch5_rad_prof_rtm(k) +  &
                            (ch5_trans_prof_rtm(k) * B_level)

 enddo

end subroutine COMPUTE_CLEAR_RAD_PROFILES_RTM

!====================================================================
! SUBROUTINE Name: GET_PIXEL_NWP_RTM
!
! Function:
! Calculates the PFAAST transmittance profiles for a given segment.
!
!====================================================================
  subroutine GET_PIXEL_NWP_RTM(jmin,numj)
   integer, intent(in):: jmin
   integer, intent(in):: numj
   integer:: i
   integer:: j
   integer:: ksfc
   integer:: ilon
   integer:: ilat
   real:: Z_sfc_km
   real:: prof_weight
   real:: satzen_mid_bin
   real:: ch3b_sfc_rad
   real:: ch4_sfc_rad
   real:: ch5_sfc_rad
   real:: rad3b_temp
   real:: rho
   real:: bt_clear_ch4_bias
   real:: bt_clear_ch5_bias
   real:: yd
   real:: yn
   real:: tau_temp
   real:: emiss_12_tropo_rtm
   real:: emiss_12_tropo_nadir_rtm
   integer:: ivza
   integer:: astatus
   integer:: lev_bnd
   integer:: ilev
   real:: Tc_lower
   real:: Pc_lower
   real:: Zc_lower
   real:: Bc_4
   real:: Bc_5
   real:: rad_clear_4
   real:: rad_clear_5
   real:: trans_ac_3b
   real:: rad_ac_3b
   real:: trans_ac_4
   real:: rad_ac_4
   real:: trans_ac_5
   real:: rad_ac_5

   !--- loop over pixels in segment
   line_loop: do j = jmin, numj + jmin - 1
     element_loop: do i = 1, num_pix
                                                                       
      !--- check for bad scans
      if (bad_pixel_mask(i,j) == sym%YES) then
        cycle
      endif

     !--- compute pixel evel tropospheric temperature
     if (smooth_nwp == sym%YES) then
       ttropo_rtm(i,j) = INTERPOLATE_NWP(t_trop_nwp(i_nwp(i,j),j_nwp(i,j)), &
                                     t_trop_nwp(i_nwp_x(i,j),j_nwp(i,j)), &
                                     t_trop_nwp(i_nwp(i,j),j_nwp_x(i,j)), &
                                     t_trop_nwp(i_nwp_x(i,j),j_nwp_x(i,j)), &
                                     lon_nwp_fac(i,j),lat_nwp_fac(i,j))
     else
       ttropo_rtm(i,j) = t_trop_nwp(i_nwp(i,j),j_nwp(i,j))
     endif


     !--- compute viewing zenith bin for rtm calculation
     ivza_rtm(i,j) = max(1,min(rtm_nvzen,ceiling(coszen(i,j)/rtm_vza_binsize)))

     !--- store cell and angular indices
     ilon = i_nwp(i,j)
     ilat = j_nwp(i,j)
     ivza = ivza_rtm(i,j)

     !--- if snow was found, modify emiss
!     if (snow(i,j) /= sym%NO_SNOW) then
!         sfc_emiss_3b(i,j) = 0.99
!         sfc_emiss_4(i,j) = 0.99
!         sfc_emiss_5(i,j) = 0.99
!     endif


     !--- if this is the first time this nwp cell is being processed do the following
     if (rtm(ilon,ilat)%flag == 0) then

       !--- copy other nwp fields into temp nwp profiles - note need to convert to mr
       T_nwp = T_prof_nwp(:,ilon,ilat)
       Z_nwp = Z_prof_nwp(:,ilon,ilat)
       ozmr_nwp = ozone_prof_nwp(:,ilon,ilat)

       !--- convert nwp rh humidty profile into mixing ratio and integrated pw profiles
       call CONVERT_NWP_H2O(ilon,ilat)  

       !--- allocate rtm arrays
       call allocate_rtm_cell(ilon,ilat,rtm_nvzen)

       !--- convert nwp rh humidty profile into mixing ratio and integrated pw profiles
       call COMPUTE_TPW_PROFILE_NWP(ilon,ilat)  

       !--- find key levels in nwp profiles
       call FIND_NWP_LEVELS(ilon,ilat)

       !--- convert the atmospheric profiles from nwp to rtm pressure coords
       call CONVERT_ATMOS_PROF_NWP_RTM(ilon,ilat)

       !--- store in rtm structures
       rtm(ilon,ilat)%T_prof = T_rtm
       rtm(ilon,ilat)%Z_prof = Z_rtm
       rtm(ilon,ilat)%wvmr_prof = wvmr_rtm
       rtm(ilon,ilat)%ozmr_prof = ozmr_rtm

       !--- find key levels (sfc, tropo, inversions) for future processing
       call FIND_RTM_LEVELS(ilon,ilat)

       !--- compute cloud parameters from nwp profiles
       call COMPUTE_NWP_CLOUD_PARAMETERS(clwmr_prof_nwp(:,ilon,ilat), &
                                         T_prof_nwp(:,ilon,ilat), &
                                         P_std_nwp, &
                                         Pc_nwp(ilon,ilat), &
                                         iwp_nwp(ilon,ilat), &
                                         lwp_nwp(ilon,ilat), &
                                         ncld_layers_nwp(ilon,ilat))
     endif

     !--- determine if RTM needs to be called for this pixel, if not do the following
     if (rtm(ilon,ilat)%d(ivza)%flag == 0) then   !check for existing RTM results


        !--- allocate memory as needed
        allocate(rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(nlevels_rtm),&
        rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(nlevels_rtm), &
        rtm(ilon,ilat)%d(ivza)%cloud_prof3b(nlevels_rtm),  &
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar(nlevels_rtm),  &
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar_total(nlevels_rtm),  &
        stat=astatus)

        allocate(rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(nlevels_rtm),&
        rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(nlevels_rtm), &
        rtm(ilon,ilat)%d(ivza)%cloud_prof4(nlevels_rtm), &
        stat=astatus)

        allocate(rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(nlevels_rtm),&
        rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(nlevels_rtm),   &
        rtm(ilon,ilat)%d(ivza)%cloud_prof5(nlevels_rtm),    &
        stat=astatus)

        !--- determine the zenith angle for the RTM
        satzen_mid_bin = acos((ivza-1)*rtm_vza_binsize + rtm_vza_binsize/2.0) / dtor

        !--- store structures into RTM arguments
        T_rtm = rtm(ilon,ilat)%T_prof
        wvmr_rtm = rtm(ilon,ilat)%wvmr_prof
        ozmr_rtm = rtm(ilon,ilat)%ozmr_prof

        !--- call the RTM
        call tranmavhrr(ancil_data_dir,T_rtm,wvmr_rtm,ozmr_rtm, &
                        satzen_mid_bin,sc_id_char,3,ch3b_trans_prof_rtm,*200)
        call tranmavhrr(ancil_data_dir,T_rtm,wvmr_rtm,ozmr_rtm,satzen_mid_bin, &
                        sc_id_char,4,ch4_trans_prof_rtm,*200)
        call tranmavhrr(ancil_data_dir,T_rtm,wvmr_rtm,ozmr_rtm,satzen_mid_bin, &
                        sc_id_char,5,ch5_trans_prof_rtm,*200)

        !--- compute a transmission profile for the solar + viewing path
        ch3b_trans_prof_solar_total_rtm  = 0.0
        if ((cossolzen(i,j) > 0.0).and.(coszen(i,j) > 0.0)) then
           ch3b_trans_prof_solar_total_rtm  = ch3b_trans_prof_rtm ** (1.0/cossolzen(i,j))
        endif

        !--- compute a transmission profile for the solar path
        ch3b_trans_prof_solar_rtm  = 0.0
        if ((cossolzen(i,j) > 0.0).and.(coszen(i,j) > 0.0)) then
           ch3b_trans_prof_solar_rtm  = ch3b_trans_prof_rtm ** (coszen(i,j)/cossolzen(i,j))
        endif

        !--- store transmission profiles into rtm structure
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar = ch3b_trans_prof_solar_rtm
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar_total = ch3b_trans_prof_solar_total_rtm
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b = ch3b_trans_prof_rtm
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr4 = ch4_trans_prof_rtm
        rtm(ilon,ilat)%d(ivza)%trans_atm_clr5 = ch5_trans_prof_rtm

        !--- compute profiles of radiance based on the transmission profiles
        call COMPUTE_CLEAR_RAD_PROFILES_RTM()
        rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b = ch3b_rad_prof_rtm
        rtm(ilon,ilat)%d(ivza)%rad_atm_clr4 = ch4_rad_prof_rtm
        rtm(ilon,ilat)%d(ivza)%rad_atm_clr5 = ch5_rad_prof_rtm
        rtm(ilon,ilat)%d(ivza)%cloud_prof3b = ch3b_cloud_prof_rtm
        rtm(ilon,ilat)%d(ivza)%cloud_prof4 = ch4_cloud_prof_rtm
        rtm(ilon,ilat)%d(ivza)%cloud_prof5 = ch5_cloud_prof_rtm

         !---   set mask  to indicate this bin or this cell has been computed
         rtm(ilon,ilat)%d(ivza)%flag = 1

  endif  !end check for previous RTM run

  !-----------------------------------------------------------------------------
  ! compute clear TOA radiance
  !-----------------------------------------------------------------------------  
  
   !--- compute pixel level surface pressure based on pixel surface elevation
   Z_sfc_km = zsfc(i,j) / 1000.0   !km
   ksfc = rtm(ilon,ilat)%sfc_level

   !--- if an sst analysis is available, use that
   if ((land_mask(i,j) == sym%NO) .and.  &
       (snow(i,j) == sym%NO_SNOW) .and.  &
       (use_sst_anal == sym%YES) .and.  &
       (sst_anal(i,j) > 270.0 )) then
       tsfc_rtm(i,j) = sst_anal(i,j)
   endif 

   !-- vertical interp weight
   prof_weight = (Z_sfc_km - rtm(ilon,ilat)%Z_prof(ksfc)) / &
       (rtm(ilon,ilat)%Z_prof(ksfc+1) - rtm(ilon,ilat)%Z_prof(ksfc))

   !--- constrain - important when high res topo differs from low res nwp topo
   prof_weight = max(0.0,min(1.0,prof_weight))

   !-- ch3b
   ch3b_sfc_rad = sfc_emiss_3b(i,j) * PLANCK_RAD_FAST(3,tsfc_rtm(i,j))
 
   rad_atm_ch3b_rtm(i,j) = rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ksfc)) * prof_weight

   trans_atm_ch3b_rtm(i,j) = rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ksfc)) * prof_weight

   trans_atm_ch3b_solar_rtm(i,j) =  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar(ksfc)) * &
                  prof_weight

   trans_atm_ch3b_solar_total_rtm(i,j) =  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar_total(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar_total(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar_total(ksfc)) * &
                  prof_weight

   rad_clear_ch3b_rtm(i,j) = rad_atm_ch3b_rtm(i,j) + trans_atm_ch3b_rtm(i,j)*ch3b_sfc_rad
   bt_clear_ch3b_rtm(i,j) = PLANCK_TEMP_FAST(3,rad_clear_ch3b_rtm(i,j))

   !--- add in solar component - does not account for glint
   rad_clear_ch3b_solar_rtm(i,j) = rad_clear_ch3b_rtm(i,j)
   bt_clear_ch3b_solar_rtm(i,j) = bt_clear_ch3b_rtm(i,j)

   if (cossolzen(i,j) >= 0.0) then

      rho = 1.0 - sfc_emiss_3b(i,j)

      rad3b_temp = rad_clear_ch3b_rtm(i,j) +  &
                   trans_atm_ch3b_solar_total_rtm(i,j) * rho * (cossolzen(i,j)*solar_3b_nu / pi)

      rad_clear_ch3b_solar_rtm(i,j) = rad3b_temp
      bt_clear_ch3b_solar_rtm(i,j) = PLANCK_TEMP_FAST(3,rad3b_temp)

   endif

   !-- ch4
   ch4_sfc_rad = sfc_emiss_4(i,j) * PLANCK_RAD_FAST(4,tsfc_rtm(i,j))

   rad_atm_ch4_rtm(i,j) = rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ksfc)) * prof_weight

   trans_atm_ch4_rtm(i,j) = rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ksfc)) * prof_weight

   rad_clear_ch4_rtm(i,j) = rad_atm_ch4_rtm(i,j) + trans_atm_ch4_rtm(i,j)*ch4_sfc_rad
   bt_clear_ch4_rtm(i,j) = PLANCK_TEMP_FAST(4,rad_clear_ch4_rtm(i,j))

   !-- ch5
   ch5_sfc_rad = sfc_emiss_5(i,j) * PLANCK_RAD_FAST(5,tsfc_rtm(i,j))

   rad_atm_ch5_rtm(i,j) = rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ksfc)) * prof_weight 

   trans_atm_ch5_rtm(i,j) = rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ksfc) +  &
                 (rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ksfc+1) -  &
                  rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ksfc)) * prof_weight 

   rad_clear_ch5_rtm(i,j) = rad_atm_ch5_rtm(i,j) + trans_atm_ch5_rtm(i,j)*ch5_sfc_rad
   bt_clear_ch5_rtm(i,j) = PLANCK_TEMP_FAST(5,rad_clear_ch5_rtm(i,j))

   !--- compute ch3b emissivities
   rad3b_temp = PLANCK_RAD_FAST(3,bt_clear_ch4_rtm(i,j))
   ems3b_clear_solar_rtm(i,j) = rad_clear_ch3b_solar_rtm(i,j) / rad3b_temp
   ems3b_clear_rtm(i,j) = rad_clear_ch3b_rtm(i,j) / rad3b_temp

   !--- atmos corrected
   ems3b_clear_solar_sfc_rtm(i,j) = ch3b_sfc_rad +  &
                    trans_atm_ch3b_solar_rtm(i,j) * (1.0 - sfc_emiss_3b(i,j))*cossolzen(i,j)*solar_3b_nu / pi

     !----------------------------------------------------------------------
     ! perform a bias correction here - only for bt's
     ! yd - daytime value, yn = nightime value
     !----------------------------------------------------------------------

     !--- ch4
     if (sfc_type(i,j) /= sym%WATER_SFC) then
        yd = 9.0
        yn = -2.0
        bt_clear_ch4_bias = 0.5*(yd-yn)*sin(2.0*pi*(pixel_local_time_hours(i,j)-8.0)/24.0) + (yd-yn)/2.0 + yn
     else
        yd = 0.4
        yn = 0.2
        bt_clear_ch4_bias = 0.5*(yd-yn)*sin(2.0*pi*(pixel_local_time_hours(i,j)-8.0)/24.0) + (yd-yn)/2.0 + yn
     endif
     bt_clear_ch4_rtm_unbiased(i,j) = bt_clear_ch4_rtm(i,j) + bt_clear_ch4_bias
     rad_clear_ch4_rtm_unbiased(i,j) = PLANCK_RAD_FAST(4,bt_clear_ch4_rtm_unbiased(i,j))

     !--- ch5
     if (sfc_type(i,j) /= sym%WATER_SFC) then
        yd = 0.82
        yn = -0.51

        !-- line below is actually bias of T4 - T5
        bt_clear_ch5_bias = 0.5*(yd-yn)*sin(2.0*pi*(pixel_local_time_hours(i,j)-8.0)/24.0) + (yd-yn)/2.0 + yn 

     else
        bt_clear_ch5_bias = -0.4
     endif

     !-- convert to a T5 bias
     bt_clear_ch5_bias = bt_clear_ch4_bias - bt_clear_ch5_bias

     bt_clear_ch5_rtm_unbiased(i,j) = bt_clear_ch5_rtm(i,j) + bt_clear_ch5_bias
     rad_clear_ch5_rtm_unbiased(i,j) = PLANCK_RAD_FAST(5,bt_clear_ch5_rtm_unbiased(i,j))

!--------------------------------------------------------------
!--- compute emissivity at tropopause
!--------------------------------------------------------------
  lev_bnd = rtm(ilon,ilat)%tropo_level  

   emiss_11_tropo_rtm(i,j) = missing_value_real4
   emiss_11_tropo_nadir_rtm(i,j) = missing_value_real4
  !--- compute 11 micron emissivity referenced to the tropopause
  if (rtm(ilon,ilat)%d(ivza)%cloud_prof4(lev_bnd) /= rad_clear_ch4_rtm(i,j)) then

    emiss_11_tropo_rtm(i,j) = (rad4(i,j) - rad_clear_ch4_rtm(i,j)) / &
               (rtm(ilon,ilat)%d(ivza)%cloud_prof4(lev_bnd)-rad_clear_ch4_rtm(i,j))

    !--- adjust to nadir
    if ((emiss_11_tropo_rtm(i,j) > 0.0) .and. (emiss_11_tropo_rtm(i,j) < 1.00)) then
       tau_temp = -coszen(i,j)*alog(1.0 - emiss_11_tropo_rtm(i,j) )
       emiss_11_tropo_nadir_rtm(i,j) = 1.0 - exp(-tau_temp)
    endif

  endif

  !--- compute 12 micron emissivity referenced to the tropopause
  if (rtm(ilon,ilat)%d(ivza)%cloud_prof5(lev_bnd) /= rad_clear_ch5_rtm(i,j)) then

    emiss_12_tropo_rtm = (rad5(i,j) - rad_clear_ch5_rtm(i,j)) / &
          (rtm(ilon,ilat)%d(ivza)%cloud_prof5(lev_bnd) - rad_clear_ch5_rtm(i,j))

    !--- adjust to nadir
    emiss_12_tropo_nadir_rtm = missing_value_real4
    if ((emiss_12_tropo_rtm > 0) .and. (emiss_12_tropo_rtm < 1.00)) then
       tau_temp = -coszen(i,j)*alog(1.0 - emiss_12_tropo_rtm )
       emiss_12_tropo_nadir_rtm = 1.0 - exp(-tau_temp)
    endif

  endif

  !--- compute split-window beta ratio at tropopause
  beta_11_12_tropo_rtm(i,j) = missing_value_real4

  if ((emiss_11_tropo_rtm(i,j) > 0.0) .and.  &
      (emiss_11_tropo_rtm(i,j) < 1.0) .and. &
      (emiss_12_tropo_rtm > 0.0) .and. &
      (emiss_12_tropo_rtm < 1.0)) then

        beta_11_12_tropo_rtm(i,j) = calculate_cloud_beta( &
                                             emiss_12_tropo_rtm,  &
                                             emiss_11_tropo_rtm(i,j)) 
  endif

  !--- compute emissivities and beta assuming an elevated surface
  Pc_lower = Psfc_nwp(ilon,ilat) - 200.0
  call KNOWING_P_COMPUTE_T_Z_RTM(ilon,ilat,Pc_lower,Tc_lower,Zc_lower,lev_bnd)
  call INTERP_RTM_KNOWING_Z(ilon,ilat,ivza,Zc_lower,lev_bnd,&
                            trans_ac_3b, trans_ac_4, trans_ac_5, &
                            rad_ac_3b,rad_ac_4,rad_ac_5)
  Bc_4 = PLANCK_RAD_FAST(4,Tc_lower)
  Bc_5 = PLANCK_RAD_FAST(5,Tc_lower)
  rad_clear_4 = rad_ac_4 + trans_ac_4*Bc_4
  rad_clear_5 = rad_ac_5 + trans_ac_5*Bc_5

 end do element_loop
end do line_loop

return

!----- Pfaast error statements
200     print *, "Error in PFAST RTM Calculation, stopping"
stop

end subroutine GET_PIXEL_NWP_RTM


!====================================================================
! SUBROUTINE Name: FIND_RTM_LEVELS
!
! Function:
! Finds various key levels in the RTM (tropopause, etc), for each RTM gridcell
!
!====================================================================
subroutine FIND_RTM_LEVELS(ilon,ilat)

 integer (kind=int4), intent(in):: ilon, ilat
 integer (kind=int4):: k

   !--- find surface level - this is closest nwp level above the actual surface
   rtm(ilon,ilat)%sfc_level = nlevels_rtm
   do k = nlevels_rtm,1,-1
       if (P_std_rtm(k) < Psfc_nwp(ilon,ilat)) then
            rtm(ilon,ilat)%sfc_level = k
            exit
       endif
   enddo

   !--------------------------------------------------------------------
   !--- find tropopause level  based on tropopause pressure
   !--- tropopause is between tropopause_level and tropopaue_level + 1
   !--------------------------------------------------------------------
   do k = 1, rtm(ilon,ilat)%sfc_level-1
      if ((P_std_rtm(k) <= P_trop_nwp(ilon,ilat)) .and. &
          (P_std_rtm(k+1) > P_trop_nwp(ilon,ilat))) then
        rtm(ilon,ilat)%tropo_level = k
      endif
   enddo

!--- check if tropopause level found
    if (rtm(ilon,ilat)%tropo_level == 0) then
       print *, "No trop  = ", t_trop_nwp(ilon,ilat), p_trop_nwp(ilon,ilat)
       print *, "Error, tropopause level not found, stopping "
       stop
    endif         

   do k = 1, rtm(ilon,ilat)%sfc_level-1
      if ((P_std_rtm(k) <= 850.0) .and. &
          (P_std_rtm(k+1) > 850.0)) then
        rtm(ilon,ilat)%level850 = k
      endif
   enddo

  !---------------------------------------------------------------------
  ! find inversion level - highest level inversion below trop
  !---------------------------------------------------------------------
   rtm(ilon,ilat)%inversion_level = 0
   do k = rtm(ilon,ilat)%tropo_level,rtm(ilon,ilat)%sfc_level-1
      if ((rtm(ilon,ilat)%T_prof(k) - rtm(ilon,ilat)%T_prof(k+1) > delta_t_inversion) .and. &
          (P_std_rtm(k) >= p_inversion_min)) then
          rtm(ilon,ilat)%inversion_level = k
          exit 
      endif
   enddo

end subroutine FIND_RTM_LEVELS

!====================================================================
! SUBROUTINE Name: COMPUTE_TPW_PROFILE_NWP
!
! Function:
! Computes the TPW profile in the NWP
!
!====================================================================
subroutine COMPUTE_TPW_PROFILE_NWP(ilon,ilat)
  integer, intent(in):: ilon, ilat
  integer:: k
  real:: e, es, w_mean,u_layer, wvmr_sfc

!--- make wvmr_nwp for use in RTM
  do k = 1, nlevels_nwp
    es = vapor(T_prof_nwp(k,ilon,ilat))
    e = rh_prof_nwp(k,ilon,ilat) * es / 100.0
    wvmr_nwp(k) = 1000.0*0.622 * (e / (P_std_nwp(k) - e))  !(g/kg)
  enddo

!--- make wvmr at sfc
    es = vapor(tmpsfc_nwp(ilon,ilat))
    e = rhsfc_nwp(ilon,ilat) * es / 100.0
    wvmr_sfc = 1000.0*0.622 * (e / (Psfc_nwp(ilon,ilat) - e))  !(g/kg)


!--- make tpw_prof_nwp for use in atmospheric correction
  tpw_prof_nwp(1,ilon,ilat) = 0.0
  do k = 1, nlevels_nwp-1   !k is now a layer index
    w_mean = 0.5*(wvmr_nwp(k+1)+wvmr_nwp(k)) / 1000.0  !(kg/kg)
    u_layer = (10.0/g)*(P_std_nwp(k+1)-P_std_nwp(k))*w_mean
    tpw_prof_nwp(k+1,ilon,ilat) = tpw_prof_nwp(k,ilon,ilat) + u_layer
  enddo

end subroutine COMPUTE_TPW_PROFILE_NWP

!====================================================================
! SUBROUTINE Name: CONVERT_NWP_H2O
!
! Function:
! Converts the water tempurature profile for use in the RTM
!
!
! e = water vapor pressure in mb
! es = water saturation vapor pressure in mb
! u_layer = mass of water in layer (g/cm^2)
! T_mean = layer mean temperature (K)
! P_mean = layer mean pressure (mb)
!====================================================================
subroutine CONVERT_NWP_H2O(ilon,ilat)
  integer, intent(in):: ilon, ilat
  integer:: k
  real:: e, es

!--- make wvmr_nwp for use in RTM
  do k = 1, nlevels_nwp
    es = vapor(t_prof_nwp(k,ilon,ilat))
    e = rh_prof_nwp(k,ilon,ilat) * es / 100.0
    wvmr_nwp(k) = 1000.0*0.622 * (e / (P_std_nwp(k) - e))  !(g/kg)
  enddo

end subroutine CONVERT_NWP_H2O

!====================================================================
! SUBROUTINE Name: ALLOCATE_RTM
!
! Function:
! Subroutine that allocates memory for nlon x nlat rtm structure.
!
!====================================================================

subroutine ALLOCATE_RTM(nx,ny)
  integer (kind=int4), intent(in) :: nx, ny

  integer :: astatus

  allocate(rtm(nx,ny),stat=astatus)
  if (astatus /= 0) then
    print "(a,'Not enough memory to allocate rtm_params structure.')",EXE_PROMPT
    stop
  endif

end subroutine ALLOCATE_RTM

!====================================================================
! SUBROUTINE Name: DEALLOCATE_RTM
!
! Function:
!  Subroutine that deallocates memory used for nlon x nlat rtm structure.
!
!====================================================================

subroutine DEALLOCATE_RTM()

  integer :: astatus

  deallocate(rtm,stat=astatus)
  if (astatus /= 0) then
    print "(a,'Error deallocating rtm_params structure.')",EXE_PROMPT
    stop
  endif

end subroutine DEALLOCATE_RTM

!====================================================================
! SUBROUTINE Name: ALLOCATE_RTM_CELL
!
! Function:
!  Subroutine to allocate memory for the RTM structure.
!
!====================================================================

subroutine ALLOCATE_RTM_CELL(ilon,ilat,nvza)
  integer (kind=int4), intent(in) :: ilon, ilat
  integer (kind=int4), intent(in) :: nvza

  integer :: astatus

  allocate(rtm(ilon, ilat)%d(nvza),stat=astatus)
  allocate(rtm(ilon, ilat)%T_prof(nlevels_rtm),stat=astatus)
  allocate(rtm(ilon, ilat)%Z_prof(nlevels_rtm),stat=astatus)
  allocate(rtm(ilon, ilat)%wvmr_prof(nlevels_rtm),stat=astatus)
  allocate(rtm(ilon, ilat)%ozmr_prof(nlevels_rtm),stat=astatus)

  if (astatus /= 0) then
    print "(a,'Not enough memory to allocate rtm_params structure.')",EXE_PROMPT
    stop
  endif

  rtm(ilon,ilat)%flag = 1

end subroutine ALLOCATE_RTM_CELL

!====================================================================
! SUBROUTINE Name: DEALLOCATE_RTM_CELL
!
! Function:
!  Subroutine to deallocates memory for the RTM structure.
!
!====================================================================

subroutine DEALLOCATE_RTM_CELL(ilon,ilat)
  integer (kind=int4), intent(in) :: ilon, ilat

  integer :: astatus
  integer :: astatus_sum

  astatus_sum = 0

  deallocate(rtm(ilon,ilat)%d,stat=astatus)
  astatus_sum = astatus_sum + astatus

  deallocate(rtm(ilon,ilat)%T_prof,stat=astatus)
  astatus_sum = astatus_sum + astatus

  deallocate(rtm(ilon,ilat)%Z_prof,stat=astatus)
  astatus_sum = astatus_sum + astatus

  deallocate(rtm(ilon,ilat)%wvmr_prof,stat=astatus)
  astatus_sum = astatus_sum + astatus

  deallocate(rtm(ilon,ilat)%ozmr_prof,stat=astatus)
  astatus_sum = astatus_sum + astatus

  if (astatus_sum /= 0) then
    print "(a,'Error deallocating rtm_params structure.')",EXE_PROMPT
    stop
  endif

  
  rtm(ilon,ilat)%flag = 0

  rtm(ilon,ilat)%sfc_level = 0

  rtm(ilon,ilat)%inversion_level = 0

  rtm(ilon,ilat)%tropo_level = 0

  rtm(ilon,ilat)%level850 = 0

end subroutine DEALLOCATE_RTM_CELL

!====================================================================
! SUBROUTINE Name: DEALLOCATE_RTM_VARS
!
! Function:
!  Subroutine to deallocate memory for the RTM profile arrays.
!
!====================================================================

subroutine DEALLOCATE_RTM_VARS(ilon,ilat,ivza)
  integer (kind=int4), intent(in) :: ilon
  integer (kind=int4), intent(in) :: ilat
  integer (kind=int4), intent(in) :: ivza

  integer :: astatus

    deallocate(rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b,&
               rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar, &
               rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b_solar_total, &
               rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b, &
               rtm(ilon,ilat)%d(ivza)%cloud_prof3b, stat=astatus)
    if (astatus /= 0) then
      print "(a,'Error destroying allocate channel 3b rtm profile.')",EXE_PROMPT
      stop
    endif

    deallocate(rtm(ilon,ilat)%d(ivza)%trans_atm_clr4,&
               rtm(ilon,ilat)%d(ivza)%rad_atm_clr4, &
               rtm(ilon,ilat)%d(ivza)%cloud_prof4, stat=astatus)

    if (astatus /= 0) then
      print "(a,'Error destroying allocate channel 4 rtm profile.')",EXE_PROMPT
      stop
    endif

    deallocate(rtm(ilon,ilat)%d(ivza)%trans_atm_clr5,&
               rtm(ilon,ilat)%d(ivza)%rad_atm_clr5, &
               rtm(ilon,ilat)%d(ivza)%cloud_prof5, stat=astatus)

    if (astatus /= 0) then
      print "(a,'Error destroying allocate channel 5 rtm profile.')",EXE_PROMPT
      stop
    endif

!--- set flag to zero (so we don't deallocate again)
    rtm(ilon,ilat)%d(ivza)%flag = 0

end subroutine DEALLOCATE_RTM_VARS

!====================================================================
! SUBROUTINE Name: INTERP_RTM_KNOWING_Z
!
! Function:
!  knowing the height (Z) profile, this interpolates various RTM parameters
!
!====================================================================

 subroutine INTERP_RTM_KNOWING_Z(ilon,ilat,ivza,Zc,ilev,trans_ac_3b,trans_ac_4,trans_ac_5, &
                       rad_ac_3b,rad_ac_4,rad_ac_5)

 real, intent(in):: Zc
 integer, intent(in):: ilon
 integer, intent(in):: ilat
 integer, intent(in):: ivza
 integer, intent(in):: ilev
 real, intent(out):: trans_ac_3b
 real, intent(out):: trans_ac_4
 real, intent(out):: trans_ac_5
 real, intent(out):: rad_ac_3b
 real, intent(out):: rad_ac_4
 real, intent(out):: rad_ac_5
 real:: dz
 
  dz = rtm(ilon,ilat)%Z_prof(ilev+1) - rtm(ilon,ilat)%Z_prof(ilev)

!--- estimate above cloud rad and trans  by interpolating precomputed rad,trans profiles in Z

       if (dz < 0.0) then

         trans_ac_3b = rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ilev) + &
           (Zc - rtm(ilon,ilat)%z_prof(ilev)) * &
           (rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ilev+1) - &
            rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ilev)) / dz

         rad_ac_3b = rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ilev) + &
           (Zc - rtm(ilon,ilat)%z_prof(ilev)) * &
           (rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ilev+1) -  &
            rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ilev)) / dz

         trans_ac_4 = rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ilev) + &
           (Zc - rtm(ilon,ilat)%Z_prof(ilev)) * &
           (rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ilev+1) - &
            rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ilev)) / dz

         rad_ac_4 = rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ilev) + &
           (Zc - rtm(ilon,ilat)%Z_prof(ilev)) * &
           (rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ilev+1) -  &
            rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ilev)) / dz

         trans_ac_5 = rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ilev) + &
           (Zc - rtm(ilon,ilat)%Z_prof(ilev)) * &
           (rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ilev+1) - &
            rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ilev)) / dz

         rad_ac_5 = rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ilev) + &
           (Zc - rtm(ilon,ilat)%Z_prof(ilev)) * &
           (rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ilev+1) -  &
            rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ilev)) / dz

       else

          trans_ac_3b = rtm(ilon,ilat)%d(ivza)%trans_atm_clr3b(ilev)
          rad_ac_3b = rtm(ilon,ilat)%d(ivza)%rad_atm_clr3b(ilev)
          trans_ac_4 = rtm(ilon,ilat)%d(ivza)%trans_atm_clr4(ilev)
          rad_ac_4 = rtm(ilon,ilat)%d(ivza)%rad_atm_clr4(ilev)
          trans_ac_5 = rtm(ilon,ilat)%d(ivza)%trans_atm_clr5(ilev)
          rad_ac_5 = rtm(ilon,ilat)%d(ivza)%rad_atm_clr5(ilev)

       endif

 end subroutine INTERP_RTM_KNOWING_Z

!====================================================================
! SUBROUTINE Name: KNOWING_P_COMPUTE_T_Z_RTM
!
! Function:
!  derive height and temperature from a profile knowing pressure
!
!====================================================================

 subroutine KNOWING_P_COMPUTE_T_Z_RTM(ilon,ilat,P,T,Z,ilev)

  !--- arguments
  integer, intent(in):: ilon
  integer, intent(in):: ilat
  real, intent(in):: P
  real, intent(out):: T
  real, intent(out):: Z
  integer, intent(out):: ilev

  !--- local variables
  real:: dp
  real:: dt
  real:: dz

  !--- verify that profile exist for this location
  if (rtm(ilon,ilat)%flag == sym%YES) then

    !--- interpolate pressure profile
    call LOCATE(P_std_rtm,nlevels_rtm,P,ilev)
    ilev = max(1,min(nlevels_rtm-1,ilev))

    dp = P_std_rtm(ilev+1) - P_std_rtm(ilev)
    dt = rtm(ilon,ilat)%T_prof(ilev+1) - rtm(ilon,ilat)%T_prof(ilev)
    dz = rtm(ilon,ilat)%Z_prof(ilev+1) - rtm(ilon,ilat)%Z_prof(ilev)

    !--- perform interpolation
    if (dp /= 0.0) then
      T = rtm(ilon,ilat)%T_prof(ilev) + dt/dp * (P - P_std_rtm(ilev))
      Z = rtm(ilon,ilat)%Z_prof(ilev) + dz/dp * (P - P_std_rtm(ilev))
    else
      T = rtm(ilon,ilat)%T_prof(ilev) 
      Z = rtm(ilon,ilat)%Z_prof(ilev)
    endif
   else
      T = missing_value_real4
      Z = missing_value_real4
      print "(a,'Error, accessing unitialized RTM profile.')",EXE_PROMPT
   endif

 end subroutine KNOWING_P_COMPUTE_T_Z_RTM

!====================================================================
! SUBROUTINE Name: KNOWING_Z_COMPUTE_T_P_RTM
!
! Function:
!  derive pressure and temperature from a profile knowing height
!
!====================================================================

 subroutine KNOWING_Z_COMPUTE_T_P_RTM(ilon,ilat,P,T,Z,ilev)

  !--- arguments
  integer, intent(in):: ilon
  integer, intent(in):: ilat
  real, intent(in):: Z
  real, intent(out):: T
  real, intent(out):: P
  integer, intent(out):: ilev

  !--- local variables
  real:: dp
  real:: dt
  real:: dz
  integer:: kstart
  integer:: kend
  integer:: nlevels_temp

  !--- compute region of atmosphere to look
  kstart = rtm(ilon,ilat)%tropo_level
  kend = rtm(ilon,ilat)%sfc_level
  nlevels_temp = kend - kstart + 1

  !--- interpolate pressure profile
  call LOCATE(rtm(ilon,ilat)%Z_prof(kstart:kend),nlevels_temp,Z,ilev)
  ilev = max(1,min(nlevels_temp-1,ilev))
  ilev = ilev + kstart - 1

  dp = P_std_rtm(ilev+1) - P_std_rtm(ilev)
  dt = rtm(ilon,ilat)%T_prof(ilev+1) - rtm(ilon,ilat)%T_prof(ilev)
  dz = rtm(ilon,ilat)%Z_prof(ilev+1) - rtm(ilon,ilat)%Z_prof(ilev)

  !--- perform interpolation
  if (dz /= 0.0) then
   T = rtm(ilon,ilat)%T_prof(ilev) + dt/dz * (Z - rtm(ilon,ilat)%Z_prof(ilev))
   P = P_std_rtm(ilev) + dp/dz * (Z - rtm(ilon,ilat)%Z_prof(ilev))
  else
   T = rtm(ilon,ilat)%T_prof(ilev) 
   P = P_std_rtm(ilev)
  endif

 end subroutine KNOWING_Z_COMPUTE_T_P_RTM

!====================================================================
! SUBROUTINE Name: KNOWING_T_COMPUTE_P_Z_RTM
!
! Function:
!  derive pressure and height from a profile knowing temperature
!
!====================================================================

 subroutine KNOWING_T_COMPUTE_P_Z_RTM(ilon,ilat,P,T,Z,klev,i,j,ierr)

  !--- arguments
  integer, intent(in):: ilon
  integer, intent(in):: ilat
  integer, intent(in):: i
  integer, intent(in):: j
  real, intent(in):: T
  real, intent(out):: Z
  real, intent(out):: P
  integer, intent(out):: klev

  !--- local variables
  real:: dp
  real:: dt
  real:: dz
  integer:: kstart
  integer:: kend
  integer:: nlevels_temp
  integer:: ierr
  integer:: level_within_inversion_flag
  real:: z1
  real:: z2
  real:: t1
  real:: t2

  !--- initialization
  ierr = sym%NO
  Z = missing_value_real4
  P = missing_value_real4

  !--- test for existence of a valid solution with troposphere
  kstart = rtm(ilon,ilat)%tropo_level
  kend = rtm(ilon,ilat)%sfc_level
  nlevels_temp = kend - kstart + 1

     
!print *, "in knowing t compute p z"
!print *, "level range = ", kstart,kend 
!print *, "T range = ", rtm(ilon,ilat)%T_prof(kstart),rtm(ilon,ilat)%T_prof(kend) 
!print *, "Z range = ", rtm(ilon,ilat)%Z_prof(kstart),rtm(ilon,ilat)%Z_prof(kend) 
  !--- interpolate temperature profile

  !--- check to see if warmer than max, than assume at surface
  if (T > maxval(rtm(ilon,ilat)%T_prof(kstart:kend))) then
    P = P_std_rtm(kend)
    Z = rtm(ilon,ilat)%Z_prof(kend)
    klev = kend - 1
    ierr = sym%NO
    return
  endif

  !--- check to see if colder than min, than assume tropopause
  if (T < minval(rtm(ilon,ilat)%T_prof(kstart:kend))) then
    P = P_std_rtm(kstart)
    Z = rtm(ilon,ilat)%Z_prof(kstart)
    klev = kstart + 1
    ierr = sym%NO
    return
  endif
  
  !--- if there is an inversion, look below first
  level_within_inversion_flag = 0
  if (rtm(ilon,ilat)%inversion_level > 0) then
   kstart = rtm(ilon,ilat)%inversion_level
   kend = rtm(ilon,ilat)%sfc_level
   nlevels_temp = kend - kstart + 1
   call LOCATE(rtm(ilon,ilat)%T_prof(kstart:kend),nlevels_temp,T,klev)
   if ((klev > 0) .and. (klev < nlevels_temp -1)) then
     klev = klev + kstart - 1
     level_within_inversion_flag = 1
   endif
  endif

  !--- if no solution within an inversion, look above
  if (level_within_inversion_flag == 0) then

    kstart = rtm(ilon,ilat)%tropo_level
    kend = rtm(ilon,ilat)%sfc_level
    nlevels_temp = kend - kstart + 1
    call LOCATE(rtm(ilon,ilat)%T_prof(kstart:kend),nlevels_temp,T,klev)
    klev = klev + kstart - 1
    klev = max(1,min(nlevels_rtm-1,klev))

  endif

  !-- if solution is above trop, set to trop values
  if (klev < rtm(ilon,ilat)%tropo_level) then

    P = P_std_rtm(rtm(ilon,ilat)%tropo_level)
    Z = rtm(ilon,ilat)%Z_prof(rtm(ilon,ilat)%tropo_level)

  else

    !--- determine derivatives
    dp = P_std_rtm(klev+1) - P_std_rtm(klev)

    if ((smooth_nwp == sym%NO) .or. &
        (rtm(i_nwp(i,j),j_nwp(i,j))%flag == sym%NO) .or. &
        (rtm(i_nwp_x(i,j),j_nwp(i,j))%flag == sym%NO) .or. &
        (rtm(i_nwp(i,j),j_nwp_x(i,j))%flag == sym%NO) .or. &
        (rtm(i_nwp_x(i,j),j_nwp_x(i,j))%flag == sym%NO)) then

     z2 = rtm(ilon,ilat)%Z_prof(klev+1)
     z1 = rtm(ilon,ilat)%Z_prof(klev)
     t2 = rtm(ilon,ilat)%T_prof(klev+1)
     t1 = rtm(ilon,ilat)%T_prof(klev)

    else


    !--- spatially smooth temperature profile

     z1 = INTERPOLATE_NWP(rtm(i_nwp(i,j),j_nwp(i,j))%Z_prof(klev), &
                          rtm(i_nwp_x(i,j),j_nwp(i,j))%Z_prof(klev), &
                          rtm(i_nwp(i,j),j_nwp_x(i,j))%Z_prof(klev), &
                          rtm(i_nwp_x(i,j),j_nwp_x(i,j))%Z_prof(klev), &
                          lon_nwp_fac(i,j),lat_nwp_fac(i,j))

     z2 = INTERPOLATE_NWP(rtm(i_nwp(i,j),j_nwp(i,j))%Z_prof(klev+1), &
                          rtm(i_nwp_x(i,j),j_nwp(i,j))%Z_prof(klev+1), &
                          rtm(i_nwp(i,j),j_nwp_x(i,j))%Z_prof(klev+1), &
                          rtm(i_nwp_x(i,j),j_nwp_x(i,j))%Z_prof(klev+1), &
                          lon_nwp_fac(i,j),lat_nwp_fac(i,j))

     t1 = INTERPOLATE_NWP(rtm(i_nwp(i,j),j_nwp(i,j))%T_prof(klev), &
                          rtm(i_nwp_x(i,j),j_nwp(i,j))%T_prof(klev), &
                          rtm(i_nwp(i,j),j_nwp_x(i,j))%T_prof(klev), &
                          rtm(i_nwp_x(i,j),j_nwp_x(i,j))%T_prof(klev), &
                          lon_nwp_fac(i,j),lat_nwp_fac(i,j))

     t2 = INTERPOLATE_NWP(rtm(i_nwp(i,j),j_nwp(i,j))%T_prof(klev+1), &
                          rtm(i_nwp_x(i,j),j_nwp(i,j))%T_prof(klev+1), &
                          rtm(i_nwp(i,j),j_nwp_x(i,j))%T_prof(klev+1), &
                          rtm(i_nwp_x(i,j),j_nwp_x(i,j))%T_prof(klev+1), &
                          lon_nwp_fac(i,j),lat_nwp_fac(i,j))

      !---- check to see if spatially interpolated values still surround T
      !---- if not, ignore spatial interpolation
      if ( (T < t1 .and. T < t2) .or. &
           (T > t1 .and. T > t2)) then
         !print *, "failure in T,Z spatial interp, turning off interp for this point"
         z2 = rtm(ilon,ilat)%Z_prof(klev+1)
         z1 = rtm(ilon,ilat)%Z_prof(klev)
         t2 = rtm(ilon,ilat)%T_prof(klev+1)
         t1 = rtm(ilon,ilat)%T_prof(klev)
      endif

    endif

    dz = z2 - z1
    dt = t2 - t1

    !--- perform interpolation
    if (dt /= 0.0) then
      P = P_std_rtm(klev) + dp/dt * (T - t1)
      Z = z1 + dz/dt * (T - t1)
    else
      P = P_std_rtm(klev) 
      Z = z1
    endif

  !--- end above trop check
  endif

  !print *, "end of routine ", P, Z


 end subroutine KNOWING_T_COMPUTE_P_Z_RTM
 
!====================================================================
! FUNCTION Name: calculate_cloud_emissivity
!
! Function:
!  Computes the emissivity for a given  temperature
!
!====================================================================

FUNCTION calculate_cloud_emissivity(Robs,Rclr,Rcld,min_emiss,max_emiss) result(emissivity)
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Rcld
  REAL(kind=real4), optional, intent(in) :: min_emiss
  REAL(kind=real4), optional, intent(in) :: max_emiss
  REAL(kind=real4) :: emissivity
  REAL(kind=real4) :: min_emiss_allowed
  REAL(kind=real4) :: max_emiss_allowed
  
  min_emiss_allowed = 1.0e-5
  max_emiss_allowed = 0.9999
  
  if (present(min_emiss)) min_emiss_allowed = min_emiss
  if (present(max_emiss)) max_emiss_allowed = max_emiss
  
  emissivity = (Robs - Rclr)/(Rcld - Rclr)
  
  emissivity = min(emissivity,max_emiss_allowed)  
  if (emissivity < min_emiss_allowed) emissivity = missing_value_real4

  return

END FUNCTION calculate_cloud_emissivity

!====================================================================
! FUNCTION Name: nadir_cloud_emissivity
!
! Function:
!  Corrects the cloud emissivity for viewing angle. 
!
!====================================================================

FUNCTION nadir_cloud_emissivity(emissivity, cos_viewing_angle, corrected_tau) result(corrected_emissivity)
  REAL(kind=real4), intent(in) :: emissivity
  REAL(kind=real4), intent(in) :: cos_viewing_angle
  REAL(kind=real4), optional, intent(out) :: corrected_tau
  REAL(kind=real4) :: tau
  REAL(kind=real4) :: corrected_emissivity
  
  corrected_emissivity = missing_value_real4
  tau = missing_value_real4
  
  if (emissivity > 0.0 .and. emissivity < 1.0) then
    tau = (-1.0)*alog(1.0 - emissivity)*cos_viewing_angle
    corrected_emissivity = 1.0 - exp((-1.0)*tau)
  endif
  
  if (present(corrected_tau)) corrected_tau = tau

  return

END FUNCTION nadir_cloud_emissivity

!====================================================================
! FUNCTION Name: calculate_cloud_beta
!
! Function:
!  Computes the beta ratio for two emissivities. 
!
!====================================================================

FUNCTION calculate_cloud_beta(emiss_top, emiss_bot) result(beta)
  REAL(kind=real4), intent(in) :: emiss_top
  REAL(kind=real4), intent(in) :: emiss_bot
  REAL(kind=real4) :: beta
  
  beta = missing_value_real4
  
  if (emiss_top > 0.0 .and. emiss_top < 1.0 .and. &
      emiss_bot > 0.0 .and. emiss_bot < 1.0) then
    beta = alog(1.0 - emiss_top)/alog(1.0 - emiss_bot)
  endif
  
  return

END FUNCTION calculate_cloud_beta

!====================================================================
! FUNCTION Name: calculate_demiss_dBsfc
!
! Description:
!   Function to compute the derivative of cloud emissivity with
!       respect to the surface blackbody radiance. 
!====================================================================

FUNCTION calculate_demiss_dBsfc(emiss_sfc, trans_atm, Rcld, Rclr, Robs) result(demiss_dBsfc)
  REAL(kind=real4), intent(in) :: emiss_sfc
  REAL(kind=real4), intent(in) :: trans_atm
  REAL(kind=real4), intent(in) :: Rcld
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4) :: demiss_dBsfc
  
  demiss_dBsfc = (emiss_sfc*trans_atm)*(((Robs-Rclr)/(Rcld-Rclr)**2) - &
                 (1.0/(Rcld-Rclr)))

  return

END FUNCTION calculate_demiss_dBsfc

!====================================================================
! FUNCTION Name: calculate_demiss_dTsfc
!
! Description:
!   Function to computes the derivative of cloud emissivity with
! respect to the surface temperature. 
!====================================================================

FUNCTION calculate_demiss_dTsfc(ichan, Tsfc, emiss_sfc, trans_atm, &
                                Rcld, Rclr, Robs, dB_dTsfc) result(demiss_dTsfc)
  INTEGER(kind=int4), intent(in) :: ichan
  REAL(kind=real4), intent(in) :: Tsfc
  REAL(kind=real4), intent(in) :: emiss_sfc
  REAL(kind=real4), intent(in) :: trans_atm
  REAL(kind=real4), intent(in) :: Rcld
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4), optional, intent(out) :: dB_dTsfc
  REAL(kind=real4) :: demiss_dTsfc
  REAL(kind=real4) :: demiss_dBsfc
  REAL(kind=real4) :: dB_dTsfc_tmp
  REAL(kind=real4) :: junk
  
  junk = planck_rad_fast(ichan,Tsfc,dB_dT=dB_dTsfc_tmp)
  demiss_dBsfc = calculate_demiss_dBsfc(emiss_sfc, trans_atm, Rcld, Rclr, Robs)
  
  demiss_dTsfc = demiss_dBsfc*dB_dTsfc_tmp
  
  if (present(dB_dTsfc)) dB_dTsfc = dB_dTsfc_tmp

  return

END FUNCTION calculate_demiss_dTsfc

!====================================================================
! FUNCTION Name: calculate_dbeta_demiss
!
! Description:
!   Function to computes the derivative of beta with
!   respect to the cloud emissivity. 
!====================================================================

FUNCTION calculate_dbeta_demiss(emiss_top, emiss_bot, dbeta_demiss_top, &
                                dbeta_demiss_bot) result(status)
  REAL(kind=real4), intent(in) :: emiss_top
  REAL(kind=real4), intent(in) :: emiss_bot
  REAL(kind=real4), intent(out) :: dbeta_demiss_top
  REAL(kind=real4), intent(out) :: dbeta_demiss_bot
  INTEGER(kind=int4) :: status
  REAL(kind=real4) :: log_top
  REAL(kind=real4) :: log_bot
  
  status = sym%FAILURE
  
  if (emiss_top > 0.0 .and. emiss_top < 1.0 .and. &
      emiss_bot > 0.0 .and. emiss_bot < 1.0) then
  
    log_top = alog(1.0 - emiss_top)
    log_bot = alog(1.0 - emiss_bot)
    
    dbeta_demiss_top = (-1.0)*(1.0/log_bot)*(1.0/(1.0 - emiss_top))
    dbeta_demiss_bot = (log_top/log_bot**2)*(1.0/(1.0 - emiss_bot))

    status = sym%SUCCESS
    
  endif
  
  return

END FUNCTION calculate_dbeta_demiss

!====================================================================
! FUNCTION Name: calculate_dbeta_dTsfc
!
! Description:
!   Function to compute the derivative of cloud beta with
!       respect to the surface temperature. 
!====================================================================

FUNCTION calculate_dbeta_dTsfc(Tsfc, ichan_top, emiss_sfc_top, trans_atm_top, &
                               Rcld_top, Rclr_top, Robs_top, &
			       ichan_bot, emiss_sfc_bot, trans_atm_bot, &
                               Rcld_bot, Rclr_bot, Robs_bot, &
			       demiss_dTsfc_top, demiss_dTsfc_bot, &
			       dB_dTsfc_top, dB_dTsfc_bot) result(dbeta_dTsfc)
  
  REAL(kind=real4), intent(in) :: Tsfc
  INTEGER(kind=int4), intent(in) :: ichan_top
  REAL(kind=real4), intent(in) :: emiss_sfc_top
  REAL(kind=real4), intent(in) :: trans_atm_top
  REAL(kind=real4), intent(in) :: Rcld_top
  REAL(kind=real4), intent(in) :: Rclr_top
  REAL(kind=real4), intent(in) :: Robs_top
  INTEGER(kind=int4), intent(in) :: ichan_bot
  REAL(kind=real4), intent(in) :: emiss_sfc_bot
  REAL(kind=real4), intent(in) :: trans_atm_bot
  REAL(kind=real4), intent(in) :: Rcld_bot
  REAL(kind=real4), intent(in) :: Rclr_bot
  REAL(kind=real4), intent(in) :: Robs_bot
  REAL(kind=real4), optional, intent(out) :: demiss_dTsfc_top
  REAL(kind=real4), optional, intent(out) :: demiss_dTsfc_bot
  REAL(kind=real4), optional, intent(out) :: dB_dTsfc_top
  REAL(kind=real4), optional, intent(out) :: dB_dTsfc_bot
  REAL(kind=real4) :: dbeta_dTsfc
  REAL(kind=real4) :: demiss_dTsfc_top_tmp
  REAL(kind=real4) :: demiss_dTsfc_bot_tmp
  REAL(kind=real4) :: dB_dTsfc_top_tmp
  REAL(kind=real4) :: dB_dTsfc_bot_tmp
  REAL(kind=real4) :: emiss_top
  REAL(kind=real4) :: emiss_bot
  REAL(kind=real4) :: dbeta_demiss_top
  REAL(kind=real4) :: dbeta_demiss_bot
  INTEGER(kind=int4) :: status
  
  demiss_dTsfc_top_tmp = calculate_demiss_dTsfc(ichan_top, &
                                                Tsfc, &
						emiss_sfc_top, &
						trans_atm_top, &
						Rcld_top, &
						Rclr_top, &
						Robs_top, &
						dB_dTsfc=dB_dTsfc_top_tmp)
  
  demiss_dTsfc_bot_tmp = calculate_demiss_dTsfc(ichan_bot, &
                                                Tsfc, &
						emiss_sfc_bot, &
						trans_atm_bot, &
						Rcld_bot, &
						Rclr_bot, &
						Robs_bot, &
						dB_dTsfc=dB_dTsfc_bot_tmp)
  
  emiss_top = calculate_cloud_emissivity(Robs_top,Rclr_top,Rcld_top)
  emiss_bot = calculate_cloud_emissivity(Robs_bot,Rclr_bot,Rcld_bot)
  
  status = calculate_dbeta_demiss(emiss_top, emiss_bot, dbeta_demiss_top, dbeta_demiss_bot)
  
  if (status == sym%SUCCESS) then
    dbeta_dTsfc = dbeta_demiss_top*demiss_dTsfc_top_tmp + dbeta_demiss_bot*demiss_dTsfc_bot_tmp
  else
    dbeta_dTsfc = missing_value_real4
  endif
  
  if (present(demiss_dTsfc_top)) demiss_dTsfc_top = demiss_dTsfc_top_tmp
  if (present(demiss_dTsfc_bot)) demiss_dTsfc_bot = demiss_dTsfc_bot_tmp
  if (present(dB_dTsfc_top)) dB_dTsfc_top = dB_dTsfc_top_tmp
  if (present(dB_dTsfc_bot)) dB_dTsfc_bot = dB_dTsfc_bot_tmp
  
  return

END FUNCTION calculate_dbeta_dTsfc

!====================================================================
! FUNCTION Name: calculate_demiss_dBcld
!
! Description:
!   Function to compute the derivative of cloud emissivity with
! respect to the cloud blackbody radiance. 
!====================================================================

FUNCTION calculate_demiss_dBcld(trans_ac, Rcld, Rclr, Robs) result(demiss_dBcld)
  REAL(kind=real4), intent(in) :: trans_ac
  REAL(kind=real4), intent(in) :: Rcld
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4) :: demiss_dBcld
  
  demiss_dBcld = (-1.0)*(Robs - Rclr)*(trans_ac)/(Rcld - Rclr)**2

  return

END FUNCTION calculate_demiss_dBcld

!====================================================================
! FUNCTION Name: calculate_demiss_dTcld
!
! Description:
!   Function to compute the derivative of cloud emissivity with
! respect to the cloud temperature. 
!====================================================================

FUNCTION calculate_demiss_dTcld(ichan, Tcld, trans_ac, &
                                Rcld, Rclr, Robs, dB_dTcld) result(demiss_dTcld)
  INTEGER(kind=int4), intent(in) :: ichan
  REAL(kind=real4), intent(in) :: Tcld
  REAL(kind=real4), intent(in) :: trans_ac
  REAL(kind=real4), intent(in) :: Rcld
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4), optional, intent(out) :: dB_dTcld
  REAL(kind=real4) :: demiss_dTcld
  REAL(kind=real4) :: demiss_dBcld
  REAL(kind=real4) :: dB_dTcld_tmp
  REAL(kind=real4) :: junk
  
  junk = planck_rad_fast(ichan,Tcld,dB_dT=dB_dTcld_tmp)
  demiss_dBcld = calculate_demiss_dBcld(trans_ac, Rcld, Rclr, Robs)
  
  demiss_dTcld = demiss_dBcld*dB_dTcld_tmp
  
  if (present(dB_dTcld)) dB_dTcld = dB_dTcld_tmp

  return

END FUNCTION calculate_demiss_dTcld

!====================================================================
! FUNCTION Name: calculate_demiss_dRcld
!
! Description:
!   Function to compute the derivative of cloud emissivity with
! respect to the cloud profile. 
!====================================================================

FUNCTION calculate_demiss_dRcld(Rcld, Rclr, Robs) result(demiss_dRcld)

  REAL(kind=real4), intent(in) :: Rcld
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4) :: demiss_dRcld
  
  demiss_dRcld = (-1.0)*(Robs-Rclr)/(Rcld-Rclr)**2
  
  return
  
END FUNCTION calculate_demiss_dRcld

!====================================================================
! FUNCTION Name: apply_rtm_sensitivity
!
! Description:
!   Function to apply an RTM related derivative to its corresponding
! function given a delta and return its min and max values. 
!====================================================================

SUBROUTINE apply_rtm_sensitivity(rtm_param, rtm_derivative, delta, &
                                 min_value, max_value)
  
  REAL(kind=real4), intent(in) :: rtm_param
  REAL(kind=real4), intent(in) :: rtm_derivative
  REAL(kind=real4), intent(in) :: delta
  REAL(kind=real4), intent(out) :: min_value
  REAL(kind=real4), intent(out) :: max_value
  
  REAL(kind=real4) :: value1
  REAL(kind=real4) :: value2
  
  min_value = missing_value_real4
  max_value = missing_value_real4
  
  if (rtm_param /= missing_value_real4 .and. &
      rtm_derivative /= missing_value_real4) then
      
    value1 = rtm_param + rtm_derivative*delta
    value2 = rtm_param - rtm_derivative*delta
    
    min_value = min(value1,value2)
    max_value = max(value1,value2)
    
  endif

END SUBROUTINE apply_rtm_sensitivity

!====================================================================
! FUNCTION Name: bias_correct_radiance_using_bt
!
! Description:
!   Function to calculate a bias corrected radiance given a
! brightness temperature and a correction in brightness temperature
! units (K). 
!====================================================================

FUNCTION bias_correct_radiance_using_bt(ichan, bt, bias) result(rad)

  INTEGER(kind=int4), intent(in) :: ichan
  REAL(kind=real4), intent(in) :: bt
  REAL(kind=real4), intent(in) :: bias
  
  REAL(kind=real4) :: rad
  
  rad = planck_rad_fast(ichan, (bt + bias))
  
  return

END FUNCTION bias_correct_radiance_using_bt

!====================================================================
! FUNCTION Name: get_cloud_emissivity_level
!
! Description:
!   This function is used to interpolate the black cloud radiance term
! to a cloud emissivity.
!====================================================================

FUNCTION get_cloud_emissivity_level(cld_emiss, Rclr, Robs, cloud_prof, nprof, weight, ilev) result(status)
  REAL(kind=real4), intent(in) :: cld_emiss
  REAL(kind=real4), intent(in) :: Rclr
  REAL(kind=real4), intent(in) :: Robs
  REAL(kind=real4), dimension(:), intent(in) :: cloud_prof
  INTEGER(kind=int4), intent(in) :: nprof
  REAL(kind=real4), intent(out) :: weight
  INTEGER(kind=int4), intent(out) :: ilev
  
  INTEGER(kind=int4) :: status
  REAL(kind=real4) :: Rcld
  
  status = sym%FAILURE
  
  if (Rclr > Robs) then
    
    Rcld = (Robs + Rclr*(cld_emiss - 1.0))/cld_emiss
    Rcld = max(Rcld, minval(cloud_prof))
	     
    call locate(cloud_prof,nprof,Rcld,ilev)
    ilev = max(1,min(nprof-1,ilev))
	      
    weight = (Rcld - cloud_prof(ilev))/(cloud_prof(ilev+1) - cloud_prof(ilev))
    
    status = sym%SUCCESS
  
  endif
  
  return

END FUNCTION get_cloud_emissivity_level

!====================================================================
! FUNCTION Name: get_cloud_emissivity_level
!
! Description:
!   This function used to compute the ratio of two byte scaled reflectance
! values.
!====================================================================

FUNCTION Scaled_Reflectance_Ratio(Ref_Num, &
                                  Min_Num, &
                                  Max_Num, &
                                  Ref_Den, &
                                  Min_Den, &
                                  Max_Den) RESULT(Rat)

  REAL(KIND=Real4), INTENT(IN) :: Ref_Num
  REAL(KIND=Real4), INTENT(IN) :: Min_Num
  REAL(KIND=Real4), INTENT(IN) :: Max_Num
  REAL(KIND=Real4), INTENT(IN) :: Ref_Den
  REAL(KIND=Real4), INTENT(IN) :: Min_Den
  REAL(KIND=Real4), INTENT(IN) :: Max_Den
  
  REAL(KIND=Real4) :: Rat
  
  Rat = Missing_Value_Real4
  
  RETURN

END FUNCTION Scaled_Reflectance_Ratio

end module RT_UTILITIES
