module SURFACE_PROPERTIES
! $Id: sfc_prop_umd.f90,v 1.9 2010/02/02 22:35:08 heidinger Exp $
!------------------------------------------------------------------
!@***h* PATMOSX/sfc_prop_umd.f90
! 
! NAME
!   sfc_prop_umd.f90
! 
! LOCATION
!   /patmosx/src/sfc_prop_umd.f90
! 
! PURPOSE
!   This is a module of surface properties (reflectance, emissivity) for
!   the UMD surface type classification
! 
! DESCRIPTION
!   This is a module of surface properties (reflectance, emissivity) for
!   the UMD surface type classification
!   Note: surface albedoes range from 0 to 1.0
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   unk
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!     0: Water
!     1: Evergreen Needleleaf Forests
!     2: Evergreen Broadleaf Forests
!     3: Deciduous Needleleaf Forests
!     4: Deciduous Broadleaf Forests
!     5: Mixed Forests
!     6: Woodlands
!     7: Wooded Grasslands/Shrubs
!     8: Closed Bushlands or Shrublands
!     9: Open Shrublands
!     10: Grasses
!     11: Cropland
!     12: Bare
!     13: Urban and Built
!     14: Snow  (not in original, based on input snow data)
!     15: Sea_Ice  (not in original, based on input snow data)
!
! EXTERNALS
!    SETUP_UMD_PROPS - assign values to radiative properties for each land type
!    READ_LAND_COVER_UMD - read the 8km UMD land cover type
!    GET_PIXEL_SFC_TYPE - spatially interpolate lover cover to each pixel
!
! INPUTS_OUTPUTS
!   Unit 7 is opened and closed in READ_LAND_COVER_UMD
!
!@*****
!------------------------------------------------------------------------
! this is module of surface properties (reflectance, emissivity) for
! the UMD surface type classification
!
!
!     0: Water
!     1: Evergreen Needleleaf Forests
!     2: Evergreen Broadleaf Forests
!     3: Deciduous Needleleaf Forests
!     4: Deciduous Broadleaf Forests
!     5: Mixed Forests
!     6: Woodlands
!     7: Wooded Grasslands/Shrubs
!     8: Closed Bushlands or Shrublands
!     9: Open Shrublands
!     10: Grasses
!     11: Cropland
!     12: Bare
!     13: Urban and Built
!     14: Snow  (not in original, based on input snow data)
!     15: Sea_Ice  (not in original, based on input snow data)
!
!  Note, surface albedoes range from 0 to 1.0
!
! Author: Andrew Heidinger, NOAA
!
! File I/O - Unit 7 is opened and closed in READ_LAND_COVER_UMD
!
! Public Routines
!  SETUP_UMD_PROPS - assign values to radiative properties for each land type
!  READ_LAND_COVER_UMD - read the 8km UMD land cover type
!  GET_PIXEL_SFC_TYPE - spatially interpolate lover cover to each pixel
!
! Private Routines: None
!
!------------------------------------------------------------------
use CONSTANTS
use AVHRR_PIXEL_COMMON

implicit none

private
public:: SETUP_UMD_PROPS,  &
         GET_PIXEL_SFC_EMISS_FROM_SFC_TYPE, &
         COMPUTE_BINARY_LAND_COAST_MASKS, &
         COMPUTE_COAST_MASK_FROM_LAND_MASK

integer, parameter, public:: ntype_sfc=16
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch1_sfc_alb_umd
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch2_sfc_alb_umd
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch3a_sfc_alb_umd
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch3b_sfc_alb_umd
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch3b_sfc_emiss_umd
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch4_sfc_emiss_umd
real(kind=real4), dimension(0:ntype_sfc-1), public,save:: ch5_sfc_emiss_umd

contains

!-----------------------------------------------------------------
! read into memory the radiative properties for each UMD type
!-----------------------------------------------------------------
subroutine SETUP_UMD_PROPS()

!--- channel 1 surface albedo based on PATMOS-x
!  ch1_sfc_alb_umd = (/0.0300, & !0
   ch1_sfc_alb_umd = (/0.0500, & !0
                       0.0406,  &  !1
                       0.0444,  &  !2
                       0.0406,  &  !3
                       0.0542,  &  !4
                       0.0423,  &  !5
                       0.0423,  &  !6
                       0.0576,  &  !7
                       0.1242,  &  !8
                       0.1781,  &  !9
                       0.0788,  &  !10
                       0.0677,  &  !11
                       0.3124,  &  !12
                       0.0918,  &  !13
                       0.9763,  &  !14
                       0.9763/)    !15

!--- channel 2 surface albedo based on PATMOS-x
     ch2_sfc_alb_umd = (/0.0163,  &  !0
                         0.1991,  &  !1
                         0.2179,  &  !2
                         0.1991,  &  !3
                         0.2943,  &  !4
                         0.2500,  &  !5
                         0.2500,  &  !6
                         0.2700,  &  !7
                         0.3072,  &  !8
                         0.2942,  &  !9
                         0.2919,  &  !10
                         0.2520,  &  !11
                         0.4019,  &  !12
                         0.2466,  &  !13
                         0.8194,  &  !14
                         0.8194/)    !15

!--- channel 3a surface albedo based on PATMOS-x
     ch3a_sfc_alb_umd = (/0.0163, & !0
                          0.2020, & !1
                          0.2210, & !2
                          0.2020, & !3
                          0.2980, & !4
                          0.2500, & !5
                          0.2500, & !6
                          0.2700, & !7
                          0.3400, & !8
                          0.2580, & !9
                          0.2980, & !10
                          0.2520, & !11
                          0.3900, & !12
                          0.2650, & !13
                          0.1743, & !14
                          0.1743/)  !15

!--- channel 3b surface emissivity based on SeeBor et al 2006
   ch3b_sfc_emiss_umd =(/0.985, &   !0
                     0.95,  &   !1
                     0.96,  &   !2
                     0.94,  &   !3
                     0.94,  &   !4
                     0.94,  &   !5
                     0.94,  &   !6
                     0.93,  &   !7
                     0.89,  &   !8
                     0.89,  &   !9
                     0.92,  &   !10
                     0.94,  &   !11
                     0.86,  &   !12
                     0.94,  &   !13
                     0.984, &   !14
                     0.984/)    !15

!--- channel 3b surface albedo based on SeeBor et al 2006
   ch3b_sfc_alb_umd = 1.0 - ch3b_sfc_emiss_umd

!--- channel 4 surface emissivity based on SeeBor et al 2006
   ch4_sfc_emiss_umd = (/0.985,  &   !0
                     0.965,  &   !1
                     0.961,  &   !2
                     0.963,  &   !3
                     0.963,  &   !4
                     0.966,  &   !5
                     0.962,  &   !6
                     0.961,  &   !7
                     0.961,  &   !8
                     0.958,  &   !9
                     0.969,  &   !10
                     0.965,  &   !11
                     0.955,  &   !12
                     0.960,  &   !13
                     0.979,  &   !14
                     0.979/)     !15

!--- channel 5 surface emissivity based on SeeBor et al 2006
   ch5_sfc_emiss_umd = (/0.985,  &   !0
                     0.967,  &   !1
                     0.965,  &   !2
                     0.965,  &   !3
                     0.966,  &   !4
                     0.968,  &   !5
                     0.965,  &   !6
                     0.966,  &   !7
                     0.969,  &   !8
                     0.968,  &   !9
                     0.964,  &   !10
                     0.969,  &   !11
                     0.971,  &   !12
                     0.962,  &   !13
                     0.977,  &   !14
                     0.977/)     !15
   
end subroutine SETUP_UMD_PROPS

!-----------------------------------------------------------------------
! If no sfc emiss data base read in, base it on the surface type.
! also derive surface reflectance from surface emissivity
! also reset sfc emiss if snow covered
!-----------------------------------------------------------------------
 subroutine GET_PIXEL_SFC_EMISS_FROM_SFC_TYPE(j1,j2)
    integer, intent(in):: j1, j2
    integer:: i, j
    
j_loop:  do j = j1,j1+j2-1

i_loop:    do i = 1,num_pix

!--- check for a bad pixel
         if (bad_pixel_mask(i,j) == sym%YES) then
            cycle
         endif
 
!--- based on surface type, assign surface emissivities if seebor not used
    if (use_seebor == sym%NO) then
     sfc_emiss_3b(i,j) = ch3b_sfc_emiss_umd(sfc_type(i,j))
     sfc_emiss_4(i,j) = ch4_sfc_emiss_umd(sfc_type(i,j))
     sfc_emiss_5(i,j) = ch5_sfc_emiss_umd(sfc_type(i,j))
    endif

!--- for ocean, use this parameterization from Nick Nalli - overwrite seebor
    if (sfc_type(i,j) == 0) then
!    sfc_emiss_4(i,j) =  0.844780 + 0.328921 * coszen(i,j)  -0.182375*(coszen(i,j)**2)
!    sfc_emiss_5(i,j) =  0.775019 + 0.474005 * coszen(i,j)  -0.261739*(coszen(i,j)**2)
!--- set to constant until we can verify the above
     sfc_emiss_3b(i,j) = 0.985
     sfc_emiss_4(i,j) = 0.985
     sfc_emiss_5(i,j) = 0.985
    endif

    !--- if (snow_mask
    if (snow(i,j) /= sym%NO_SNOW) then

      if ((sfc_type(i,j) /= sym%EVERGREEN_NEEDLE_SFC) .and. &
          (sfc_type(i,j) /= sym%EVERGREEN_BROAD_SFC)) then

      sfc_emiss_3b(i,j) = ch3b_sfc_emiss_umd(14)
      sfc_emiss_4(i,j) = ch4_sfc_emiss_umd(14)
      sfc_emiss_5(i,j) = ch5_sfc_emiss_umd(14)

     endif
    endif

    !--- define desert based on surface type
    desert_mask(i,j) = sym%NO
    if ((sfc_type(i,j) == 12) .and. (snow(i,j) == sym%NO_SNOW)) then
       desert_mask(i,j) = sym%YES
    endif

  end do i_loop
 end do j_loop

 end subroutine GET_PIXEL_SFC_EMISS_FROM_SFC_TYPE
!----------------------------------------------------------------------
! based on the coast and land flags, derive binary land and coast
! masks (yes/no)
!
! Note, coast mask is dependent on sensor resolution
!----------------------------------------------------------------------
 subroutine COMPUTE_BINARY_LAND_COAST_MASKS(j1,j2)
    integer, intent(in):: j1, j2
    integer:: i, j
    
j_loop:  do j = j1,j1+j2-1

i_loop:    do i = 1,num_pix

!--- check for a bad pixel
         if (bad_pixel_mask(i,j) == sym%YES) then
            cycle
         endif

!--- binary land mask
   land_mask(i,j) = sym%NO

!--- if land mask read in, use it
   if (read_land_mask == sym%YES) then
     if (land(i,j) == sym%LAND) then
       land_mask(i,j) = sym%YES
     endif
!--- if land mask not read in, base off of surface type
   else   
     if (sfc_type(i,j) /= sym%WATER_SFC) then
       land_mask(i,j) = sym%YES
     endif
   endif

!--- binary coast mask
   if (read_coast_mask == sym%YES) then
    coast_mask(i,j) = sym%NO

!-- for gac data
     if ((gac == sym%YES) .and. (coast(i,j) /= sym%NO_COAST)) then
      if (coast(i,j) <= sym%COAST_10KM) then
         coast_mask(i,j) = sym%YES
      endif
     endif

!-- for lac,hrpt or frac data
     if ((gac == sym%NO) .and. (coast(i,j) /= sym%NO_COAST)) then
      if (coast(i,j) <= sym%COAST_5KM) then
         coast_mask(i,j) = sym%YES
      endif
     endif

   endif



  end do i_loop
 end do j_loop



!------------- compute coast mask if coast data not read in
 if (read_coast_mask == sym%NO) then 

   call COMPUTE_COAST_MASK_FROM_LAND_MASK(j1,j2)

 endif

 end subroutine COMPUTE_BINARY_LAND_COAST_MASKS


!------------------------------------------------------------------------
! compute the data_mask array - this holds information pertaining to the
! type of pixel this is and how the cloud mask treats it
!------------------------------------------------------------------------------
 subroutine COMPUTE_COAST_MASK_FROM_LAND_MASK(jmin,jmax)
  integer, intent(in):: jmin,jmax
  integer:: i, j, i1, i2, j1,j2,nland,nmax,n

  n = 2    !determine size of box, 2 = 5x5
  do j = jmin,jmax

!--- check for bad scans
     if (bad_scan_flag(j) /= 0) then
       cycle
     endif

    j1 = max(jmin,j-n)
    j2 = min(j+n,jmax)

    do i = 1, num_pix

     i1 = max(1,i-n)
     i2 = min(i+n,int(num_pix, kind=int4))

!--- compute number of land pixels in box
     nland = sum(land_mask(i1:i2,j1:j2))

!--- compute maximum number of land pixels in box
     nmax = (j2-j1+1)*(i2-i1+1)

!--- compute coast mask
     coast_mask(i,j) = sym%YES
     if ((nland == nmax).or.(nland == 0)) then
      coast_mask(i,j) = sym%NO
     endif

   enddo
  enddo

end subroutine COMPUTE_COAST_MASK_FROM_LAND_MASK


end module SURFACE_PROPERTIES
